/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

(function ()
{
  Ext.BLANK_IMAGE_URL = 'images/s.gif';

  Ext.Loader.setConfig( {
    enabled : true, // enables dynamic (on-demand) class-loading
    disableCaching : true, // false = removes the _dc param from all loader's HTTP requests
    paths : {
      'Ext' : 'js/extjs',
      'QDesk' : 'js/quartzdesk'
    }
  } );

  Ext.require( [
    'QDesk.ApplicationConfig',
    'QDesk.Log',
    'QDesk.Msg',
    'QDesk.Properties',

    'QDesk.util.UxUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',

    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.Viewport'
  ] );


  /*
   * Register a global error handler for errors raised throughout the ExtJS framework and this application.
   * Note: to raise an error the application should use the Ext.Error.raise function.
   */
  Ext.Error.handle = function ( err )
  {
    Ext.log( { level : 'error', stack : true }, err );
    return QDesk.util.UxUtils.handleError( err );
  };

  Ext.application( {
    name : 'QDesk',
    appFolder : 'js/quartzdesk',

    controllers : [
      'navigation.HelpController',
      'navigation.NavigationController',

      'navigation.ConnectionsTreeToolbarController',

      'navigation.SettingsController',

      'scheduler.quartz.SchedulerController',

      'top.SwitchLocaleController',
      'top.LicenseInfoController'
    ],

    models : [
      'AboutData',
      'ConnectionsTreeNode',
      'JSScriptingObjectTreeNode',
      'Locale',
      'MessageMacroTreeNode',
      'NameValuePair',
      'RemoteJmxService',
      'Scheduler',
      'SettingsTreeNode',
      'TimeZone',

      'customization.gui.navigation.Link',
      'customization.gui.navigation.Node',
      'customization.gui.navigation.Tab',

      'message.Message',
      'message.MessageAttachment',
      'message.MessageChannelProfile',
      'message.MessageChannelProfilesTreeNode',

      'scheduler.ExecHealthIndicator',
      'scheduler.ExecHealthIndicatorEntry',

      'scheduler.quartz.QuartzCalendar',
      'scheduler.quartz.QuartzExecHistory',
      'scheduler.quartz.QuartzExecNotificationRule',
      'scheduler.quartz.QuartzExecutingJob',
      'scheduler.quartz.QuartzJobChain',
      'scheduler.quartz.QuartzJobDataMapEntry',
      'scheduler.quartz.QuartzJobDetail',
      'scheduler.quartz.QuartzJobGroup',
      'scheduler.quartz.QuartzTrigger',
      'scheduler.quartz.QuartzTriggerGroup',
      'scheduler.quartz.QuartzTriggerWithJobDetail',
      'scheduler.quartz.stats.QuartzExecStatData'
    ],

    /* Singleton stores that are automatically instantiated and registered with the Ext.StoreManager */
    stores : [
      'EmptyStore',

      'LocaleStore',
      'TimestampFormatStore',

      'enums.ExecStatusStore',
      'enums.ExpressionLanguageStore',
      'enums.JsLoggingLevelStore',
      'enums.MessageChannelStore',
      'enums.MessageReceiverWSVersionStore',
      'enums.NotificationSeverityStore',
      'enums.ProxyTypeStore',
      'enums.SnmpV3AuthenticationProtocolStore',
      'enums.SnmpV3PrivacyProtocolStore',
      'enums.SnmpV3SecurityLevelStore',
      'enums.WSAuthenticationTypeStore',

      'enums.JmxProtocolStore',

      'enums.scheduler.ExecNotificationRuleConditionTypeStore',
      'enums.scheduler.JobChainConditionTypeStore',
      'enums.scheduler.JobChainTargetExecTypeStore',

      'customization.gui.navigation.TabStore',

      'scheduler.quartz.QuartzJSScriptingObjectsStore',
      'scheduler.quartz.QuartzExecNotificationRuleMessageMacrosStore',
      'scheduler.quartz.stats.QuartzStatisticsTimePeriodLengthStore',
      'scheduler.quartz.stats.QuartzJobStatisticsTypeStore',
      'scheduler.quartz.stats.QuartzSchedulerStatisticsTypeStore',
      'scheduler.quartz.stats.QuartzTriggerStatisticsTypeStore'
    ],

    /*
     * Executed by ExtJS from Ext.onReady
     */
    launch : function ()
    {
      /* Stop propagating the contextmenu event from the application. This prevents the browser context menu from showing up. */
      Ext.getDoc().on( 'contextmenu', function ( event, targetHtmlEl )
      {
        // we need to enable context menus in "Chart preview" windows so that users can save images
        var targetEl = Ext.get( targetHtmlEl.id );
        if ( targetEl && targetEl.hasCls( 'x-chart-image' ) )
        {
          //
          return false;
        }
        event.stopEvent();
      } );

      /* Start of tooltip manager initialization. */
      // Init the singleton.  Any tag-based quick tips will start working.;
      Ext.tip.QuickTipManager.init();

      // Apply a set of config properties to the singleton
      Ext.apply( Ext.tip.QuickTipManager.getQuickTip(), {
        dismissDelay : 0, /* do not dismiss tooltips after 5000 ms period */
        maxWidth : 350,
        showDelay : 50 /* default is 500 and that is too long */
      } );
      /* End of tooltip manager initialization. */


      /* Start of ExtJS overrides */

      Ext.override( Ext.data.proxy.Server, {
        /* Parameter names used in Ajax data store requests */
        startParam : 'startAt',
        groupParam : 'groupBy',
        sortParam : 'sort',
        filterParam : 'filter',

        noCache : true  // true is default (a _dc param s appended to all proxy request URLs)
      } );

      /* Timeout for all Ext.Ajax.request calls. */
      Ext.Ajax.setTimeout( 1000 * QDesk.ApplicationConfig.get( 'ajaxTimeout' ) );

      /* Ext.Ajax.setTimeout() does not seem to affect Ajax proxies in stores, so we need this override. */
      Ext.override( Ext.data.Store, {
        load : function ()
        {
          var me = this;
          var proxy = me.getProxy();
          proxy.timeout = 1000 * QDesk.ApplicationConfig.get( 'ajaxTimeout' );
          me.callParent( arguments );
        }
      } );


      // SVG no longer available in ExtJS 5 and 6 charts. Override for the default sencha-charts SVG rasterizer URL.
      //Ext.override( Ext.draw.Container, {
      //  /*
      //   * Replaces the default ExtJS SVG export action URL "http://svg.sencha.io".
      //   */
      //  defaultDownloadServerUrl : QDesk.util.constant.ActionUrl.svgActionRasterize
      //} );


      /*
       * Forms are validated after rendering so that invalid fields are immediately visually
       * distinguished.
       *
       * Form validation also occurs upon form field change.
       */
      Ext.override( Ext.form.Panel, {
        initComponent : function ()
        {
          var me = this;

          me.callParent( arguments );

          me.on( 'afterrender', function ()
          {
            me.isValid();
          } );

          Ext.each( Ext.ComponentQuery.query( 'field', me ), function ( field )
          {
            field.on( 'change', function ()
            {
              me.isValid();
            } );
          } );
        }
      } );


      /*
       * Tooltip in label support for form fields.
       * Inspired by http://www.sencha.com/forum/showthread.php?19301
       */
      Ext.override( Ext.form.Field, {
        afterRender : function ()
        {
          var label = null;
          if ( this.fieldTooltip )
          {
            label = this._findHeaderDiv( this, 'label.x-form-item-label span.x-form-item-label-inner' );
            this._maybeAddToolTipToHeaderDiv( label, this.fieldTooltip );
          }

          if ( this.boxTooltip )
          {
            label = this._findHeaderDiv( this, 'label.x-form-cb-label' );
            this._maybeAddToolTipToHeaderDiv( label, this.boxTooltip );
          }

          this.callSuper();
        },

        _findHeaderDiv : function ( field, selector )
        {
          var wrapDiv = field.getEl().select( selector );
          return wrapDiv ? wrapDiv.child( 'label' ) : null;
        },

        _maybeAddToolTipToHeaderDiv : function ( label, tooltipText )
        {
          if ( label )
          {
            if ( this.fieldTooltipClosable )
            {
              label.createChild( {
                tag : 'img',
                src : 'images/icon/tooltip-non-auto-hide-10x10.png',
                style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
                'data-qtip' : Ext.htmlEncode( tooltipText ),
                'data-qtitle' : Ext.htmlEncode( QDesk.Msg.get( 'QDesk.tooltipNonAutoHideTitle' ) ),
                //'data-closable' : true,
                'data-hide' : 'user'  // disables automatic tooltip hiding when the user moves the cursor outside the tooltip icon
              } );
            }
            else
            {
              label.createChild( {
                tag : 'img',
                src : 'images/icon/tooltip-10x10.png',
                style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
                'data-qtip' : Ext.htmlEncode( tooltipText )
              } );
            }
          }
        }
      } );


      /*
       * Tooltip in label support for form fields.
       * Inspired by http://www.sencha.com/forum/showthread.php?19301
       */
      Ext.override( Ext.form.FieldSet, {
        afterRender : function ()
        {
          var headerDiv = null;
          if ( this.tooltip )
          {
            headerDiv = this._findHeaderDiv( this, 'div.x-fieldset-header-text' );
            this._maybeAddToolTipToHeaderDiv( headerDiv, this.tooltip );
          }

          this.callSuper();
        },

        _findHeaderDiv : function ( field, selector )
        {
          var wrapDiv = field.getEl().select( selector );
          return wrapDiv ? wrapDiv.child( 'div' ) : null;
        },

        _maybeAddToolTipToHeaderDiv : function ( headerDiv, tooltipText )
        {
          if ( headerDiv )
          {
            headerDiv.createChild( {
              tag : 'img',
              src : 'images/icon/tooltip-10x10.png',
              style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
              'data-qtip' : Ext.htmlEncode( tooltipText )
            } );
          }
        }
      } );


      /**
       * setCursorPosition and getCursorPosition for textfield, textarea etc.
       * http://www.sencha.com/forum/showthread.php?95486-Cursor-Position-in-TextField
       */
      Ext.override( Ext.form.field.Text, {

        setCaretPosition : function ( pos )
        {
          var el = this.inputEl.dom;
          if ( typeof(el.selectionStart) === "number" )
          {
            el.focus();
            el.setSelectionRange( pos, pos );
          }
          else if ( el.createTextRange )
          {
            var range = el.createTextRange();
            range.move( "character", pos );
            range.select();
          }
          else
          {
            //throw 'setCaretPosition() not supported';
          }
        },

        getCaretPosition : function ()
        {
          var el = this.inputEl.dom;
          if ( typeof(el.selectionStart) === "number" )
          {
            return el.selectionStart;
          }
          else if ( document.selection && el.createTextRange )
          {
            var range = document.selection.createRange();
            range.collapse( true );
            range.moveStart( "character", -el.value.length );
            return range.text.length;
          }
          else
          {
            //throw 'getCaretPosition() not supported';
          }
        }
      } );


      /**
       * JSON encoded Date instances are serialized using the ISO8601 format. Without this override the
       * format would be: yyyy-mm-ddThh:mm:ss (i.e. no time zone).
       */
      Ext.override( Ext.JSON, {
        encodeDate : function ( d )
        {
          return '"' + Ext.Date.format( d, QDesk.util.constant.DateFormat.timestampFormatIso8601 ) + '"';
        }
      } );


      /**
       * Localization of Ext.ux.TreeIconCombo texts.
       */
      Ext.override( Ext.ux.form.TreeIconCombo, {
        emptyValueText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.valueNotFoundText' ),
        valueNotFoundText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.valueNotFoundText' ),
        loadFailedText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.loadFailedText' ),
        loadingText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.loadingText' )
      } );


      /**
       * Default datetime format of the datetime value used by the DateTime component.
       */
      Ext.override( Ext.ux.form.field.DateTime, {
        dateTimeFormat : QDesk.util.constant.DateFormat.timestampFormatLong
      } );


      /**
       * Localization of the Chart Preview window title.
       */
      Ext.override( Ext.draw.ContainerBase, {
        preview : function ()
        {
          var image = this.getImage();
          new Ext.window.Window( {
            title : QDesk.Msg.get( 'Ext.draw.ContainerBase.previewWindowTitle' ),
            closeable : true,
            renderTo : Ext.getBody(),
            autoShow : true,
            maximizeable : true,
            maximized : true,
            border : true,
            layout : {
              type : 'hbox',
              pack : 'center',
              align : 'middle'
            },
            items : {
              xtype : 'container',
              items : {
                xtype : 'image',
                mode : 'img',
                cls : Ext.baseCSSPrefix + 'chart-image',
                src : image.data,
                listeners : {
                  afterrender : function ()
                  {
                    var me = this,
                        img = me.imgEl.dom,
                        ratio = image.type === 'svg' ? 1 : (window[ 'devicePixelRatio' ] || 1),
                        size;
                    if ( !img.naturalWidth || !img.naturalHeight )
                    {
                      img.onload = function ()
                      {
                        var width = img.naturalWidth,
                            height = img.naturalHeight;
                        me.setWidth( Math.floor( width / ratio ) );
                        me.setHeight( Math.floor( height / ratio ) );
                      };
                    }
                    else
                    {
                      size = me.getSize();
                      me.setWidth( Math.floor( size.width / ratio ) );
                      me.setHeight( Math.floor( size.height / ratio ) );
                    }
                  }
                }
              }
            }
          } );
        }
      } );


      /* Start of Temporary fixes */

      /*
       * This fixes the problem with the time field ignoring the seconds and milliseconds by
       * setValue( Date ).
       *
       * Sencha does not seem to consider this a bug, but we do. This problem is reported at
       * http://www.sencha.com/forum/showthread.php?275972-TimeField-don-t-consider-the-seconds-and-milliseconds
       */

      Ext.override( Ext.form.field.Time, {
        getInitDate : function ( hours, minutes, seconds, milliseconds )
        {
          var parts = this.initDateParts;
          return new Date( parts[ 0 ], parts[ 1 ], parts[ 2 ], hours || 0, minutes || 0, seconds || 0,
              milliseconds || 0 );
        },

        setValue : function ( v )
        {
          this.getPicker();
          if ( Ext.isDate( v ) )
          {
            v = this.getInitDate( v.getHours(), v.getMinutes(), v.getSeconds(), v.getMilliseconds() );
          }
          return this.callSuper( [ v ] );
        }
      } );

      /* End of Temporary fixes */

      /* End of ExtJS overrides */

      Ext.create( 'QDesk.view.Viewport' );
    }
  } );
})();
