/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * A custom form submit action that expects the user to provide all parameter values
 * in the {@code params} action config option.
 * To use this custom submit action, the form must be submitted using the {@code doAction}
 * function like so:
 * <pre>
 *   var formPanel = ...
 *   var form = formPanel.getForm();
 *   form.doAction('ParamsSubmit', {
 *     url : ...
 *     params : {
 *       param1 : param1Value,
 *       ...
 *       paramN : paramNValue,
 *     }
 *   });
 * </pre>
 * @param form the form to submit
 * @param options the options of the HTTP Request
 */
Ext.define( 'Ext.ux.ParamsSubmit', {
  extend : 'Ext.form.action.Submit',
  alternateClassName : 'Ext.form.Action.ParamsSubmit',
  alias : 'formaction.ParamsSubmit',

  type : 'ParamsSubmit',

  run : function ()
  {
    var method = this.getMethod();
    var isGet = method == 'GET';

    if ( this.clientValidation === false || this.form.isValid() )
    {
      Ext.Ajax.request( Ext.apply( this.createCallback( this.form ), {
        url : this.getUrl( isGet ),
        method : method,
        params : this.params,
        isUpload : this.form.fileUpload
      } ) );
    }
    else if ( this.clientValidation !== false )
    {
      // client validation failed
      this.failureType = Ext.form.Action.CLIENT_INVALID;
      this.form.afterAction( this, false );
    }
  }
} );
