/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Simple logging facade built on top of ExtJS logging API.
 */
Ext.define( 'QDesk.Log', {
  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.enums.JsLoggingLevel'
  ],

  singleton : true,

  LEVEL_DEBUG : 1,
  LEVEL_INFO : 2,
  LEVEL_WARN : 3,
  LEVEL_ERROR : 4,
  LEVEL_NONE : 99,


  dump : function ( obj, msg )
  {
    Ext.log( {dump : obj}, msg );
  },


  debug : function ( msg )
  {
    var loggingLevelNum = QDesk.Log._getCurrentLevelNum();
    if ( loggingLevelNum <= QDesk.Log.LEVEL_DEBUG )
    {
      Ext.log( {level : 'log'}, msg );
    }
  },


  info : function ( msg )
  {
    var loggingLevelNum = QDesk.Log._getCurrentLevelNum();
    if ( loggingLevelNum <= QDesk.Log.LEVEL_INFO )
    {
      Ext.log( {level : 'info'}, msg );
    }
  },


  warn : function ( msg )
  {
    var loggingLevelNum = QDesk.Log._getCurrentLevelNum();
    if ( loggingLevelNum <= QDesk.Log.LEVEL_WARN )
    {
      Ext.log( {level : 'warn'}, msg );
    }
  },


  error : function ( msg )
  {
    var loggingLevelNum = QDesk.Log._getCurrentLevelNum();
    if ( loggingLevelNum <= QDesk.Log.LEVEL_ERROR )
    {
      Ext.log( {level : 'error'}, msg );
    }
  },


  _getCurrentLevelNum : function ()
  {
    var currentLevel = QDesk.ApplicationConfig.get( 'jsLoggingLevel', QDesk.enums.JsLoggingLevel.NONE );

    if ( currentLevel == QDesk.enums.JsLoggingLevel.NONE )
    {
      return QDesk.Log.LEVEL_NONE;
    }
    else if ( currentLevel == QDesk.enums.JsLoggingLevel.DEBUG )
    {
      return QDesk.Log.LEVEL_DEBUG;
    }
    else if ( currentLevel == QDesk.enums.JsLoggingLevel.INFO )
    {
      return QDesk.Log.LEVEL_INFO;
    }
    else if ( currentLevel == QDesk.enums.JsLoggingLevel.WARN )
    {
      return QDesk.Log.LEVEL_WARN;
    }
    else if ( currentLevel == QDesk.enums.JsLoggingLevel.ERROR )
    {
      return QDesk.Log.LEVEL_ERROR;
    }
    else
    {
      Ext.log( {level : 'error'}, 'Unrecognized JavaScript logging level: ' + currentLevel );
      return QDesk.Log.LEVEL_NONE;
    }
  }
} );
