/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.config.ConnectionConfig', {
  singleton : true,

  /**
   * Maps connection ID to connection configuration data object (key-value pairs).
   */
  configs : Ext.create( 'Ext.util.MixedCollection' ),

  /**
   * Returns the value of the specified configuration entry.
   * @param key a configuration entry key.
   * @return the configuration entry value.
   */
  get : function ( key, connectionId )
  {
    var me = this;

    var data = me.configs.get( connectionId );
    if ( data == null )
    {
      //Ext.Error.raise( 'Configuration for connection ID: ' + connectionId + ' not found.' );
      QDesk.Log.error( 'Configuration for connection ID: ' + connectionId + ' not found.' );
      return null;
    }
    else
    {
      var entryValue = data[key];
      if ( entryValue == null )
      {
        //Ext.Error.raise( 'Undefined connection configuration entry key: ' + key );
        QDesk.Log.error( 'Undefined connection configuration entry key: ' + key );
      }
      return entryValue;
    }
  },

  /**
   * Applies the specified configuration changes to the current configuration.
   * @param changes an object containing configuration changes.
   * @param connectionId a connection ID.
   */
  applyChanges : function ( changes, connectionId )
  {
    var me = this;

    var data = me.configs.get( connectionId );
    if ( data == null )
    {
      //Ext.Error.raise( 'Configuration for connection ID: ' + connectionId + ' not found.' );
      QDesk.Log.error( 'Configuration for connection ID: ' + connectionId + ' not found.' );
    }
    else
    {
      for ( var entryKey in changes )
      {
        var entryValue = changes[entryKey];
        if ( data[entryKey] == null )
        {
          QDesk.Log.error( 'Skipped undefined connection configuration entry with key: ' + entryKey );
        }
        else
        {
          data[entryKey] = entryValue;
        }
      }
    }
  }
} );
