/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Various Quartz-specific controller related utility methods.
 */
Ext.define( 'QDesk.controller.QuartzControllerUtils', {
  singleton : true,

  requires : [
    'QDesk.controller.command.connection.ConnectionUtils'
  ],


  /**
   * Returns the jobs-tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getJobsTab : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    var jobsTab = schedulerTab.down( '#jobsTab' );
    if ( !jobsTab )
      Ext.Error.raise( 'Jobs tab for connection ID: ' + me.connectionId + ' not found.' );

    return jobsTab;
  },


  /**
   * Returns the currently executing jobs tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getExecutingJobsTab : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    var executingJobsTab = schedulerTab.down( '#executingJobsTab' );
    if ( !executingJobsTab )
      Ext.Error.raise( 'Executing jobs tab for connection ID: ' + me.connectionId + ' not found.' );

    return executingJobsTab;
  },


  /**
   * Returns the triggers-tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getTriggersTab : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    return schedulerTab.down( '#triggersTab' );
  },


  /**
   * Returns the info-tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getInfoTab : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    return schedulerTab.down( '#infoTab' );
  },


  /**
   * Returns the scheduler execution history tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerExecHistoryTab : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    return schedulerTab.down( '#schedulerExecHistoryTab' );
  },


  /**
   * Returns the scheduler statistics tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerStatisticsTab : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    return schedulerTab.down( '#schedulerStatisticsTab' );
  },


  /**
   * Returns the scheduler notifications tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerNotificationsTab : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    return schedulerTab.down( '#schedulerNotificationsTab' );
  },


  /**
   * Returns the scheduler job chains tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerJobChainsTab : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    return schedulerTab.down( '#schedulerJobChainsTab' );
  }
} );
