/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Base class for all commands.
 */
Ext.define( 'QDesk.controller.command.BaseCommand', {

  requires : [
    'QDesk.util.UxUtils'
  ],

  /*
   * Property that is automatically set to point to the QuartzDesk application.
   */
  application: null,

  /**
   * Optional config property with the function to invoke when this command succeeds.
   */
  success : undefined,

  /**
   * Optional config property with the function to invoke when this command fails.
   */
  failure : undefined,

  /**
   * Optional element used by the mask and unmask methods. If not specified, then the
   * element returned by Ext.getBody() element is used.
   */
  maskEl : undefined,

  /**
   * Creates an empty command instance.
   * @param config a configuration.
   */
  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.maskEl )
    {
      me.maskEl = Ext.getBody();
    }

    Ext.apply( config, {
      application : QDesk.getApplication()
    } );

    Ext.apply( me, config );

    me.callParent( [config] );
  },


  /**
   * Executes this command.
   */
  execute : function ()
  {
  },


  mask : function ( msg )
  {
    var me = this;
    QDesk.util.UxUtils.maskEl( me.maskEl, msg );
  },


  unmask : function ()
  {
    var me = this;
    QDesk.util.UxUtils.unmaskEl( me.maskEl );
  }

} );
