/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the specified error.
 */
Ext.define( 'QDesk.controller.command.SaveGridConfig', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const'
  ],

  /**
   * Required config property containing the grid configuration data
   * (column visibility and column order).
   */
  gridConfig : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.gridConfig )
      Ext.Error.raise( 'Required config parameter missing: gridConfig' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.saveGridConfig' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.SaveGridConfig.confirmMsg' ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._saveChart();
            }
          }
      );
    }
    else
    {
      me._saveChart();
    }
  },


  _saveChart : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.SaveGridConfig.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.applicationConfigActionSaveChanges,
      params : {
        changes : Ext.encode( me.gridConfig.configChanges )
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            /*
             * If saved successfully, apply the changes to the current configuration.
             */
            QDesk.ApplicationConfig.applyChanges( me.gridConfig.configChanges );

            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.SaveGridConfig.successMsg' ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.SaveGridConfig.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
