/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Saves the specified HTML contents extracted from the HTML editor in
 * the job execution log window.
 */
Ext.define( 'QDesk.controller.command.SaveLog', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /**
   * Required config property containing the formatted HTML data
   * with job execution log records.
   */
  htmlLogData : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.htmlLogData )
      Ext.Error.raise( 'Required config parameter missing: htmlLogData' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;
    me._saveLogData();
  },


  _saveLogData : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.SaveLog.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.saveLogActionPrepare,
      params : {
        htmlLogData : me.htmlLogData
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var handle = json.data.handle;
            me._downloadLogData( handle );

            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.SaveLog.successMsg' ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.SaveLog.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  _downloadLogData : function ( handle )
  {
    var downloadUrl = QDesk.util.constant.ActionUrl.saveLogActionDownload + '&handle=' + handle;

    var iframe = Ext.getBody().createChild( {
      tag : 'iframe',
      width : 0,
      height : 0,
      frameborder : 0,
      src : downloadUrl,
      cls : Ext.baseCSSPrefix + 'hide-display'
    } );

    // give the backend 10 seconds to initiate the download and then silently drop the iframe
    Ext.defer( function ()
        {
          iframe.remove();
        },
        10000 );
  }
} );
