/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified folder and refreshes (reloads) its parent folder in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.SwitchLocale', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required configuration parameter with the current application locale.
   */
  oldLocale : undefined,

  /*
   * Required configuration parameter with the new (desired) application locale.
   */
  newLocale : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.oldLocale )
      Ext.Error.raise( 'Required config parameter missing: oldLocale' );

    if ( !config.newLocale )
      Ext.Error.raise( 'Required config parameter missing: newLocale' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.localeActionSwitch,
      params : {
        newLocale : me.newLocale
      },
      callback : function ( opts, success, resp )
      {
        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.SwitchLocale.maskMsg' ) );
            window.location.href = window.location.href;
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.SwitchLocale.errorMsg',
              me.oldLocale, me.newLocale ) );
        }
      }
    } );
  }
} );
