/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the specified error.
 */
Ext.define( 'QDesk.controller.command.ViewError', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const',
    'QDesk.view.main.ViewTextWindow'
  ],

  /**
   * Required config property containing the text to view.
   */
  errorText : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.errorText )
      Ext.Error.raise( 'Required config parameter missing: errorText' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.ViewError.maskMsg' ) );

    // give the mask a chance to show up
    Ext.defer( me._viewError, 100, me );
  },


  _viewError : function ()
  {
    var me = this;

    var window = Ext.widget( 'ViewTextWindow', {
      cls : 'viewErrorWindow',
      iconCls : 'viewErrorWindowIcon',
      title : QDesk.Msg.get( 'QDesk.controller.command.ViewError.windowTitle' )
    } );

    window.on( 'afterrender', function ()
    {
      window.maximize();
      window.toFront( false );

      var textColor = QDesk.ApplicationConfig.get( 'viewError.textColor' );

      var formattedError = '<pre style="margin-bottom: 0px; margin-top: 0px;">';
      formattedError += '<span style="color: ' + textColor + '">';
      formattedError += me.errorText;
      formattedError += '</span>';
      formattedError += '</pre>';

      var editor = window.down( '#editor' );
      editor.setValue( formattedError );

      /*
       * Set focus on the editor so that the user can immediately use CTRL+A to
       * select all text in the editor. The delay is used to defer setting of the
       * focus to avoid a known issue (see http://forums.ext.net/showthread.php?3315-CLOSED-Set-Focus-to-textfield-in-logon-window).
       */
      editor.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    window.on( 'show', function ()
    {
      me.unmask();
    } );

    window.show();
  }
} );
