/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Applies the application settings changes by saving them in DB and updating
 * the QDesk.ApplicationConfig singleton.
 */
Ext.define( 'QDesk.controller.command.config.ApplyApplicationConfigChanges', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const'
  ],

  /*
   * Required parameter with application configuration changes to apply.
   */
  changes : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.changes )
      Ext.Error.raise( 'Required config parameter missing: changes' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.ComponentQuery.query( 'ApplicationConfigWindow' )[0];
    var cardPanel = window.down( '#cardPanel' );
    var formPanel = cardPanel.getLayout().getActiveItem();

    var form = formPanel.getForm();

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.applicationConfigActionSaveChanges,
      params : {
        changes : Ext.encode( me.changes )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.config.ApplyApplicationConfigChanges.waitMsg' ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          /*
           * If saved successfully, apply the changes to the current configuration.
           */
          QDesk.ApplicationConfig.applyChanges( me.changes );

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.config.ApplyApplicationConfigChanges.successMsg' ),
              function ()
              {
                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }

                me.application.fireEvent( 'applicationConfigChanged', {
                  changes : me.changes
                } );
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  }
} );
