/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the applications settings window that allows one to edit the application configuration.
 */
Ext.define( 'QDesk.controller.command.config.EditApplicationConfig', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.config.ApplyApplicationConfigChanges',
    'QDesk.util.UxUtils',
    'QDesk.view.main.config.ApplicationConfigWindow'
  ],

  qdesk : {
    /**
     * MixedCollection instance with unsaved changes.
     */
    changes : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      qdesk : {
        changes : Ext.create( 'Ext.util.MixedCollection' )
      }
    } );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.config.EditApplicationConfig.maskMsg' ) );

    // deferred so that the mask message gets displayed
    Ext.defer( function ()
    {
      var me = this;

      var window = Ext.widget( 'ApplicationConfigWindow' );

      me._bindEventHandlers( window );

      window.show();
      //window.toFront();
    }, 100, me );
  },


  /**
   * Bind application settings tree panel event handlers.
   * @param window a application settings window.
   */
  _bindEventHandlers : function ( window )
  {
    var me = this;

    window.on( 'afterrender', function ()
    {
      me.unmask();
    } );

    window.down( '#okButton' ).on( 'click', function ()
    {
      var changes = window.getChanges();

      if ( Ext.Object.isEmpty( changes ) )
      {
        // no changes
        window.close();
      }
      else
      {
        Ext.create( 'QDesk.controller.command.config.ApplyApplicationConfigChanges', {
          changes : changes,
          success : function ()
          {
            window.resetChanges();
            window.close();
          }
        } ).execute();
      }
    } );

    window.down( '#applyButton' ).on( 'click', function ( button )
    {
      var changes = window.getChanges();

      Ext.create( 'QDesk.controller.command.config.ApplyApplicationConfigChanges', {
        changes : changes,
        success : function ()
        {
          window.resetChanges();
          button.setDisabled( true );
        }
      } ).execute();
    } );
  }
} );
