/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Adds a new folder to the specified parent folder in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.AddConnectionFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.navigation.connections.EditConnectionFolderWindow'
  ],

  /*
   * Required ID of the parent folder the new folder should be added to.
   */
  parentFolderId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.parentFolderId )
      Ext.Error.raise( 'Required config parameter missing: parentFolderId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditFolderWindow', {
      qdesk : {
        mode : QDesk.view.navigation.connections.EditConnectionFolderFormMode.ADD
      }
    } );

    var formPanel = window.down( '#form' );

    formPanel.on( 'afterrender', function ()
    {
      var folderNameField = formPanel.down( '#folderName' );
      folderNameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      var form = formPanel.getForm();
      var folderName = formPanel.down( '#folderName' ).getValue();

      form.doAction( 'ParamsSubmit', {
        url : QDesk.util.constant.ActionUrl.schedulerConnectionActionAddFolder,
        params : {
          parentFolderId : me.parentFolderId,
          folderName : folderName
        },
        waitMsg : QDesk.Msg.get( 'QDesk.controller.command.connection.AddConnectionFolder.waitMsg', folderName ),
        success : function ( form, action )
        {
          var json = Ext.decode( action.response.responseText );
          if ( json.success )
          {
            window.close();

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.AddConnectionFolder.successMsg', folderName ),
                function ()
                {
                  me.application.fireEvent( 'folderAdded', {
                    folderId : json.data.id
                  } );
                } );

            me._refreshTargetFolder();
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        },
        failure : function ( form, action )
        {
          QDesk.util.UxUtils.showExtJsActionError( action );
        }
      } );
    } );

    window.show();
  },


  _refreshTargetFolder : function ()
  {
    var me = this;

    // reload the folder's parent folder in the connections tree view
    Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
      folderId : me.parentFolderId
    } ).execute();
  }
} );
