/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Adds a new connection to the specified parent folder in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.AddSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.connection.SaveSchedulerConnection',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.navigation.connections.EditSchedulerConnectionWindow'
  ],

  /*
   * Required ID of the parent folder the new connection should be added to.
   */
  parentFolderId : undefined,

  /**
   * Required config parameter with the scheduler type.
   * See QDesk.enums.scheduler.SchedulerType.
   */
  schedulerType : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.parentFolderId )
      Ext.Error.raise( 'Required config parameter missing: parentFolderId' );

    if ( !config.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: schedulerType' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditSchedulerConnectionWindow', {
      qdesk : {
        mode : QDesk.view.navigation.connections.EditSchedulerConnectionWindow.MODE_ADD,
        schedulerType : me.schedulerType
      }
    } );

    // Save/Update button
    window.down( '#actionButton' ).on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.connection.SaveSchedulerConnection', {
        parentFolderId : me.parentFolderId,
        connection : QDesk.controller.command.connection.ConnectionUtils.createSchedulerConnection( me.schedulerType, window ),
        window : window
      } ).execute();
    } );


    /* Load the form data */
    QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.connection.AddSchedulerConnection.maskLoadMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionGetDefaultConnectionConfig,
      params : {
        schedulerType : me.schedulerType
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            // set data in all forms
            Ext.each( Ext.ComponentQuery.query( 'form', window ), function ( formPanel )
            {
              formPanel.getForm().setValues( json.data );
            } );

            window.show();
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.AddSchedulerConnection.errorMsg' ) );
        }
      }
    } );
  }
} );
