/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified folder and refreshes (reloads) its parent folder in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.DeleteConnectionFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const'
  ],

  /*
   * Required ID of the folder to delete.
   */
  folderId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.folderDelete' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteConnectionFolder.confirmMsg',
              QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteFolder();
            }
          }
      );
    }
    else
    {
      me._deleteFolder();
    }
  },


  _deleteFolder : function ()
  {
    var me = this;

    /* Close scheduler tabs of all connections registered under the deleted folder */
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( me.folderId );

    var folderNode = store.getNodeById( nodeId );
    me._closeSchedulerTabsOfAllNestedConnections( folderNode );

    var folderName = QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId );

    QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteConnectionFolder.maskMsg',
        folderName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionDeleteFolder,
      params : {
        folderId : me.folderId
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteConnectionFolder.successMsg',
                folderName ),
                function ()
                {
                  me.application.fireEvent( 'folderDeleted', {
                    folderId : me.folderId
                  } );
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteConnectionFolder.errorMsg',
              folderName ) );
        }

        // reload the folder's parent folder in the connections tree view
        var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getFolderParentFolderId( me.folderId );
        if ( parentFolderId )
        {
          Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
            folderId : parentFolderId
          } ).execute();
        }
        else
        {
          Ext.Error.raise( 'Parent folder ID for folder ID: ' + me.folderId + ' not found.' );
        }
      }
    } );
  },


  /**
   * Closes scheduler tabs (scheduler views) of all connections nested underneath the specified folder node.
   * @param folderNode a folder node.
   */
  _closeSchedulerTabsOfAllNestedConnections : function ( folderNode )
  {
    var me = this;

    Ext.each( folderNode.childNodes, function ( childNode )
    {
      if ( childNode.get( 'nodeType' ) == 'connection' )
      {
        var connectionId = childNode.get( 'id' );
        var schedulerTabItemId = "schedulerTabPanel-" + connectionId;
        Ext.each( Ext.ComponentQuery.query( '#' + schedulerTabItemId ), function ( tabPanel )
        {
          tabPanel.close();
        } );
      }
      else if ( childNode.get( 'nodeType' ) == 'folder' )
      {
        me._closeSchedulerTabsOfAllNestedConnections( childNode );
      }
    } );
  }
} );
