/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified connection and refreshes (reloads) its parent folder in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.DeleteSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required ID of the connection to delete.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.connectionDelete' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteSchedulerConnection.confirmMsg',
              QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteConnection();
            }
          }
      );
    }
    else
    {
      me._deleteConnection();
    }
  },


  _deleteConnection : function ()
  {
    var me = this;

    me._closeSchedulerTabForConnection();

    var connectionName = QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId );

    QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteSchedulerConnection.maskMsg',
        connectionName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionDeleteConnection,
      params : {
        connectionId : me.connectionId
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteSchedulerConnection.successMsg',
                connectionName ),
                function ()
                {
                  me.application.fireEvent( 'connectionDeleted', {
                    connectionId : me.connectionId
                  } );
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteSchedulerConnection.errorMsg',
              connectionName ) );
        }

        // reload the connection's parent folder in the connections tree view
        var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getConnectionParentFolderId( me.connectionId );
        if ( parentFolderId )
        {
          Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
            folderId : parentFolderId
          } ).execute();
        }
        else
        {
          Ext.Error.raise( 'Parent folder ID for connection ID: ' + me.connectionId + ' not found.' );
        }
      }
    } );
  },


  /**
   * Closes the scheduler tab (scheduler view) of the connection that is to be deleted.
   */
  _closeSchedulerTabForConnection : function ()
  {
    var me = this;

    var schedulerTabItemId = "schedulerTabPanel-" + me.connectionId;
    Ext.each( Ext.ComponentQuery.query( '#' + schedulerTabItemId ), function ( tabPanel )
    {
      tabPanel.close();
    } );
  }
} );
