/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Scheduler discovery used by the AddConnection and EditConnection commands.
 */
Ext.define( 'QDesk.controller.command.connection.DiscoverSchedulers', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ServerErrorType',
    'QDesk.util.UxUtils'
  ],

  /*
   * Required config parameter with the SchedulerConnectionDataForm form panel
   * initiating the discovery.
   */
  form : undefined,

  /**
   * Required config parameter with the scheduler type.
   * See QDesk.enums.scheduler.SchedulerType.
   */
  schedulerType : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.form )
      Ext.Error.raise( 'Required config parameter missing: form' );

    if ( !config.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: schedulerType' );

    Ext.apply( config, {
      maskEl : config.form.up( 'window' ) ? config.form.up( 'window' ).getEl() : config.form.getEl()
    } );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerObjectNameCombo = me.form.down( '#schedulerObjectName' );
    schedulerObjectNameCombo.setValue( '' );
    schedulerObjectNameCombo.getStore().loadData( [] );

    var useServiceParams = me.form.down( '#serviceParamsRadio' ).getValue();
    var useServiceUrl = me.form.down( '#serviceUrlRadio' ).getValue();

    var fieldValues = me.form.getForm().getFieldValues();

    var remoteJmxService = Ext.create( 'QDesk.model.RemoteJmxService',
        {
          protocol : useServiceParams ? fieldValues.protocol : null,
          host : useServiceParams ? fieldValues.host : null,
          port : useServiceParams ? fieldValues.port : null,
          serviceUrl : useServiceUrl ? fieldValues.serviceUrl : null,
          authentication : {
            username : fieldValues.username ? fieldValues.username : null,
            password : fieldValues.password ? fieldValues.password : null
          },
          secure : fieldValues.secure == null ? false : fieldValues.secure
        } );

    if ( fieldValues.username == '' && fieldValues.password == '' )
      remoteJmxService.set( 'authentication', null );

    /*
     * Important to drop id property from the data to avoid JSOn deserialization issues on the server-side.
     */
    var remoteJmxServiceData = remoteJmxService.getData();
    delete remoteJmxServiceData.id;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.connection.DiscoverSchedulers.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerActionDiscover,
      params : {
        schedulerType : me.schedulerType,
        remoteJmxService : Ext.encode( remoteJmxServiceData )
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            schedulerObjectNameCombo.getStore().loadData( json.schedulers );

            if ( json.schedulers.length == 0 )
            {
              QDesk.util.UxUtils.showInfoDialog(
                  QDesk.Msg.get( 'QDesk.controller.command.connection.DiscoverSchedulers.noSchedulersFoundMsg' ) );
            }
            else
            {
              /* select first scheduler in the list */
              schedulerObjectNameCombo.setValue( json.schedulers[0].objectName );

              QDesk.util.UxUtils.showInfoDialog(
                  QDesk.Msg.get( 'QDesk.controller.command.connection.DiscoverSchedulers.schedulersFoundMsg',
                      json.schedulers.length ) );
            }
          }
          else
          {
            // application errors are shown in a warning dialog
            if ( json.errorType && json.errorType == QDesk.enums.ServerErrorType.APPLICATION )
            {
              QDesk.util.UxUtils.showWarningDialog( json.errors[0].msg );
            }
            else
            {
              QDesk.util.UxUtils.showJsonResponseErrors( json );
            }
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.DiscoverSchedulers.errorMsg' ) );
        }
      }
    } );
  }
} );
