/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the name of an existing folder in the connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.EditConnectionFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.navigation.connections.EditConnectionFolderWindow'
  ],

  /*
   * Required ID of the folder that is edited.
   */
  folderId : undefined,

  /**
   * Required config parameter with the existing folder name.
   */
  folderName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    if ( !config.folderName )
      Ext.Error.raise( 'Required config parameter missing: folderName' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditFolderWindow', {
      qdesk : {
        mode : QDesk.view.navigation.connections.EditConnectionFolderFormMode.EDIT
      }
    } );

    var formPanel = window.down( '#form' );

    formPanel.on( 'afterrender', function ()
    {
      var folderNameField = formPanel.down( '#folderName' );
      folderNameField.setValue( me.folderName );
      folderNameField.focus( true, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      var form = formPanel.getForm();
      var folderName = formPanel.down( '#folderName' ).getValue();

      form.doAction( 'ParamsSubmit', {
        url : QDesk.util.constant.ActionUrl.schedulerConnectionActionUpdateFolder,
        params : {
          folderId : me.folderId,
          folderName : folderName
        },
        waitMsg : QDesk.Msg.get( 'QDesk.controller.command.connection.EditConnectionFolder.waitMsg', folderName ),
        success : function ( form, action )
        {
          var json = Ext.decode( action.response.responseText );
          if ( json.success )
          {
            window.close();

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.EditConnectionFolder.successMsg', folderName ),
                function ()
                {
                  me.application.fireEvent( 'folderUpdated', {
                    folderId : me.folderId
                  } );
                } );

            me._refreshTargetFolder();
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        },
        failure : function ( form, action )
        {
          QDesk.util.UxUtils.showExtJsActionError( action );
        }
      } );
    } )
    ;

    window.show();
  },


  /**
   * Reloads the connection's parent folder.
   */
  _refreshTargetFolder : function ()
  {
    var me = this;

    // reload the connection's parent folder in the connections tree view
    var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getFolderParentFolderId( me.folderId );
    if ( parentFolderId )
    {
      Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
        folderId : parentFolderId
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Parent folder ID for folder ID: ' + me.folderId + ' not found.' );
    }
  }
} );
