/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Moves the folder to a new target folder.
 */
Ext.define( 'QDesk.controller.command.connection.MoveConnectionFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.connection.RefreshConnectionFolder',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required ID of the folder the folder should be moved to.
   */
  targetFolderId : undefined,

  /*
   * Required ID of the folder to move.
   */
  folderId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.targetFolderId )
      Ext.Error.raise( 'Required config parameter missing: targetFolderId' );

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var folderName = QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.connection.MoveConnectionFolder.maskMsg', folderName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionMoveFolder,
      params : {
        targetFolderId : me.targetFolderId,
        folderId : me.folderId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me._refreshFolder( me.targetFolderId );

            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.connection.MoveConnectionFolder.successMsg',
                folderName ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            me._refreshFolder( null );

            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          me._refreshFolder( null );

          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.MoveConnectionFolder.errorMsg',
                  folderName ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  /**
   * Refreshes the specified folder.
   */
  _refreshFolder : function ( folderId )
  {
    Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
      folderId : folderId
    } ).execute();
  }
} );
