/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes (reloads) the specified connections tree folder.
 */
Ext.define( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const'
  ],

  /*
   * Optional ID of the folder node to reload. If not specified, then the entire tree (root node) is reloaded.
   */
  folderId : null,

  /*
   * Optional flag indicating of the reloaded folder should be expanded after the reload operation. Defaults to
   * true.
   */
  expand : true,

  /*
   * Maximum number of attempts to load the scheduler connections tree.
   */
  retryCount : 10,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( config.folderId == null )
      config.folderId = QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID;

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];

    var selectionModel = treePanel.getSelectionModel();

    // array of selected records (in our case at most one record)
    var selectedRecords = selectionModel.getSelection();
    selectionModel.deselectAll();

    var store = treePanel.getStore();

    // if the store is not being loaded, reload the panel
    if ( !store.isLoading() )
    {
      //QDesk.Log.debug( 'About to load connections store, folderId: ' + me.folderId );

      var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( me.folderId );

      var node = store.getNodeById( nodeId );
      if ( node == null )
      {
        node = store.getRootNode();
      }

      if ( node )
      {
        node.removeAll( false );

        //var folderName = QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId );

        // load the data and if the load succeeds expand the requested path
        store.load(
            {
              node : node,
              callback : function ( records, operation, success )
              {
                if ( success )
                {
                  /*
                   * Important to update the node's 'leaf' flag depending on the number of loaded children nodes.
                   */
                  node.set( 'leaf', records.length = 0 );

                  if ( me.expand )
                  {
                    // expand the reloaded folder
                    treePanel.expandPath( node.getPath() );
                  }

                  // restore the original selection
                  if ( !Ext.isEmpty( selectedRecords ) )
                  {
                    var selectedNode = store.getNodeById( selectedRecords[0].get( 'id' ) );
                    if ( selectedNode )
                      selectionModel.select( [selectedNode] );
                  }

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                }
                else
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                }
              }
            }
        );
      }
      else
      {
        // invoke the failure callback handler
        if ( me.failure && Ext.isFunction( me.failure ) )
        {
          me.failure.apply( me );
        }
      }
    }
    else
    {
      if ( me.retryCount > 0 )
      {
        QDesk.Log.warn( 'Cannot refresh connections store (folderId: ' + me.folderId +
        ') because it is being loaded. Will automatically retry in 200 ms.' );
        Ext.defer( me.execute, 200, me );
        //Ext.Error.raise( 'Cannot refresh folder ID: ' + me.folderId + ' because store is loading.' );

        me.retryCount--;
      }
    }
  }
} );
