/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens message channel profile editor.
 */
Ext.define( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.message.channel.MessageChannelProfileUtils',
    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.SendTestMessage',
    'QDesk.util.CommonUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const'
  ],

  /*
   * Required ID of the folder the profile should be added to.
   */
  parentFolderId : undefined,

  /*
   * Required parameter with the message channel ({@link QDesk.enums.message.MessageChannel})
   * to open the editor for.
   */
  messageChannel : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.parentFolderId )
      Ext.Error.raise( 'Required config parameter missing: parentFolderId' );

    if ( !config.messageChannel )
      Ext.Error.raise( 'Required config parameter missing: messageChannel' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.controller.command.message.channel.MessageChannelProfileUtils.cleanupEditPanel();

    var formPanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.createEmptyProfilePanel(
        me.messageChannel );

    formPanel.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    // Send Test Message button
    formPanel.down( '#sendTestMessageButton' ).on( 'click', function ()
    {
      me._sendTestMessage();
    } );

    // Apply button
    formPanel.down( '#saveButton' ).on( 'click', function ()
    {
      me._submitForm();
    } );

    QDesk.controller.command.message.channel.MessageChannelProfileUtils.addAndShowPanel( formPanel );
  },


  _sendTestMessage : function ()
  {
    var me = this;

    var editPanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getEditPanel();

    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    // model.MessageChannelProfile populated with message channel specific values
    var messageChannelProfile = formPanel.createMessageChannelProfile();

    var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();

    Ext.create( 'QDesk.controller.command.message.channel.SendTestMessage', {
      maskEl : window.getEl(),
      profile : messageChannelProfile
    } ).execute();
  },


  _submitForm : function ()
  {
    var me = this;

    var editPanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getEditPanel();
    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    // model.MessageChannelProfile populated with message channel specific values
    var messageChannelProfile = formPanel.createMessageChannelProfile();

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionAddProfile,
      params : {
        parentFolderId : me.parentFolderId,
        profile : QDesk.util.CommonUtils.encode( messageChannelProfile )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.message.channel.AddMessageChannelProfile.waitMsg',
          messageChannelProfile.get( 'name' ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          QDesk.controller.command.message.channel.MessageChannelProfileUtils.cleanupEditPanel();

          me._refreshTargetFolder();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.AddMessageChannelProfile.successMsg',
                  messageChannelProfile.get( 'name' ) ),
              function ()
              {
                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  },


  /**
   * Refreshes the parent folder.
   * @private
   */
  _refreshTargetFolder : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
      folderId : me.parentFolderId
    } ).execute();
  }
} );
