/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified message channel profile folder and refreshes (reloads)
 * its parent folder in the message channel profiles tree panel.
 */
Ext.define( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.message.channel.MessageChannelProfileUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required ID of the message channel profile folder to delete.
   */
  folderId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var folderName = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getFolderName( me.folderId );

    if ( QDesk.ApplicationConfig.get( 'confirm.msgChannelProfileFolderDelete' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder.confirmMsg',
              folderName ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteFolder( folderName );
            }
          }
      );
    }
    else
    {
      me._deleteFolder( folderName );
    }
  },


  _deleteFolder : function ( folderName )
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder.maskMsg',
        folderName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionDeleteFolder,
      params : {
        folderId : me.folderId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder.successMsg',
                folderName ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {// invoke the success callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder.errorMsg',
                  folderName ),
              function ()
              {// invoke the success callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }

        // reload the folder's parent folder
        var parentFolderId = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getFolderParentFolderId( me.folderId );
        if ( parentFolderId )
        {
          Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
            folderId : parentFolderId
          } ).execute();
        }
        else
        {
          Ext.Error.raise( 'Parent folder ID for message channel profile folder ID: ' + me.folderId + ' not found.' );
        }
      }
    } );
  }
} );
