/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the specified message channel profiles folder.
 */
Ext.define( 'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.message.channel.MessageChannelProfileUtils',
    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.main.message.channel.FolderPanel'
  ],

  /*
   * Required folder record ({@link QDesk.model.message.MessageChannelProfilesTreeNode} instance)
   * to edit.
   */
  folder : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folder )
      Ext.Error.raise( 'Required config parameter missing: folder' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.controller.command.message.channel.MessageChannelProfileUtils.cleanupEditPanel();

    var formPanel = Ext.widget( 'FolderPanel' );

    // focus the name text field
    formPanel.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    QDesk.controller.command.message.channel.MessageChannelProfileUtils.addAndShowPanel( formPanel );

    // Apply button
    formPanel.down( '#saveButton' ).on( 'click', function ()
    {
      me._submitForm();
    } );

    formPanel.populate.apply( formPanel, [me.folder.data] );
  },


  _submitForm : function ()
  {
    var me = this;

    var editPanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getEditPanel();
    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    var fieldValues = form.getFieldValues();
    var folderId = fieldValues['id'];
    var folderName = fieldValues['name'];

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionUpdateFolder,
      params : {
        folderId : folderId,
        folderName : folderName
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder.waitMsg',
          folderName ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          //QDesk.controller.command.message.channel.MessageChannelProfileUtils.cleanupEditPanel();

          me._refreshTargetFolder();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder.successMsg',
                  folderName ),
              function ()
              {
                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  },


  /**
   * Refreshes the parent folder.
   */
  _refreshTargetFolder : function ()
  {
    var me = this;

    var folderId = me.folder.get( 'folderId' );
    var parentFolderId = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getFolderParentFolderId( folderId );
    if ( parentFolderId )
    {
      Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
        folderId : parentFolderId
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Parent folder ID for profile ID: ' + me.profileId + ' not found.' );
    }
  }
} );
