/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the message channel profiles window that allows one to add/edit and delete
 * message channel profiles.
 */
Ext.define( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.message.channel.AddMessageChannelProfile',
    'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.DeleteMessageChannelProfile',
    'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.EditMessageChannelProfile',
    'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.MessageChannelProfileUtils',
    'QDesk.controller.command.message.channel.MoveMessageChannelProfile',
    'QDesk.controller.command.message.channel.MoveMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.main.message.channel.MessageChannelProfilesWindow'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'MessageChannelProfilesWindow', {
      qdesk : {
      }
    } );

    me._bindEventHandlers( window );

    window.show();

    window.toFront();
  },


  /**
   * Bind message channel profiles window event handlers.
   * @param window a message channel profiles window.
   */
  _bindEventHandlers : function ( window )
  {
    var me = this;

    // tree reload events
    var treePanel = window.down( '#messageChannelProfilesTreePanel' );
    var toolBar = treePanel.down( '#toolBar' );
    var treeStore = treePanel.getStore();

    // disable tree toolbar buttons before tree load
    treeStore.on( 'beforeload',
        function ( store, operation )
        {
          Ext.each( Ext.ComponentQuery.query( 'button', toolBar ), function ( button )
          {
            button.disable();
          } );
        }, me );

    treeStore.on( 'load',
        function ( store, node, records, success )
        {
          var me = this;
          toolBar.down( '#refreshButton' ).enable();

          if ( success )
          {
            toolBar.down( '#collapseButton' ).enable();
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.errorLoadingProfilesMsg' ) );
          }
        }, me );

    // context menu
    treePanel.on( 'itemcontextmenu',
        function ( view, record, item, index, event )
        {
          var nodeId = record.get( 'id' );
          event.stopEvent();

          var nodeType = record.get( 'nodeType' );
          if ( nodeType == 'folder' )
          {
            var folderCtxMenu = me._createFolderCtxMenu( record );
            folderCtxMenu.showAt( event.getXY() );
          }
          else if ( nodeType == 'profile' )
          {
            var profileCtxMenu = me._createProfileCtxMenu( record );
            profileCtxMenu.showAt( event.getXY() );
          }
        }, me );

    // selection changes
    treePanel.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length > 0 )  // selection change event is also fired after deselection
          {
            var selectedRecord = selectedRecords[0];
            var nodeId = selectedRecord.get( 'id' );
            var nodeType = selectedRecord.get( 'nodeType' );
            if ( nodeType == 'folder' )
            {
              me._onFolderSelected( selectedRecord );
            }
            else if ( nodeType == 'profile' )
            {
              me._onProfileSelected( selectedRecord );
            }
          }
          else
          {
            me._onUnselected();
          }
        }, me );

    var treeView = treePanel.getView();

    treeView.on( 'beforedrop', function ( node, data, overModel, dropPosition, dropFunction, eOpts )
    {
      var draggedRecord = data.records[0];
      var targetRecord = overModel;

      var draggedRecordParent = draggedRecord.parentNode;

      // if node is dragged to its existing parent, then we cancel the operation
      if ( draggedRecordParent.get( 'id' ) == targetRecord.get( 'id' ) )
        return false;

      if ( !me._checkNodeMove( draggedRecord, targetRecord ) )
      {
        return false;
      }

      me._onNodeDrag( draggedRecord, targetRecord );

      return 1;
    } );

// Commented out - target folder refresh is done in the MoveMessageChannelProfile
// and MoveMessageChannelProfileFolder commands.
//
//    treeView.on( 'drop', function ( node, data, overModel, dropPosition, eOpts )
//    {
//      var draggedRecord = data.records[0];
//      var targetRecord = overModel;
//      var targetFolderId = targetRecord.get( 'folderId' );
//      Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
//        folderId: targetFolderId
//      } ).execute();
//    } );

    me._bindToolbarEventHandlers( window );
  },


  /**
   * Binds event handlers to message channel profiles tree panel's tool bar.
   * @param window a message channel profiles window.
   */
  _bindToolbarEventHandlers : function ( window )
  {
    var me = this;

    // tree reload events
    var treePanel = window.down( '#messageChannelProfilesTreePanel' );
    var toolBar = treePanel.down( '#toolBar' );

    toolBar.down( '#refreshButton' ).on( 'click', me._onRefreshButtonClick, me );
    toolBar.down( '#collapseButton' ).on( 'click', me._onCollapseButtonClick, me );
    toolBar.down( '#deleteFolderOrProfileButton' ).on( 'click', me._onDeleteFolderOrProfileButtonClick, me );

    // add folder
    toolBar.down( '#addFolderButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder', {
          maskEl : window.getEl(),
          parentFolderId : folderId
        } ).execute();
      }
    }, me );

    /* Add message channel profile */

    // EMAIL
    toolBar.down( '#addEmailProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.EMAIL
        } ).execute();
      }
    }, me );

    // IM_AIM
    toolBar.down( '#addImAimProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.IM_AIM
        } ).execute();
      }
    }, me );

    // IM_GTALK
    toolBar.down( '#addImGTalkProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.IM_GTALK
        } ).execute();
      }
    }, me );

    // IM_ICQ
    toolBar.down( '#addImIcqProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.IM_ICQ
        } ).execute();
      }
    }, me );

    // IM_SKYPE
//    toolBar.down( '#addImSkypeProfileButton' ).on( 'click', function ()
//    {
//      var selection = treePanel.getSelectionModel().getSelection();
//      if ( !Ext.isEmpty( selection ) )
//      {
//        var folderId = selection[0].get( 'folderId' );
//        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
//          maskEl : window.getEl(),
//          parentFolderId : folderId,
//          messageChannel : QDesk.enums.message.MessageChannel.IM_SKYPE
//        } ).execute();
//      }
//    }, me );

    // IM_XMPP
    toolBar.down( '#addImXmppProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.IM_XMPP
        } ).execute();
      }
    }, me );

    // IM_YAHOO
    toolBar.down( '#addImYahooProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.IM_YAHOO
        } ).execute();
      }
    }, me );

    // SNMPV1_TRAP
    toolBar.down( '#addSnmpV1TrapProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.SNMPV1_TRAP
        } ).execute();
      }
    }, me );

    // SNMPV2C_TRAP
    toolBar.down( '#addSnmpV2cTrapProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.SNMPV2C_TRAP
        } ).execute();
      }
    }, me );

    // SNMPV3_TRAP
    toolBar.down( '#addSnmpV3TrapProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.SNMPV3_TRAP
        } ).execute();
      }
    }, me );

    // WEB_SERVICE
    toolBar.down( '#addWebServiceProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[0].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.enums.message.MessageChannel.WEB_SERVICE
        } ).execute();
      }
    }, me );
  },


  /**
   * Invoked when the refreshButton in the message channel profiles tree panel is clicked.
   */
  _onRefreshButtonClick : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
    } ).execute();
  },


  /**
   * Invoked when the collapseButton in the message channel profiles tree panel is clicked.
   */
  _onCollapseButtonClick : function ()
  {
    var me = this;
    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    treePanel.collapseAll();
  },


  /**
   * Invoked when the deleteFolderOrProfileButton in the message channel profiles tree panel is clicked.
   */
  _onDeleteFolderOrProfileButtonClick : function ()
  {
    var me = this;

    var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();

    var selectedRecord = me._getSelectedTreeRecord();
    if ( selectedRecord )
    {
      var nodeId = selectedRecord.get( 'id' );
      var nodeType = selectedRecord.get( 'nodeType' );
      if ( nodeType == 'folder' )
      {
        var folderId = selectedRecord.get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder', {
          maskEl : window.getEl(),
          folderId : folderId
        } ).execute();
      }
      else if ( nodeType == 'profile' )
      {
        var profileId = selectedRecord.get( 'profileId' );
        Ext.create( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfile', {
          maskEl : window.getEl(),
          profileId : profileId
        } ).execute();
      }
    }
  },


  /**
   * Invoked when a folder is selected in the message channel profiles tree panel.
   * @param folderRecord a folder record.
   */
  _onFolderSelected : function ( folderRecord )
  {
    var me = this;

    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var toolBar = treePanel.down( '#toolBar' );

    var addButton = toolBar.down( '#addFolderOrProfileButton' );
    var deleteButton = toolBar.down( '#deleteFolderOrProfileButton' );

    // enable add action
    addButton.enable();

    // delete button tooltips differ based on whether a folder, or profile is selected
    deleteButton.setTooltip(
        QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.tooltipDeleteFolderButton' ) );

    // delete button enabled only for non-root folders
    var nodeId = folderRecord.get( 'id' );
    if ( nodeId == QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID )
    {
      deleteButton.disable();

      QDesk.controller.command.message.channel.MessageChannelProfileUtils.cleanupEditPanel();
    }
    else
    {
      deleteButton.enable();

      var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();

      Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder', {
        maskEl : window.getEl(),
        folder : folderRecord
      } ).execute();
    }
  },


  /**
   * Invoked when a profile is selected in the message channel profiles tree panel.
   * @param profileRecord a profile record.
   */
  _onProfileSelected : function ( profileRecord )
  {
    var me = this;

    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var toolBar = treePanel.down( '#toolBar' );

    var addButton = toolBar.down( '#addFolderOrProfileButton' );
    var deleteButton = toolBar.down( '#deleteFolderOrProfileButton' );

    // disable add action
    addButton.disable();

    // delete button tooltips differ based on whether a folder, or profile is selected
    deleteButton.setTooltip(
        QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.tooltipDeleteProfileButton' ) );

    deleteButton.enable();

    var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();

    var profileId = profileRecord.get( 'profileId' );
    var messageChannel = profileRecord.get( 'messageChannel' );

    Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfile', {
      maskEl : window.getEl(),
      profileId : profileId,
      messageChannel : messageChannel
    } ).execute();
  },


  /**
   * Invoked when a profile, or folder is unselected in the message channel profiles tree panel.
   */
  _onUnselected : function ()
  {
    var me = this;

    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var toolBar = treePanel.down( '#toolBar' );

    var addButton = toolBar.down( '#addFolderOrProfileButton' );
    var deleteButton = toolBar.down( '#deleteFolderOrProfileButton' );

    // disable add action
    addButton.disable();

    // disable delete button and reset its tooltips (tooltips change dynamically)
    deleteButton.setTooltip( '' );
    deleteButton.disable();

    //QDesk.controller.command.message.channel.MessageChannelProfileUtils.cleanupEditPanel();
  },


  /**
   * Creates and returns the context menu for a folder node in the message channel profile tree.
   * @param folderRecord a folder record (see QDesk.model.MessageChannelProfilesTreeNode}.
   * @return the created context menu.
   */
  _createFolderCtxMenu : function ( folderRecord )
  {
    var me = this;

    var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();

    var nodeId = folderRecord.get( 'id' );
    var folderId = folderRecord.get( 'folderId' );
    var isRootFolder = nodeId == QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        {
          text : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddFolderOrProfile' ),
          menu : {
            items : [
              {
                text : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddFolder' ),
                iconCls : 'addFolder',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder', {
                    maskEl : window.getEl(),
                    parentFolderId : folderId
                  } ).execute();
                }
              },

              {
                text : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddProfile' ),
                menu : {
                  items : [
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.EMAIL ),
                      iconCls : QDesk.enums.message.MessageChannel.EMAIL,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.EMAIL
                        } ).execute();
                      }
                    },
                    '-',
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.IM_AIM ),
                      iconCls : QDesk.enums.message.MessageChannel.IM_AIM,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.IM_AIM
                        } ).execute();
                      }
                    },
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.IM_GTALK ),
                      iconCls : QDesk.enums.message.MessageChannel.IM_GTALK,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.IM_GTALK
                        } ).execute();
                      }
                    },
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.IM_ICQ ),
                      iconCls : QDesk.enums.message.MessageChannel.IM_ICQ,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.IM_ICQ
                        } ).execute();
                      }
                    },
//                    {
//                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.IM_SKYPE ),
//                      iconCls : QDesk.enums.message.MessageChannel.IM_SKYPE,
//                      handler : function ()
//                      {
//                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
//                          maskEl : window.getEl(),
//                          parentFolderId : folderId,
//                          messageChannel : QDesk.enums.message.MessageChannel.IM_SKYPE
//                        } ).execute();
//                      }
//                    },
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.IM_XMPP ),
                      iconCls : QDesk.enums.message.MessageChannel.IM_XMPP,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.IM_XMPP
                        } ).execute();
                      }
                    },
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.IM_YAHOO ),
                      iconCls : QDesk.enums.message.MessageChannel.IM_YAHOO,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.IM_YAHOO
                        } ).execute();
                      }
                    },
                    '-',
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.SNMPV1_TRAP ),
                      iconCls : QDesk.enums.message.MessageChannel.SNMPV1_TRAP,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.SNMPV1_TRAP
                        } ).execute();
                      }
                    },
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.SNMPV2C_TRAP ),
                      iconCls : QDesk.enums.message.MessageChannel.SNMPV2C_TRAP,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.SNMPV2C_TRAP
                        } ).execute();
                      }
                    },
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.SNMPV3_TRAP ),
                      iconCls : QDesk.enums.message.MessageChannel.SNMPV3_TRAP,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.SNMPV3_TRAP
                        } ).execute();
                      }
                    },
                    '-',
                    {
                      text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.message.MessageChannel.WEB_SERVICE ),
                      iconCls : QDesk.enums.message.MessageChannel.WEB_SERVICE,
                      handler : function ()
                      {
                        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                          maskEl : window.getEl(),
                          parentFolderId : folderId,
                          messageChannel : QDesk.enums.message.MessageChannel.WEB_SERVICE
                        } ).execute();
                      }
                    }
                  ]
                }
              }
            ]
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuDelete' ),
          iconCls : 'deleteFolder',
          disabled : isRootFolder,
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder', {
              maskEl : window.getEl(),
              folderId : folderId
            } ).execute();
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuRefresh' ),
          iconCls : 'refreshFolder',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
              maskEl : window.getEl(),
              folderId : folderId
            } ).execute();
          }
        }
      ]
    } );
  },


  /**
   * Creates and returns the context menu for a profile node in the message channel profile tree.
   * @param profileRecord a profile record (see QDesk.model.MessageChannelProfilesTreeNode}.
   * @return the created context menu.
   */
  _createProfileCtxMenu : function ( profileRecord )
  {
    var me = this;

    var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();

    var profileId = profileRecord.get( 'profileId' );
    var messageChannel = profileRecord.get( 'messageChannel' );

    return Ext.create( 'Ext.menu.Menu', {
      items : [
//
//        // edit profile
//        {
//          text: QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.profileCtxMenuEdit' ),
//          iconCls: 'editProfile',
//          handler: function ()
//          {
//            Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfile', {
//              maskEl: window.getEl(),
//              profileId: profileId,
//              messageChannel: messageChannel
//            } ).execute();
//          }
//        },
        // delete profile
        {
          text : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.profileCtxMenuDelete' ),
          iconCls : 'deleteProfile',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfile', {
              maskEl : window.getEl(),
              profileId : profileId
            } ).execute();
          }
        }
      ]
    } );
  },


  /**
   * Returns the currently selected record (a {@link QDesk.model.message.MessageChannelProfilesTreeNode} instance),
   * or null if no record is selected.
   */
  _getSelectedTreeRecord : function ()
  {
    var me = this;

    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var selectedRecords = treePanel.getSelectionModel().getSelection();
    if ( Ext.isEmpty( selectedRecords ) )
      return null;
    else
      return selectedRecords[0];
  },


  _onNodeDrag : function ( draggedRecord, targetRecord )
  {
    var me = this;

    var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();

    var targetFolderId = targetRecord.get( 'folderId' );
    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      var folderId = draggedRecord.get( 'folderId' );
      Ext.create( 'QDesk.controller.command.message.channel.MoveMessageChannelProfileFolder', {
        maskEl : window.getEl(),
        folderId : folderId,
        targetFolderId : targetFolderId
      } ).execute();
    }
    else if ( nodeType == 'profile' )
    {
      var profileId = draggedRecord.get( 'profileId' );
      Ext.create( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile', {
        maskEl : window.getEl(),
        profileId : profileId,
        targetFolderId : targetFolderId
      } ).execute();
    }
  },


  /**
   * Checks if the specified record can be dragged to the specified target record.
   * @param draggedRecord the dragged folder/profile record.
   * @param targetRecord the target record (target folder).
   * @return {boolean} true if the dragged folder/profile can be moved, false otherwise.
   * @private
   */
  _checkNodeMove : function ( draggedRecord, targetRecord )
  {
    var canBeMoved = true;

    var targetFolderId = targetRecord.get( 'folderId' );
    var targetFolderName = targetRecord.get( 'name' );

    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      var folderId = draggedRecord.get( 'folderId' );
      var folderName = draggedRecord.get( 'name' );

      Ext.each( targetRecord.childNodes, function ( node )
      {
        if ( node.get( 'nodeType' ) == 'folder' && node.get( 'name' ) == folderName )
        {
          QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCannotBeMovedMsg',
              folderName, targetFolderName ) );

          canBeMoved = false;
          return false;
        }
      } );
    }
    else if ( nodeType == 'profile' )
    {
      var profileId = draggedRecord.get( 'profileId' );
      var profileName = draggedRecord.get( 'name' );

      Ext.each( targetRecord.childNodes, function ( node )
      {
        if ( node.get( 'nodeType' ) == 'profile' && node.get( 'name' ) == profileName )
        {
          QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.profileCannotBeMovedMsg',
              profileName, targetFolderName ) );

          canBeMoved = false;
          return false;
        }
      } );
    }

    return canBeMoved;
  }
} );
