/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Moves the message channel profile to a new target folder.
 */
Ext.define( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.message.channel.MessageChannelProfileUtils',
    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',
    'QDesk.util.UxUtils'
  ],

  /*
   * Required ID of the folder the profile should be moved to.
   */
  targetFolderId : undefined,

  /*
   * Required ID of the profile to move.
   */
  profileId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.targetFolderId )
      Ext.Error.raise( 'Required config parameter missing: targetFolderId' );

    if ( !config.profileId )
      Ext.Error.raise( 'Required config parameter missing: profileId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.controller.command.message.channel.MessageChannelProfileUtils.cleanupEditPanel();

    var profileName = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfileName( me.profileId );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile.maskMsg',
        profileName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionMoveProfile,
      params : {
        targetFolderId : me.targetFolderId,
        profileId : me.profileId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me._refreshFolder( me.targetFolderId );

            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile.successMsg',
                profileName ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            me._refreshFolder( null );

            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          me._refreshFolder( null );

          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile.errorMsg',
                  profileName ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  /**
   * Refreshes the specified folder.
   */
  _refreshFolder : function ( folderId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
      folderId : folderId
    } ).execute();
  }
} );
