/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes (reloads) the specified message channel profiles tree folder.
 */
Ext.define( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const'
  ],

  /*
   * Optional ID of the folder to reload. If not specified, then the entire tree (root node) is reloaded.
   */
  folderId : QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_FOLDER_ID,

  /*
   * Optional flag indicating of the reloaded folder should be expanded after the reload operation. Defaults to
   * true.
   */
  expand : true,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( config.folderId == null )
      config.folderId = QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_FOLDER_ID;

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.controller.command.message.channel.MessageChannelProfileUtils.cleanupEditPanel();

    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();

    // unselect all nodes in the tree panel
    treePanel.getSelectionModel().deselectAll( false );

    var store = treePanel.getStore();

    // if the store is not being loaded, reload the panel
    if ( !store.isLoading() )
    {
      var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( me.folderId );

      var node = store.getNodeById( nodeId );

      if ( node )
      {
        node.removeAll( false );

        // load the data and if the load succeeds expand the requested path
        store.load(
            {
              node : node,
              callback : function ( records, operation, success )
              {
                if ( success )
                {
                  /*
                   * Important to set the 'leaf' flag to false so that the folder can be a drag&drop target
                   */
                  node.set( 'leaf', false );

                  if ( me.expand )
                  {
                    // expand the reloaded folder
                    treePanel.expandPath( node.getPath() );
                  }

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                }
                else
                {
                  QDesk.util.UxUtils.showErrorDialog(
                      QDesk.Msg.get( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder.errorMsg' ),
                      function ()
                      {
                        // invoke the failure callback handler
                        if ( me.failure && Ext.isFunction( me.failure ) )
                        {
                          me.failure.apply( me );
                        }
                      } );
                }
              }
            }
        );
      }
      else
      {
        // invoke the failure callback handler
        if ( me.failure && Ext.isFunction( me.failure ) )
        {
          me.failure.apply( me );
        }
      }
    }
    else
    {
      Ext.Error.raise( 'Cannot refresh folder ID: ' + me.folderId + ' because store is loading.' );
    }
  }
} );
