/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Sends a test message using the specified message channel profile.
 */
Ext.define( 'QDesk.controller.command.message.channel.SendTestMessage', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.CommonUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const',
    'QDesk.view.main.message.channel.SendTestMessageRecipientsWindow'
  ],

  /*
   * Required message channel profile (QDesk.model.message.MessageChannelProfile instance).
   */
  profile : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.profile )
      Ext.Error.raise( 'Required config parameter missing: profile' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var messageChannel = me.profile.get( 'channel' );

    if ( messageChannel == QDesk.enums.message.MessageChannel.SNMPV1_TRAP
        || messageChannel == QDesk.enums.message.MessageChannel.SNMPV2C_TRAP
        || messageChannel == QDesk.enums.message.MessageChannel.SNMPV3_TRAP
        || messageChannel == QDesk.enums.message.MessageChannel.WEB_SERVICE )
    {
      // recipients not required by the SNMPV* and WEB_SERVICE message channels
      me._sendTestMessage();
    }
    else
    {
      var window = Ext.widget( 'SendTestMessageRecipientsWindow', {
        qdesk : {
          messageChannel : messageChannel
        }
      } );

      var formPanel = window.down( '#form' );

      formPanel.on( 'afterrender', function ()
      {
        var folderNameField = formPanel.down( '#recipients' );
        folderNameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
      } );

      formPanel.down( '#sendButton' ).on( 'click', function ()
      {
        var recipients = window.down( '#recipients' ).getValue();
        window.close();
        me._sendTestMessage( recipients );
      } );

      window.show();
    }
  },


  _sendTestMessage : function ( recipients )
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.message.channel.SendTestMessage.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionSendTestMessage,
      params : {
        recipients : recipients,
        profile : QDesk.util.CommonUtils.encode( me.profile )
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfoDialog(
                QDesk.Msg.get( 'QDesk.controller.command.message.channel.SendTestMessage.successMsg' ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.SendTestMessage.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
