/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Generates the SNMP MIB configuration file.
 */
Ext.define( 'QDesk.controller.command.message.channel.snmp.GenerateMib', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils'
  ],

  /*
   * Required SNMP message channel (QDesk.enums.message.MessageChannel).
   */
  channel : undefined,

  /*
   * Required SNMP message channel configuration.
   */
  channelConfig : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.channel )
      Ext.Error.raise( 'Required config parameter missing: channel' );

    if ( !config.channelConfig )
      Ext.Error.raise( 'Required config parameter missing: channelConfig' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.message.channel.snmp.GenerateMib.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.snmpGenerateMib,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfoDialog( QDesk.Msg.get( 'QDesk.controller.command.message.channel.snmp.GenerateMib.successMsg' ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.snmp.GenerateMib.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
