/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Pauses the scheduler whose connection ID is specified in the configuration parameters.
 */
Ext.define( 'QDesk.controller.command.scheduler.common.PauseScheduler', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils'
  ],

  /*
   * Required ID of the connection of the scheduler to pause.
   */
  connectionId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.schedulerPause' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.PauseScheduler.confirmMsg',
              QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._pauseScheduler();
            }
          }
      );
    }
    else
    {
      me._pauseScheduler();
    }
  },


  _pauseScheduler : function ()
  {
    var me = this;

    var connectionName = QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId );

    QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.PauseScheduler.maskMsg',
        connectionName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerActionPause,
      params : {
        connectionId : me.connectionId
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.PauseScheduler.successMsg',
                connectionName ),
                function ()
                {
                  me.application.fireEvent( 'schedulerPaused', {
                    connectionId : me.connectionId
                  } );
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.PauseScheduler.errorMsg',
              connectionName ) );
        }

        // reload the connection's parent folder in the connections tree view
        var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getConnectionParentFolderId( me.connectionId );
        if ( parentFolderId )
        {
          Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
            folderId : parentFolderId
          } ).execute();
        }
        else
        {
          Ext.Error.raise( 'Parent folder ID for connection ID: ' + me.connectionId + ' not found.' );
        }
      }
    } );
  }
} );
