/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Adds a new trigger.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.AddTrigger', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.Const',

    'QDesk.util.quartz.QuartzTriggerUtils',

    'QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode',
    'QDesk.view.main.scheduler.quartz.triggers.EditTriggerWindow'
  ],

  /*
   * Required ID of the scheduler connection.
   */
  connectionId : undefined,

  /*
   * Required type of the trigger to add (see QDesk.enums.scheduler.quartz.TriggerType).
   */
  triggerType : undefined,

  /**
   * Required job group name.
   */
  jobGroupName : undefined,

  /**
   * Required job name.
   */
  jobName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.triggerType )
      Ext.Error.raise( 'Required config parameter missing: triggerType' );

    if ( !config.jobGroupName )
      Ext.Error.raise( 'Required config parameter missing: jobGroupName' );

    if ( !config.jobName )
      Ext.Error.raise( 'Required config parameter missing: jobName' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.AddTrigger.maskLoadMsg' ) );

    var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( me.connectionId );
    var schedulerInfo = QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo( me.connectionId );

    var window = Ext.widget( 'QuartzEditTriggerWindow', {
      qdesk : {
        connectionId : me.connectionId,
        mode : QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.ADD,
        triggerType : me.triggerType,
        triggerJobGroupName : me.jobGroupName,
        triggerJobName : me.jobName,
        jvmAgentInfo : jvmAgentInfo,
        schedulerInfo : schedulerInfo
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );

    var formStores = formPanel.getFormStores();

    /* Load the form data */
    QDesk.util.StoreUtils.onAllStoresLoad( formStores, function ( success )
    {
      me.unmask();

      if ( success )
      {
        window.show();
      }
      else
      {
        // error loading stores
        QDesk.util.UxUtils.showErrorDialog(
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.AddTrigger.errorLoadMsg' ) );
      }
    } );
    QDesk.util.StoreUtils.loadStores( formStores );
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    var form = formPanel.getForm();
    var fieldValues = form.getFieldValues();
    var jobDataMap = formPanel.getJobDataMap();

    var triggerName = fieldValues.name;
    var triggerGroupName = fieldValues.group;

    var trigger = QDesk.util.quartz.QuartzTriggerUtils.createTrigger( fieldValues, jobDataMap );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.quartz.triggerActionAdd,
      params : {
        connectionId : me.connectionId,
        trigger : QDesk.util.CommonUtils.encode( trigger )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.AddTrigger.waitMsg',
          triggerName, triggerGroupName ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.AddTrigger.successMsg',
                  triggerName, triggerGroupName ),
              function ()
              {
                me.application.fireEvent( 'quartzTriggerAdded', {
                  connectionId : me.connectionId
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  }
} );
