/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the view panel for the Quartz scheduler described by the specified
 * connection description configuration object.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.controller.command.scheduler.quartz.RefreshActiveTab',

    'QDesk.controller.command.scheduler.common.StartScheduler',
    'QDesk.controller.command.scheduler.common.PauseScheduler',
    'QDesk.controller.command.scheduler.common.StopScheduler',

    'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',

    'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab',

    'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',

    'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab',

    'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab',

    'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab',
    'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab',

    'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab',

    'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab',
    'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab',

    'QDesk.enums.ProductEdition',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProduct'
  ],

  /*
   * Required connection ID configuration attribute.
   */
  connectionId : undefined,

  /**
   * Required scheduler info object with the following structure:
   * {
   *   type : QUARTZ
   *   version : {
   *     major : 1,
   *     minor : 2,
   *     maintenance : 3
   *   }
   * }
   */
  schedulerInfo : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: schedulerInfo' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[ 0 ];
    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( me.connectionId );

    // if there is no tab panel for the scheduler, create a new one and add it to the mainTabPanel, otherwise
    // just activate the existing tab panel
    if ( schedulerTab == null )
    {
      QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.maskMsg' ) );

      // determine if the JVM Agent is available for the specified connection
      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.schedulerActionAgentInfo,
        params : {
          connectionId : me.connectionId
        },
        callback : function ( opts, success, resp )
        {
          QDesk.util.UxUtils.unmaskBody();

          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              if ( json.data.available )
              {
                if ( QDesk.util.license.LicensedProduct.productEdition == QDesk.enums.ProductEdition.LITE )
                {
                  // For QuartzDesk Lite edition we SKIP QuartzDesk JVM Agent version check!
                  me._openSchedulerTabPanel( mainTabPanel, json.data );
                }
                else
                {
                  // For QuartzDesk Standard and Enterprise editions we PERFORM QuartzDesk JVM Agent version and edition check!

                  /* If the QuartzDesk JVM Agent version is compatible with the QuartzDesk web application */
                  if ( json.data.compatible )
                  {
                    // all is sweet (versions and editions are compatible)
                    me._openSchedulerTabPanel( mainTabPanel, json.data );
                  }
                  else
                  {
                    if ( !json.data.compatibleVersion )
                    {
                      var jvmAgentVersion = json.data.version;
                      var productVersion = QDesk.util.license.LicensedProduct.productVersion;

                      QDesk.util.UxUtils.showWarningDialog(
                          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentIncompatibleVersionMsg',
                              QDesk.util.FormatUtils.formatProductVersion( jvmAgentVersion ),
                              QDesk.util.FormatUtils.formatProductVersion( productVersion )
                          ),
                          function ()
                          {
                            me._openSchedulerTabPanel( mainTabPanel, json.data );
                          } );
                    }
                    else
                    {
                      var jvmAgentEdition = json.data.edition;
                      var productEdition = QDesk.util.license.LicensedProduct.productEdition;

                      if ( !json.data.compatibleEdition )
                      {
                        QDesk.util.UxUtils.showWarningDialog(
                            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentIncompatibleEditionMsg',
                                jvmAgentEdition,
                                productEdition
                            ),
                            function ()
                            {
                              me._openSchedulerTabPanel( mainTabPanel, json.data );
                            } );
                      }
                    }

                  }
                  /* End of QuartzDesk JVM Agent vs QuartzDesk app version check */
                }
              }
              else
              {
                // Lite edition
                if ( QDesk.util.license.LicensedProduct.productEdition == QDesk.enums.ProductEdition.LITE )
                {
                  // For Lite editions we IGNORE that the QuartzDesk JVM Agent is not available!
                  me._openSchedulerTabPanel( mainTabPanel, json.data );
                }
                else
                {
                  // Standard and Enterprise editions
                  if ( QDesk.ApplicationConfig.get( 'warning.jvmAgentNotAvailable' ) )
                  {
                    QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentNotAvailableMsg' ),
                        function ()
                        {
                          me._openSchedulerTabPanel( mainTabPanel, json.data );
                        } );
                  }
                  else
                  {
                    me._openSchedulerTabPanel( mainTabPanel, json.data );
                  }
                }
              }
            }
            else
            {
              // Server side error
              QDesk.util.UxUtils.showJsonResponseErrors( json );
            }
          }
          else
          {
            // Ajax error
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.errorMsg' ) );
          }
        }
      } );
    }
    else
    {
      // make the panel active
      mainTabPanel.setActiveTab( schedulerTab );
    }
  },


  /**
   * Opens the Quartz scheduler tab and adds it to the specified main tab panel.
   * @param mainTabPanel the main tab panel.
   * @param jvmAgentInfo an object encapsulating the agent information for the scheduler connection.
   */
  _openSchedulerTabPanel : function ( mainTabPanel, jvmAgentInfo )
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.maskMsg' ) );

    // deferred so that the mask message gets displayed
    Ext.defer( function ()
    {
      var me = this;

      // open a new Quartz scheduler tab panel and add it to the main tab panel
      var schedulerTab = mainTabPanel.add( {
        xtype : 'QuartzSchedulerTab',
        itemId : QDesk.controller.command.connection.ConnectionUtils.getSchedulerTabItemId( me.connectionId ),
        title : QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId ),
        qdesk : {
          connectionId : me.connectionId,
          schedulerInfo : me.schedulerInfo,
          jvmAgentInfo : jvmAgentInfo
        }
      } );

      mainTabPanel.setActiveTab( schedulerTab );

      me._initializeSchedulerTabPanel( schedulerTab, me.connectionId );

      me.unmask();
    }, 100, me );
  },


  /**
   * Initializes the scheduler tab panel by binding all UI panel controls with the logic.
   * @param tabPanel a Quartz scheduler tab panel.
   */
  _initializeSchedulerTabPanel : function ( tabPanel )
  {
    var me = this;

    var refreshButton = tabPanel.down( '#schedulerTabToolBar > #refreshButton' );
    refreshButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var startButton = tabPanel.down( '#schedulerTabToolBar > #startSchedulerButton' );
    startButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.StartScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var pauseButton = tabPanel.down( '#schedulerTabToolBar > #pauseSchedulerButton' );
    pauseButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.PauseScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var stopButton = tabPanel.down( '#schedulerTabToolBar > #stopSchedulerButton' );
    stopButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.StopScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    me._initializeSubTabs( tabPanel );
  },


  /**
   * Initializes sub tabs of the specified scheduler tab.
   * @param tabPanel a scheduler tab (the main tab).
   */
  _initializeSubTabs : function ( tabPanel )
  {
    var me = this;

    var jobsTab = tabPanel.down( '#jobsTab' );
    jobsTab.on( 'activate', function ( jobsTab )
    {
      /* If the jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !jobsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var executingJobsTab = tabPanel.down( '#executingJobsTab' );
    executingJobsTab.on( 'activate', function ( executingJobsTab )
    {
      /* If the executing jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !executingJobsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var triggersTab = tabPanel.down( '#triggersTab' );
    triggersTab.on( 'activate', function ( triggersTab )
    {
      /* If the triggers tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !triggersTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var infoTab = tabPanel.down( '#infoTab' );
    infoTab.on( 'activate', function ( infoTab )
    {
      /* If the info tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !infoTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var schedulerExecHistoryTab = tabPanel.down( '#schedulerExecHistoryTab' );
    schedulerExecHistoryTab.on( 'activate', function ( schedulerExecHistoryTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerExecHistoryTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab', {
          connectionId : me.connectionId
        } ).execute();


        Ext.create( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var schedulerStatisticsTab = tabPanel.down( '#schedulerStatisticsTab' );
    schedulerStatisticsTab.on( 'activate', function ( schedulerStatisticsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerStatisticsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var schedulerNotificationsTab = tabPanel.down( '#schedulerNotificationsTab' );
    schedulerNotificationsTab.on( 'activate', function ( schedulerNotificationsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerNotificationsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var schedulerJobChainsTab = tabPanel.down( '#schedulerJobChainsTab' );
    schedulerJobChainsTab.on( 'activate', function ( schedulerJobChainsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerJobChainsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    /* Initially selected tab (this could be made configurable...) */
    tabPanel.setActiveTab( jobsTab );

    // setActiveTab does not fire the activate event if the tab is already active so we need to
    // simulate the active event...

    /* If the jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
    if ( !jobsTab.getInitialized() )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
        connectionId : me.connectionId
      } ).execute();
    }

    Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
      connectionId : me.connectionId
    } ).execute();
  }
} );
