/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Pauses the specified job group.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.PauseJobGroup', {
  extend: 'QDesk.controller.command.BaseCommand',

  requires: [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required config property containing the connection ID.
   */
  connectionId: undefined,

  /**
   * Required config property containing the job group name.
   */
  jobGroupName: undefined,


  constructor: function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.jobGroupName )
      Ext.Error.raise( 'Required config parameter missing: jobGroupName' );

    me.callParent( [config] );
  },


  execute: function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.jobGroupPause' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.PauseJobGroup.confirmMsg',
              me.jobGroupName ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._pauseJobGroup();
            }
          }
      );
    }
    else
    {
      me._pauseJobGroup();
    }
  },


  _pauseJobGroup: function ()
  {
    var me = this;

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.PauseJobGroup.maskMsg',
            me.jobGroupName ) );

    Ext.Ajax.request( {
      url: QDesk.util.constant.ActionUrl.quartz.jobGroupActionPause,
      params: {
        connectionId: me.connectionId,
        jobGroupName: me.jobGroupName
      },
      callback: function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.PauseJobGroup.successMsg',
                    me.jobGroupName ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }

                  me.application.fireEvent( 'quartzJobGroupPaused', {
                    connectionId: me.connectionId,
                    jobGroupName: me.jobGroupName
                  } );
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.PauseJobGroup.errorMsg',
                  me.jobGroupName ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
