/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the currently active tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.enums.scheduler.SchedulerStatus',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var tabPanel = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( me.connectionId );
    //assert tabPanel != null

    /*
     * Refreshes the scheduler tab tool bar (depends on the scheduler status).
     */
    me._updateSchedulerTabToolBarStatus( tabPanel );

    var activeTab = tabPanel.getActiveTab();

    if ( activeTab.itemId == 'jobsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
        connectionId : me.connectionId
      } ).execute();
    }
    else if ( activeTab.itemId == 'executingJobsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab', {
        connectionId : me.connectionId
      } ).execute();
    }
    else if ( activeTab.itemId == 'triggersTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
        connectionId : me.connectionId
      } ).execute();
    }
    else if ( activeTab.itemId == 'infoTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab', {
        connectionId : me.connectionId
      } ).execute();
    }
    else if ( activeTab.itemId == 'schedulerExecHistoryTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab', {
        connectionId : me.connectionId
      } ).execute();
    }
    else if ( activeTab.itemId == 'schedulerStatisticsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
        connectionId : me.connectionId
      } ).execute();
    }
    else if ( activeTab.itemId == 'schedulerNotificationsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
        connectionId : me.connectionId
      } ).execute();
    }
    else if ( activeTab.itemId == 'schedulerJobChainsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
        connectionId : me.connectionId
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Unrecognized scheduler tab itemId: ' + activeTab.itemId );
    }
  },


  /**
   * Fetches info data for the specified scheduler and updates the scheduler tab
   * toolbar using the data.
   * @param tabPanel the scheduler tab panel.
   */
  _updateSchedulerTabToolBarStatus : function ( tabPanel )
  {
    var me = this;

    /* Disable all toolbar buttons */
    var refreshButton = Ext.ComponentQuery.query( '#schedulerTabToolBar > #refreshButton', tabPanel )[ 0 ];
    refreshButton.disable();

    var startSchedulerButton = Ext.ComponentQuery.query( '#schedulerTabToolBar > #startSchedulerButton',
        tabPanel )[ 0 ];
    startSchedulerButton.disable();

    var pauseSchedulerButton = Ext.ComponentQuery.query( '#schedulerTabToolBar > #pauseSchedulerButton',
        tabPanel )[ 0 ];
    pauseSchedulerButton.disable();

    var stopSchedulerButton = Ext.ComponentQuery.query( '#schedulerTabToolBar > #stopSchedulerButton', tabPanel )[ 0 ];
    stopSchedulerButton.disable();

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerActionInfo,
      params : {
        connectionId : me.connectionId
      },
      success : function ( response )
      {
        var json = Ext.decode( response.responseText );
        if ( json.success )
        {
          me._updateSchedulerTabToolBarButtons( tabPanel, json.data.status );
        }
        else
        {
          QDesk.Log.error( 'Error invoking ' + QDesk.util.constant.ActionUrl.schedulerActionInfo +
              ' to obtain scheduler data.' );

          me._updateSchedulerTabToolBarButtons( tabPanel /* unknown scheduler status */ );
        }
      },
      failure : function ( response )
      {
        QDesk.Log.error( 'Error invoking ' + QDesk.util.constant.ActionUrl.schedulerActionInfo +
            ' to obtain scheduler data.' );

        me._updateSchedulerTabToolBarButtons( tabPanel /* unknown scheduler status */ );
      }
    } );
  },


  /**
   * Enables/disabled the scheduler toolbar buttons based on the current scheduler status.
   * @param tabPanel a scheduler tab panel.
   * @param schedulerStatus the current scheduler status.
   */
  _updateSchedulerTabToolBarButtons : function ( tabPanel, schedulerStatus )
  {
    var me = this;

    var refreshButton = Ext.ComponentQuery.query( '#schedulerTabToolBar > #refreshButton', tabPanel )[ 0 ];

    // scheduler buttons
    var startSchedulerButton = Ext.ComponentQuery.query( '#schedulerTabToolBar > #startSchedulerButton',
        tabPanel )[ 0 ];
    var pauseSchedulerButton = Ext.ComponentQuery.query( '#schedulerTabToolBar > #pauseSchedulerButton',
        tabPanel )[ 0 ];
    var stopSchedulerButton = Ext.ComponentQuery.query( '#schedulerTabToolBar > #stopSchedulerButton', tabPanel )[ 0 ];

    // refresh button should always be enabled regardless of the scheduler state
    refreshButton.enable();

    if ( schedulerStatus )
    {
      switch ( schedulerStatus )
      {
        case QDesk.enums.scheduler.SchedulerStatus.STARTED:
          startSchedulerButton.disable();
          pauseSchedulerButton.enable();
          stopSchedulerButton.enable();
          break;

        case QDesk.enums.scheduler.SchedulerStatus.PAUSED:
          startSchedulerButton.enable();
          pauseSchedulerButton.disable();
          stopSchedulerButton.enable();
          break;

        case QDesk.enums.scheduler.SchedulerStatus.STOPPED:
          // not necessary as the tab of a stopped Quartz scheduler gets closed
          startSchedulerButton.disable();
          pauseSchedulerButton.disable();
          stopSchedulerButton.disable();
          break;

        default:
          Ext.Error.raise( 'Unrecognized scheduler status: ' + schedulerStatus )
      }
    }
  }
} );
