/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Resumes the specified trigger.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.ResumeTrigger', {
  extend: 'QDesk.controller.command.BaseCommand',

  requires: [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required config property containing the connection ID.
   */
  connectionId: undefined,

  /**
   * Required config property containing the trigger to resume.
   */
  trigger: undefined,


  constructor: function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.trigger )
      Ext.Error.raise( 'Required config parameter missing: trigger' );

    me.callParent( [config] );
  },


  execute: function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.triggerResume' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ResumeTrigger.confirmMsg',
              me.trigger.get( 'group' ).name, me.trigger.get( 'name' ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._resumeTrigger();
            }
          }
      );
    }
    else
    {
      me._resumeTrigger();
    }
  },


  _resumeTrigger: function ()
  {
    var me = this;

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ResumeTrigger.maskMsg',
            me.trigger.get( 'group' ).name, me.trigger.get( 'name' ) ) );

    Ext.Ajax.request( {
      url: QDesk.util.constant.ActionUrl.quartz.triggerActionResume,
      params: {
        connectionId: me.connectionId,
        triggerGroupName: me.trigger.get( 'group' ).name,
        triggerName: me.trigger.get( 'name' )
      },
      callback: function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ResumeTrigger.successMsg',
                    me.trigger.get( 'group' ).name, me.trigger.get( 'name' ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }

                  me.application.fireEvent( 'quartzTriggerResumed', {
                    connectionId: me.connectionId,
                    triggerGroupName: me.trigger.get( 'group' ).name,
                    triggerName: me.trigger.get( 'name' )
                  } );
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ResumeTrigger.errorMsg',
                  me.trigger.get( 'group' ).name, me.trigger.get( 'name' ) ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
