/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a modal window with the list of all jobs and prompts the user to select a job.
 * Upon job selection, the selected JobDetail is passed to the success handler if defined.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.SelectJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.Const',

    'QDesk.util.quartz.QuartzUtils',

    'QDesk.view.main.scheduler.quartz.SelectJobWindow'
  ],

  /*
   * Required ID of the scheduler connection.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

//    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.SelectJob.maskLoadMsg' ) );

    var window = Ext.widget( 'QuartzSelectJobWindow', {
      qdesk : {
        connectionId : me.connectionId
      }
    } );

    me._bindEventHandlers( window );

    window.show();
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );
    var selectJobGrid = formPanel.down( '#selectJobGrid' );
    var actionButton = formPanel.down( '#actionButton' );

    // the action button is disabled by default
    actionButton.disable();


    // job selection
    selectJobGrid.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          actionButton.setDisabled( selectedRecords.length == 0 );
        } );


    // Save/Update button
    actionButton.on( 'click',
        function ()
        {
          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            if ( selectJobGrid.getSelectionModel().hasSelection() )
            {
              var jobDetail = selectJobGrid.getSelectionModel().getSelection()[0];

              window.close();
              me.success.apply( me, [jobDetail] );
            }
          }
        } );
  }
} );
