/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the configured execution history log data.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryLog', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.LoggingEventPriority',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const',
    'QDesk.view.main.ViewLogWindow'
  ],

  /*
   * Required config property containing the connection ID.
   */
  connectionId : undefined,

  /**
   * Required config property containing the execution history record ID.
   */
  execHistoryId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.execHistoryId )
      Ext.Error.raise( 'Required config parameter missing: execHistoryId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryLog.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execHistoryActionLog,
      params : {
        connectionId : me.connectionId,
        execHistoryId : me.execHistoryId
      },
      scope : me,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var formattedLog = me._formatLogData( json.data );

            var window = Ext.widget( 'ViewLogWindow' );
            window.on( 'afterrender', function ()
            {
              window.maximize();
              window.toFront( false );
            } );

            window.down( 'toolbar #saveButton' ).setDisabled( false );

            var editor = window.down( '#editor' );
            editor.setValue( formattedLog );

            window.show();

            /*
             * Set focus on the editor so that the user can immediately use CTRL+A to
             * select all text in the editor. The delay is used to defer setting of the
             * focus to avoid a known issue (see http://forums.ext.net/showthread.php?3315-CLOSED-Set-Focus-to-textfield-in-logon-window).
             */
            editor.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryLog.loadErrorMsg',
                  me.execHistoryId ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  /**
   * Formats the specified list of logging events.
   * @param loggingEvents a list of logging event data.
   * @return the formatted logging events as an HTML string.
   * @private
   */
  _formatLogData : function ( loggingEvents )
  {
    var result = '<pre style="margin-bottom: 0px; margin-top: 0px;">';
    Ext.each( loggingEvents, function ( loggingEvent )
    {
      var priority = loggingEvent.priority;

      var priorityColor;
      if ( priority == QDesk.enums.LoggingEventPriority.TRACE )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.trace' )
      }
      else if ( priority == QDesk.enums.LoggingEventPriority.DEBUG )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.debug' )
      }
      else if ( priority == QDesk.enums.LoggingEventPriority.INFO )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.info' )
      }
      else if ( priority == QDesk.enums.LoggingEventPriority.WARN )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.warn' )
      }
      else if ( priority == QDesk.enums.LoggingEventPriority.ERROR )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.error' )
      }

      var msg = '<span style="color: ' + priorityColor + '">';
      msg += loggingEvent.message;
      msg += '</span><br/>';

      result += msg;
    } );
    result += '</pre>';

    return result;
  }
} );
