/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the configured execution history result.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryResult', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const',
    'QDesk.view.main.ViewTextWindow'
  ],

  /*
   * Required config property containing the connection ID.
   */
  connectionId : undefined,

  /**
   * Required config property containing the execution history record ID.
   */
  execHistoryId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.execHistoryId )
      Ext.Error.raise( 'Required config parameter missing: execHistoryId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryResult.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execHistoryActionResult,
      params : {
        connectionId : me.connectionId,
        execHistoryId : me.execHistoryId
      },
      scope : me,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var formattedResult = me._formatResult( json.data );

            var window = Ext.widget( 'ViewTextWindow', {
              cls : 'viewResultWindow',
              iconCls : 'viewResultWindowIcon',
              title : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryResult.windowTitle' )
            } );

            window.on( 'afterrender', function ()
            {
              window.maximize();
              window.toFront( false );
            } );

            var editor = window.down( '#editor' );
            editor.setValue( formattedResult );

            /*
             * Set focus on the editor so that the user can immediately use CTRL+A to
             * select all text in the editor. The delay is used to defer setting of the
             * focus to avoid a known issue (see http://forums.ext.net/showthread.php?3315-CLOSED-Set-Focus-to-textfield-in-logon-window).
             */
            editor.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );

            window.show();

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.scheduler.quartz.JobController.errorViewResult', me.execHistoryId ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  /**
   * Formats the specified result.
   * @param result the result.
   * @return the formatted result as an HTML string.
   * @private
   */
  _formatResult : function ( result )
  {
    var textColor = QDesk.ApplicationConfig.get( 'viewResult.textColor' );

    var formattedResult = '<pre style="margin-bottom: 0px; margin-top: 0px;">';
    formattedResult += '<span style="color: ' + textColor + '">';
    formattedResult += result;
    formattedResult += '</span>';
    formattedResult += '</pre>';

    return formattedResult;
  }
} );
