/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the configured job data map.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
  extend: 'QDesk.controller.command.BaseCommand',

  requires: [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.enums.scheduler.SchedulerStatus',
    'QDesk.model.scheduler.quartz.QuartzJobDataMapEntry',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.main.scheduler.quartz.JobDataMapWindow'
  ],

  /*
   * Required config property containing the job data map.
   */
  jobDataMap: undefined,

  /**
   * Optional config property containing the effective job data map.
   */
  effectiveJobDataMap: undefined,

  constructor: function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.jobDataMap )
      Ext.Error.raise( 'Required config parameter missing: jobDataMap' );

    me.callParent( [config] );
  },


  execute: function ()
  {
    var me = this;

    var window = Ext.widget( 'QuartzJobDataMapWindow',
        {
          title: QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap.windowTitle' ),

          qdesk: {
            jobDataMap: me.jobDataMap,
            effectiveJobDataMap: me.effectiveJobDataMap
          }
        } );

    window.down( '#closeButton' ).on( 'click', function ()
    {
      window.close();
    } );

    window.down( '#effectiveCheckbox' ).on( 'change', me._onEffectiveCheckboxChange, me );

    window.show();
  },


  /**
   * Invoked when the Effective checkbox selection changes.
   * @param checkbox the Effective checkbox.
   * @param checked a checkbox selection status.
   */
  _onEffectiveCheckboxChange: function ( checkbox, checked )
  {
    var me = this;

    var jobDataMapGrid = checkbox.up( 'window' ).down( '#jobDataMapGrid' );

    var jobDataMap = jobDataMapGrid.qdesk.jobDataMap;
    var effectiveJobDataMap = jobDataMapGrid.qdesk.effectiveJobDataMap;

    var records;

    if ( checked )
    {
      records = me._createModelRecords( effectiveJobDataMap );
    }
    else
    {
      records = me._createModelRecords( jobDataMap );
    }

    jobDataMapGrid.getStore().loadData( records );
  },


  /**
   * Converts the specified array of JSON instances representing job data map entries
   * to an array of {@code QDesk.model.scheduler.quartz.QuartzJobDataMapEntry} instances.
   * @param rawData raw data.
   * @return the created model records.
   */
  _createModelRecords: function ( rawData )
  {
    var records = [];
    Ext.each( rawData, function ( entry )
    {
      records.push(
          Ext.create( 'QDesk.model.scheduler.quartz.QuartzJobDataMapEntry', {
            name: entry.name,
            value: entry.value
          } )
      );
    } );

    return records;
  }
} );
