/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified job chain.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,

  /**
   * Required config parameter with the job chain to delete.
   */
  jobChain : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.jobChain )
      Ext.Error.raise( 'Required config parameter missing: jobChain' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.jobChainDelete' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain.confirmMsg',
              me.jobChain.get( 'name' ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteJobChain();
            }
          }
      );
    }
    else
    {
      me._deleteJobChain();
    }
  },


  /**
   * Deletes the job chain.
   */
  _deleteJobChain : function ()
  {
    var me = this;

    var jobChainId = me.jobChain.get( 'id' );
    var name = me.jobChain.get( 'name' );

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain.maskMsg', name ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.jobChainActionDelete,
      params : {
        connectionId : me.connectionId,
        jobChainId : jobChainId
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain.successMsg',
                    name ),
                function ()
                {
                  me.application.fireEvent( 'quartzJobChainDeleted', {
                    connectionId : me.connectionId,
                    jobChain : me.jobChain
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain.errorMsg', name ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
