/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the scheduler Job Chains grid in the scheduler Job Chains panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobChaining &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var schedulerJobChainsTab = QDesk.controller.QuartzControllerUtils.getSchedulerJobChainsTab( me.connectionId );
      var jobChainsGrid = schedulerJobChainsTab.down( '#jobChainsGrid' );

      var jobChainsGridStore = jobChainsGrid.getStore();

      jobChainsGridStore.getProxy().url =
          QDesk.util.constant.ActionUrl.quartz.jobChainActionListForScheduler;

      var extraStoreParams = {
        connectionId : me.connectionId
      };

      // if job chains index enabled, we add the filter query text param
      if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzJobChainsIndexEnabled( me.connectionId ) )
      {
        var filterQueryText = jobChainsGrid.down( '#filterQueryText' ).getValue();
        if ( !Ext.isEmpty( filterQueryText ) )
        {
          Ext.apply( extraStoreParams, {
            filterQuery : filterQueryText
          } );
        }
      }

      jobChainsGridStore.getProxy().extraParams = extraStoreParams;

      jobChainsGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
