/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the scheduler Execution History grid in the scheduler Execution History panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionHistory &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var schedulerExecHistoryTab = QDesk.controller.QuartzControllerUtils.getSchedulerExecHistoryTab( me.connectionId );

      var execHistoryGrid = schedulerExecHistoryTab.down( '#execHistoryGrid' );

      var execHistoryGridStore = execHistoryGrid.getStore();

      var extraStoreParams = {
        connectionId : me.connectionId
      };

      var maxFetchDate = execHistoryGrid.down( '#maxFetchDateSelect' ).getValue();
      Ext.apply( extraStoreParams, {
        maxFetchDate : maxFetchDate
      } );

      var maxFetchSize = execHistoryGrid.down( '#maxFetchSizeSelect' ).getValue();
      if ( maxFetchSize != -1 )  // -1 == unlimited
      {
        Ext.apply( extraStoreParams, {
          maxFetchSize : maxFetchSize
        } );
      }

      // if exec history index enabled, we add the filter query text param
      if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecHistoryIndexEnabled( me.connectionId ) )
      {
        var filterQueryText = execHistoryGrid.down( '#filterQueryText' ).getValue();
        if ( !Ext.isEmpty( filterQueryText ) )
        {
          Ext.apply( extraStoreParams, {
            filterQuery : filterQueryText
          } );
        }
      }

      execHistoryGridStore.getProxy().url =
          QDesk.util.constant.ActionUrl.quartz.execHistoryActionListForScheduler;

      execHistoryGridStore.getProxy().extraParams = extraStoreParams;

      var reloadStores = true;

      // optimization to avoid repeated reloads of empty stores
      if ( reloadStores )
      {
        execHistoryGridStore.loadPage( 1, {
          scope : me,
          callback : function ( records, operation, success )
          {
            if ( success )
            {
              // invoke the success callback handler
              if ( me.success && Ext.isFunction( me.success ) )
              {
                me.success.apply( me );
              }
            }
            else
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            }
          }
        } );
      }
    }
  }
} );
