/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Info form in the Info tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoForm', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var infoTab = QDesk.controller.QuartzControllerUtils.getInfoTab( me.connectionId );

    var infoForm = infoTab.down( '#infoForm' );

    QDesk.util.UxUtils.maskEl( infoTab.getEl(),
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoForm.maskMsg' ) );

    infoForm.load( {
      url : QDesk.util.constant.ActionUrl.quartz.infoActionInfo,
      params : {
        connectionId : me.connectionId
      },
      success : function ( form, action )
      {
        QDesk.util.UxUtils.unmaskEl( infoTab.getEl() );

        var json = Ext.decode( action.response.responseText );

        if ( json.success )
        {
          if ( json.data.jvmAgentAvailable )
          {
            // QuartzDesk agent available -> show all QuartzDesk Agent related form fields
            me._jvmAgentAvailable( infoForm );

            var jvmAgentConfigurationGrid = infoForm.down( '#jvmAgentConfigurationGrid' );
            jvmAgentConfigurationGrid.getStore().loadData( json.data.jvmAgentCfg );
          }
          else
          {
            // QuartzDesk agent not available -> hide all QuartzDesk Agent related form fields
            me._jvmAgentNotAvailable( infoForm );
          }
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json );
        }
      },
      failure : function ( form, action )
      {
        // clear the form
        infoForm.getForm().reset();
        var jvmAgentConfigurationGrid = infoForm.down( '#jvmAgentConfigurationGrid' );
        jvmAgentConfigurationGrid.getStore().loadData( [] );

        QDesk.util.UxUtils.unmaskEl( infoTab.getEl() );
        QDesk.util.UxUtils.showExtJsActionError( action );
      }
    } );
  },


  /**
   * Invoked when the QuartzDesk Agent is available.
   * @param infoForm the info form.
   */
  _jvmAgentAvailable : function ( infoForm )
  {
    infoForm.down( '#jvmAgentNotAvailableMsg' ).setVisible( false );

    infoForm.down( '#jvmAgentNameAndVersion' ).setVisible( true );
    infoForm.down( '#jvmAgentVersion' ).setVisible( true );
    infoForm.down( '#jvmAgentEdition' ).setVisible( true );

    infoForm.down( '#jvmAgentLocale' ).setVisible( true );
    infoForm.down( '#jvmAgentTimeZone' ).setVisible( true );

    infoForm.down( '#jvmAgentClassLoaderInfo' ).setVisible( true );
    infoForm.down( '#jvmAgentCurrentThreadContextClassLoaderInfo' ).setVisible( true );

    infoForm.down( '#jvmAgentConfiguration' ).setVisible( true );
  },


  /**
   * Invoked when the QuartzDesk Agent is not available.
   * @param infoForm the info form.
   */
  _jvmAgentNotAvailable : function ( infoForm )
  {
    infoForm.down( '#jvmAgentNotAvailableMsg' ).setVisible( true );

    infoForm.down( '#jvmAgentNameAndVersion' ).setVisible( false );
    infoForm.down( '#jvmAgentVersion' ).setVisible( false );
    infoForm.down( '#jvmAgentEdition' ).setVisible( false );

    infoForm.down( '#jvmAgentLocale' ).setVisible( false );
    infoForm.down( '#jvmAgentTimeZone' ).setVisible( false );

    infoForm.down( '#jvmAgentClassLoaderInfo' ).setVisible( false );
    infoForm.down( '#jvmAgentCurrentThreadContextClassLoaderInfo' ).setVisible( false );

    infoForm.down( '#jvmAgentConfiguration' ).setVisible( false );
  }
} );
