/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified job.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,

  /**
   * Required config parameter with the job detail of the job to delete.
   */
  jobDetail : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.jobDetail )
      Ext.Error.raise( 'Required config parameter missing: jobDetail' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.jobDelete' ) )
    {
      var jobGroupName = me.jobDetail.get( 'group' ).name;
      var jobName = me.jobDetail.get( 'name' );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob.confirmMsg',
              jobGroupName, jobName ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteJob();
            }
          }
      );
    }
    else
    {
      me._deleteJob();
    }
  },


  /**
   * Deletes the job.
   */
  _deleteJob : function ()
  {
    var me = this;

    var jobGroupName = me.jobDetail.get( 'group' ).name;
    var jobName = me.jobDetail.get( 'name' );

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob.maskMsg',
            jobGroupName, jobName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.jobActionDelete,
      params : {
        connectionId : me.connectionId,
        jobGroupName : jobGroupName,
        jobName : jobName
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob.successMsg',
                    jobGroupName, jobName ),
                function ()
                {
                  me.application.fireEvent( 'quartzJobDeleted', {
                    connectionId : me.connectionId,
                    jobDetail : me.jobDetail
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob.errorMsg',
                  jobGroupName, jobName ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
