/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the specified job.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.quartz.QuartzUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.main.scheduler.quartz.jobs.JobDetailFormMode',
    'QDesk.view.main.scheduler.quartz.jobs.JobDetailWindow'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,

  /**
   * Required config parameter with the job detail of the job to edit.
   */
  jobDetail : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.jobDetail )
      Ext.Error.raise( 'Required config parameter missing: jobDetail' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var jobDetailClone = QDesk.util.quartz.QuartzUtils.cloneJobDetail( me.jobDetail );

    var formWindow = Ext.widget( 'QuartzJobDetailWindow',
        {
          title : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob.windowTitle' ),

          qdesk : {
            jobDetailFormMode : QDesk.view.main.scheduler.quartz.jobs.JobDetailFormMode.EDIT,
            jobDetail : jobDetailClone,
            callback : function ( jobDetail )
            {
              var jobGroup = jobDetail.get( 'group' );
              var jobName = jobDetail.get( 'name' );

              /*
               * Important to drop id property from the data to avoid JSOn deserialization issues on the server-side.
               */
              var jobDetailData = jobDetail.getData();
              delete jobDetailData.id;

              var form = formWindow.down( '#jobDetailForm' ).getForm();
              form.doAction( 'ParamsSubmit', {
                url : QDesk.util.constant.ActionUrl.quartz.jobActionUpdate,
                params : {
                  connectionId : me.connectionId,
                  jobDetail : Ext.encode( jobDetailData )
                },
                waitMsg : QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.jobs.EditJob.waitMsg',
                    jobGroup.name, jobName ),
                success : function ( form, action )
                {
                  formWindow.close();

                  QDesk.util.UxUtils.showInfo(
                      QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob.successMsg',
                          jobGroup.name, jobName ),
                      function ()
                      {
                        me.application.fireEvent( 'quartzJobUpdated', {
                          connectionId : me.connectionId,
                          jobDetail : jobDetailClone
                        } );

                        // invoke the success callback handler
                        if ( me.success && Ext.isFunction( me.success ) )
                        {
                          me.success.apply( me );
                        }
                      } );
                },
                failure : function ( form, action )
                {
                  QDesk.util.UxUtils.showExtJsActionError( action,
                      function ()
                      {
                        // invoke the failure callback handler
                        if ( me.failure && Ext.isFunction( me.failure ) )
                        {
                          me.failure.apply( me );
                        }
                      } );
                }
              } );
            },
            scope : me
          }
        } );

    formWindow.show();
  }
} );
