/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Interrupts the specified executing job.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,

  /**
   * Required config parameter with the job detail of the job to delete.
   */
  executingJob : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.executingJob )
      Ext.Error.raise( 'Required config parameter missing: executingJob' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    // jobExecutionId is available for Quartz >= 2.1.1 and jobs can be interrupted in Quartz >= 2.1.1
    var jobExecutionId = me.executingJob.get( 'jobExecutionId' );
    if ( Ext.isEmpty( jobExecutionId ) )
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob.warnInterruptJobNotSupported' ) );
    }
    else
    {
      if ( QDesk.ApplicationConfig.get( 'quartz.confirm.jobInterrupt' ) )
      {
        var jobDetail = me.executingJob.get( 'jobDetail' );

        var jobGroupName = jobDetail.group.name;
        var jobName = jobDetail.name;

        QDesk.util.UxUtils.showConfirmDialog(
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob.confirmMsg',
                jobGroupName, jobName ),
            function ( btn )
            {
              if ( btn == 'yes' )
              {
                me._interruptJob();
              }
            }
        );
      }
      else
      {
        me._interruptJob();
      }
    }
  },


  /**
   * Interrupts the job.
   */
  _interruptJob : function ()
  {
    var me = this;

    var jobDetail = me.executingJob.get( 'jobDetail' );

    var jobGroupName = jobDetail.group.name;
    var jobName = jobDetail.name;

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob.maskMsg',
            jobGroupName, jobName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.jobActionInterrupt,
      params : {
        connectionId : me.connectionId,
        jobName : jobName,
        jobGroupName : jobGroupName,
        jobExecutionId : me.executingJob.get( 'jobExecutionId' )
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob.successMsg',
                    jobGroupName, jobName ),
                function ()
                {
                  me.application.fireEvent( 'quartzJobInterrupted', {
                    connectionId : me.connectionId,
                    executingJob : me.executingJob
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          var msg;
          if ( resp.timedout )
          {
            msg = QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob.timeoutErrorMsg',
                jobGroupName, jobName );
          }
          else
          {
            msg = QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob.errorMsg',
                jobGroupName, jobName );
          }

          QDesk.util.UxUtils.showErrorDialog( msg,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
