/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the jobs grid in the Currently Executing Jobs tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.QuartzControllerUtils'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var executingJobsTab = QDesk.controller.QuartzControllerUtils.getExecutingJobsTab( me.connectionId );

    var executingJobsGrid = executingJobsTab.down( '#executingJobsGrid' );
    var filterByJobGroupCombo = executingJobsGrid.down( '#filterByJobGroupCombo' );

    // reloads and resets the job group filter combo and then reloads the jobs grid
    filterByJobGroupCombo.getStore().load( {
      scope : me,
      callback : function ( records, operation, success )
      {
        if ( success )
        {
          // reload the jobs grid
          if ( filterByJobGroupCombo.getValue() == null )
          {
            // no need to reset the combo, just reload the grid
            executingJobsGrid.getStore().loadPage( 1, {
              scope : me,
              callback : function ( records, operation, success )
              {
                if ( success )
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                }
                else
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                }
              }
            } );
          }
          else
          {
            // triggers the change event which reloads the job grid
            filterByJobGroupCombo.reset( null );

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
        }
        else
        {
          // invoke the failure callback handler
          if ( me.failure && Ext.isFunction( me.failure ) )
          {
            me.failure.apply( me );
          }
        }
      }
    } );
  }
} );
