/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Job Chains view in the scheduler view panel's Jobs tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobJobChainsView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected job.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobChaining &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var jobsTab = QDesk.controller.QuartzControllerUtils.getJobsTab( me.connectionId );
      var jobsGrid = jobsTab.down( '#jobsGrid' );

      var selectedJobDetails = jobsGrid.getSelectionModel().getSelection();

      var jobChainsGrid = jobsTab.down( '#jobChainsGrid' );
      var jobChainsGridStore = jobChainsGrid.getStore();

      var reloadStores = true;

      if ( me.populate )
      {
        if ( Ext.isEmpty( selectedJobDetails ) )
        {
          jobChainsGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
          jobChainsGridStore.getProxy().extraParams = {};
        }
        else
        {
          var jobDetail = selectedJobDetails[0];

          jobChainsGridStore.getProxy().url =
              QDesk.util.constant.ActionUrl.quartz.jobChainActionListForJob;

          var extraStoreParams = {
            connectionId : me.connectionId,
            jobGroupName : jobDetail.get( 'group' ).name,
            jobName : jobDetail.get( 'name' )
          };

          // if job chains index enabled, we add the filter query text param
          if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzJobChainsIndexEnabled( me.connectionId ) )
          {
            var filterQueryText = jobChainsGrid.down( '#filterQueryText' ).getValue();
            if ( !Ext.isEmpty( filterQueryText ) )
            {
              Ext.apply( extraStoreParams, {
                filterQuery : filterQueryText
              } );
            }
          }

          jobChainsGridStore.getProxy().extraParams = extraStoreParams;
        }
      }
      else
      {
        if ( jobChainsGridStore.getProxy().url == QDesk.util.constant.ActionUrl.emptyStoreAction )
          reloadStores = false;

        jobChainsGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
        jobChainsGridStore.getProxy().extraParams = {};
      }

      // optimization to avoid repeated reloads of empty stores
      if ( reloadStores )
        jobChainsGridStore.loadPage( 1 );
    }
  }
} );
