/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Jobs tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsGrid',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobHistoryView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobTriggersView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobStatisticsView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobExecNotificationsView'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    // refreshes the jobs grid
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsGrid', {
      connectionId : me.connectionId,
      success : function ()
      {
        me._refreshHistoryView( me.connectionId, true );
        me._refreshTriggersView( me.connectionId, true );
        me._refreshStatisticsView( me.connectionId, true );
        me._refreshNotificationsView( me.connectionId, true );
      },
      failure : function ()
      {
        me._refreshHistoryView( me.connectionId, false );
        me._refreshTriggersView( me.connectionId, false );
        me._refreshStatisticsView( me.connectionId, false );
        me._refreshNotificationsView( me.connectionId, false );
      }
    } ).execute();
  },

  /**
   * Invoked to refresh the data in the job execution history view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshHistoryView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobHistoryView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job triggers view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshTriggersView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobTriggersView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job statistics view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshStatisticsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobStatisticsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job notifications view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshNotificationsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobExecNotificationsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  }
} );
