/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Notifications tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.QuartzControllerUtils',

    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',

    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid',
    'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule',

    'QDesk.enums.message.MessageCreatorType',

    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerNotificationsTab = QDesk.controller.QuartzControllerUtils.getSchedulerNotificationsTab( me.connectionId );

    me._bindEventHandlers( schedulerNotificationsTab );

    schedulerNotificationsTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Notifications tab to their respective handlers.
   */
  _bindEventHandlers : function ( schedulerNotificationsTab )
  {
    var me = this;
    me._bindEventHandlersNotificationRulesGrid( schedulerNotificationsTab );
  },


  /**
   * Binds the notification rules grid events to their handlers.
   */
  _bindEventHandlersNotificationRulesGrid : function ( schedulerNotificationsTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionNotifications &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var execNotificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

      // refresh button
      execNotificationRulesGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshNotificationRulesGrid( me.connectionId );
          } );

      execNotificationRulesGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onNotificationRuleUnselected( schedulerNotificationsTab );
        }
        else
        {
          me._onNotificationRuleSelected( selectedRecords[ 0 ], schedulerNotificationsTab );
        }
      } );

      var filterQueryText = execNotificationRulesGrid.down( '#filterQueryText' );

      var agentAvailable = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId );
      if ( agentAvailable )
      {
        var notificationRulesIndexEnabled = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecNotificationRulesIndexEnabled( me.connectionId );
        if ( !notificationRulesIndexEnabled )
        {
          filterQueryText.emptyText =
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab.notificationRulesIndexDisabledEmptyText' );

          filterQueryText.applyEmptyText();
        }
      }
      else
      {
        filterQueryText.emptyText =
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab.indexNoAgentEmptyText' );

        filterQueryText.applyEmptyText();
      }

      execNotificationRulesGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        execNotificationRulesGrid.down( '#refreshButton' ).disable();
        execNotificationRulesGrid.down( '#addNotificationRuleButton' ).disable();
        execNotificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
        execNotificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
        execNotificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();
        filterQueryText.disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        execNotificationRulesGrid.getSelectionModel().deselectAll();
      } );

      execNotificationRulesGrid.getStore().on( 'load', function ( store, records, successful, operation )
      {
        execNotificationRulesGrid.down( '#refreshButton' ).enable();

        // enable only if the notification rules index is enabled in the agent
        if ( notificationRulesIndexEnabled )
        {
          filterQueryText.enable();
        }

        if ( successful )
        {
          execNotificationRulesGrid.down( '#addNotificationRuleButton' ).enable();
        }
        else
        {
          store.removeAll();

          var rawData = this.getProxy().getReader().rawData;
          if ( rawData && !rawData.success )  // we better check the success flag to make sure there are actually some errors
          {
            QDesk.util.UxUtils.showJsonResponseErrors( rawData );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab.errorLoadingNotifRulesMsg' ) );
          }
        }
      } );

      execNotificationRulesGrid.down( '#addNotificationRuleButton' ).on( 'click', function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
          connectionId : me.connectionId,
          /*
           * Upon success refresh the notifications tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
              connectionId : me.connectionId
            } ).execute();
          }
        } ).execute();
      } );

      execNotificationRulesGrid.down( '#deleteNotificationRuleButton' ).on( 'click', function ()
      {
        var execNotificationRule = execNotificationRulesGrid.getSelectionModel().getSelection()[ 0 ];
        me._deleteExecNotificationRule( execNotificationRule );
      } );

      execNotificationRulesGrid.down( '#editNotificationRuleButton' ).on( 'click', function ()
      {
        var execNotificationRule = execNotificationRulesGrid.getSelectionModel().getSelection()[ 0 ];
        me._editExecNotificationRule( execNotificationRule );
      } );

      execNotificationRulesGrid.down( '#cloneNotificationRuleButton' ).on( 'click', function ()
      {
        var execNotificationRule = execNotificationRulesGrid.getSelectionModel().getSelection()[ 0 ];
        me._cloneExecNotificationRule( execNotificationRule );
      } );

      // opens a window with messages enqueued by the notification rule
      execNotificationRulesGrid.down( '#messagesColumn' ).on( 'requestViewMessages',
          function ( execNotificationRuleId )
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
              connectionId : me.connectionId,
              creatorId : execNotificationRuleId,
              creatorType : QDesk.enums.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
            } ).execute();
          } );

      /* ExecNotificationRulesGrid context menu actions */
      execNotificationRulesGrid.on( 'enableRule', me._enableExecNotificationRule, me );
      execNotificationRulesGrid.on( 'disableRule', me._disableExecNotificationRule, me );
      execNotificationRulesGrid.on( 'deleteRule', me._deleteExecNotificationRule, me );
      execNotificationRulesGrid.on( 'editRule', me._editExecNotificationRule, me );
      execNotificationRulesGrid.on( 'cloneRule', me._cloneExecNotificationRule, me );
    }
  },


  /**
   * Invoked when a notification rule record in the SchedulerExecNotificationRulesGrid in the SchedulerNotificationsTab
   * is selected.
   * @param notificationRule a QDesk.model.scheduler.quartz.QuartzExecNotificationRule instance representing the selected
   * notification rule.
   * @param schedulerNotificationsTab the scheduler Notifications tab.
   */
  _onNotificationRuleSelected : function ( notificationRule, schedulerNotificationsTab )
  {
    var me = this;

    var notificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

    notificationRulesGrid.down( '#deleteNotificationRuleButton' ).enable();
    notificationRulesGrid.down( '#editNotificationRuleButton' ).enable();
    notificationRulesGrid.down( '#cloneNotificationRuleButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the SchedulerExecNotificationRulesGrid in the SchedulerNotificationsTab
   * is unselected.
   * @param schedulerNotificationsTab the scheduler Notifications tab.
   */
  _onNotificationRuleUnselected : function ( schedulerNotificationsTab )
  {
    var me = this;

    var notificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

    notificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
    notificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
    notificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();
  },


  /**
   * Invoked to refresh the data in the scheduler execution notification rules grid.
   * @param connectionId a connection ID.
   */
  _refreshNotificationRulesGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid', {
      connectionId : connectionId
    } ).execute();
  },


  _enableExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotificationRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _disableExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotificationRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _deleteExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotificationRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _editExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRuleId : execNotificationRule.get( 'id' ),
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _cloneExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRuleId : execNotificationRule.get( 'id' ),
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  }
} );
