/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Sends a test execution notification message.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.SendTestMessage', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required message channel profile ID property.
   */
  channelProfileId : undefined,

  /**
   * Required message channel property.
   */
  channel : undefined,

  /*
   * Property with the message recipients (required for channels != WEB_SERVICE, != SNMPV*).
   */
  recipients : undefined,

  /*
   * Property with the message subject template (required for EMAIL, SNMPV* and WEB_SERVICE channels).
   */
  subjectTemplate : undefined,

  /*
   * Required property with the message subject template.
   */
  bodyTemplate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.channelProfileId )
      Ext.Error.raise( 'Required config parameter missing: channelProfileId' );

    if ( !config.channel )
      Ext.Error.raise( 'Required config parameter missing: channel' );

    // recipients are not used by SNMPV* and WEB_SERVICE channels
    if ( config.channel != QDesk.enums.message.MessageChannel.SNMPV1_TRAP
        && config.channel != QDesk.enums.message.MessageChannel.SNMPV2C_TRAP
        && config.channel != QDesk.enums.message.MessageChannel.SNMPV3_TRAP
        && config.channel != QDesk.enums.message.MessageChannel.WEB_SERVICE )
    {
      if ( !config.recipients )
        Ext.Error.raise( 'Required config parameter missing: recipients' );
    }

    // subject required only for EMAIL, SNMPV* and WEB_SERVICE channels
    if ( config.channel == QDesk.enums.message.MessageChannel.EMAIL
        || config.channel == QDesk.enums.message.MessageChannel.SNMPV1_TRAP
        || config.channel == QDesk.enums.message.MessageChannel.SNMPV2C_TRAP
        || config.channel == QDesk.enums.message.MessageChannel.SNMPV3_TRAP
        || config.channel == QDesk.enums.message.MessageChannel.WEB_SERVICE )
    {
      if ( !config.subjectTemplate )
        Ext.Error.raise( 'Required config parameter missing: subjectTemplate' );
    }

    if ( !config.bodyTemplate )
      Ext.Error.raise( 'Required config parameter missing: bodyTemplate' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.SendTestMessage.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleSendTestMessageActionTest,
      params : {
        channelProfileId : me.channelProfileId,
        recipients : me.recipients,
        subjectTemplate : me.subjectTemplate,
        bodyTemplate : me.bodyTemplate
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfoDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.SendTestMessage.successMsg' ),
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.SendTestMessage.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
