/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Statistics tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',

    'QDesk.controller.command.PreviewChart',
    'QDesk.controller.command.SaveChart',
    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerStatisticsTab = QDesk.controller.QuartzControllerUtils.getSchedulerStatisticsTab( me.connectionId );

    me._bindEventHandlers( schedulerStatisticsTab );

    schedulerStatisticsTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Statistics tab to their respective handlers.
   */
  _bindEventHandlers : function ( schedulerStatisticsTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionStatistics &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var statisticsPanel = schedulerStatisticsTab.down( '#statisticsPanel' );

      // refresh button
      statisticsPanel.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshStatisticsView( me.connectionId );
          } );

      // preview chart button
      statisticsPanel.down( '#previewChartButton' ).on( 'click',
          function ()
          {
            me._previewStatisticsChart( me.connectionId );
          } );

      // save chart button
      //statisticsPanel.down( '#saveChartButton #saveChartAsJpeg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/jpeg' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsPng' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/png' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsSvg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/svg+xml' );
      //    } );


      statisticsPanel.on( 'statisticsCriteriaChange',
          function ( statisticsType, periodLength, periodStart )
          {
            me._refreshStatisticsView( me.connectionId );
          } );
    }
  },


  _refreshStatisticsView : function ( connectionId )
  {
    Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
      connectionId : connectionId
    } ).execute();
  },


  /**
   * Invoked to view the chart in a new modal window which allows the user to save the chart
   * through the browser context menu.
   * @param connectionId a connection ID.
   */
  _previewStatisticsChart : function ( connectionId )
  {
    var me = this;

    var statisticsTab = QDesk.controller.QuartzControllerUtils.getSchedulerStatisticsTab( connectionId );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = statisticsTab.down( '#statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.PreviewChart', {
      chart : chart
    } ).execute();
  },


  /**
   * Invoked to save the chart in the scheduler statistics tab.
   * @param connectionId a connection ID.
   * @param mimeType the chart image mime type.
   */
  _saveStatisticsChart : function ( connectionId, mimeType )
  {
    var statisticsTab = QDesk.controller.QuartzControllerUtils.getSchedulerStatisticsTab( connectionId );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = statisticsTab.down( '#statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.SaveChart', {
      chart : chart,
      mimeType : mimeType
    } ).execute();
  }
} );
