/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Triggers tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.QuartzControllerUtils',

    'QDesk.controller.command.PreviewChart',
    'QDesk.controller.command.SaveChart',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',

    'QDesk.controller.command.scheduler.quartz.AddTrigger',
    'QDesk.controller.command.scheduler.quartz.CloneTrigger',
    'QDesk.controller.command.scheduler.quartz.DeleteTrigger',
    'QDesk.controller.command.scheduler.quartz.EditTrigger',
    'QDesk.controller.command.scheduler.quartz.PauseTrigger',
    'QDesk.controller.command.scheduler.quartz.ResumeTrigger',

    'QDesk.controller.command.scheduler.quartz.SelectJob',

    'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule',

    'QDesk.controller.command.scheduler.quartz.chain.AddJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EditJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain',

    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerJobChainsView',

    'QDesk.enums.scheduler.quartz.TriggerGroupState',
    'QDesk.enums.scheduler.quartz.TriggerState',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( me.connectionId );

    me._bindEventHandlers( triggersTab );

    triggersTab.setInitialized( true );
  },


  /**
   * Binds events from the triggers tab to their respective handlers.
   */
  _bindEventHandlers : function ( triggersTab )
  {
    var me = this;
    me._bindEventHandlersTriggersGrid( triggersTab );

    // individual views
    me._bindEventHandlersHistoryView( triggersTab );
    me._bindEventHandlersStatisticsView( triggersTab );
    me._bindEventHandlersNotificationsView( triggersTab );
    me._bindEventHandlersJobChainsView( triggersTab );
  },


  /**
   * Binds the triggers grid events to their handlers.
   */
  _bindEventHandlersTriggersGrid : function ( triggersTab )
  {
    var me = this;

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    // refresh button
    triggersGrid.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me._refreshTriggersGrid( me.connectionId );
        } );

    var resumeTriggerButton = triggersGrid.down( '#resumeTriggerButton' );
    resumeTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._resumeTrigger( trigger );
    }, me );

    var pauseTriggerButton = triggersGrid.down( '#pauseTriggerButton' );
    pauseTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._pauseTrigger( trigger );
    }, me );

    var addTriggerButton = triggersGrid.down( '#addTriggerButton' );
    Ext.each( Ext.ComponentQuery.query( 'menuitem', addTriggerButton ),
        function ( menuitem )
        {
          menuitem.on( 'click', function ()
          {
            me._addTrigger( menuitem.qdesk.triggerType );
          }, me );
        } );

    var deleteTriggerButton = triggersGrid.down( '#deleteTriggerButton' );
    deleteTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._deleteTrigger( trigger );
    }, me );

    var editTriggerButton = triggersGrid.down( '#editTriggerButton' );
    editTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._editTrigger( trigger );
    }, me );

    var cloneTriggerButton = triggersGrid.down( '#cloneTriggerButton' );
    cloneTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._cloneTrigger( trigger );
    }, me );

    var filterByTriggerGroupCombo = triggersGrid.down( '#filterByTriggerGroupCombo' );
    filterByTriggerGroupCombo.on( 'change', function ( combo, newValue )
    {
      if ( newValue == null )
      {
        resumeTriggerGroupButton.disable();
        pauseTriggerGroupButton.disable();
      }
      else
      {
        // based on the selected trigger group state enable either the resume, or pause button
        var triggerGroup = filterByTriggerGroupCombo.getStore().findRecord( 'name', newValue );
        var triggerGroupState = triggerGroup.get( 'state' );

        switch ( triggerGroupState )
        {
          case QDesk.enums.scheduler.quartz.TriggerGroupState.NORMAL:
            resumeTriggerGroupButton.disable();
            pauseTriggerGroupButton.enable();
            break;

          case QDesk.enums.scheduler.quartz.TriggerGroupState.PAUSED:
            resumeTriggerGroupButton.enable();
            pauseTriggerGroupButton.disable();
            break;

          default:
            Ext.Error.raise( 'Unexpected trigger group state: ' + triggerGroupState );
        }
      }

      triggersGrid.getStore().clearFilter( true );

      if ( newValue != null )
      {
        triggersGrid.getStore().addFilter( [
          Ext.create( 'Ext.util.Filter', {
            property : 'trigger.group.name',
            value : newValue
          } )
        ], false );
      }

      // apply, or clear the filter and load the first page
      triggersGrid.getStore().loadPage( 1 );
    } );

    var resumeTriggerGroupButton = triggersGrid.down( '#resumeTriggerGroupButton' );
    resumeTriggerGroupButton.on( 'click', function ()
    {
      var triggerGroupName = filterByTriggerGroupCombo.getValue();
      me._resumeTriggerGroup( triggerGroupName );
    }, me );

    var pauseTriggerGroupButton = triggersGrid.down( '#pauseTriggerGroupButton' );
    pauseTriggerGroupButton.on( 'click', function ()
    {
      var triggerGroupName = filterByTriggerGroupCombo.getValue();
      me._pauseTriggerGroup( triggerGroupName );
    }, me );

    var filterQueryText = triggersGrid.down( '#filterQueryText' );

    var agentAvailable = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId );
    if ( agentAvailable )
    {
      var triggersIndexEnabled = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzTriggersIndexEnabled( me.connectionId );
      if ( !triggersIndexEnabled )
      {
        filterQueryText.emptyText =
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.triggersIndexDisabledEmptyText' );

        filterQueryText.applyEmptyText();
      }
    }
    else
    {
      filterQueryText.emptyText =
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexNoAgentEmptyText' );

      filterQueryText.applyEmptyText();
    }

    // event fired by the actioncolumn in the TriggersGrid
    triggersGrid.down( '#triggerJobDataMapColumn' ).on( 'requestViewJobDataMap',
        function ( jobDataMap, effectiveJobDataMap )
        {
          me._viewJobDataMap( jobDataMap, effectiveJobDataMap );
        } );

    triggersGrid.getStore().on( 'beforeload', function ( store, operation )
    {
      triggersGrid.down( '#refreshButton' ).disable();
      triggersGrid.down( '#addTriggerButton' ).disable();
      filterByTriggerGroupCombo.disable();
      filterQueryText.disable();

      /*
       * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
       * Therefore we need to force-fire this event by clearing the selection.
       */
      triggersGrid.getSelectionModel().deselectAll();
    } );

    triggersGrid.getStore().on( 'load', function ( triggerStore, records, successful, operation )
    {
      triggersGrid.down( '#refreshButton' ).enable();

      // enable only if the triggers index is enabled in the agent
      if ( triggersIndexEnabled )
      {
        filterQueryText.enable();
      }

      if ( successful )
      {
        // enable only if a compatible JVM Agent is installed
        if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
        {
          triggersGrid.down( '#addTriggerButton' ).enable();
        }

        filterByTriggerGroupCombo.enable();
      }
      else
      {
        triggerStore.removeAll();

        var rawData = this.getProxy().getReader().rawData;
        if ( rawData && !rawData.success )  // we better check the success flag to make sure there are actually some errors
        {
          QDesk.util.UxUtils.showJsonResponseErrors( rawData );
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingTriggersMsg' ) );
        }
      }
    } );

    triggersGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
    {
      if ( selectedRecords.length == 0 )
      {
        me._onTriggerUnselected( triggersTab );
      }
      else
      {
        me._onTriggerSelected( selectedRecords[ 0 ], triggersTab );
      }
    } );

    /* JobTriggersGrid context menu actions */
    triggersGrid.on( 'resumeTrigger', me._resumeTrigger, me );
    triggersGrid.on( 'pauseTrigger', me._pauseTrigger, me );
    triggersGrid.on( 'deleteTrigger', me._deleteTrigger, me );
    triggersGrid.on( 'editTrigger', me._editTrigger, me );
    triggersGrid.on( 'cloneTrigger', me._cloneTrigger, me );
  },


  /**
   * Binds the trigger execution history panel events to their handlers.
   */
  _bindEventHandlersHistoryView : function ( triggersTab )
  {
    var me = this;

    var historyView = triggersTab.down( '#historyView' );
    historyView.on( 'activate', function ()
    {
      me._refreshHistoryView( me.connectionId, true );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshExecNotificationsView( me.connectionId, false );
      me._refreshJobChainsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionHistory &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var execHistoryGrid = triggersTab.down( '#execHistoryGrid' );
      // assert execHistoryGrid != null - the trigger execution history grid should exist

      // refresh button
      execHistoryGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // max fetch date picker
      execHistoryGrid.down( '#maxFetchDateSelect' ).on( 'changeMaxFetchDate',
          function ( newValue )
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // max fetch size combo
      execHistoryGrid.down( '#maxFetchSizeSelect' ).on( 'changeMaxFetchSize',
          function ( newValue )
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // event fired by the actioncolumn in the ExecHistoryGrid
      execHistoryGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      execHistoryGrid.down( '#errorColumn' ).on( 'requestViewException',
          function ( exception )
          {
            me._viewException( me.connectionId, exception );
          } );

      execHistoryGrid.down( '#resultColumn' ).on( 'requestViewResult',
          function ( execHistoryId )
          {
            me._viewResult( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#userDataColumn' ).on( 'requestViewUserData',
          function ( execHistoryId )
          {
            me._viewUserData( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#logColumn' ).on( 'requestViewLog',
          function ( execHistoryId )
          {
            me._viewLog( me.connectionId, execHistoryId );
          } );

      var filterQueryText = execHistoryGrid.down( '#filterQueryText' );

      var agentAvailable = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId );
      if ( agentAvailable )
      {
        var execHistoryIndexEnabled = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecHistoryIndexEnabled( me.connectionId );
        if ( !execHistoryIndexEnabled )
        {
          filterQueryText.emptyText =
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.execHistoryIndexDisabledEmptyText' );

          filterQueryText.applyEmptyText();
        }
      }
      else
      {
        filterQueryText.emptyText =
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexNoAgentEmptyText' );

        filterQueryText.applyEmptyText();
      }

      execHistoryGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        execHistoryGrid.down( '#refreshButton' ).disable();
        execHistoryGrid.down( '#maxFetchDateSelect' ).disable();
        execHistoryGrid.down( '#maxFetchSizeSelect' ).disable();
        filterQueryText.disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        execHistoryGrid.getSelectionModel().deselectAll();
      } );

      execHistoryGrid.getStore().on( 'load', function ( execHistoryStore, records, successful, operation )
      {
        var triggersGrid = triggersTab.down( '#triggersGrid' );
        var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

        if ( triggerSelected )
        {
          execHistoryGrid.down( '#refreshButton' ).enable();
          execHistoryGrid.down( '#maxFetchDateSelect' ).enable();
          execHistoryGrid.down( '#maxFetchSizeSelect' ).enable();

          // enable only if the exec history index is enabled in the agent
          if ( execHistoryIndexEnabled )
          {
            filterQueryText.enable();
          }

          if ( successful )
          {
            // currently no logic
          }
        }

        if ( successful )
        {
          // currently no logic
        }
        else
        {
          execHistoryStore.removeAll();

          var rawData = this.getProxy().getReader().rawData;
          if ( rawData && !rawData.success )  // we better check the success flag to make sure there are actually some errors
          {
            QDesk.util.UxUtils.showJsonResponseErrors( rawData );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingExecHistoryMsg' ) );
          }
        }
      } );
    }
  },


  /**
   * Binds the trigger statistics panel events to their handlers.
   */
  _bindEventHandlersStatisticsView : function ( triggersTab )
  {
    var me = this;

    var statisticsView = triggersTab.down( '#statisticsView' );
    statisticsView.on( 'activate', function ()
    {
      me._refreshStatisticsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshExecNotificationsView( me.connectionId, false );
      me._refreshJobChainsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionStatistics &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var statisticsPanel = statisticsView.down( '#statisticsPanel' );

      // refresh button
      statisticsPanel.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshStatisticsView( me.connectionId, true );
          } );

      // preview chart button
      statisticsPanel.down( '#previewChartButton' ).on( 'click',
          function ()
          {
            me._previewStatisticsChart( me.connectionId );
          } );

      // save chart button
      //statisticsPanel.down( '#saveChartButton #saveChartAsJpeg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/jpeg' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsPng' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/png' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsSvg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/svg+xml' );
      //    } );


      statisticsPanel.on( 'statisticsCriteriaChange',
          function ( statisticsType, periodLength, periodStart )
          {
            me._refreshStatisticsView( me.connectionId, true );
          } );
    }
  },


  /**
   * Binds the trigger notifications panel events to their handlers.
   */
  _bindEventHandlersNotificationsView : function ( triggersTab )
  {
    var me = this;

    var notificationsView = triggersTab.down( '#notificationsView' );
    notificationsView.on( 'activate', function ()
    {
      me._refreshExecNotificationsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshJobChainsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionNotifications &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var execNotificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );

      // refresh button
      execNotificationRulesGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshExecNotificationsView( me.connectionId, true );
          } );

      execNotificationRulesGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onNotificationRuleUnselected( triggersTab );
        }
        else
        {
          me._onNotificationRuleSelected( selectedRecords[ 0 ], triggersTab );
        }
      } );

      var filterQueryText = execNotificationRulesGrid.down( '#filterQueryText' );

      var agentAvailable = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId );
      if ( agentAvailable )
      {
        var notificationRulesIndexEnabled = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecNotificationRulesIndexEnabled( me.connectionId );
        if ( !notificationRulesIndexEnabled )
        {
          filterQueryText.emptyText =
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.notificationRulesIndexDisabledEmptyText' );

          filterQueryText.applyEmptyText();
        }
      }
      else
      {
        filterQueryText.emptyText =
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexNoAgentEmptyText' );

        filterQueryText.applyEmptyText();
      }

      execNotificationRulesGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        execNotificationRulesGrid.down( '#refreshButton' ).disable();
        execNotificationRulesGrid.down( '#addNotificationRuleButton' ).disable();
        execNotificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
        execNotificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
        execNotificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();
        filterQueryText.disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        execNotificationRulesGrid.getSelectionModel().deselectAll();
      } );

      execNotificationRulesGrid.getStore().on( 'load', function ( store, records, successful, operation )
      {
        var triggersGrid = triggersTab.down( '#triggersGrid' );
        var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

        if ( triggerSelected )
        {
          execNotificationRulesGrid.down( '#refreshButton' ).enable();

          if ( successful )
          {
            execNotificationRulesGrid.down( '#addNotificationRuleButton' ).enable();
          }

          // enable only if the notification rules index is enabled in the agent
          if ( notificationRulesIndexEnabled )
          {
            filterQueryText.enable();
          }
        }

        if ( successful )
        {
          // currently no logic
        }
        else
        {
          store.removeAll();

          var rawData = this.getProxy().getReader().rawData;
          if ( rawData && !rawData.success )  // we better check the success flag to make sure there are actually some errors
          {
            QDesk.util.UxUtils.showJsonResponseErrors( rawData );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingNotifRulesMsg' ) );
          }
        }
      } );

      execNotificationRulesGrid.down( '#addNotificationRuleButton' ).on( 'click', function ()
      {
        var triggersGrid = triggersTab.down( '#triggersGrid' );
        var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
        var trigger = triggerWithJobDetail.get( 'trigger' );

        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
          connectionId : me.connectionId,
          triggerGroupName : trigger.group.name,
          triggerName : trigger.name,
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshExecNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      execNotificationRulesGrid.down( '#deleteNotificationRuleButton' ).on( 'click', function ()
      {
        var execNotificationRule = execNotificationRulesGrid.getSelectionModel().getSelection()[ 0 ];
        me._deleteExecNotificationRule( execNotificationRule );
      } );

      execNotificationRulesGrid.down( '#editNotificationRuleButton' ).on( 'click', function ()
      {
        var execNotificationRule = execNotificationRulesGrid.getSelectionModel().getSelection()[ 0 ];
        me._editExecNotificationRule( execNotificationRule );
      } );

      execNotificationRulesGrid.down( '#cloneNotificationRuleButton' ).on( 'click', function ()
      {
        var execNotificationRule = execNotificationRulesGrid.getSelectionModel().getSelection()[ 0 ];
        me._cloneExecNotificationRule( execNotificationRule );
      } );

      // opens a window with messages enqueued by the notification rule
      execNotificationRulesGrid.down( '#messagesColumn' ).on( 'requestViewMessages',
          function ( execNotificationRuleId )
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
              connectionId : me.connectionId,
              creatorId : execNotificationRuleId,
              creatorType : QDesk.enums.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
            } ).execute();
          } );

      /* ExecNotificationRulesGrid context menu actions */
      execNotificationRulesGrid.on( 'enableRule', me._enableExecNotificationRule, me );
      execNotificationRulesGrid.on( 'disableRule', me._disableExecNotificationRule, me );
      execNotificationRulesGrid.on( 'deleteRule', me._deleteExecNotificationRule, me );
      execNotificationRulesGrid.on( 'editRule', me._editExecNotificationRule, me );
      execNotificationRulesGrid.on( 'cloneRule', me._cloneExecNotificationRule, me );
    }
  },


  /**
   * Binds the trigger job chains panel events to their handlers.
   */
  _bindEventHandlersJobChainsView : function ( triggersTab )
  {
    var me = this;

    var jobChainsView = triggersTab.down( '#jobChainsView' );
    jobChainsView.on( 'activate', function ()
    {
      me._refreshJobChainsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshExecNotificationsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobChaining &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var jobChainsGrid = triggersTab.down( '#jobChainsGrid' );

      // refresh button
      jobChainsGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshJobChainsView( me.connectionId, true );
          } );

      jobChainsGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onJobChainUnselected( triggersTab );
        }
        else
        {
          me._onJobChainSelected( selectedRecords[ 0 ], triggersTab );
        }
      } );

      var filterQueryText = jobChainsGrid.down( '#filterQueryText' );

      var agentAvailable = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId );
      if ( agentAvailable )
      {
        var jobChainsIndexEnabled = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzJobChainsIndexEnabled( me.connectionId );
        if ( !jobChainsIndexEnabled )
        {
          filterQueryText.emptyText =
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.jobChainsIndexDisabledEmptyText' );

          filterQueryText.applyEmptyText();
        }
      }
      else
      {
        filterQueryText.emptyText =
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexNoAgentEmptyText' );

        filterQueryText.applyEmptyText();
      }

      jobChainsGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        jobChainsGrid.down( '#refreshButton' ).disable();
        jobChainsGrid.down( '#addJobChainButton' ).disable();
        jobChainsGrid.down( '#deleteJobChainButton' ).disable();
        jobChainsGrid.down( '#editJobChainButton' ).disable();
        jobChainsGrid.down( '#cloneJobChainButton' ).disable();
        filterQueryText.disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        jobChainsGrid.getSelectionModel().deselectAll();
      } );

      jobChainsGrid.getStore().on( 'load', function ( store, records, successful, operation )
      {
        var triggersGrid = triggersTab.down( '#triggersGrid' );
        var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

        if ( triggerSelected )
        {
          jobChainsGrid.down( '#refreshButton' ).enable();

          if ( successful )
          {
            jobChainsGrid.down( '#addJobChainButton' ).enable();
          }

          // enable only if the job chains index is enabled in the agent
          if ( jobChainsIndexEnabled )
          {
            filterQueryText.enable();
          }
        }

        if ( successful )
        {
          // currently no logic
        }
        else
        {
          store.removeAll();

          var rawData = this.getProxy().getReader().rawData;
          if ( rawData && !rawData.success )  // we better check the success flag to make sure there are actually some errors
          {
            QDesk.util.UxUtils.showJsonResponseErrors( rawData );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingJobChainsMsg' ) );
          }
        }
      } );

      jobChainsGrid.down( '#addJobChainButton' ).on( 'click', function ()
      {
        var triggersGrid = triggersTab.down( '#triggersGrid' );
        var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
        var trigger = triggerWithJobDetail.get( 'trigger' );

        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.AddJobChain', {
          connectionId : me.connectionId,
          triggerGroupName : trigger.group.name,
          triggerName : trigger.name,
          /*
           * Upon success refresh the job chains view.
           */
          success : function ()
          {
            me._refreshJobChainsView( me.connectionId, true );
          }
        } ).execute();
      } );

      jobChainsGrid.down( '#deleteJobChainButton' ).on( 'click', function ()
      {
        var jobChain = jobChainsGrid.getSelectionModel().getSelection()[ 0 ];
        me._deleteJobChain( jobChain );
      } );

      jobChainsGrid.down( '#editJobChainButton' ).on( 'click', function ()
      {
        var jobChain = jobChainsGrid.getSelectionModel().getSelection()[ 0 ];
        me._editJobChain( jobChain );
      } );

      jobChainsGrid.down( '#cloneJobChainButton' ).on( 'click', function ()
      {
        var jobChain = jobChainsGrid.getSelectionModel().getSelection()[ 0 ];
        me._cloneJobChain( jobChain );
      } );

      /* JobChainsGrid context menu actions */
      jobChainsGrid.on( 'enableJobChain', me._enableJobChain, me );
      jobChainsGrid.on( 'disableJobChain', me._disableJobChain, me );
      jobChainsGrid.on( 'deleteJobChain', me._deleteJobChain, me );
      jobChainsGrid.on( 'editJobChain', me._editJobChain, me );
      jobChainsGrid.on( 'cloneJobChain', me._cloneJobChain, me );
    }
  },


  /**
   * Refreshes the triggers grid.
   * @param connectionId a connection ID.
   * @private
   */
  _refreshTriggersGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersGrid', {
      connectionId : connectionId
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger execution history view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshHistoryView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerHistoryView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger statistics view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshStatisticsView : function ( connectionId, populate )
  {
    var me = this;

    var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( connectionId );
    var statisticsView = triggersTab.down( '#statisticsView' );

    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerStatisticsView', {
      connectionId : connectionId,
      populate : populate,
      maskEl : statisticsView.getEl()
    } ).execute();
  },


  /**
   * Invoked to view the chart in a new modal window which allows the user to save the chart
   * through the browser context menu.
   * @param connectionId a connection ID.
   */
  _previewStatisticsChart : function ( connectionId )
  {
    var me = this;

    var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( connectionId );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = triggersTab.down( '#statisticsView #statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.PreviewChart', {
      chart : chart
    } ).execute();
  },


  /**
   * Invoked to save the chart in the trigger statistics view.
   * @param connectionId a connection ID.
   * @param mimeType the chart image mime type.
   */
  _saveStatisticsChart : function ( connectionId, mimeType )
  {
    var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( connectionId );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = triggersTab.down( '#statisticsView #statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.SaveChart', {
      chart : chart,
      mimeType : mimeType
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger notifications view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshExecNotificationsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerExecNotificationsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger job chains view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshJobChainsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerJobChainsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked when a trigger in the TriggersGrid in the TriggerTab is selected.
   * @param triggerWithJobDetail a QDesk.model.scheduler.quartz.QuartzTriggerWithJobDetail instance representing the selected trigger.
   * @param triggersTab a TriggersTab.
   */
  _onTriggerSelected : function ( triggerWithJobDetail, triggersTab )
  {
    var me = this;

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    var triggerState = triggerWithJobDetail.get( 'trigger' ).state;
    switch ( triggerState )
    {
      case QDesk.enums.scheduler.quartz.TriggerState.NONE:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.NORMAL:
        triggersGrid.down( '#resumeTriggerButton' ).disable();
        triggersGrid.down( '#pauseTriggerButton' ).enable();
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.PAUSED:
        triggersGrid.down( '#resumeTriggerButton' ).enable();
        triggersGrid.down( '#pauseTriggerButton' ).disable();
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.COMPLETE:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.ERROR:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.BLOCKED:
        break;

      default:
        Ext.Error.raise( 'Unexpected trigger state value: ' + triggerState );
    }

    triggersGrid.down( '#deleteTriggerButton' ).enable();

    // enable only if a compatible JVM Agent is installed
    if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      triggersGrid.down( '#editTriggerButton' ).enable();
      triggersGrid.down( '#cloneTriggerButton' ).enable();
    }

    var views = triggersTab.down( '#views' );

    var connectionId = triggersTab.getConnectionId();
    var activeView = views.getActiveTab();

    me._refreshHistoryView( connectionId, activeView.itemId == 'historyView' );
    me._refreshStatisticsView( connectionId, activeView.itemId == 'statisticsView' );
    me._refreshExecNotificationsView( connectionId, activeView.itemId == 'notificationsView' );
    me._refreshJobChainsView( connectionId, activeView.itemId == 'jobChainsView' );
  },


  /**
   * Invoked when a trigger in the TriggersGrid in the TriggersTab is unselected.
   * @param triggersTab a TriggersTab.
   */
  _onTriggerUnselected : function ( triggersTab )
  {
    var me = this;

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    triggersGrid.down( '#resumeTriggerButton' ).disable();
    triggersGrid.down( '#pauseTriggerButton' ).disable();
    triggersGrid.down( '#deleteTriggerButton' ).disable();
    triggersGrid.down( '#editTriggerButton' ).disable();
    triggersGrid.down( '#cloneTriggerButton' ).disable();

    var connectionId = triggersTab.getConnectionId();

    me._refreshHistoryView( connectionId, false );
    me._refreshStatisticsView( connectionId, false );
    me._refreshExecNotificationsView( connectionId, false );
    me._refreshJobChainsView( connectionId, false );
  },


  /**
   * Invoked when an execution notification rule record in the TriggerNotificationRulesGrid in the TriggerNotificationsView
   * is selected.
   * @param execNotificationRule a QDesk.model.scheduler.quartz.QuartzExecNotificationRule instance representing the selected
   * notification rule record.
   * @param triggersTab the TriggersTab.
   */
  _onNotificationRuleSelected : function ( execNotificationRule, triggersTab )
  {
    var me = this;

    var execNotificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );

    execNotificationRulesGrid.down( '#deleteNotificationRuleButton' ).enable();
    execNotificationRulesGrid.down( '#editNotificationRuleButton' ).enable();
    execNotificationRulesGrid.down( '#cloneNotificationRuleButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the TriggerNotificationRulesGrid in the TriggerNotificationsView
   * is unselected.
   * @param triggersTab the TriggersTab.
   */
  _onNotificationRuleUnselected : function ( triggersTab )
  {
    var me = this;

    var execNotificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );

    execNotificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
    execNotificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
    execNotificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();
  },


  /**
   * Invoked when a job chain record in the JobChainsGrid in the TriggerJobChainsView is selected.
   * @param jobChain a QDesk.model.scheduler.quartz.QuartzJobChain instance representing the selected
   * job chain.
   * @param triggersTab the TriggersTab.
   */
  _onJobChainSelected : function ( jobChain, triggersTab )
  {
    var me = this;

    var jobChainsGrid = triggersTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteJobChainButton' ).enable();
    jobChainsGrid.down( '#editJobChainButton' ).enable();
    jobChainsGrid.down( '#cloneJobChainButton' ).enable();
  },


  /**
   * Invoked when a job chain record in the JobChainsGrid in the TriggerJobChainsView is unselected.
   * @param triggersTab the TriggersTab.
   */
  _onJobChainUnselected : function ( triggersTab )
  {
    var me = this;

    var jobChainsGrid = triggersTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteJobChainButton' ).disable();
    jobChainsGrid.down( '#editJobChainButton' ).disable();
    jobChainsGrid.down( '#cloneJobChainButton' ).disable();
  },


  _resumeTriggerGroup : function ( triggerGroupName )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.ResumeTriggerGroup', {
      connectionId : me.connectionId,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _pauseTriggerGroup : function ( triggerGroupName )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.PauseTriggerGroup', {
      connectionId : me.connectionId,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _resumeTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ResumeTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _pauseTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.PauseTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _addTrigger : function ( triggerType )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.SelectJob', {
      connectionId : me.connectionId,
      /*
       * Upon job selection, open the Add trigger window.
       */
      success : function ( jobDetail )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.AddTrigger', {
          connectionId : me.connectionId,
          triggerType : triggerType,
          jobGroupName : jobDetail.get( 'group' ).name,
          jobName : jobDetail.get( 'name' ),

          /*
           * Upon success refresh the triggers tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
              connectionId : me.connectionId
            } ).execute();
          }
        } ).execute();
      }
    } ).execute();
  },


  _deleteTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.DeleteTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _editTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.EditTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _cloneTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.CloneTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _enableExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotificationRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _disableExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotificationRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _deleteExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotificationRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _editExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRuleId : execNotificationRule.get( 'id' ),
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _cloneExecNotificationRule : function ( execNotificationRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRuleId : execNotificationRule.get( 'id' ),
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _enableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _disableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _deleteJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _editJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain', {
      connectionId : me.connectionId,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _cloneJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain', {
      connectionId : me.connectionId,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap a job data map.
   * @param effectiveJobDataMap an effective job data map.
   */
  _viewJobDataMap : function ( jobDataMap, effectiveJobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      jobDataMap : jobDataMap,
      effectiveJobDataMap : effectiveJobDataMap
    } ).execute();
  },


  /**
   * Opens a window showing the job execution exception.
   * @param connectionId a connection ID.
   * @param exception the exception data.
   */
  _viewException : function ( connectionId, exception )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.ViewError', {
      errorText : exception
    } ).execute();
  },


  /**
   * Opens a window showing the job execution log.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewLog : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryLog', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution result.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewResult : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryResult', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution user data.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewUserData : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryUserData', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  }
} );
