/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Execution History view in the scheduler view panel's Triggers tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerHistoryView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected job.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionHistory &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( me.connectionId );
      var triggersGrid = triggersTab.down( '#triggersGrid' );

      var selectedTriggers = triggersGrid.getSelectionModel().getSelection();

      var execHistoryGrid = triggersTab.down( '#execHistoryGrid' );

      var execHistoryGridStore = execHistoryGrid.getStore();

      var reloadStores = true;

      if ( me.populate )
      {
        if ( Ext.isEmpty( selectedTriggers ) )
        {
          execHistoryGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
          execHistoryGridStore.getProxy().extraParams = {};
        }
        else
        {
          var triggerWithJobDetail = selectedTriggers[0];

          var trigger = triggerWithJobDetail.get( 'trigger' );

          execHistoryGridStore.getProxy().url =
              QDesk.util.constant.ActionUrl.quartz.execHistoryActionListForTrigger;

          var extraStoreParams = {
            connectionId : me.connectionId,
            triggerGroupName : trigger.group.name,
            triggerName : trigger.name
          };

          var maxFetchDate = execHistoryGrid.down( '#maxFetchDateSelect' ).getValue();
          Ext.apply( extraStoreParams, {
            maxFetchDate : maxFetchDate
          } );

          var maxFetchSize = execHistoryGrid.down( '#maxFetchSizeSelect' ).getValue();
          if ( maxFetchSize != -1 )  // -1 == unlimited
          {
            Ext.apply( extraStoreParams, {
              maxFetchSize : maxFetchSize
            } );
          }

          // if exec history index enabled, we add the filter query text param
          if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecHistoryIndexEnabled( me.connectionId ) )
          {
            var filterQueryText = execHistoryGrid.down( '#filterQueryText' ).getValue();
            if ( !Ext.isEmpty( filterQueryText ) )
            {
              Ext.apply( extraStoreParams, {
                filterQuery : filterQueryText
              } );
            }
          }

          execHistoryGridStore.getProxy().extraParams = extraStoreParams;
        }
      }
      else
      {
        if ( execHistoryGridStore.getProxy().url == QDesk.util.constant.ActionUrl.emptyStoreAction )
          reloadStores = false;

        execHistoryGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
        execHistoryGridStore.getProxy().extraParams = {};
      }

      // optimization to avoid repeated reloads of empty stores
      if ( reloadStores )
      {
        execHistoryGridStore.loadPage( 1, {
          scope : me,
          callback : function ( records, operation, success )
          {
            if ( success )
            {
              // invoke the success callback handler
              if ( me.success && Ext.isFunction( me.success ) )
              {
                me.success.apply( me );
              }
            }
            else
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            }
          }
        } );
      }
    }
  }
} );
