/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Controller that manages the accordion based NavigationPanel. Currently this controller
 * performs the following actions:
 * <ul>
 *   <li>Upon the selection of a child panel of the navigation panel this controller
 *   adds the {@code selected} class to the selected panel's header. Effectively this
 *   means that the title of the selected child panel will be rendered in bold.
 *   </li>
 * </ul>
 */
Ext.define( 'QDesk.controller.navigation.NavigationController', {
  extend : 'Ext.app.Controller',

  init : function ()
  {
    this.control( {
      // key is a component query, value is a listener configuration
      '#navigationPanel > panel' : {
        expand : this.onExpandChildPanel,
        collapse : this.onCollapseChildPanel
      },
      '#navigationPanel' : {
        afterrender : this.onAfterRender
      }
    } );
  },


  /**
   * Invoked when the navigationPanel's (accordion) child panel is expanded.
   * This method adds the 'selected' CSS class to the title element of the
   * expanded accordion panel.
   * @param childPanel a navigationPanel's child panel.
   */
  onExpandChildPanel : function ( childPanel )
  {
    var titleHtmlEl = Ext.query( "span.x-panel-header-text", childPanel.getEl().dom )[0];
    if ( titleHtmlEl )
    {
      new Ext.Element( titleHtmlEl ).addCls( 'selected' );
    }
  },


  /**
   * Invoked when the navigationPanel's (accordion) child panel is collapsed.
   * This method removes the 'selected' CSS class from the title element of the
   * collapsed accordion panel.
   * @param childPanel a navigationPanel's child panel.
   */
  onCollapseChildPanel : function ( childPanel )
  {
    var titleHtmlEl = Ext.query( "span.x-panel-header-text", childPanel.getEl().dom )[0];
    if ( titleHtmlEl )
    {
      new Ext.Element( titleHtmlEl ).removeCls( 'selected' );
    }
  },


  /**
   * Invoked when the navigationPanel component renders. This method
   * adds the 'selected' CSS class to the initially selected accordion panel.
   * @param navigPanel a navigationPanel's child panel.
   */
  onAfterRender : function ( navigPanel )
  {
    var me = this;
    navigPanel.items.each( function ( childPanel, index, childPanels )
    {
      if ( !childPanel.collapsed )
      {
        me.onExpandChildPanel( childPanel );
      }
    } );
  }
} );