/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.scheduler.quartz.SchedulerController', {
  extend : 'QDesk.controller.scheduler.quartz.BaseController',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.scheduler.quartz.BaseController',
    'QDesk.enums.scheduler.SchedulerType',
    'QDesk.util.UxUtils',
    'QDesk.view.main.scheduler.quartz.SchedulerTab'
  ],


  init : function ()
  {
    var me = this;

    me.application.addListener( 'schedulerStarted', me.onSchedulerStarted, me );
    me.application.addListener( 'schedulerPaused', me.onSchedulerPaused, me );
    me.application.addListener( 'schedulerStopped', me.onSchedulerStopped, me );

    me.callParent( arguments );
  },


  onLaunch : function ( application )
  {
    var me = this;
    me.callParent( arguments );
  },


  /**
   * Invoked when a scheduler has been started. This method checks if there is a tab for the
   * given scheduler and if so, it refreshes its sub-panels by firing appropriate events.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  onSchedulerStarted : function ( eventData )
  {
    var me = this;
    var schedulerType = QDesk.controller.command.connection.ConnectionUtils.getSchedulerType( eventData.connectionId );

    if ( schedulerType == QDesk.enums.scheduler.SchedulerType.QUARTZ )
    {
      var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( eventData.connectionId );
      if ( schedulerTab )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
          connectionId : eventData.connectionId
        } ).execute();
      }
    }
  },


  /**
   * Invoked when a scheduler has been paused. This method checks if there is a tab for the
   * given scheduler and if so, it refreshes its sub-panels by firing appropriate events.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  onSchedulerPaused : function ( eventData )
  {
    var me = this;
    var schedulerType = QDesk.controller.command.connection.ConnectionUtils.getSchedulerType( eventData.connectionId );

    if ( schedulerType == QDesk.enums.scheduler.SchedulerType.QUARTZ )
    {
      var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( eventData.connectionId );
      if ( schedulerTab )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
          connectionId : eventData.connectionId
        } ).execute();
      }
    }
  },


  /**
   * Invoked when a scheduler has been stopped. This method checks if there is a tab for the
   * given scheduler and if so, it destroys the tab panel.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  onSchedulerStopped : function ( eventData )
  {
    var me = this;
    var schedulerType = QDesk.controller.command.connection.ConnectionUtils.getSchedulerType( eventData.connectionId );

    if ( schedulerType == QDesk.enums.scheduler.SchedulerType.QUARTZ )
    {
      var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( eventData.connectionId );
      if ( schedulerTab )
      {
        var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[0];
        mainTabPanel.remove( schedulerTab );
      }
    }
  }
} );
