/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.top.LicenseInfoController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],


  onLaunch : function ( application )
  {
    var me = this;
    me.callParent( arguments );

    var licenseInfoPanel = Ext.ComponentQuery.query( '#licenseInfoPanel' )[0];
    //assert licenseInfoPanel != null;

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.licenseActionV10Info,
      callback : function ( opts, success, resp )
      {
        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            licenseInfoPanel.setProductNameAndVersion( json.productName, json.productVersion );

            licenseInfoPanel.setLicenseeNameAndExpiry( json.license.licensee.name, json.licenseExpiresInDays,
                json.licenseExpiresInDaysText );

            licenseInfoPanel.setVisible( true );

            /*
             * FIXME: In ExtJS 4.2 for some reason the license info data do not show up until the browser
             * gets resized to force relayout of the entire viewport. The manual doLayout of the top container
             * does the trick..
             */
            licenseInfoPanel.up( 'TopContainer' ).updateLayout();

            /*
             * If license has an expiry date, then keep refreshing the number of days till expiration
             * in the info panel.
             */
            if ( json.licenseExpiresInDays != null )
              Ext.defer( me._refreshLicenseExpiresInDays, 50000, me, [licenseInfoPanel] );
          }
          else
          {
            QDesk.Log.error( 'Error invoking ' + QDesk.util.constant.ActionUrl.licenseActionV10Info +
                ' to obtain license info.' );
          }
        }
        else
        {
          QDesk.Log.error( 'Error invoking ' + QDesk.util.constant.ActionUrl.licenseActionV10Info +
              ' to obtain license info.' );
        }
      }
    } );
  },


  _refreshLicenseExpiresInDays : function ( licenseInfoPanel )
  {
    var me = this;

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.licenseActionV10Info,
      callback : function ( opts, success, resp )
      {
        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            licenseInfoPanel.setLicenseeNameAndExpiry( json.license.licensee.name, json.licenseExpiresInDays,
                json.licenseExpiresInDaysText );

            /*
             * FIXME: In ExtJS 4.2 for some reason the license info data do not show up until the browser
             * gets resized to force relayout of the entire viewport. The manual doLayout of the top container
             * does the trick..
             */
            licenseInfoPanel.up( 'TopContainer' ).updateLayout();
          }
          else
          {
            QDesk.Log.error( 'Error invoking ' + QDesk.util.constant.ActionUrl.licenseActionV10Info +
                ' to obtain license info.' );
          }
        }
        else
        {
          QDesk.Log.error( 'Error invoking ' + QDesk.util.constant.ActionUrl.licenseActionV10Info +
              ' to obtain license info.' );
        }

        Ext.defer( me._refreshLicenseExpiresInDays, 50000, me, [licenseInfoPanel] );
      }
    } );
  }
} );
