/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * The Quartz scheduler trigger data model based on the
 * {@link com.quartzdesk.domain.model.scheduler.quartz.Trigger} class.
 */
Ext.define( 'QDesk.model.scheduler.quartz.QuartzTrigger', {
  extend : 'Ext.data.Model',

  requires : [
    'QDesk.model.TimeZone',
    'QDesk.model.scheduler.quartz.QuartzCalendar',
    'QDesk.model.scheduler.quartz.QuartzJobGroup',
    'QDesk.model.scheduler.quartz.QuartzTriggerGroup',
    'QDesk.util.constant.DateFormat'
  ],

  /*    Trigger Type               1.8.5  1.8.6  2.0.0  2.0.1  2.0.2  2.1.0  2.1.1  2.1.2  2.1.3  2.1.4  2.1.5  2.1.6  2.1.7  2.2.0  2.2.1
   *    CronTrigger                  y      y      y      y      y      y      y      y      y      y      y      y      y      y      y
   *    SimpleTrigger                y      y      y      y      y      y      y      y      y      y      y      y      y      y      y
   *    CalendarIntervalTrigger      n      n      y      y      y      y      y      y      y      y      y      y      y      y      y
   *    DailyIntervalTimeTrigger     n      n      n      n      n      y      y      y      y      y      y      y      y      y      y
   *    DateIntervalTrigger          y      y      n      n      n      n      n      n      n      n      n      n      n      n      n
   *    NthIncludedDayTrigger        y      y      n      n      n      n      n      n      n      n      n      n      n      n      n
   */

  fields : [
    /* QDesk.enums.scheduler.quartz.TriggerType */
    {name : 'type', type : 'string', allowNull : true},

    /*
     * Start of common trigger attributes.
     */
    {name : 'name', type : 'string', allowNull : true},
    {name : 'group', reference : 'scheduler.quartz.QuartzTriggerGroup'},

    /* com.quartzdesk.domain.model.scheduler.quartz.TriggerState */
    {name : 'state', type : 'string', allowNull : true},

    {name : 'jobName', type : 'string', allowNull : true},
    {name : 'jobGroup', reference : 'scheduler.quartz.QuartzJobGroup'},

    {name : 'calendar', reference : 'scheduler.quartz.QuartzCalendar'},

    {name : 'description', type : 'string', allowNull : true},

    {name : 'priority', type : 'int', allowNull : true},

    {name : 'misfireInstruction', type : 'int', allowNull : true},

    /* All dates are expected in ISO 8601 format (dateFormat: QDesk.util.constant.DateFormat.timestampFormatIso8601) */
    {name : 'startTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601},
    {name : 'endTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601},

    {name : 'previousFireTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601},
    {name : 'nextFireTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601},
    {name : 'finalFireTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601},

    {name : 'jobDataMap', type : 'auto'},
    {name : 'effectiveJobDataMap', type : 'auto'},

    /*
     * Quartz 1.x.
     */
    {name : 'volatility', type : 'boolean'},

    /*
     * QuartzDesk trigger monitoring URL.
     * Read-only property.
     */
    {name : 'monitoringUrl', type : 'string'},

    /*
     * See QDesk.model.scheduler.ExecHealthIndicator
     * Read-only property.
     *
     * Commented out because the Ext.sparkline.TriState widget cannot use values from the execHealthIndicator
     * indicator field.
     */
    //{name : 'execHealthIndicator', reference : 'scheduler.ExecHealthIndicator', allowNull : true},

    /*
     * Job execution health indicator numeric values in a format suitable for the sparklinetristate widget.
     * The format is:
     *   positive value = successful execution
     *   negative value = failed execution
     *   0 value = veto execution
     *
     * Read-only property.
     */
    {name : 'execHealthIndicatorValues', type : 'auto', allowNull : true},

    /* End of common trigger attributes */


    /* Start of trigger-type specific attributes */

    {name : 'cronExpression', type : 'string', allowNull : true}, // CronTrigger

    {name : 'timeZone', reference : 'TimeZone'},          // CronTrigger, CalendarIntervalTrigger (>=2.1.3), NthIncludedDayTrigger

    {name : 'repeatCount', type : 'int', allowNull : true},       // SimpleTrigger, DailyTimeIntervalTrigger

    {name : 'repeatInterval', type : 'int', allowNull : true},    // SimpleTrigger, CalendarIntervalTrigger, DailyTimeIntervalTrigger, DateIntervalTrigger
    /* com.quartzdesk.domain.model.scheduler.quartz.IntervalUnit */
    {name : 'repeatIntervalUnit', type : 'string', allowNull : true}, // CalendarIntervalTrigger, DailyTimeIntervalTrigger, DateIntervalTrigger

    {name : 'preserveHourOfDayAcrossDaylightSavings', type : 'boolean', allowNull : true},  // CalendarIntervalTrigger (>=2.1.3)
    {name : 'skipDayIfHourDoesNotExist', type : 'boolean', allowNull : true},               // CalendarIntervalTrigger (>=2.1.3)

//    { name : 'startTimeOfDay', type : 'date', dateFormat : QDesk.util.constant.DateFormat.dayTimeFormat },   // DailyTimeIntervalTrigger
//    { name : 'endTimeOfDay', type : 'date', dateFormat : QDesk.util.constant.DateFormat.dayTimeFormat },     // DailyTimeIntervalTrigger

    {name : 'startTimeOfDay', type : 'string', allowNull : true},   // DailyTimeIntervalTrigger
    {name : 'endTimeOfDay', type : 'string', allowNull : true},     // DailyTimeIntervalTrigger

    /* List of QDesk.enums.Weekday */
    {name : 'daysOfWeek', type : 'auto'},   // DailyTimeIntervalTrigger

    {name : 'nthDay', type : 'int', allowNull : true},                // NthIncludedDayTrigger
    /* com.quartzdesk.domain.model.scheduler.quartz.IntervalType */
    {name : 'intervalType', type : 'string', allowNull : true},       // NthIncludedDayTrigger

//    { name : 'fireAt', type : 'date', dateFormat : QDesk.util.constant.DateFormat.dayTimeFormat },  // NthIncludedDayTrigger
    {name : 'fireAt', type : 'string', allowNull : true},  // NthIncludedDayTrigger

    {name : 'nextFireCutoffInterval', type : 'int', allowNull : true} // NthIncludedDayTrigger

    /* End of trigger-type specific attributes */
  ]
} );
