/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.store.TimestampFormatStore', {
  extend : 'Ext.data.Store',

  storeId : 'TimestampFormatStore',

  fields : [ 'value', 'valueLabel' ],

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    //var sampleDate = new Date(2015, 4, 25, 16, 0, 0, 123);
    var sampleDate = new Date();

    Ext.apply( config, {
      data : [
        {
          value : 'Y-m-d h:i:s.u A T',  // 2015-05-31 04:00:00.123 PM CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'Y-m-d h:i:s.u A T' )
        },
        {
          value : 'Y-m-d H:i:s.u T',    // 2015-05-31 16:00:00.123 CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'Y-m-d H:i:s.u T' )
        },

        {
          value : 'd-m-Y h:i:s.u A T',  // 31-05-2015 04:00:00.123 PM CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'd-m-Y h:i:s.u A T' )
        },
        {
          value : 'd-m-Y H:i:s.u T',    // 31-05-2015 16:00:00.123 CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'd-m-Y H:i:s.u T' )
        },

        {
          value : 'm/d/Y h:i:s.u A T',  // 05/31/2015 04:00:00.123 PM CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'm/d/Y h:i:s.u A T' )
        },
        {
          value : 'm/d/Y H:i:s.u T',    // 05/31/2015 16:00:00.123 CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'm/d/Y H:i:s.u T' )
        },

        {
          value : 'd/m/Y h:i:s.u A T',  // 31/05/2015 04:00:00.123 PM CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'd/m/Y h:i:s.u A T' )
        },
        {
          value : 'd/m/Y H:i:s.u T',    // 31/05/2015 16:00:00.123 CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'd/m/Y H:i:s.u T' )
        },

        {
          value : 'd.m.Y h:i:s.u A T',  // 31.05.2015 04:00:00.123 PM CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'd.m.Y h:i:s.u A T' )
        },
        {
          value : 'd.m.Y H:i:s.u T',    // 31.05.2015 16:00:00.123 CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'd.m.Y H:i:s.u T' )
        },

        {
          value : 'Y.m.d h:i:s.u A T',  // 2015.05.31 04:00:00.123 PM CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'Y.m.d h:i:s.u A T' )
        },
        {
          value : 'Y.m.d H:i:s.u T',    // 2015.05.31 16:00:00.123 CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'Y.m.d H:i:s.u T' )
        },

        {
          value : 'M d, Y h:i:s.u A T',  // May 31, 2015 04:00:00.123 PM CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'M d, Y h:i:s.u A T' )
        },
        {
          value : 'M d, Y H:i:s.u T',    // May 31, 2015 16:00:00.123 CEDT
          valueLabel : me._formatValueLabel( sampleDate, 'M d, Y H:i:s.u T' )
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  _formatValueLabel : function(date, format)
  {
    //return format + '<br/>[' + Ext.Date.format( date, format ) + ']';
    return format;
  }
} );
