/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various common utility methods.
 */
Ext.define( 'QDesk.util.CommonUtils', {
  requires : [
    'QDesk.Log'
  ],

  singleton : true,


  /**
   * Returns -1, 0, or 1 if the specified version is lower, equal, or greater then the other version.
   * @param version a version.
   * @param otherVersion a minimum version.
   */
  versionCompare : function ( version, otherVersion )
  {
    if ( version.major > otherVersion.major )
    {
      return 1;
    }
    else if ( version.major == otherVersion.major )
    {
      if ( version.minor > otherVersion.minor )
      {
        return 1;
      }
      else if ( version.minor == otherVersion.minor )
      {
        if ( version.maintenance > otherVersion.maintenance )
        {
          return 1;
        }
        else if ( version.maintenance == otherVersion.maintenance )
        {
          return 0;
        }
        else
        {
          // version.maintenance < minVersion.maintenance
          return -1;
        }
      }
      else
      {
        // version.minor < minVersion.minor
        return -1;
      }
    }
    else
    {
      // version.major < minVersion.major
      return -1;
    }
  },


  /**
   * Encodes the data wrapped in the specified ExtJS model instance.
   * This method is useful when POSTing extJS model instances as
   * serialized JSON objects.
   * @param model an ExtJS model instance.
   * @return the JSON serialized model instance.
   */
  encode : function ( model )
  {
    var me = this;
    var clone = me._cloneAndDropEmpty( model.getData() );

    /*
     * If the ID property is not a number, then it means it is the ExtJS generated ID
     * and we need to drop it.
     */
    if ( !Ext.isNumber( clone.id ) )
    {
      delete clone.id;
    }

    return Ext.encode( clone );
  },


  /**
   * Deep clones the specified object and drops
   * all properties with an empty value.
   * @param o an object to clone.
   * @return the clone result.
   */
  _cloneAndDropEmpty : function ( o )
  {
    var me = this;

    if ( !o || 'object' !== typeof o )
    {
      return o;
    }
    if ( 'function' === typeof o.clone )
    {
      return o.clone();
    }
    var c = '[object Array]' === Object.prototype.toString.call( o ) ? [] : {};
    var p, v;
    for ( p in o )
    {
      if ( o.hasOwnProperty( p ) )
      {
        v = o[p];
        if ( v && Ext.typeOf( v ) == 'date' )
        {
          c[p] = new Date( v.getTime() );
        }
        else if ( v && 'object' === typeof v )
        {
          c[p] = me._cloneAndDropEmpty( v );
        }
        else
        {
          // copy only non-empty properties
          if ( !Ext.isEmpty( v ) )
            c[p] = v;
        }
      }
    }
    return c;
  }
} );
