/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


Ext.define( 'QDesk.util.EnumUtils', {
  requires : [
    'QDesk.Log',
    'QDesk.Msg'
  ],

  singleton : true,

  /**
   * Returns a localized name for the specified QDesk.enums.JsLoggingLevel value.
   * @param jsLoggingLevel a QDesk.enums.JsLoggingLevel value.
   * @return the localized JavaScript logging level name.
   */
  getJsLoggingLevelName : function ( jsLoggingLevel )
  {
    return QDesk.Msg.get( 'QDesk.enums.JsLoggingLevel.' + jsLoggingLevel );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.message.MessageChannel value.
   * @param messageChannel a QDesk.enums.message.MessageChannel value.
   * @return the localized message channel name.
   */
  getMessageChannelName : function ( messageChannel )
  {
    return QDesk.Msg.get( 'QDesk.enums.message.MessageChannel.' + messageChannel );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.MessageCreatorType value.
   * @param messageCreatorType a QDesk.enums.MessageCreatorType value.
   * @return the localized message creator type name.
   */
  getMessageCreatorTypeName : function ( messageCreatorType )
  {
    return QDesk.Msg.get( 'QDesk.enums.MessageCreatorType.' + messageCreatorType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.MessageStatus value.
   * @param messageStatus a QDesk.enums.MessageStatus value.
   * @return the localized message status name.
   */
  getMessageStatusName : function ( messageStatus )
  {
    return QDesk.Msg.get( 'QDesk.enums.MessageStatus.' + messageStatus );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.NotificationSeverity value.
   * @param notificationSeverity a QDesk.enums.NotificationSeverity value.
   * @return the localized notification severity name.
   */
  getNotificationSeverityName : function ( notificationSeverity )
  {
    return QDesk.Msg.get( 'QDesk.enums.NotificationSeverity.' + notificationSeverity );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.ExecNotificationRuleConditionType value.
   * @param notificationConditionType a QDesk.enums.scheduler.ExecNotificationRuleConditionType value.
   * @return the localized notification condition type name.
   */
  getExecNotificationRuleConditionTypeName : function ( notificationConditionType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.ExecNotificationRuleConditionType.' + notificationConditionType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.ExecStatus value.
   * @param execStatus a QDesk.enums.ExecStatus value.
   * @return the localized execution status name.
   */
  getExecStatusName : function ( execStatus )
  {
    return QDesk.Msg.get( 'QDesk.enums.ExecStatus.' + execStatus );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.ExpressionLanguage value.
   * @param expressionLanguage a QDesk.enums.ExpressionLanguage value.
   * @return the localized expression language name.
   */
  getExpressionLanguageName : function ( expressionLanguage )
  {
    return QDesk.Msg.get( 'QDesk.enums.ExpressionLanguage.' + expressionLanguage );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.JobChainConditionType value.
   * @param jobChainConditionType a QDesk.enums.scheduler.JobChainConditionType value.
   * @return the localized job chain condition type name.
   */
  getJobChainConditionTypeName : function ( jobChainConditionType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.JobChainConditionType.' + jobChainConditionType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.JobChainTargetExecType value.
   * @param jobChainTargetExecType a QDesk.enums.scheduler.JobChainTargetExecType value.
   * @return the localized job chain target exec type name.
   */
  getJobChainTargetExecTypeName : function ( jobChainTargetExecType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.JobChainTargetExecType.' + jobChainTargetExecType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.JobChainTargetType value.
   * @param jobChainTargetType a QDesk.enums.scheduler.JobChainTargetType value.
   * @return the localized job chain target type name.
   */
  getJobChainTargetTypeName : function ( jobChainTargetType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.JobChainTargetType.' + jobChainTargetType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.JmxProtocol value.
   * @param protocol a QDesk.enums.JmxProtocol value.
   * @return the localized JMX protocol name.
   */
  getProtocolName : function ( protocol )
  {
    return QDesk.Msg.get( 'QDesk.enums.JmxProtocol.' + protocol );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.ProxyType value.
   * @param proxyType a QDesk.enums.ProxyType value.
   * @return the localized proxy type name.
   */
  getProxyTypeName : function ( proxyType )
  {
    return QDesk.Msg.get( 'QDesk.enums.ProxyType.' + proxyType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.SnmpV3AuthenticationProtocol value.
   * @param authenticationProtocol a QDesk.enums.SnmpV3AuthenticationProtocol value.
   * @return the localized authentication protocol name.
   */
  getSnmpV3AuthenticationProtocolName : function ( authenticationProtocol )
  {
    return QDesk.Msg.get( 'QDesk.enums.SnmpV3AuthenticationProtocol.' + authenticationProtocol );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.SnmpV3PrivacyProtocol value.
   * @param privacyProtocol a QDesk.enums.SnmpV3PrivacyProtocol value.
   * @return the localized privacy protocol name.
   */
  getSnmpV3PrivacyProtocolName : function ( privacyProtocol )
  {
    return QDesk.Msg.get( 'QDesk.enums.SnmpV3PrivacyProtocol.' + privacyProtocol );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.SnmpV3SecurityLevel value.
   * @param securityLevel a QDesk.enums.SnmpV3SecurityLevel value.
   * @return the localized security level name.
   */
  getSnmpV3SecurityLevelName : function ( securityLevel )
  {
    return QDesk.Msg.get( 'QDesk.enums.SnmpV3SecurityLevel.' + securityLevel );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.TimePeriodLength value.
   * @param periodLength a QDesk.enums.TimePeriodLength value.
   * @return the localized period length name.
   */
  getTimePeriodLengthName : function ( periodLength )
  {
    return QDesk.Msg.get( 'QDesk.enums.TimePeriodLength.' + periodLength );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.Weekday value.
   * @param weekday a QDesk.enums.Weekday value.
   * @return the localized weekday name.
   */
  getWeekdayName : function ( weekday )
  {
    return QDesk.Msg.get( 'QDesk.enums.Weekday.' + weekday );
  },


  /**
   * Returns a localized short name for the specified QDesk.enums.Weekday value.
   * @param weekday a QDesk.enums.Weekday value.
   * @return the localized short weekday name.
   */
  getWeekdayShortName : function ( weekday )
  {
    return QDesk.Msg.get( 'QDesk.enums.Weekday.shortName.' + weekday );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.WSAuthenticationType value.
   * @param wsAuthenticationType a QDesk.enums.WSAuthenticationType value.
   * @return the localized WS authentication type name.
   */
  getWSAuthenticationTypeName : function ( wsAuthenticationType )
  {
    return QDesk.Msg.get( 'QDesk.enums.WSAuthenticationType.' + wsAuthenticationType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.MessageReceiverWSVersion value.
   * @param wsVersion a QDesk.enums.MessageReceiverWSVersion value.
   * @return the localized WS version name.
   */
  getMessageReceiverWSVersionName : function ( wsVersion )
  {
    return QDesk.Msg.get( 'QDesk.enums.MessageReceiverWSVersion.' + wsVersion );
  }
} );
