/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various utility methods for data stores.
 */
Ext.define( 'QDesk.util.StoreUtils', {
  requires : [
    'QDesk.Log'
  ],

  singleton : true,


  /**
   * Loads (asynchronously) all of the specified stores.
   * @param stores the array with stores to load.
   */
  loadStores : function ( stores )
  {
    Ext.each( stores, function ( store )
    {
      store.load();
    } );
  },


  /**
   * Invokes the specified callback method once all of the specified stores
   * are loaded. The callback method is passed a single argument - successful
   * flag indicating the overall result.
   * @param stores an array of stores.
   * @param callback a callback method to invoke once all stores have been loaded.
   * @param scope the scope used for the callback method.
   */
  onAllStoresLoad : function ( stores, callback, scope )
  {
    if ( !stores )
      Ext.Error.raise( 'Required stores argument.' );

    if ( !callback )
      Ext.Error.raise( 'Required callback argument.' );

    if ( !Ext.isFunction( callback ) )
      Ext.Error.raise( 'Callback argument must be a function.' );

    var loadedCount = 0;
    var loadedSuccessful = true;

    Ext.each( stores, function ( store, index )
    {
      // tree store load event handler takes different arguments then other data store load event handlers
      if ( store instanceof Ext.data.TreeStore )
      {
        var loadHandler = function ( store, records, successful )
        {
          store.un( 'load', loadHandler );

          loadedCount++;
          loadedSuccessful &= successful;

          if ( loadedCount == stores.length )
          {
            callback.apply( scope ? scope : this, [loadedSuccessful] );
          }
        };
        store.on( 'load', loadHandler );
      }
      else
      {
        // ordinary data store
        var loadHandler = function ( store, records, successful )
        {
          store.un( 'load', loadHandler );

          loadedCount++;
          loadedSuccessful &= successful;

          if ( loadedCount == stores.length )
          {
            callback.apply( scope ? scope : this, [loadedSuccessful] );
          }
        };
        store.on( 'load', loadHandler );
      }
    } );
  }
} );
