/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * QDesk configuration singleton that provides access to the application configuration data.
 */
Ext.define( 'QDesk.util.constant.DateFormat', {

  singleton : true,

  /**
   * Format used to parse and render date values.
   * 'Y-m-d' -> '2012-01-25'
   */
  dateFormat : 'Y-m-d',

  /**
   * Format used to parse and render timestamp values.
   * 'c' (ISO 8601 format) -> '2012-01-25T14:48:08.000+0000'
   */
  timestampFormatIso8601 : 'c',

  /**
   * Format used to render timestamp values.
   * 'Y-m-d H:i:s.u T' -> '2012-01-25 15:48:08.000 CEST'
   */
  //timestampFormatLong : 'Y-m-d h:i:s.u A T',  // 2015-05-31 04:00:00.123 PM CEDT
  //timestampFormatLong : 'Y-m-d H:i:s.u T',    // 2015-05-31 16:00:00.123 CEDT

  //timestampFormatLong : 'd-m-Y h:i:s.u A T',  // 31-05-2015 04:00:00.123 PM CEDT
  //timestampFormatLong : 'd-m-Y H:i:s.u T',    // 31-05-2015 16:00:00.123 CEDT

  //timestampFormatLong : 'm/d/Y h:i:s.u A T',  // 05/31/2015 04:00:00.123 PM CEDT
  //timestampFormatLong : 'm/d/Y H:i:s.u T',    // 05/31/2015 16:00:00.123 CEDT

  //timestampFormatLong : 'd/m/Y h:i:s.u A T',  // 31/05/2015 04:00:00.123 PM CEDT
  //timestampFormatLong : 'd/m/Y H:i:s.u T',    // 31/05/2015 16:00:00.123 CEDT

  //timestampFormatLong : 'd.m.Y h:i:s.u A T',  // 31.05.2015 04:00:00.123 PM CEDT
  //timestampFormatLong : 'd.m.Y H:i:s.u T',    // 31.05.2015 16:00:00.123 CEDT

  //timestampFormatLong : 'Y.m.d h:i:s.u A T',  // 2015.05.31 04:00:00.123 PM CEDT
  //timestampFormatLong : 'Y.m.d H:i:s.u T',    // 2015.05.31 16:00:00.123 CEDT

  //timestampFormatLong : 'M d, Y h:i:s.u A T', // May 31, 2015 04:00:00.123 PM CEDT
  //timestampFormatLong : 'M d, Y H:i:s.u T',   // May 31, 2015 16:00:00.123 CEDT

  /**
   * Format used to parse and render time values.
   * 'H:i' -> '15:48'
   */
  timeFormatShort : 'H:i',

  /**
   * Format used to parse and render time values.
   * 'H:i:s' -> '15:48:08'
   */
  timeFormatMedium : 'H:i:s',

  /**
   * Format used to parse and render time values.
   * 'H:i:s.u' -> '15:48:08.000'
   */
  timeFormatLong : 'H:i:s.u',

  /**
   * Format used to parse and render com.quartzdesk.domain.type.DayTime values.
   * 'H:i:s' -> '15:48:08'
   */
  dayTimeFormat : 'H:i:s'
} );
