/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


Ext.define( 'QDesk.util.quartz.QuartzEnumUtils', {
  requires : [
    'QDesk.Log',
    'QDesk.Msg'
  ],

  singleton : true,

  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.TriggerType value.
   * @param triggerType a QDesk.enums.scheduler.quartz.TriggerType value.
   * @return the localized repeat interval unit name.
   */
  getTriggerTypeName : function ( triggerType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.TriggerType.' + triggerType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.RepeatIntervalUnit value.
   * @param repeatIntervalUnit a QDesk.enums.scheduler.quartz.RepeatIntervalUnit value.
   * @return the localized repeat interval unit name.
   */
  getRepeatIntervalUnitName : function ( repeatIntervalUnit )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.RepeatIntervalUnit.' + repeatIntervalUnit );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.IntervalType value.
   * @param intervalType a QDesk.enums.scheduler.quartz.IntervalType value.
   * @return the localized interval type name.
   */
  getIntervalTypeName : function ( intervalType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.IntervalType.' + intervalType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.TriggerStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.TriggerStatisticsType value.
   * @return the localized JavaScript trigger statistics type name.
   */
  getTriggerStatisticsTypeName : function ( statisticsType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.TriggerStatisticsType.' + statisticsType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.JobStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.JobStatisticsType value.
   * @return the localized JavaScript job statistics type name.
   */
  getJobStatisticsTypeName : function ( statisticsType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.JobStatisticsType.' + statisticsType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.SchedulerStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.SchedulerStatisticsType value.
   * @return the localized JavaScript scheduler statistics type name.
   */
  getSchedulerStatisticsTypeName : function ( statisticsType )
  {
    var args = Array.prototype.slice.call( arguments ).slice( 1 );
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.SchedulerStatisticsType.' + statisticsType, args );
  },


  /**
   * Returns the misfire instruction value for the specified misfire instruction number (int).
   * @param triggerType a trigger type.
   * @param misfireInstructionNumber a misfire instruction number.
   * @returns the trigger type specific misfire instruction name.
   */
  getTriggerMisfireInstructionFromNumber : function ( triggerType, misfireInstructionNumber )
  {
    switch ( triggerType )
    {
      case QDesk.enums.scheduler.quartz.TriggerType.SIMPLE:
        switch ( misfireInstructionNumber )
        {
          case -1:
            return QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY;
          case 0:
            return QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.SMART_POLICY;
          case  1:
            return QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.FIRE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.RESCHEDULE_NOW_WITH_EXISTING_REPEAT_COUNT;
          case 3:
            return QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.RESCHEDULE_NOW_WITH_REMAINING_REPEAT_COUNT;
          case 4:
            return QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.RESCHEDULE_NEXT_WITH_REMAINING_COUNT;
          case 5:
            return QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.RESCHEDULE_NEXT_WITH_EXISTING_COUNT;
        }
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.CRON:
        switch ( misfireInstructionNumber )
        {
          case -1:
            return QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY;
          case  0:
            return QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.CALENDAR_INTERVAL:
        switch ( misfireInstructionNumber )
        {
          case -1:
            return  QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY;
          case 0:
            return QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.DAILY_TIME_INTERVAL:
        switch ( misfireInstructionNumber )
        {
          case -1:
            return QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY;
          case 0:
            return QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.DATE_INTERVAL:
        switch ( misfireInstructionNumber )
        {
          case 0:
            return QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.NTH_INCLUDED_DAY:
        switch ( misfireInstructionNumber )
        {
          case 0:
            return QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      default:
        throw Ext.Error.raise( 'Unexpected trigger type: ' + triggerType );
    }

    Ext.Error.raise( 'Unrecognized misfire instruction number: ' + misfireInstructionNumber +
        ' for trigger type: ' + triggerType );
  },


  /**
   * Returns the localized name for the specified trigger misfire instruction.
   * @param triggerType the trigger type (a QDesk.enums.scheduler.quartz.TriggerType value).
   * @param misfireInstruction a misfire instruction.
   * @return the localized JavaScript misfire instruction name.
   */
  getTriggerMisfireInstructionName : function ( triggerType, misfireInstruction )
  {
    switch ( triggerType )
    {
      case QDesk.enums.scheduler.quartz.TriggerType.SIMPLE:
        return QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.TriggerType.CRON:
        return QDesk.util.quartz.QuartzEnumUtils.getCronTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.TriggerType.CALENDAR_INTERVAL:
        return QDesk.util.quartz.QuartzEnumUtils.getCalendarIntervalTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.TriggerType.DAILY_TIME_INTERVAL:
        return QDesk.util.quartz.QuartzEnumUtils.getDailyTimeIntervalTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.TriggerType.DATE_INTERVAL:
        return QDesk.util.quartz.QuartzEnumUtils.getDateIntervalTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.TriggerType.NTH_INCLUDED_DAY:
        return QDesk.util.quartz.QuartzEnumUtils.getNthIncludedDayTriggerMisfireInstructionName( misfireInstruction );

      default:
        throw Ext.Error.raise( 'Unexpected trigger type: ' + triggerType );
    }
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getCalendarIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getCronTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getDailyTimeIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getDateIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getNthIncludedDayTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getSimpleTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.' +
        misfireInstruction );
  }
} );
