/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various job chain related utility methods.
 */
Ext.define( 'QDesk.util.quartz.QuartzJobChainUtils', {
  singleton : true,

  requires : [
    'QDesk.Msg',
    'QDesk.Properties',
    'QDesk.enums.Weekday',

    'QDesk.enums.scheduler.JobChainConditionType',
    'QDesk.enums.scheduler.JobChainTargetExecType',
    'QDesk.enums.scheduler.JobChainTargetType',

    'QDesk.model.scheduler.quartz.QuartzJobChain',
    'QDesk.model.scheduler.quartz.QuartzJobChainTarget',

    'QDesk.util.EnumUtils'
  ],


  /**
   * Creates a QDesk.model.scheduler.quartz.QuartzJobChain model instance and populates it with
   * the specified EditJobChainForm field values.
   * @param fieldValues a EditJobChainForm field values.
   * @param jobChainTargets an array of objects representing job chains targets (QDesk.model.scheduler.quartz.QuartzJobChainTarget data).
   * @return the created QDesk.model.scheduler.quartz.QuartzJobChain model instance.
   */
  createJobChain : function ( fieldValues, jobChainTargets )
  {
    var me = this;

    var jobChain = Ext.create( 'QDesk.model.scheduler.quartz.QuartzJobChain' );

    // set ID only for an updated job chain that has a numerical ID; newly added job chains have textual ExtJS IDs
    if ( Ext.isNumeric( fieldValues.id ) )
    {
      jobChain.set( 'id', fieldValues.id );
    }
    else
    {
      jobChain.set( 'id', null );
    }

    if ( !Ext.isEmpty( fieldValues.schedulerObjectName ) )
      jobChain.set( 'schedulerObjectName', fieldValues.schedulerObjectName );

    if ( !Ext.isEmpty( fieldValues.createdAt ) )
      jobChain.set( 'createdAt', fieldValues.createdAt );

    if ( !Ext.isEmpty( fieldValues.updatedAt ) )
      jobChain.set( 'updatedAt', fieldValues.updatedAt );

    /* Start of Quartz specific fields */
    if ( !Ext.isEmpty( fieldValues.jobGroupName ) )
      jobChain.set( 'jobGroupName', fieldValues.jobGroupName );

    if ( !Ext.isEmpty( fieldValues.jobName ) )
      jobChain.set( 'jobName', fieldValues.jobName );

    if ( !Ext.isEmpty( fieldValues.triggerGroupName ) )
      jobChain.set( 'triggerGroupName', fieldValues.triggerGroupName );

    if ( !Ext.isEmpty( fieldValues.triggerName ) )
      jobChain.set( 'triggerName', fieldValues.triggerName );
    /* End of Quartz specific fields */

    if ( !Ext.isEmpty( fieldValues.name ) )
      jobChain.set( 'name', fieldValues.name );

    if ( !Ext.isEmpty( fieldValues.description ) )
      jobChain.set( 'description', fieldValues.description );

    jobChain.set( 'enabled', fieldValues.enabled );

    if ( !Ext.isEmpty( fieldValues.enabledFromTime ) )
      jobChain.set( 'enabledFromTime', QDesk.util.FormatUtils.formatDayTime( fieldValues.enabledFromTime ) );

    if ( !Ext.isEmpty( fieldValues.enabledToTime ) )
      jobChain.set( 'enabledToTime', QDesk.util.FormatUtils.formatDayTime( fieldValues.enabledToTime ) );

    jobChain.set( 'enabledOnWeekday', me._createEnabledOnWeekdaysArray( fieldValues ) );

    if ( !Ext.isEmpty( fieldValues.conditionType ) )
      jobChain.set( 'conditionType', fieldValues.conditionType );

    if ( fieldValues.conditionType == QDesk.enums.scheduler.JobChainConditionType.EXEC_STATUS )
      jobChain.set( 'conditionExecStatus', fieldValues.conditionExecStatus );

    if ( fieldValues.conditionType == QDesk.enums.scheduler.JobChainConditionType.MAX_DURATION )
      jobChain.set( 'conditionMaxDuration', fieldValues.conditionMaxDuration );

    if ( fieldValues.conditionType == QDesk.enums.scheduler.JobChainConditionType.EXPRESSION )
    {
      jobChain.set( 'conditionExpressionLanguage', fieldValues.conditionExpressionLanguage );
      jobChain.set( 'conditionExpression', fieldValues.conditionExpression );
    }

    jobChain.set( 'target', jobChainTargets );

    return jobChain;
  },


  /**
   * Creates a QDesk.model.scheduler.quartz.QuartzJobChainTarget model instance and populates it with
   * the specified EditJobChainTargetForm field values.
   * @param fieldValues a EditJobChainTargetForm field values.
   * @return the created QDesk.model.scheduler.quartz.QuartzJobChainTarget model instance.
   */
  createJobChainTarget : function ( fieldValues )
  {
    var me = this;

    var jobChainTarget = Ext.create( 'QDesk.model.scheduler.quartz.QuartzJobChainTarget' );

    // set ID only for an updated target that has a numerical ID; newly added targets have textual ExtJS IDs
    if ( Ext.isNumeric( fieldValues.id ) )
    {
      jobChainTarget.set( 'id', fieldValues.id );
    }
    else
    {
      jobChainTarget.set( 'id', null );
    }

    if ( !Ext.isEmpty( fieldValues.type ) )
      jobChainTarget.set( 'type', fieldValues.type );

    if ( fieldValues.type == QDesk.enums.scheduler.JobChainTargetType.LOCAL )
    {
      // no remote connection fields
    }
    else if ( fieldValues.type == QDesk.enums.scheduler.JobChainTargetType.REMOTE )
    {
      if ( !Ext.isEmpty( fieldValues.remoteConnectionId ) )
        jobChainTarget.set( 'remoteConnectionId', fieldValues.remoteConnectionId );

      if ( !Ext.isEmpty( fieldValues.remoteConnectionName ) )
        jobChainTarget.set( 'remoteConnectionName', fieldValues.remoteConnectionName );
    }
    else
    {
      Ext.Error.raise( 'Unexpected job chain target type: ' + fieldValues.type );
    }

    jobChainTarget.set( 'enabled', fieldValues.enabled );

    if ( !Ext.isEmpty( fieldValues.jobGroupName ) )
      jobChainTarget.set( 'jobGroupName', fieldValues.jobGroupName );

    if ( !Ext.isEmpty( fieldValues.jobName ) )
      jobChainTarget.set( 'jobName', fieldValues.jobName );

    if ( !Ext.isEmpty( fieldValues.execType ) )
      jobChainTarget.set( 'execType', fieldValues.execType );

    if ( fieldValues.execType == QDesk.enums.scheduler.JobChainTargetExecType.IMMEDIATE )
    {
      // no fields
    }
    else if ( fieldValues.execType == QDesk.enums.scheduler.JobChainTargetExecType.DELAYED )
    {
      jobChainTarget.set( 'execTypeDelay', fieldValues.execTypeDelay );
    }
    else if ( fieldValues.execType == QDesk.enums.scheduler.JobChainTargetExecType.DATE_EXPRESSION )
    {
      jobChainTarget.set( 'execTypeDateExpressionLanguage', fieldValues.execTypeDateExpressionLanguage );
      jobChainTarget.set( 'execTypeDateExpression', fieldValues.execTypeDateExpression );
    }
    else
    {
      Ext.Error.raise( 'Unexpected job chain target exec type: ' + fieldValues.execType );
    }

    return jobChainTarget;
  },


  /**
   * Creates and returns an HTML table for the specified job chain value object.
   * This method is invoked from the job chains grids to obtain a formatted
   * HTML code to display job chain targets properties.
   * @param jobChainValues a job chain value object (not a QuartzJobChain model instance!).
   * @returns {string}
   */
  createJobChainPropertiesTable : function ( jobChainValues )
  {
    var me = this;

    var html = '<table class="jobChainProperties">' +
        '<thead>' +
        '<tr><th class="scheduler">' +
        QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobChainUtils.jobChainProperty.targetScheduler' ) +
        '</th><th class="jobName">' +
        QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobChainUtils.jobChainProperty.targetJobName' ) +
        '</th><th class="jobGroup">' +
        QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobChainUtils.jobChainProperty.targetJobGroup' ) +
        '</th><th class="enabled">' +
        QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobChainUtils.jobChainProperty.targetEnabled' ) +
        '</th><th class="execType">' +
        QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobChainUtils.jobChainProperty.targetExecType' ) +
        '</th></tr>' +
        '</thead>' +
        '<tbody>';

    var targets = jobChainValues.target;
    if ( Ext.isEmpty( targets ) )
    {
      html += '<tr><td colspan="5">' +
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobChainUtils.noTargetsMsg' ) +
          '</td></tr>';
    }
    else
    {
      Ext.each( targets, function ( target )
      {
        html +=
            '<tr><td>' +
            me.getTargetSchedulerDescription( target.type, target.remoteConnectionName ) +
            '</td><td>' +
            target.jobName +
            '</td><td>' +
            target.jobGroupName +
            '</td><td>' +
            (target.enabled ? QDesk.Msg.get( 'QDesk.yes' ) : QDesk.Msg.get( 'QDesk.no' ) ) +
            '</td><td>' +
            me.getTargetExecTypeDescription( target.execType, target.execTypeDelay,
                target.execTypeDateExpressionLanguage ) +
            '</td></tr>';
      } );
    }

    html += '</tbody>' +
        '</table>';

    return html;
  },


  /**
   * Returns a textual description of the scheduler of the specified job chain target.
   * @param type a job chain target type.
   * @param remoteConnectionName a job chain target remote connection name.
   * @returns {*}
   */
  getTargetSchedulerDescription : function ( type, remoteConnectionName )
  {
    switch ( type )
    {
      case QDesk.enums.scheduler.JobChainTargetType.LOCAL:
        return QDesk.util.EnumUtils.getJobChainTargetTypeName( type );
        break;

      case QDesk.enums.scheduler.JobChainTargetType.REMOTE:
        if ( !remoteConnectionName )  // can happen for connections that have been deleted
          remoteConnectionName = QDesk.Msg.get( 'QDesk.notAvailable' );

        return QDesk.util.EnumUtils.getJobChainTargetTypeName( type ) + ' [' +
            remoteConnectionName + ']';
        break;

      default:
        return QDesk.util.EnumUtils.getJobChainTargetTypeName( type );
    }
  },


  /**
   * Returns a textual description of the execution type of the specified job chain target.
   * @param execType a target exec type.
   * @param execTypeDelay a delay value.
   * @param execTypeDateExpressionLanguage an expression language.
   * @returns {*}
   */
  getTargetExecTypeDescription : function ( execType, execTypeDelay, execTypeDateExpressionLanguage )
  {
    switch ( execType )
    {
      case QDesk.enums.scheduler.JobChainTargetExecType.IMMEDIATE:
        return QDesk.util.EnumUtils.getJobChainTargetExecTypeName( execType );
        break;

      case QDesk.enums.scheduler.JobChainTargetExecType.DELAYED:
        return QDesk.util.EnumUtils.getJobChainTargetExecTypeName( execType ) + ' [' +
            execTypeDelay + 'ms]';
        break;

      case QDesk.enums.scheduler.JobChainTargetExecType.DATE_EXPRESSION:
        return QDesk.util.EnumUtils.getJobChainTargetExecTypeName( execType ) + ' [' +
            QDesk.util.EnumUtils.getExpressionLanguageName( execTypeDateExpressionLanguage ) + ']';
        break;

      default:
        return QDesk.util.EnumUtils.getJobChainTargetExecTypeName( execType );
    }
  },


  /**
   * Returns an array with weekdays the job chain is enabled on.
   * @param fieldValues form field values.
   * @return the weekday array.
   */
  _createEnabledOnWeekdaysArray : function ( fieldValues )
  {
    var enabledOnWeekdays = [];

    if ( fieldValues.enabledOnMonday )
      enabledOnWeekdays.push( QDesk.enums.Weekday.MONDAY );

    if ( fieldValues.enabledOnTuesday )
      enabledOnWeekdays.push( QDesk.enums.Weekday.TUESDAY );

    if ( fieldValues.enabledOnWednesday )
      enabledOnWeekdays.push( QDesk.enums.Weekday.WEDNESDAY );

    if ( fieldValues.enabledOnThursday )
      enabledOnWeekdays.push( QDesk.enums.Weekday.THURSDAY );

    if ( fieldValues.enabledOnFriday )
      enabledOnWeekdays.push( QDesk.enums.Weekday.FRIDAY );

    if ( fieldValues.enabledOnSaturday )
      enabledOnWeekdays.push( QDesk.enums.Weekday.SATURDAY );

    if ( fieldValues.enabledOnSunday )
      enabledOnWeekdays.push( QDesk.enums.Weekday.SUNDAY );

    return enabledOnWeekdays;
  }
} );
