/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various Quartz scheduler specific utility methods.
 */
Ext.define( 'QDesk.util.quartz.QuartzUtils', {
  requires : [
  ],

  singleton : true,

  /**
   * Creates a clone of the specified QDesk.model.scheduler.quartz.QuartzJobDetail instance.
   * @param jobDetail a QDesk.model.scheduler.quartz.QuartzJobDetail instance.
   * @return the clone.
   */
  cloneJobDetail : function ( jobDetail )
  {
    var clone = jobDetail.copy();  // does not clone the jobDataMap array

    var jobDataMap = jobDetail.get( 'jobDataMap' );

    if ( jobDataMap )
    {
      // assert Ext.isArray(jobDataMap)
      clone.set( 'jobDataMap', Ext.Array.clone( jobDataMap ) );
    }

    return clone;
  },


  /**
   * Create a job data map as an array of objects with the following structure:
   * <pre>
   * {
   *   name : 'some name',
   *   value : 'some value'
   * }
   * </pre>
   * @param jobDataMapGrid the job data map grid to extract values from.
   * @returns {Array}
   */
  createJobDataMap : function ( jobDataMapGrid )
  {
    var jobDataMap = [];

    Ext.Array.each( jobDataMapGrid.getStore().data.getRange(), function ( entry )
    {
      var entryName = Ext.String.trim( entry.get( 'name' ) );
      var entryValue = Ext.String.trim( entry.get( 'value' ) );

      // copy only entries with non-blank name
      if ( !Ext.isEmpty( entryName ) )
      {
        jobDataMap.push( {
          name : entryName,
          value : entryValue
        } );
      }
    } );

    return jobDataMap;
  },


  /**
   * Populates the specified job data map grid using the provided array of data
   * objects with the following structure:
   * <pre>
   * {
     *   name : 'some name',
     *   value : 'some value'
     * }
   * </pre>
   * @param jobDataMapGrid the job data map grid to extract values from.
   * @param jobDataMap the job data map.
   */
  populateJobDataMap : function ( jobDataMapGrid, jobDataMap )
  {
    jobDataMapGrid.getStore().loadData( jobDataMap, false );
  }
} );
