/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.FormNavigationMenuTreePanel', {
  extend : 'Ext.tree.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.model.FormNavigationMenuTreeNode',
    'QDesk.util.EnumUtils',
    'QDesk.util.constant.Const'
  ],

  alias : 'widget.FormNavigationMenuTreePanel',

  cls : 'formNavigationMenuTreePanel',

  scrollable : true,

  header : false,

  useArrows : true,

  rootVisible : false,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.root )
      Ext.Error.raise( 'Required config parameter missing: root' );

    var store = Ext.create( 'Ext.data.TreeStore', {
      model : 'QDesk.model.FormNavigationMenuTreeNode',
      proxy : {
        data : config.root,
        type : 'memory',
        reader : {
          type : 'json'
        }
      }
    } );

    Ext.apply( config, {
      store : store,

      columns : [
        {
          xtype : 'treecolumn',
          dataIndex : 'name',
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var licensedFormCount = record.get( 'licensedFormCount' );
            if ( licensedFormCount > 0 )
            {
              /* At least a single form panel bound to this navigation node, or any of its descendant nodes, is licensed. */
              var invalidFormCount = record.get( 'invalidFormCount' );
              if ( invalidFormCount > 0 )
              {
                return '<span class="invalid">' + value + '</span>'
              }
              else
              {
                return '<span>' + value + '</span>'
              }
            }
            else
            {
              /* None of the forms is licensed. */
              return '<span class="notLicensed">' + value + '</span>'
            }
          }
        }
      ]

//    In the future we might want to provide config option live search capability in the settings tree.
//
//      dockedItems : [
//        {
//          xtype : 'toolbar',
//          itemId : 'toolBar',
//          dock : 'top',
//          items : [
//          ]
//        }
//      ]
    } );

    me.callParent( [config] );
  }
} );
