/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.base.BaseConfigFormPanel', {
  extend : 'Ext.form.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.util.license.LicensedProduct'
  ],

  border : false,

  bodyPadding : 10,

  // anchor is the default layout for form panels
  layout : {
    type : 'anchor'
  },

  defaults : {
    anchor : '100%'
  },

  fieldDefaults : {
    labelWidth : 60,
    msgTarget : 'side'
  },

  qdesk : {
    /*
     * ID of the tree node in the config navigation tree upon whose selection this
     * panel should be activated.
     */
    nodeId : undefined,

    /*
     * Boolean flag set by this class to indicate if access to this form is allowed by
     * the installed license.
     */
    licensed : undefined
  },


  initComponent : function ()
  {
    var me = this;

    if ( me.items )
    {
      Ext.Array.insert( me.items, 0, [
        {
          xtype : 'WarningMessagePanel',
          itemId : 'licenseWarningMessagePanel',
          hidden : true,
          style : {
            marginBottom : '5px'
          },
          qdesk : {
            msg : QDesk.Msg.get( 'QDesk.view.base.BaseConfigFormPanel.licenseRestrictionsApplied',
                QDesk.util.license.LicensedProduct.productName )
          }
        }
      ] );
    }

    me.callParent();

    me._applyLicenseRestrictions();
  },


  _applyLicenseRestrictions : function ()
  {
    var me = this;

    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;

    var licensed = false;
    var restrictedByLicense = false;

    /* Disabled all form elements that are not licensed. */
    Ext.each( Ext.ComponentQuery.query( 'container, field', me ),
        function ( component )
        {
          if ( component.qdesk && component.qdesk.licensedFor )
          {
            if ( Ext.Array.contains( component.qdesk.licensedFor, licensedProductEdition ) )
            {
              licensed = true;
            }
            else
            {
              /* disable component if it is not licensed */
              component.setDisabled( true );
              restrictedByLicense = true;
            }
          }
          else
          {
            // licensed = true; - intentionally commented out, because forms can contain fields and containers
            // without the qdesk.licensedFor config values
          }
        } );

    me.qdesk.licensed = licensed;

    /* */
    if ( restrictedByLicense )
    {
      me.down( '#licenseWarningMessagePanel' ).setVisible( true );
    }
  }
} );
