/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Base class for a scheduler tab sub-tabs. This class provides access to the
 * sub-tab initialization flag and scheduler description object in the parent
 * tab.
 */
Ext.define( 'QDesk.view.base.scheduler.BaseSchedulerSubTab', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.util.license.LicensedProduct'
  ],

  bodyCls : 'baseSchedulerSubTabBody',

  closable : false,

  scrollable : true,

  /**
   * Returns the value of the QDesk boolean flag indicating whether tab has been initialized.
   * @return the QDesk initialization flag.
   */
  getInitialized : function ()
  {
    return this.qdesk.initialized;
  },


  /**
   * Sets the value of the QDesk boolean flag indicating whether tab has been initialized.
   * @param initialized the QDesk initialization flag.
   */
  setInitialized : function ( initialized )
  {
    this.qdesk.initialized = initialized;
  },


  /**
   * Returns the connection ID this tab has been created for.
   * @return the connection ID.
   */
  getConnectionId : function ()
  {
    var me = this;
    var schedulerTab = me.up( 'panel' );
    // assert schedulerTab != null
    return schedulerTab.getConnectionId();
  },


  /**
   * Returns true if the QuartzDesk JVM Agent is available and compatible with the
   * QuartzDesk GUI, false otherwise.
   * @param jvmAgentInfo QuartzDesk JVM Agent info.
   * @returns true if the QuartzDesk JVM Agent is available and compatible with the
   * QuartzDesk GUI, false otherwise.
   */
  isJvmAgentAvailableAndCompatible : function ( jvmAgentInfo )
  {
    return jvmAgentInfo.available && jvmAgentInfo.compatible;
  },


  /**
   * Returns the QuartzDesk JVM Agent availability and compatibility warning message
   * that gets shown in the yellow box in the GUI if the agent is either not available,
   * or its version/edition is not compatible with the GUI.
   * @param jvmAgentInfo the JVM Agent info.
   * @returns the warning message.
   */
  getJvmAgentWarningMessage : function ( jvmAgentInfo )
  {
    if ( jvmAgentInfo.available )
    {
      if ( !jvmAgentInfo.compatibleVersion )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionMsg',
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.version ),
            QDesk.util.FormatUtils.formatProductVersion( QDesk.util.license.LicensedProduct.productVersion ) );
      }
      else if ( !jvmAgentInfo.compatibleEdition )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionMsg',
            jvmAgentInfo.edition,
            QDesk.util.license.LicensedProduct.productEdition );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotAvailableMsg' );
    }
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      qdesk : {
        // tab initialization flag
        initialized : false
      }
    } );

    me.callParent( [config] );
  }
} );
