/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.ViewLogWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.SaveLog'
  ],

  alias : 'widget.ViewLogWindow',

  cls : 'viewLogWindow',
  iconCls : 'viewLogWindowIcon',

  scrollable : true,

  modal : true,
  resizable : true,
  maximizable : true,

  width : 800,
  height : 500,

  minWidth : 800,
  minHeight : 500,

  layout : 'fit',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.ViewLogWindow.windowTitle' ),

      items : [
        {
          xtype : 'htmleditor',
          itemId : 'editor',
          enableAlignments : false,
          enableColors : false,
          enableFont : false,
          enableFontSize : false,
          enableFormat : false,
          enableLinks : false,
          enableLists : false,
          enableSourceEdit : false,
          border : false
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'top',
          //ui : 'footer',
          items : [
            {
              xtype : 'button',
              itemId : 'saveButton',
              scale : 'small',
              iconCls : 'save',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.ViewLogWindow.tooltipSaveButton' ),
              disabled : true
            },
            {
              xtype : 'button',
              itemId : 'autoScrollButton',
              scale : 'small',
              iconCls : 'autoScroll',
              pressed : false,
              enableToggle : true,
              tooltip : QDesk.Msg.get( 'QDesk.view.main.ViewLogWindow.tooltipAutoScrollButton' ),
              disabled : true
            },
            '-',
            {
              xtype : 'button',
              itemId : 'interruptJobButton',
              scale : 'small',
              iconCls : 'interruptJob',
              pressed : false,
              enableToggle : true,
              tooltip : QDesk.Msg.get( 'QDesk.view.main.ViewLogWindow.tooltipInterruptJobButton' ),
              disabled : true
            }
          ]
        },
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'closeButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelClose' )
            }
          ]
        }
      ]
    } );

    me.callParent( [config] );

    var editor = me.down( '#editor' );
    // make the editor read-only
    editor.setReadOnly( true );

    // hide the toolbar that is visible even though all toolbar controls have been disabled
    // FIXME: in ExtJS 4.2 this resets the height of the editor to its default height of 100px
    // editor.getToolbar().hide();

    me.down( '#closeButton' ).on( 'click', function ()
    {
      me.close();
    } );
  },


  initComponent : function ()
  {
    var me = this;
    me.callParent();

    var editor = me.down( '#editor' );
    var saveButton = me.down( 'toolbar #saveButton' );

    saveButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.SaveLog', {
        htmlLogData : editor.getValue()
      } ).execute();
    }, me );
  }
} );
