/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.config.ApplicationConfigWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.FormNavigationMenuTreePanel',
    'QDesk.view.main.config.colors.ColorsPanel',
    'QDesk.view.main.config.colors.ColorsExecHistoryPanel',
    'QDesk.view.main.config.confirm.ConfirmationsPanel',
    'QDesk.view.main.config.confirm.QuartzConfirmationsPanel',
    'QDesk.view.main.config.general.GeneralPanel'
  ],

  statics : {
    NAVIG_NODE_ID_ROOT : 'root',

    NAVIG_NODE_ID_COLORS : 'colors',
    NAVIG_NODE_ID_COLORS_EXEC_HISTORY : 'colorsExecHistory',

    NAVIG_NODE_ID_CONFIRMATIONS : 'confirmations',
    NAVIG_NODE_ID_CONFIRMATIONS_QUARTZ : 'quartzConfirmations',

    NAVIG_NODE_ID_GENERAL : 'general'
  },

  alias : 'widget.ApplicationConfigWindow',

  cls : 'applicationConfigWindow',
  iconCls : 'applicationConfigWindowIcon',

  modal : true,
  resizable : true,

  width : 700,
  minWidth : 700,

  height : 600,
  minHeight : 600,

  layout : {
    type : 'border'
  },


  qdesk : {
    /**
     * MixedCollection instance with unsaved changes.
     */
    changes : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      qdesk : {
        changes : Ext.create( 'Ext.util.MixedCollection' )
      }
    } );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.config.ApplicationConfigWindow.windowTitle' ),

      items : [
        {
          xtype : 'FormNavigationMenuTreePanel',
          itemId : 'navigationTreeMenuPanel',
          region : 'west',
          split : true,
          header : false,
          collapsible : true,
          collapseMode : 'mini',
          width : 250,
          root : {
            id : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_ROOT,  // ROOT folder ID
            name : QDesk.Msg.get( 'QDesk.view.main.config.ApplicationConfigWindow.navigationRoot' ),
            expanded : true,

            children : [
              /* General */
              {
                id : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_GENERAL,
                name : QDesk.Msg.get( 'QDesk.view.main.config.ApplicationConfigWindow.navigationGeneral' ),
                leaf : true
              },

              /* Colors */
              {
                id : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_COLORS,
                name : QDesk.Msg.get( 'QDesk.view.main.config.ApplicationConfigWindow.navigationColors' ),
                children : [
                  {
                    id : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_COLORS_EXEC_HISTORY,
                    name : QDesk.Msg.get( 'QDesk.view.main.config.ApplicationConfigWindow.navigationColors.execHistory' ),
                    leaf : true
                  }
                ]
              },

              /* Confirmations */
              {
                id : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_CONFIRMATIONS,
                name : QDesk.Msg.get( 'QDesk.view.main.config.ApplicationConfigWindow.navigationConfirmations' ),
                children : [
                  {
                    id : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_CONFIRMATIONS_QUARTZ,
                    name : QDesk.Msg.get( 'QDesk.view.main.config.ApplicationConfigWindow.navigationConfirmations.quartz' ),
                    leaf : true
                  }
                ]
              }
            ]
          }
        },
        {
          xtype : 'panel',
          itemId : 'cardPanel',
          region : 'center',
          layout : 'card',
          minWidth : 360,
          items : [
            {
              xtype : 'panel',
              itemId : 'infoPanel',
              border : false,
              cls : 'infoPanel',
              bodyPadding : '10px',
              html : QDesk.Msg.get( 'QDesk.view.main.config.ApplicationConfigWindow.infoText' )
            },

            /* General */
            {
              xtype : 'GeneralPanel',
              itemId : 'generalPanel',
              qdesk : {
                nodeId : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_GENERAL
              }
            },

            /* Confirmations */
            {
              xtype : 'ConfirmationsPanel',
              itemId : 'confirmationsPanel',
              qdesk : {
                nodeId : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_CONFIRMATIONS
              }
            },

            /* Quartz Confirmations */
            {
              xtype : 'QuartzConfirmationsPanel',
              itemId : 'quartzConfirmationsPanel',
              qdesk : {
                nodeId : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_CONFIRMATIONS_QUARTZ
              }
            },

            /* Colors */
            {
              xtype : 'ColorsPanel',
              itemId : 'colorsPanel',
              qdesk : {
                nodeId : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_COLORS
              }
            },
            {
              xtype : 'ColorsExecHistoryPanel',
              itemId : 'colorsExecHistoryPanel',
              qdesk : {
                nodeId : QDesk.view.main.config.ApplicationConfigWindow.NAVIG_NODE_ID_COLORS_EXEC_HISTORY
              }
            }
          ]
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'okButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelOk' )
            },
            {
              xtype : 'button',
              itemId : 'cancelButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
            },
            {
              xtype : 'button',
              itemId : 'applyButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelApply' ),
              disabled : true  // disabled by default and enabled upon app settings changes
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Returns the current configuration changes that have not been applied yet.
   * @return the object containing the changes.
   */
  getChanges : function ()
  {
    var me = this;

    var changes = {};

    me.qdesk.changes.eachKey( function ( key )
    {
      var value = me.qdesk.changes.getByKey( key );
      changes[ key ] = value;
    } );

    return changes;
  },


  /**
   * Returns the current configuration changes that have not been applied yet.
   * @return the object containing the changes.
   */
  resetChanges : function ()
  {
    var me = this;
    me.qdesk.changes.clear();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    // set data in all forms
    Ext.each( Ext.ComponentQuery.query( 'form', me ), function ( formPanel )
    {
      formPanel.getForm().setValues( QDesk.ApplicationConfig.data );
    } );

    me.on( 'beforerender', function ()
    {
      /* Keep track of changes to the settings */
      me._bindChangeDetectors();

      /* Sets the initial licensedFormCount and invalidFormCount flags in the navigation tree nodes. */
      me._initializeForms();
    } );

    // select the first node in the navigation panel (Connection)
    me.on( 'afterrender', function ()
    {
      /* Important to reset the changes. */
      me.qdesk.changes.clear();

      var navigationTreeMenuPanel = me.down( '#navigationTreeMenuPanel' );
      var navigationRootNode = navigationTreeMenuPanel.getStore().getRootNode();

      var selectionModel = navigationTreeMenuPanel.getSelectionModel();
      selectionModel.select( navigationRootNode.firstChild );
    } );

    // cancel button closes the window
    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } );

    var navigationTreeMenuPanel = me.down( '#navigationTreeMenuPanel' );

    var cardPanel = me.down( '#cardPanel' );

    // selection changes
    navigationTreeMenuPanel.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length > 0 )  // selection change event is also fired after deselection
          {
            var selectedRecord = selectedRecords[ 0 ];
            var nodeId = selectedRecord.get( 'id' );

            var formPanel = me._getFormPanelByNodeId( nodeId );

            if ( formPanel )
              cardPanel.getLayout().setActiveItem( formPanel );
          }
        }, me );
  },


  /**
   * Returns the form panel for the specified node ID.
   * @param nodeId a node ID.
   * @return {Ext.form.Panel} the form panel, or null.
   * @private
   */
  _getFormPanelByNodeId : function ( nodeId )
  {
    var me = this;

    var result = null;
    Ext.each( Ext.ComponentQuery.query( '#cardPanel form', me ), function ( formPanel )
    {
      if ( formPanel.qdesk.nodeId == nodeId )
      {
        result = formPanel;
        return false;
      }
    } );

    return result;
  },


  /**
   * Bind change detectors to all form fields.
   * @private
   */
  _bindChangeDetectors : function ()
  {
    var me = this;

    var fields = Ext.ComponentQuery.query( '#cardPanel form field', me );
    Ext.each( fields, function ( field )
    {
      if ( field.qdesk && field.qdesk.ignore )
      {
        // skip form fields marked with qdesk.ignore flag (various helper fields)
      }
      else
      {
        field.on( 'change', function ( field, newValue, oldValue )
        {
          var fieldName = field.getName();

          if ( newValue == QDesk.ApplicationConfig.get( fieldName ) )
          {
            // new value is equal to the currently configured value
            me.qdesk.changes.removeAtKey( fieldName );
            //QDesk.Log.debug( 'Reverted field value, name: ' + fieldName + ', value: ' + newValue );
          }
          else
          {
            me.qdesk.changes.add( fieldName, newValue );
            //QDesk.Log.debug( 'Changed field value, name: ' + fieldName + ', value: ' + newValue );
          }

          var formPanel = field.up( 'form' );
          me._updateInvalidFormCounts( formPanel );
        }, me );
      }
    } );
  },


  /**
   * Iterates through and validates all forms and sets the initial licensedFormCount and
   * invalidFormCount flag values.
   * @private
   */
  _initializeForms : function ()
  {
    var me = this;

    var formPanels = Ext.ComponentQuery.query( '#cardPanel form', me );
    Ext.each( formPanels, function ( formPanel )
    {
      me._updateLicensedFormCounts( formPanel );
      me._updateInvalidFormCounts( formPanel );
    }, me );
  },


  /**
   * Check to see if the specified form panel is licensed and based on the result
   * it updates values of the licensedFormCount flags in the navigation tree menu.
   * @private
   */
  _updateLicensedFormCounts : function ( formPanel )
  {
    var me = this;

    // find the navigation node for the form panel
    var treePanel = me.down( '#navigationTreeMenuPanel' );
    var navigationNode = treePanel.getStore().getNodeById( formPanel.qdesk.nodeId );

    if ( !navigationNode )
      Ext.Error.raise( 'Cannot find navigation node for form panel with nodeId: ' + formPanel.qdesk.nodeId );

    var licensed = formPanel.qdesk.licensed;

    if ( licensed )
    {
      while ( navigationNode != null )
      {
        navigationNode.set( 'licensedFormCount', navigationNode.get( 'licensedFormCount' ) + 1 );
        navigationNode = navigationNode.parentNode;
      }
    }
  },


  /**
   * Validates the specified form panel and based on the result it updates the
   * invalidFormCount flags in the navigation tree menu
   * @param formPanel a form panel to validate.
   * @private
   */
  _updateInvalidFormCounts : function ( formPanel )
  {
    var me = this;

    // find the navigation node for the form panel
    var treePanel = me.down( '#navigationTreeMenuPanel' );
    var navigationNode = treePanel.getStore().getNodeById( formPanel.qdesk.nodeId );

    if ( !navigationNode )
      Ext.Error.raise( 'Cannot find navigation node for form panel with nodeId: ' + formPanel.qdesk.nodeId );

    var invalidFormCount = navigationNode.get( 'invalidFormCount' );

    var validNow = formPanel.isValid();
    var validBefore = invalidFormCount == 0;

    while ( navigationNode != null )
    {
      if ( validNow )
      {
        if ( validBefore )
        {
          // do nothing, form was valid and still is valid
        }
        else
        {
          navigationNode.set( 'invalidFormCount', navigationNode.get( 'invalidFormCount' ) - 1 );
        }
      }
      else
      {
        if ( validBefore )
        {
          navigationNode.set( 'invalidFormCount', navigationNode.get( 'invalidFormCount' ) + 1 );
        }
        else
        {
          // do nothing, form was and still is invalid
        }
      }

      navigationNode = navigationNode.parentNode;
    }

    var navigationRootNode = treePanel.getRootNode();
    var rootInvalidFormCount = navigationRootNode.get( 'invalidFormCount' );

    var okButton = me.down( '#okButton' );
    okButton.setDisabled( rootInvalidFormCount > 0 );

    var applyButton = me.down( '#applyButton' );
    applyButton.setDisabled( rootInvalidFormCount > 0 || me.qdesk.changes.getCount() == 0 );
  }
} );
