/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel', {
  extend : 'QDesk.view.base.BaseConfigFormPanel',

  requires : [
    'QDesk.Msg'
  ],

  alias : 'widget.ColorsExecHistoryPanel',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        /* Log priority colors (used by ViewExecHistoryLog command) */
        {
          xtype : 'fieldset',
          fieldDefaults : {
            labelWidth : 65
          },
          title : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldsetLabelLog' ),
          items : [
            {
              xtype : 'colorfield',
              itemId : 'traceColor',
              name : 'viewLog.priorityColors.trace',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldLabelLogTrace' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldTooltipLogTrace' ),
              allowBlank : false
            },
            {
              xtype : 'colorfield',
              itemId : 'debugColor',
              name : 'viewLog.priorityColors.debug',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldLabelLogDebug' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldTooltipLogDebug' ),
              allowBlank : false
            },
            {
              xtype : 'colorfield',
              itemId : 'infoColor',
              name : 'viewLog.priorityColors.info',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldLabelLogInfo' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldTooltipLogInfo' ),
              allowBlank : false
            },
            {
              xtype : 'colorfield',
              itemId : 'warnColor',
              name : 'viewLog.priorityColors.warn',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldLabelLogWarn' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldTooltipLogWarn' ),
              allowBlank : false
            },
            {
              xtype : 'colorfield',
              itemId : 'errorColor',
              name : 'viewLog.priorityColors.error',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldLabelLogError' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldTooltipLogError' ),
              allowBlank : false
            },

            {
              xtype : 'fieldset',
              title : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldsetLabelLogExample' ),
              items : [
                {
                  xtype : 'htmleditor',
                  itemId : 'editor',
                  enableAlignments : false,
                  enableColors : false,
                  enableFont : false,
                  enableFontSize : false,
                  enableFormat : false,
                  enableLinks : false,
                  enableLists : false,
                  enableSourceEdit : false,
                  border : true,
                  height : 90,
                  anchor : '100%',
                  qdesk : {
                    ignore : true
                  }
                }
              ]
            }
          ],
          qdesk : {
            licensedFor : [
              QDesk.enums.ProductEdition.STANDARD,
              QDesk.enums.ProductEdition.ENTERPRISE
            ]
          }
        },

        {
          xtype : 'fieldcontainer',
          fieldDefaults : {
            labelWidth : 80
          },
          items : [
            {
              xtype : 'colorfield',
              itemId : 'resultColor',
              name : 'viewResult.textColor',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldLabelResult' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldTooltipResult' ),
              allowBlank : false
            },

            {
              xtype : 'colorfield',
              itemId : 'userDataColor',
              name : 'viewUserData.textColor',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldLabelUserData' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.config.colors.ColorsExecHistoryPanel.fieldTooltipUserData' ),
              allowBlank : false
            }
          ],
          qdesk : {
            licensedFor : [
              QDesk.enums.ProductEdition.STANDARD,
              QDesk.enums.ProductEdition.ENTERPRISE
            ]
          }
        }
      ]
    } );

    me.callParent( [config] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var previewEditor = me.down( '#editor' );

    // for some reason the empty toolbar is not hidden
    previewEditor.getToolbar().hide();
    previewEditor.setReadOnly( true );

    me.down( '#traceColor' ).on( 'change', me._updatePreviewEditor, me );
    me.down( '#debugColor' ).on( 'change', me._updatePreviewEditor, me );
    me.down( '#infoColor' ).on( 'change', me._updatePreviewEditor, me );
    me.down( '#warnColor' ).on( 'change', me._updatePreviewEditor, me );
    me.down( '#errorColor' ).on( 'change', me._updatePreviewEditor, me );
  },


  _updatePreviewEditor : function ()
  {
    var me = this;

    var previewEditor = me.down( '#editor' );

    var traceColor = me.down( '#traceColor' ).getValue();
    var debugColor = me.down( '#debugColor' ).getValue();
    var infoColor = me.down( '#infoColor' ).getValue();
    var warnColor = me.down( '#warnColor' ).getValue();
    var errorColor = me.down( '#errorColor' ).getValue();

    var previewValue =
        '<span style="color:#' + traceColor + '">This is TRACE message.</span><br/>' +
        '<span style="color:#' + debugColor + '">This is DEBUG message.</span><br/>' +
        '<span style="color:#' + infoColor + '">This is INFO message.</span><br/>' +
        '<span style="color:#' + warnColor + '">This is WARN message.</span><br/>' +
        '<span style="color:#' + errorColor + '">This is ERROR message.</span><br/>';

    previewEditor.setValue( previewValue );
  }
} );
