/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.ImProxyPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.store.enums.ProxyTypeStore'
  ],

  alias : 'widget.ImProxyPanel',

  border : false,

  layout : 'anchor',

  defaults : {
    anchor : '100%',
    labelWidth : 85
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.tabLabelProxy' ),

      items : [
        {
          xtype : 'combo',
          itemId : 'proxyType',
          name : 'proxyType',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldLabelProxyType' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldTooltipProxyType' ),
          value : '',
          allowBlank : false,
          editable : false,
          forceSelection : true,
          queryMode : 'local',
          store : 'enums.ProxyTypeStore',
          valueField : 'value',
          displayField : 'valueLabel'
        },

        {
          xtype : 'textfield',
          itemId : 'proxyHost',
          name : 'proxyHost',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldLabelProxyHost' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldTooltipProxyHost' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldEmptyTxtProxyHost' ),
          allowBlank : true,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'numberfield',
          itemId : 'proxyPort',
          name : 'proxyPort',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldLabelProxyPort' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldTooltipProxyPort' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldEmptyTxtProxyPort' ),
          minValue : 1,
          maxValue : 65535,
          anchor : '60%',
          allowBlank : true,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'textfield',
          itemId : 'proxyUsername',
          name : 'proxyUsername',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldLabelProxyUsername' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldTooltipProxyUsername' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldEmptyTxtProxyUsername' ),
          allowBlank : true,
          qdesk : {
            required : false
          }
        },
        {
          xtype : 'textfield',
          inputType : 'password',
          itemId : 'proxyPassword',
          name : 'proxyPassword',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldLabelProxyPassword' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldTooltipProxyPassword' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.ImProxyPanel.fieldEmptyTxtProxyPassword' ),
          allowBlank : true,
          qdesk : {
            required : false
          }
        }
      ]
    } );

    me.callParent( [config] );
  },

  /**
   * Updates the allowBlank flags of the proxy host and port form fields based on the
   * specified proxy configuration object.
   * @param proxy a proxy configuration object.
   */
  updateProxyFieldsAllowsBlank : function ( proxy )
  {
    var me = this;

    var proxyIsSet = !Ext.isEmpty( proxy ) && !Ext.isEmpty( proxy.type );

    var proxyHostField = me.down( '#proxyHost' );
    proxyHostField.allowBlank = !proxyIsSet;
    proxyHostField.validate();
    proxyHostField.clearInvalid();

    var proxyPortField = me.down( '#proxyPort' );
    proxyPortField.allowBlank = !proxyIsSet;
    proxyPortField.validate();
    proxyPortField.clearInvalid();
  },


  /**
   * Extracts proxy configuration values from the specified proxy configuration
   * object.
   *
   * @param proxy proxy configuration object.
   * @return the form field values.
   */
  proxyConfig2FieldValues : function ( proxy )
  {
    var fieldValues = {};

    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( fieldValues, {
        proxyType : proxy.type,
        proxyHost : proxy.host,
        proxyPort : proxy.port,
        proxyUsername : proxy.username,
        proxyPassword : proxy.password
      } );
    }

    return fieldValues;
  },


  /**
   * Extracts proxy related data from the specified form field values object
   * and creates a proxy configuration object. If the proxy type is set to
   * an empty value (Proxy type = "None"), then this method returns null.
   *
   * @param fieldValues form field values.
   */
  fieldValues2ProxyConfig : function ( fieldValues )
  {
    var me = this;

    // if proxy set to "None"
    if ( Ext.isEmpty( fieldValues.proxyType ) )
    {
      return null;
    }
    else
    {
      var proxy = {};

      Ext.apply( proxy, {
        type : fieldValues.proxyType,
        host : fieldValues.proxyHost,
        port : fieldValues.proxyPort
      } );

      if ( !Ext.isEmpty( fieldValues.proxyUsername ) )
      {
        Ext.apply( proxy, {
          username : fieldValues.proxyUsername
        } )
      }

      if ( !Ext.isEmpty( fieldValues.proxyPassword ) )
      {
        Ext.apply( proxy, {
          password : fieldValues.proxyPassword
        } )
      }

      return proxy;
    }
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    /*
     * Upon changing the proxy type to None, reset all proxy related form fields and make these
     * fields optional.
     */
    me.down( '#proxyType' ).on( 'change', function ( combo, newValue )
    {
      var proxyIsSet = !Ext.isEmpty( newValue );

      if ( !proxyIsSet )
      {
        me.down( '#proxyHost' ).setValue( null );
        me.down( '#proxyPort' ).setValue( null );
        me.down( '#proxyUsername' ).setValue( null );
        me.down( '#proxyPassword' ).setValue( null );
      }

      var proxyHostField = me.down( '#proxyHost' );
      proxyHostField.allowBlank = !proxyIsSet;
      proxyHostField.validate();
      proxyHostField.clearInvalid();

      var proxyPortField = me.down( '#proxyPort' );
      proxyPortField.allowBlank = !proxyIsSet;
      proxyPortField.validate();
      proxyPortField.clearInvalid();
    }, me );
  }
} )
;
