/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.im_icq.ImIcqProfilePanel', {
  extend : 'Ext.form.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.view.main.message.channel.ImProxyPanel',
    'QDesk.view.main.message.channel.im_icq.ImIcqAdvancedPanel',
    'QDesk.view.main.message.channel.im_icq.ImIcqBasicPanel'
  ],

  alias : 'widget.ImIcqProfilePanel',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3px 10px 5px 10px',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'IM_ICQ',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_icq.ImIcqProfilePanel.title' )
            },

            {
              xtype : 'displayfield',
              value : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_icq.ImIcqProfilePanel.disclaimerMsg' ),
              fieldCls : 'disclaimerMessageChannel'
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_icq.ImIcqProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_icq.ImIcqProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_icq.ImIcqProfilePanel.fieldEmptyTxtName' ),
              allowBlank : false,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'ImIcqBasicPanel'
            },
            {
              xtype : 'ImIcqAdvancedPanel'
            },
            {
              xtype : 'ImProxyPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessageEllipsis' ),
          formBind : true
        },
        {
          itemId : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true
        }
      ]
    } );

    me.callParent( [config] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name,
      username : profile.channelConfig.username,
      password : profile.channelConfig.password
    } );

    if ( profile.channelConfig.loginHost )
    {
      Ext.apply( values, {
        loginHost : profile.channelConfig.loginHost
      } );
    }

    if ( profile.channelConfig.loginPort )
    {
      Ext.apply( values, {
        loginPort : profile.channelConfig.loginPort
      } );
    }

    var proxyPanel = me.down( 'ImProxyPanel' );
    Ext.apply( values, proxyPanel.proxyConfig2FieldValues( profile.channelConfig.proxy ) );

    // based on the selected proxy type make the proxy host & port form fields required
    proxyPanel.updateProxyFieldsAllowsBlank( profile.channelConfig.proxy );

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with
   * form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.enums.message.MessageChannel.IM_ICQ );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      username : fieldValues.username,
      password : fieldValues.password
    } );

    if ( !Ext.isEmpty( fieldValues.loginHost ) )
    {
      Ext.apply( channelConfig, {
        loginHost : fieldValues.loginHost
      } );
    }

    if ( !Ext.isEmpty( fieldValues.loginPort ) )
    {
      Ext.apply( channelConfig, {
        loginPort : fieldValues.loginPort
      } );
    }

    var proxyPanel = me.down( 'ImProxyPanel' );
    var proxy = proxyPanel.fieldValues2ProxyConfig( fieldValues );
    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( channelConfig, {
        proxy : proxy
      } );
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
