/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.im_skype.ImSkypeProfilePanel', {
  extend : 'Ext.form.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.message.channel.skype.CheckLocalSkypeInstallation'
  ],

  alias : 'widget.ImSkypeProfilePanel',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3px 10px 5px 10px',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'IM_SKYPE',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_skype.ImSkypeProfilePanel.title' )
            },

            {
              xtype : 'displayfield',
              value : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_skype.ImSkypeProfilePanel.disclaimerMsg' ),
              fieldCls : 'disclaimerMessageChannel'
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_skype.ImSkypeProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_skype.ImSkypeProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_skype.ImSkypeProfilePanel.fieldEmptyTxtName' ),
              allowBlank : false,
              qdesk : {
                required : true
              }
            },

            {
              xtype : 'panel',
              border : false,
              layout : 'hbox',
              items : [
                {
                  xtype : 'button',
                  itemId : 'checkSkypeInstallationButton',
                  text : QDesk.Msg.get( 'QDesk.view.main.message.channel.im_skype.ImSkypeProfilePanel.buttonLabelCheckSkypeInstallation' )
                }
              ]
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessageEllipsis' ),
          formBind : true
        },
        {
          itemId : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true
        }
      ]
    } );

    me.callParent( [config] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.down( '#checkSkypeInstallationButton' ).on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.message.channel.skype.CheckLocalSkypeInstallation', {
        maskEl : me.up( 'window' ).getEl()
      } ).execute();
    } );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name
    } );

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with
   * form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.enums.message.MessageChannel.IM_SKYPE );

    var channelConfig = {};
    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
