/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.snmp.SnmpV2cTrapProfilePanel', {
  extend : 'Ext.form.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.view.main.message.channel.snmp.SnmpV2cTrapBasicPanel'
  ],

  alias : 'widget.SnmpV2cTrapProfilePanel',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3px 10px 5px 10px',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'SNMPV2C_TRAP',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV2cTrapProfilePanel.title' )
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV2cTrapProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV2cTrapProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV2cTrapProfilePanel.fieldEmptyTxtName' ),
              allowBlank : false,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'SnmpV2cTrapBasicPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessage' ),
          formBind : true
        },
        {
          itemId : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true
        }
      ]
    } );

    me.callParent( [config] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name
    } );

    if ( profile.channelConfig.host )
    {
      Ext.apply( values, {
        host : profile.channelConfig.host
      } );
    }

    if ( profile.channelConfig.port )
    {
      Ext.apply( values, {
        port : profile.channelConfig.port
      } );
    }

    if ( profile.channelConfig.community )
    {
      Ext.apply( values, {
        community : profile.channelConfig.community
      } );
    }

//    if ( profile.channelConfig.trapOid )
//    {
//      Ext.apply( values, {
//        trapOid : profile.channelConfig.trapOid
//      } );
//    }

    if ( profile.channelConfig.systemDescription )
    {
      Ext.apply( values, {
        systemDescription : profile.channelConfig.systemDescription
      } );
    }

    if ( profile.channelConfig.variableMessageTrapCustomType )
    {
      Ext.apply( values, {
        variableMessageTrapCustomType : profile.channelConfig.variableMessageTrapCustomType
      } );
    }

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with
   * form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.enums.message.MessageChannel.SNMPV2C_TRAP );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      host : fieldValues.host,
      port : fieldValues.port,
      community : fieldValues.community,
      //trapOid : fieldValues.trapOid,
      systemDescription : fieldValues.systemDescription,
      variableMessageTrapCustomType : fieldValues.variableMessageTrapCustomType
    } );

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
