/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.store.enums.SnmpV3AuthenticationProtocolStore',
    'QDesk.store.enums.SnmpV3PrivacyProtocolStore',
    'QDesk.store.enums.SnmpV3SecurityLevelStore',
    'QDesk.enums.SnmpV3SecurityLevel'
  ],

  alias : 'widget.SnmpV3TrapSecurityPanel',

  border : false,

  layout : 'anchor',

  defaults : {
    anchor : '100%',
    labelWidth : 75
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.tabLabelSecurity' ),

      items : [
        {
          xtype : 'textfield',
          itemId : 'username',
          name : 'username',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldLabelUsername' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldTooltipUsername' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldEmptyTxtUsername' ),
          value : 'quartzdesk', /* default username */
          allowBlank : false,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'combo',
          itemId : 'securityLevel',
          name : 'securityLevel',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldLabelSecurityLevel' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldTooltipSecurityLevel' ),
          allowBlank : true,
          editable : false,
          forceSelection : true,
          queryMode : 'local',
          store : 'enums.SnmpV3SecurityLevelStore',
          valueField : 'value',
          displayField : 'valueLabel'
        },

        {
          xtype : 'fieldset',
          itemId : 'authenticationFieldSet',
          title : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldsetLabelAuthentication' ),
          padding : '5 0 0 5',
          defaults : {
            labelWidth : 70
          },
          items : [
            {
              xtype : 'combo',
              itemId : 'authProtocol',
              name : 'authProtocol',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldLabelAuthProtocol' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldTooltipAuthProtocol' ),
              value : QDesk.enums.SnmpV3AuthenticationProtocol.MD5,
              allowBlank : true,
              editable : false,
              forceSelection : true,
              queryMode : 'local',
              store : 'enums.SnmpV3AuthenticationProtocolStore',
              valueField : 'value',
              displayField : 'valueLabel'
            },
            {
              xtype : 'textfield',
              inputType : 'password',
              itemId : 'authPassword',
              name : 'authPassword',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldLabelAuthPassword' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldTooltipAuthPassword' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldEmptyTxtAuthPassword' ),
              minLength : 8, /* USM passphrases must be at least 8 bytes long (RFC3414 §11.2) */
              allowBlank : true,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'fieldset',
          itemId : 'privacyFieldSet',
          title : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldsetLabelPrivacy' ),
          padding : '5 0 0 5',
          defaults : {
            labelWidth : 70
          },
          items : [
            {
              xtype : 'combo',
              itemId : 'privacyProtocol',
              name : 'privacyProtocol',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldLabelPrivacyProtocol' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldTooltipPrivacyProtocol' ),
              value : QDesk.enums.SnmpV3PrivacyProtocol.DES,
              allowBlank : true,
              editable : false,
              forceSelection : true,
              queryMode : 'local',
              store : 'enums.SnmpV3PrivacyProtocolStore',
              valueField : 'value',
              displayField : 'valueLabel'
            },
            {
              xtype : 'textfield',
              inputType : 'password',
              itemId : 'privacyPassword',
              name : 'privacyPassword',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldLabelPrivacyPassword' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldTooltipPrivacyPassword' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.snmp.SnmpV3TrapSecurityPanel.fieldEmptyTxtPrivacyPassword' ),
              minLength : 8, /* USM passphrases must be at least 8 bytes long (RFC3414 §11.2) */
              allowBlank : true,
              qdesk : {
                required : true
              }
            }
          ]
        }
      ]
    } );

    me.callParent( [config] );
  },


  /**
   * Updates the allowBlank flags of the proxy host and port form fields based on the
   * specified proxy configuration object.
   * @param proxy a proxy configuration object.
   */
  updateProxyFieldsAllowsBlank : function ( proxy )
  {
    var me = this;

    var proxyIsSet = !Ext.isEmpty( proxy ) && !Ext.isEmpty( proxy.type );

    var proxyHostField = me.down( '#proxyHost' );
    proxyHostField.allowBlank = !proxyIsSet;
    proxyHostField.validate();
    proxyHostField.clearInvalid();

    var proxyPortField = me.down( '#proxyPort' );
    proxyPortField.allowBlank = !proxyIsSet;
    proxyPortField.validate();
    proxyPortField.clearInvalid();
  },


  /**
   * Extracts security configuration values from the specified security
   * configuration object.
   *
   * @param security a security configuration object.
   * @return the form field values.
   */
  securityConfig2FieldValues : function ( security )
  {
    var fieldValues = {};

    if ( !Ext.isEmpty( security ) )
    {
      Ext.apply( fieldValues, {
        username : security.username,
        securityLevel : security.level,
        authProtocol : security.authProtocol,
        authPassword : security.authPassword,
        privacyProtocol : security.privacyProtocol,
        privacyPassword : security.privacyPassword
      } );
    }

    return fieldValues;
  },


  /**
   * Extracts security related data from the specified form field values object
   * and creates a security configuration object.
   *
   * @param fieldValues form field values.
   */
  fieldValues2SecurityConfig : function ( fieldValues )
  {
    var security = {};

    Ext.apply( security, {
      username : fieldValues.username,
      level : fieldValues.securityLevel
    } );

    switch ( fieldValues.securityLevel )
    {
      case QDesk.enums.SnmpV3SecurityLevel.NO_AUTH_NO_PRIV:
        break;

      case QDesk.enums.SnmpV3SecurityLevel.AUTH_NO_PRIV:
        Ext.apply( security, {
          authProtocol : fieldValues.authProtocol,
          authPassword : fieldValues.authPassword
        } );
        break;

      case QDesk.enums.SnmpV3SecurityLevel.AUTH_PRIV:
        Ext.apply( security, {
          authProtocol : fieldValues.authProtocol,
          authPassword : fieldValues.authPassword,
          privacyProtocol : fieldValues.privacyProtocol,
          privacyPassword : fieldValues.privacyPassword
        } );
        break;

      default:
        Ext.Error.raise( 'Unexpected SNMPv3 security level: ' + fieldValues.securityLevel );
    }

    return security;
  },


  initComponent : function ()
  {
    var me = this;
    me.callParent();

    /*
     * Upon changing the security level, update form fields on this security panel
     */
    me.down( '#securityLevel' ).on( 'change', function ( combo, newValue )
    {
      var authenticationFieldSet = me.down( '#authenticationFieldSet' );
      var authProtocolField = authenticationFieldSet.down( '#authProtocol' );
      var authPasswordField = authenticationFieldSet.down( '#authPassword' );

      var privacyFieldSet = me.down( '#privacyFieldSet' );
      var privacyProtocolField = privacyFieldSet.down( '#privacyProtocol' );
      var privacyPasswordField = privacyFieldSet.down( '#privacyPassword' );

      switch ( newValue )
      {
        case QDesk.enums.SnmpV3SecurityLevel.NO_AUTH_NO_PRIV:
          //authProtocolField.setValue( null );
          authProtocolField.allowBlank = true;

          //authPasswordField.setValue( null );
          authPasswordField.allowBlank = true;

          //privacyProtocolField.setValue( null );
          privacyProtocolField.allowBlank = true;

          //privacyPasswordField.setValue( null );
          privacyPasswordField.allowBlank = true;

          authenticationFieldSet.setVisible( false );
          privacyFieldSet.setVisible( false );
          break;

        case QDesk.enums.SnmpV3SecurityLevel.AUTH_NO_PRIV:
          authProtocolField.allowBlank = false;
          authPasswordField.allowBlank = false;

          //privacyProtocolField.setValue( null );
          privacyProtocolField.allowBlank = true;

          //privacyPasswordField.setValue( null );
          privacyPasswordField.allowBlank = true;

          authenticationFieldSet.setVisible( true );
          privacyFieldSet.setVisible( false );
          break;

        case QDesk.enums.SnmpV3SecurityLevel.AUTH_PRIV:
          authProtocolField.allowBlank = false;
          authPasswordField.allowBlank = false;
          privacyProtocolField.allowBlank = false;
          privacyPasswordField.allowBlank = false;

          authenticationFieldSet.setVisible( true );
          privacyFieldSet.setVisible( true );
          break;

        default:
          Ext.Error.raise( 'Unexpected SNMPv3 security level: ' + newValue );
      }

      authProtocolField.validate();
      authProtocolField.clearInvalid();

      authPasswordField.validate();
      authPasswordField.clearInvalid();

      privacyProtocolField.validate();
      privacyProtocolField.clearInvalid();

      privacyPasswordField.validate();
      privacyPasswordField.clearInvalid();
    }, me );

    // initial value
    me.down( '#securityLevel' ).setValue( QDesk.enums.SnmpV3SecurityLevel.NO_AUTH_NO_PRIV );
  }
} );
