/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.MessageReceiverWSVersion',
    'QDesk.store.enums.MessageReceiverWSVersionStore'
  ],

  alias : 'widget.WebServiceBasicPanel',

  border : false,

  layout : 'anchor',

  defaults : {
    anchor : '100%',
    labelWidth : 80
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.tabLabelBasic' ),

      items : [
        {
          xtype : 'combo',
          itemId : 'version',
          name : 'version',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldLabelVersion' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldTooltipVersion' ),
          value : QDesk.enums.MessageReceiverWSVersion.V5_0,
          allowBlank : false,
          editable : false,
          forceSelection : true,
          queryMode : 'local',
          store : 'enums.MessageReceiverWSVersionStore',
          valueField : 'value',
          displayField : 'valueLabel'
        },
        {
          xtype : 'textfield',
          itemId : 'endpointUrl',
          name : 'endpointUrl',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldLabelEndpoint' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldTooltipEndpoint' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldEmptyTxtEndpoint' ),
          allowBlank : false,
          qdesk : {
            required : true
          }
        },
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldsetLabelAuthentication' ),
          padding : '5 0 0 5',
          defaults : {
            labelWidth : 75
          },
          items : [
            {
              xtype : 'combo',
              itemId : 'authenticationType',
              name : 'authenticationType',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldLabelAuthenticationType' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldTooltipAuthenticationType' ),
              value : '',
              allowBlank : false,
              editable : false,
              forceSelection : true,
              queryMode : 'local',
              store : 'enums.WSAuthenticationTypeStore',
              valueField : 'value',
              displayField : 'valueLabel'
            },

            {
              xtype : 'textfield',
              itemId : 'authenticationUsername',
              name : 'authenticationUsername',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldLabelUsername' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldTooltipUsername' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldEmptyTxtUsername' ),
              allowBlank : true,
              qdesk : {
                required : true
              }
            },
            {
              xtype : 'textfield',
              inputType : 'password',
              itemId : 'authenticationPassword',
              name : 'authenticationPassword',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldLabelPassword' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldTooltipPassword' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel.fieldEmptyTxtPassword' ),
              allowBlank : true,
              qdesk : {
                required : true
              }
            }
          ]
        }
      ]
    } );

    me.callParent( [config] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    /*
     * Upon changing the authentication to None, reset the username and password form fields and make these
     * fields optional.
     */
    me.down( '#authenticationType' ).on( 'change', function ( combo, newValue )
    {
      var authIsSet = !Ext.isEmpty( newValue );

      if ( !authIsSet )
      {
        me.down( '#authenticationUsername' ).setValue( null );
        me.down( '#authenticationPassword' ).setValue( null );
      }

      var usernameField = me.down( '#authenticationUsername' );
      usernameField.allowBlank = !authIsSet;
      usernameField.validate();
      usernameField.clearInvalid();

      var passwordField = me.down( '#authenticationPassword' );
      passwordField.allowBlank = !authIsSet;
      passwordField.validate();
      passwordField.clearInvalid();
    }, me );
  }
} );
