/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.web_service.WebServiceProfilePanel', {
  extend : 'Ext.form.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.view.main.message.channel.web_service.WebServiceBasicPanel'
  ],

  alias : 'widget.WebServiceProfilePanel',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3px 10px 5px 10px',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'WEB_SERVICE',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceProfilePanel.title' )
            },

            {
              xtype : 'displayfield',
              value : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceProfilePanel.infoMsg' ),
              fieldCls : 'infoMessageChannel'
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.message.channel.web_service.WebServiceProfilePanel.fieldEmptyTxtName' ),
              allowBlank : false,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'WebServiceBasicPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessage' ),
          formBind : true
        },
        {
          itemId : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true
        }
      ]
    } );

    me.callParent( [config] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name,
      version : profile.channelConfig.version,
      endpointUrl : profile.channelConfig.endpointUrl
    } );

    if ( profile.channelConfig.authentication )
    {
      Ext.apply( values, {
        authenticationType : profile.channelConfig.authentication.type,
        authenticationUsername : profile.channelConfig.authentication.username,
        authenticationPassword : profile.channelConfig.authentication.password
      } );
    }

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with
   * form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.enums.message.MessageChannel.WEB_SERVICE );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      version : fieldValues.version,
      endpointUrl : fieldValues.endpointUrl
    } );

    if ( !Ext.isEmpty( fieldValues.authenticationType ) )
    {
      Ext.apply( channelConfig, {
        authentication : {
          type : fieldValues.authenticationType,
          username : fieldValues.authenticationUsername,
          password : fieldValues.authenticationPassword
        }
      } );

      // make the authentication username and password form fields required
      var usernameField = me.down( '#authenticationUsername' );
      usernameField.allowBlank = false;
      usernameField.validate();
      usernameField.clearInvalid();

      var passwordField = me.down( '#authenticationPassword' );
      passwordField.allowBlank = false;
      passwordField.validate();
      passwordField.clearInvalid();
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
