/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.enums.message.MessageChannel',
    'QDesk.enums.message.MessageCreatorType',
    'QDesk.enums.message.MessageStatus',

    'QDesk.store.message.MessageStore',

    'QDesk.util.constant.DateFormat',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.view.widget.MaxTodayDateField',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.MessageQueueGrid',

  cls : 'messageQueueGrid',

  //scrollable : true,

  qdesk : {
    /*
     * Required ID of the scheduler connection.
     */
    connectionId : undefined
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
        '<tpl if="subject"><span style="white-space: nowrap"><b>{[QDesk.Msg.get( \"QDesk.view.main.scheduler.common.message.MessageQueueGrid.textSubject\" )]}:</b> <code>{[Ext.htmlEncode(values.subject)]}</code></span><br/></tpl>' +
        '<tpl if="body"><b>{[QDesk.Msg.get( \"QDesk.view.main.scheduler.common.message.MessageQueueGrid.textBody\" )]}:</b> <pre style="padding-left: 5px; margin-top: 0px">{[Ext.htmlEncode(values.body)]}</pre><br/></tpl>' +
        '<tpl if="attachment &amp;&amp; attachment.length &gt; 0"><span style="white-space: nowrap"><b style="padding-left:5px">{[QDesk.Msg.get( \"QDesk.view.main.scheduler.common.message.MessageQueueGrid.textAttachments\" )]}:</b> ' +
        '<tpl for="attachment" between=", ">' +
        '<a href="{[this._getDownloadUrl(values.id)]}" target="msg-{parent.id}">{name}</a>' +
        '</tpl>' +
        '</span>' +
        '<iframe name="msg-{id}" height="0" width="0" style="display:none"></iframe>' +
        '</tpl>' +
        '</p>',
        {
          /**
           * Returns the download link URL for the specified attachment.
           * @param attachmentId the message attachment ID.
           * @return {String} the download URL.
           * @private
           */
          _getDownloadUrl : function ( attachmentId )
          {
            var grid = this.owner.grid;
            var qdesk = grid.qdesk;

            return QDesk.util.constant.ActionUrl.messageActionDownloadAttachment +
                '&connectionId=' + qdesk.connectionId +
                '&attachmentId=' + attachmentId;
          }
        }
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var store = Ext.create( 'QDesk.store.message.MessageStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'MaxTodayDateField',
          itemId : 'maxFetchDateSelect',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.fieldLabelMaxFetchDateSelect' ),
          editable : false,
          format : QDesk.util.constant.DateFormat.dateFormat,
          value : new Date(),
          maxValue : new Date(),
          labelAlign : 'right',
          labelWidth : 80,
          width : 180,
          disabled : true
        },
        {
          xtype : 'combo',
          itemId : 'maxFetchSizeSelect',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.fieldLabelMaxFetchSizeSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [10, '10'],
              [20, '20'],
              [50, '50'],
              [100, '100'],
              [500, '500'],
              [1000, '1000']
              //          [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 80,
          width : 150,
          editable : false,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ],

      columns : [
        // id
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.4
        },

        // schedulerObjectName
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : false,
          groupable : false,
          flex : 1.1
        },

        // channel profile
        {
          itemId : 'channelProfileColumn',
          dataIndex : 'channelProfileName',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textChannelProfileColumn' ),
          sortable : false,
          groupable : false,
          flex : 1.3,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metadata, record )
          {
            var iconClass, tooltipText;

            var messageChannel = record.get( 'channelProfileChannel' );

            if ( value )
            {

              tooltipText = QDesk.util.EnumUtils.getMessageChannelName( messageChannel );

              if ( messageChannel == QDesk.enums.message.MessageChannel.EMAIL )
                iconClass = 'messageChannelEmail';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_AIM )
                iconClass = 'messageChannelImAim';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_GTALK )
                iconClass = 'messageChannelImGtalk';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_ICQ )
                iconClass = 'messageChannelImIcq';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_SKYPE )
                iconClass = 'messageChannelImSkype';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_XMPP )
                iconClass = 'messageChannelImXmpp';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_YAHOO )
                iconClass = 'messageChannelImYahoo';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.SNMPV1_TRAP )
                iconClass = 'messageChannelSnmpV1Trap';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.SNMPV2C_TRAP )
                iconClass = 'messageChannelSnmpV2cTrap';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.SNMPV3_TRAP )
                iconClass = 'messageChannelSnmpV3Trap';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.WEB_SERVICE )
                iconClass = 'messageChannelWebService';
            }
            else
            {
              value = QDesk.Msg.get( 'QDesk.notAvailable' );

              iconClass = 'messageChannelUnknown';
              var messageChannelProfileId = record.get( 'channelProfileId' );
              tooltipText =
                  QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.tooltipChannelProfileNotFound',
                      messageChannelProfileId );
            }

            metadata.tdAttr = 'data-qtip="' + tooltipText + '"';

            return '<div class="x-action-col-icon ' + iconClass +
                '" style="padding-top: 1px; background-position: 0 1px">' +
                '<span style="padding-left: 20px;">' + value + '</span></div>';
          }
        },

        // status
        {
          xtype : 'actioncolumn',
          itemId : 'statusColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textStatusColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textStatusColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metadata, record )
          {
            var status = record.get( 'status' );
            metadata.tdAttr = 'data-qtip="' + QDesk.util.EnumUtils.getMessageStatusName( status ) + '"';
            return value;
          },
          getClass : function ( value, metadata, record )
          {
            var status = record.get( 'status' );
            if ( status == QDesk.enums.message.MessageStatus.PENDING )
              return 'messageStatusPending';
            else if ( status == QDesk.enums.message.MessageStatus.PROCESSED )
              return 'messageStatusProcessed';
            else if ( status == QDesk.enums.message.MessageStatus.FAILED_TEMP )
              return 'messageStatusFailedTemp';
            else if ( status == QDesk.enums.message.MessageStatus.FAILED_PERM )
              return 'messageStatusFailedPerm';
          }
        },

        // correlationId
        {
          itemId : 'correlationIdColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textCorrelationIdColumn' ),
          dataIndex : 'correlationId',
          sortable : false,
          groupable : false,
          flex : 0.7
        },

        // creator
        {
          itemId : 'creatorColumn',
          dataIndex : 'creatorName',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textCreatorColumn' ),
          sortable : false,
          groupable : false,
          flex : 1,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metadata, record )
          {
            var iconClass;

            var creatorType = record.get( 'creatorType' );
            var tooltipText = QDesk.util.EnumUtils.getMessageCreatorTypeName( creatorType );

            if ( creatorType == QDesk.enums.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE )
            {
              iconClass = 'messageCreatorTypeQuartzExecNotifRule';
            }
            else
            {
              iconClass = 'messageCreatorTypeUnknown';
            }

            // creator name not available
            if ( !value )
            {
              var creatorId = record.get( 'creatorId' );
              value = QDesk.Msg.get( 'QDesk.notAvailable' );
            }

            metadata.tdAttr = 'data-qtip="' + tooltipText + '"';

            return '<div class="x-action-col-icon ' + iconClass +
                '" style="padding-top: 1px; background-position: 0 1px">' +
                '<span style="padding-left: 20px;">' + value + '</span></div>';
          }
        },

        // createdAt
        {
          xtype : 'datecolumn',
          itemId : 'createdAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textCreatedAtColumn' ),
          dataIndex : 'createdAt',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          sortable : true,
          groupable : false,
          flex : 1
        },

        // createdBy
        {
          itemId : 'createdByColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textCreatedByColumn' ),
          dataIndex : 'createdBy',
          sortable : false,
          groupable : false,
          flex : 1
        },

        // postponedTill
        {
          xtype : 'datecolumn',
          itemId : 'postponedTillColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textPostponedTillColumn' ),
          dataIndex : 'postponedTill',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          sortable : true,
          groupable : false,
          flex : 1
        },

        // processingStartedAt
        {
          xtype : 'datecolumn',
          itemId : 'processingStartedAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textProcessingStartedAtColumn' ),
          dataIndex : 'processingStartedAt',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          sortable : true,
          groupable : false,
          flex : 1
        },

        // processingFinishedAt
        {
          xtype : 'datecolumn',
          itemId : 'processingFinishedAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textProcessingFinishedAtColumn' ),
          dataIndex : 'processingFinishedAt',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          sortable : true,
          groupable : false,
          flex : 1
        },

        // processedBy
        {
          itemId : 'processedByColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textProcessedByColumn' ),
          dataIndex : 'processedBy',
          sortable : false,
          groupable : false,
          flex : 1
        },

        // errorCount
        {
          xtype : 'numbercolumn',
          itemId : 'errorCountColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textErrorCountColumn' ),
          dataIndex : 'errorCount',
          sortable : false,
          groupable : false,
          flex : 0.7,
          renderer : function ( value, metadata, record )
          {
            var maxErrorCount = record.get( 'maxErrorCount' );
            if ( !maxErrorCount )
              maxErrorCount = '?';

            return value + ' / ' + maxErrorCount;
          }
        },

        // lastErrorAt
        {
          xtype : 'datecolumn',
          itemId : 'lastErrorAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textLastErrorAtColumn' ),
          dataIndex : 'lastErrorAt',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          sortable : true,
          groupable : false,
          flex : 1
        },

        // lastErrorDesc
        {
          xtype : 'actioncolumn',
          itemId : 'lastErrorDescColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textLastErrorDescColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.textLastErrorDescColumn' ),
          sortable : false,
          groupable : false,
          width : 80,
          items : [
            {
              icon : 'images/icon/message-error-16x16.png',
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var lastErrorDesc = record.get( 'lastErrorDesc' );
                this.fireEvent( 'requestViewError', lastErrorDesc );
              },
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.message.MessageQueueGrid.tooltipLastErrorDescColumn' ),
              getClass : function ( value, metadata, record )
              {
                // hide the icon in case the log data is not available
                var lastErrorDesc = record.get( 'lastErrorDesc' );
                return lastErrorDesc ? 'clickable' : 'x-hidden';
              }
            }
          ]
        }
      ],

      store : store,

      qdesk : {
        gridId : 'messageQueueGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [config] );

    var maxFetchSize = QDesk.ApplicationConfig.get( me.qdesk.gridId + '.maxFetchSize' );
    me.down( '#maxFetchSizeSelect' ).setValue( maxFetchSize );
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var maxFetchSize = me.down( '#maxFetchSizeSelect' ).getValue();

    var extra = {};
    extra[me.qdesk.gridId + '.maxFetchSize'] = maxFetchSize;

    return extra;
  }
} );
