/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.scheduler.quartz.notif.exec.SendTestMessage',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.ExpressionLanguage',
    'QDesk.enums.message.MessageChannel',
    'QDesk.enums.scheduler.ExecNotificationRuleConditionType',
    'QDesk.enums.NotificationSeverity',
    'QDesk.enums.Weekday',

    'QDesk.store.enums.scheduler.ExecNotificationRuleConditionTypeStore',
    'QDesk.store.enums.ExecStatusStore',
    'QDesk.store.enums.ExpressionLanguageStore',
    'QDesk.store.enums.NotificationSeverityStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel',
    'QDesk.view.main.scheduler.common.JSScriptingObjectsWindow',
    'QDesk.view.main.scheduler.common.notif.exec.MessageMacrosWindow',

    'QDesk.view.widget.MessageChannelProfileCombo'
  ],

  alias : 'widget.EditExecNotificationRuleForm',

  fieldDefaults : {
    labelWidth : 80
  },

  qdesk : {
    /*
     * Mode of the form (see EditExecNotificationRuleFormMode).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.enums.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    var enabledOnWeekdaysPanel =
    {
      xtype : 'panel',
      border : false,
      height : 50,
      layout : {
        type : 'hbox',
        pack : 'start',
        align : 'stretch'
      },
      items : [
        {
          xtype : 'checkboxgroup',
          style : {
            paddingLeft : '10px',
            backgroundColor : '#E6F7E6'
          },
          flex : 2,

          vertical : true,
          columns : 3,
          items : [
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.MONDAY ),
              name : 'enabledOnMonday',
              itemId : 'enabledOnMonday',
              qdesk : {
                value : QDesk.enums.Weekday.MONDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.TUESDAY ),
              name : 'enabledOnTuesday',
              itemId : 'enabledOnTuesday',
              qdesk : {
                value : QDesk.enums.Weekday.TUESDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.WEDNESDAY ),
              name : 'enabledOnWednesday',
              itemId : 'enabledOnWednesday',
              qdesk : {
                value : QDesk.enums.Weekday.WEDNESDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.THURSDAY ),
              name : 'enabledOnThursday',
              itemId : 'enabledOnThursday',
              qdesk : {
                value : QDesk.enums.Weekday.THURSDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.FRIDAY ),
              name : 'enabledOnFriday',
              itemId : 'enabledOnFriday',
              qdesk : {
                value : QDesk.enums.Weekday.FRIDAY
              }
            }
          ]
        },
        {
          xtype : 'checkboxgroup',
          style : {
            paddingLeft : '10px',
            backgroundColor : '#F7E6E6'
          },
          flex : 1,
          vertical : true,
          columns : 1,
          items : [
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.SATURDAY ),
              name : 'enabledOnSaturday',
              itemId : 'enabledOnSaturday',
              qdesk : {
                value : QDesk.enums.Weekday.SATURDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.SUNDAY ),
              name : 'enabledOnSunday',
              itemId : 'enabledOnSunday',
              qdesk : {
                value : QDesk.enums.Weekday.SUNDAY
              }
            }
          ]
        }
      ]
    };


    var enabledFromToPanel =
    {
      xtype : 'panel',
      border : false,
      layout : 'column',
      style : {
        marginBottom : '5px'
      },
      items : [
        {
          xtype : 'timefield',
          itemId : 'enabledFromTime',
          name : 'enabledFromTime',
          labelWidth : 95,
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelEnabledFromTime' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipEnabledFromTime' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtEnabledFromTime' ),
          format : QDesk.util.constant.DateFormat.dayTimeFormat,
          increment : 30
        },
        {
          xtype : 'timefield',
          itemId : 'enabledToTime',
          name : 'enabledToTime',
          labelWidth : 82,
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelEnabledToTime' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipEnabledToTime' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtEnabledToTime' ),
          format : QDesk.util.constant.DateFormat.dayTimeFormat,
          increment : 30,
          margin : '0 0 0 15'
        }
      ]
    };

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          itemId : 'notificationRuleId',
          name : 'id'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'schedulerObjectName',
          name : 'schedulerObjectName'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'messageChannelProfileChannel',
          name : 'messageChannelProfileChannel'
        },

        /* Start of Quartz specific fields */
        {
          xtype : 'hiddenfield',
          itemId : 'jobGroupName',
          name : 'jobGroupName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'jobName',
          name : 'jobName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'triggerGroupName',
          name : 'triggerGroupName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'triggerName',
          name : 'triggerName'
        },
        /* End of Quartz specific fields */

        {
          xtype : 'hiddenfield',
          itemId : 'createdAt',
          name : 'createdAt'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'updatedAt',
          name : 'updatedAt'
        },

        {
          xtype : 'textfield',
          itemId : 'name',
          name : 'name',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelName' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipName' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtName' ),
          allowBlank : false
        },
        {
          xtype : 'textareafield',
          itemId : 'description',
          name : 'description',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelDescription' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipDescription' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtDescription' ),
          labelAlign : 'top',
          allowBlank : true
        },
        {
          xtype : 'iconcombo',
          itemId : 'severity',
          name : 'severity',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelSeverity' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipSeverity' ),
          allowBlank : false,
          editable : false,
          forceSelection : true,
          queryMode : 'local',
          store : 'enums.NotificationSeverityStore',
          valueField : 'value',
          displayField : 'valueLabel',
          iconClsField : 'value'
        },

        {
          xtype : 'checkboxfield',
          itemId : 'enabled',
          name : 'enabled',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelEnabled' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipEnabled' ),
          allowBlank : true
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetLabelEnabledOnWeekdays' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetTooltipEnabledOnWeekdays' ),
          itemId : 'enabledOnWeekdaysFieldSet',
          defaults : {
            labelWidth : 100,
            anchor : '100%'
          },
          padding : '0 10px 7px 10px',
          items : [
            enabledOnWeekdaysPanel
          ]
        },

        enabledFromToPanel,

        {
          xtype : 'tabpanel',
          bodyPadding : 10,
          items : [
            {
              xtype : 'panel',
              border : false,
              iconCls : 'execNotificationConditionTabIcon',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.tabLabelTriggerCondition' ),
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.tabTooltipTriggerCondition' ),
              layout : 'anchor',
              //height: 265,
              defaults : {
                labelWidth : 120,
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'combo',
                  itemId : 'conditionType',
                  name : 'conditionType',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelConditionType' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipConditionType' ),
                  allowBlank : false,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.scheduler.ExecNotificationRuleConditionTypeStore',
                  valueField : 'value',
                  displayField : 'valueLabel'
                },

                /* Condition exec status */
                {
                  xtype : 'combo',
                  itemId : 'conditionExecStatus',
                  name : 'conditionExecStatus',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelConditionExecStatus' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipConditionExecStatus' ),
                  allowBlank : true,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.ExecStatusStore',
                  valueField : 'value',
                  displayField : 'valueLabel',
                  qdesk : {
                    required : true
                  }
                },

                /* Condition max duration */
                {
                  xtype : 'numberfield',
                  itemId : 'conditionMaxDuration',
                  name : 'conditionMaxDuration',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMaxDuration' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMaxDuration' ),
                  emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtMaxDuration' ),
                  minValue : 1,
                  qdesk : {
                    required : true
                  }
                },

                /* Condition expression panel */
                {
                  xtype : 'iconcombo',
                  itemId : 'conditionExpressionLanguage',
                  name : 'conditionExpressionLanguage',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelConditionExpressionLanguage' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipConditionExpressionLanguage' ),
                  allowBlank : true,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.ExpressionLanguageStore',
                  valueField : 'value',
                  displayField : 'valueLabel',
                  iconClsField : 'value',
                  qdesk : {
                    required : true
                  }
                },

                {
                  xtype : 'panel',
                  itemId : 'conditionExpressionPanel',
                  border : false,
                  layout : 'column',
                  items : [
                    {
                      xtype : 'textareafield',
                      itemId : 'conditionExpression',
                      name : 'conditionExpression',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelConditionExpression' ),
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipConditionExpression' ),
                      emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtConditionExpression' ),
                      labelAlign : 'top',
                      columnWidth : 1,
                      height : 200,
                      qdesk : {
                        required : true
                      }
                    },
                    {
                      xtype : 'button',
                      itemId : 'conditionExpressionObjects',
                      style : {
                        marginLeft : '3px',
                        marginTop : '22px'
                      },
                      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonTooltipConditionExpressionObjects' ),
                      iconCls : 'expressionObjects'
                    }
                  ]
                }
              ]
            },
            {
              xtype : 'panel',
              itemId : 'messageTab',
              border : false,
              iconCls : 'execNotificationMessageTabIcon',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.tabLabelMessage' ),
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.tabTooltipMessage' ),
              layout : 'anchor',
              defaults : {
                labelWidth : 105,
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'panel',
                  itemId : 'messageChannelProfilePanel',
                  border : false,
                  layout : 'column',
                  style : {
                    marginBottom : '5px'
                  },
                  defaults : {
                    labelWidth : 105
                  },
                  items : [
                    {
                      xtype : 'MessageChannelProfileCombo',
                      itemId : 'messageChannelProfileCombo',
                      name : 'messageChannelProfileId',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMessageChannelProfile' ),
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMessageChannelProfile' ),
                      allowBlank : false,
                      columnWidth : 1
                    },
                    {
                      xtype : 'button',
                      itemId : 'messageChannelProfilesButton',
                      style : {
                        marginLeft : '3px'
                      },
                      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonTooltipMessageChannelProfiles' ),
                      iconCls : 'messageChannelProfiles'
                    }
                  ]
                },

                {
                  xtype : 'textfield',
                  itemId : 'messageToAddr',
                  name : 'messageToAddr',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMessageToAddr' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMessageToAddr' ),
                  emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtMessageToAddr' ),
                  allowBlank : false,
                  hidden : true  /* recipients field is normally hidden and it is made visible only when channel != WEB_SERVICE is selected */
                },

                {
                  xtype : 'panel',
                  itemId : 'messageSubjectTemplatePanel',
                  border : false,
                  hidden : true, /* subject panel is normally hidden and it is made visible only when EMAIL/WEB_SERVICE channel is selected */
                  layout : 'column',
                  defaults : {
                    labelWidth : 105
                  },
                  items : [
                    {
                      xtype : 'textfield',
                      itemId : 'messageSubjectTemplate',
                      name : 'messageSubjectTemplate',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMessageSubjectTemplate' ),
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMessageSubjectTemplate' ),
                      emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtMessageSubjectTemplate' ),
                      allowBlank : false,
                      columnWidth : 1,
                      qdesk : {
                        required : true
                      }
                    },
                    {
                      xtype : 'button',
                      itemId : 'messageSubjectTemplateMacros',
                      style : {
                        marginLeft : '3px'
                      },
                      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonTooltipMessageSubjectTemplateMacros' ),
                      iconCls : 'messageSubjectTemplateMacros'
                    }
                  ]
                },

                {
                  xtype : 'panel',
                  itemId : 'messageBodyTemplatePanel',
                  border : false,
                  layout : 'column',
                  style : {
                    marginBottom : '5px'
                  },
                  items : [
                    {
                      xtype : 'textareafield',
                      itemId : 'messageBodyTemplate',
                      name : 'messageBodyTemplate',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMessageBodyTemplate' ),
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMessageBodyTemplate' ),
                      emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtMessageBodyTemplate' ),
                      labelAlign : 'top',
                      allowBlank : false,
                      columnWidth : 1,
                      height : 200
                    },
                    {
                      xtype : 'button',
                      itemId : 'messageBodyTemplateMacros',
                      style : {
                        marginLeft : '3px',
                        marginTop : '22px'
                      },
                      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonTooltipMessageBodyTemplateMacros' ),
                      iconCls : 'messageBodyTemplateMacros'
                    }
                  ]
                },


                {
                  xtype : 'fieldset',
                  title : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetLabelMessageAttachments' ),
                  tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetTooltipMessageAttachments' ),
                  itemId : 'messageAttachmentsFieldSet',
                  defaults : {
                    labelWidth : 100,
                    anchor : '100%'
                  },
                  items : [
                    {
                      xtype : 'checkboxgroup',
                      //vertical : true,
                      columns : 4,
                      items : [
                        {
                          name : 'messageAttachLog',
                          itemId : 'messageAttachLog',
                          boxLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.boxLabelMessageAttachLog' )
                        },
                        {
                          name : 'messageAttachError',
                          itemId : 'messageAttachError',
                          boxLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.boxLabelMessageAttachError' )
                        },
                        {
                          name : 'messageAttachResult',
                          itemId : 'messageAttachResult',
                          boxLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.boxLabelMessageAttachResult' )
                        },
                        {
                          name : 'messageAttachUserData',
                          itemId : 'messageAttachUserData',
                          boxLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.boxLabelMessageAttachUserData' )
                        }
                      ]
                    }
                  ]
                },

                {
                  xtype : 'panel',
                  border : false,
                  height : 25,
                  layout : {
                    type : 'hbox',
                    pack : 'start',
                    align : 'stretch'
                  },
                  defaults : {
                    labelWidth : 80,
                    anchor : '100%'
                  },
                  items : [
                    {
                      xtype : 'button',
                      itemId : 'sendTestMessage',
                      text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonLabelSendTestMessage' ),
                      anchor : '20%'
                    },
                    {
                      xtype : 'displayfield',
                      value : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleForm.noteSendTestMessage' ),
                      style : {
                        marginLeft : '5px'
                      }
                    }
                  ]
                }
              ]
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    /* Defaults */
    me.down( '#severity' ).setValue( QDesk.enums.NotificationSeverity.INFO );
    me.down( '#enabled' ).setValue( true );
    me.down( '#enabledOnMonday' ).setValue( true );
    me.down( '#enabledOnTuesday' ).setValue( true );
    me.down( '#enabledOnWednesday' ).setValue( true );
    me.down( '#enabledOnThursday' ).setValue( true );
    me.down( '#enabledOnFriday' ).setValue( true );
    me.down( '#enabledOnSaturday' ).setValue( true );
    me.down( '#enabledOnSunday' ).setValue( true );

    me.down( '#conditionType' ).setValue( QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS );
    me.down( '#conditionExecStatus' ).setValue( QDesk.enums.ExecStatus.SUCCESS );
    me.down( '#conditionExpressionLanguage' ).setValue( QDesk.enums.ExpressionLanguage.JAVASCRIPT );
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    // notification condition type change
    me.down( '#conditionType' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#conditionExecStatus' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS );
        field.allowBlank = newValue != QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS;
        field.validate();
      } );

      Ext.each( me.down( '#conditionMaxDuration' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.ExecNotificationRuleConditionType.MAX_DURATION );
        field.allowBlank = newValue != QDesk.enums.scheduler.ExecNotificationRuleConditionType.MAX_DURATION;
        field.validate();
      } );

      Ext.each( me.down( '#conditionExpression' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXPRESSION );
        field.allowBlank = newValue != QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXPRESSION;
        field.validate();
      } );

      Ext.each( me.down( '#conditionExpressionLanguage' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXPRESSION );
        field.allowBlank = newValue != QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXPRESSION;
        field.validate();
      } );

      Ext.each( me.down( '#conditionExpressionObjects' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXPRESSION );
      } );
    } );

    // message channel profile change
    me.down( '#messageChannelProfileCombo' ).on( 'change', function ( combo, newValue, oldValue )
    {
      var messageChannel = combo.getMessageChannel();

      me.down( '#messageChannelProfileChannel' ).setValue( messageChannel );

      if ( messageChannel == QDesk.enums.message.MessageChannel.EMAIL )
      {
        // required recipient(s)
        Ext.each( me.down( '#messageToAddr' ), function ( field )
        {
          field.setVisible( true );
          field.allowBlank = false;
          field.validate();
        } );

        // required message subject
        Ext.each( me.down( '#messageSubjectTemplatePanel' ), function ( panel )
        {
          panel.setVisible( true );

          var field = panel.down( '#messageSubjectTemplate' );
          field.allowBlank = false;
          field.validate();
        } );

        // required message body
        Ext.each( me.down( '#messageBodyTemplatePanel' ), function ( panel )
        {
          panel.setVisible( true );

          var field = panel.down( '#messageBodyTemplate' );
          field.allowBlank = false;
          field.validate();
        } );

        // visible message attachments
        me.down( '#messageAttachmentsFieldSet' ).setVisible( true );
      }
      else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_AIM ||
          messageChannel == QDesk.enums.message.MessageChannel.IM_GTALK ||
          messageChannel == QDesk.enums.message.MessageChannel.IM_ICQ ||
          messageChannel == QDesk.enums.message.MessageChannel.IM_SKYPE ||
          messageChannel == QDesk.enums.message.MessageChannel.IM_XMPP ||
          messageChannel == QDesk.enums.message.MessageChannel.IM_YAHOO )
      {
        // required recipient(s)
        Ext.each( me.down( '#messageToAddr' ), function ( field )
        {
          field.setVisible( true );
          field.allowBlank = false;
          field.validate();
        } );

        // hidden message subject
        Ext.each( me.down( '#messageSubjectTemplatePanel' ), function ( panel )
        {
          panel.setVisible( false );

          var field = panel.down( '#messageSubjectTemplate' );
          field.allowBlank = true;
          field.validate();
        } );

        // required message body
        Ext.each( me.down( '#messageBodyTemplatePanel' ), function ( panel )
        {
          panel.setVisible( true );

          var field = panel.down( '#messageBodyTemplate' );
          field.allowBlank = false;
          field.validate();
        } );

        // hidden message attachments
        me.down( '#messageAttachmentsFieldSet' ).setVisible( false );
      }
      else if ( messageChannel == QDesk.enums.message.MessageChannel.SNMPV1_TRAP
          || messageChannel == QDesk.enums.message.MessageChannel.SNMPV2C_TRAP
          || messageChannel == QDesk.enums.message.MessageChannel.SNMPV3_TRAP )
      {
        // hidden recipient(s)
        Ext.each( me.down( '#messageToAddr' ), function ( field )
        {
          field.setVisible( false );
          field.allowBlank = true;
          field.validate();
        } );

        // required message subject
        Ext.each( me.down( '#messageSubjectTemplatePanel' ), function ( panel )
        {
          panel.setVisible( true );

          var field = panel.down( '#messageSubjectTemplate' );
          field.allowBlank = false;
          field.validate();
        } );

        // required message body
        Ext.each( me.down( '#messageBodyTemplatePanel' ), function ( panel )
        {
          panel.setVisible( true );

          var field = panel.down( '#messageBodyTemplate' );
          field.allowBlank = false;
          field.validate();
        } );

        // hidden message attachments
        me.down( '#messageAttachmentsFieldSet' ).setVisible( false );
      }
      else if ( messageChannel == QDesk.enums.message.MessageChannel.WEB_SERVICE )
      {
        // hidden recipient(s)
        Ext.each( me.down( '#messageToAddr' ), function ( field )
        {
          field.setVisible( false );
          field.allowBlank = true;
          field.validate();
        } );

        // required message subject
        Ext.each( me.down( '#messageSubjectTemplatePanel' ), function ( panel )
        {
          panel.setVisible( true );

          var field = panel.down( '#messageSubjectTemplate' );
          field.allowBlank = false;
          //field.validate();
          //field.clearInvalid();
        } );

        // required message body
        Ext.each( me.down( '#messageBodyTemplatePanel' ), function ( panel )
        {
          panel.setVisible( true );

          var field = panel.down( '#messageBodyTemplate' );
          field.allowBlank = false;
          field.validate();
        } );

        // visible message attachments
        me.down( '#messageAttachmentsFieldSet' ).setVisible( true );
      }

      // revalidates the form
      me.getForm().isValid();
    }, me );


    // expression scripting objects
    me.down( '#conditionExpressionObjects' ).on( 'click', function ( button )
    {
      // for JavaScript language, open the JSScriptingObjectsWindow window
      var selectedExprLang = me.down( '#conditionExpressionLanguage' ).getValue();

      var window;
      switch ( selectedExprLang )
      {
        case QDesk.enums.ExpressionLanguage.JAVASCRIPT:
          window = Ext.widget( 'JSScriptingObjectsWindow', {
            qdesk : {
              schedulerType : me.qdesk.schedulerType
            }
          } );
          break;

        default:
          Ext.Error.raise( 'Expression properties window not available for language: ' + selectedExprLang );
      }

      window.down( '#treePanel' ).on( 'itemdblclick', function ( treePanel, record, item, index, event )
      {
        var propertyName = record.get( 'propertyName' );
        var propertyPath = record.getPath( 'propertyName', '.' ).substring( 2 );
        window.close();

        var conditionExpression = me.down( '#conditionExpression' );
        QDesk.util.FormUtils.insertTextAtCursor( conditionExpression, propertyPath );
        event.stopPropagation();
      }, me );

      window.show();
    }, me );


    // message subject macros
    me.down( '#messageSubjectTemplateMacros' ).on( 'click', function ( button )
    {
      var window = Ext.widget( 'ExecNotificationRuleMessageMacrosWindow', {
        qdesk : {
          schedulerType : me.qdesk.schedulerType
        }
      } );

      window.down( '#treePanel' ).on( 'itemdblclick', function ( treePanel, record, item, index, event )
      {
        var leaf = record.get( 'leaf' );
        if ( leaf )
        {
          var macroName = record.get( 'macroName' );
          window.close();

          var messageSubjectField = me.down( '#messageSubjectTemplate' );
          QDesk.util.FormUtils.insertTextAtCursor( messageSubjectField, '${' + macroName + '}' );
          event.stopPropagation();
        }
      }, me );

      window.show();
    }, me );


    // message body macros
    me.down( '#messageBodyTemplateMacros' ).on( 'click', function ( button )
    {
      var window = Ext.widget( 'ExecNotificationRuleMessageMacrosWindow', {
        qdesk : {
          schedulerType : me.qdesk.schedulerType
        }
      } );

      window.down( '#treePanel' ).on( 'itemdblclick', function ( treePanel, record, item, index, event )
      {
        var leaf = record.get( 'leaf' );
        if ( leaf )
        {
          var macroName = record.get( 'macroName' );
          window.close();

          var messageBodyField = me.down( '#messageBodyTemplate' );
          QDesk.util.FormUtils.insertTextAtCursor( messageBodyField, '${' + macroName + '}' );
          event.stopPropagation();
        }
      }, me );


      window.show();
    }, me );

    // manage message channel profiles
    me.down( '#messageChannelProfilesButton' ).on( 'click', function ( button )
    {
      Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles', {} ).execute();

      // upon message channel profiles window close reload the message channel profile
      // combo tree (profiles might have been added/removed)
      var window = Ext.ComponentQuery.query( 'MessageChannelProfilesWindow' )[ 0 ];
      window.on( 'close', function ()
      {
        var treeStore = me.down( '#messageChannelProfileCombo' ).getStore();
        var rootNode = treeStore.getRootNode();
        rootNode.removeAll( false );
        treeStore.load( { node : rootNode } );
      }, me );
    }, me );

    /*
     * Send Test Message button functionality - enabled only when all allowBlank=false fields
     * on the messageTab have a value.
     */
    var messageTab = me.down( '#messageTab' );
    Ext.each( Ext.ComponentQuery.query( 'field', messageTab ), function ( field )
    {
      field.on( 'change', function ()
      {
        me._maybeEnableSendTestMessageButton();
      }, me );
    }, me );

    me.down( '#sendTestMessage' ).on( 'click', function ( button )
    {
      me._sendTestMessage();
    }, me );

    me._maybeEnableSendTestMessageButton();
  },


  _getActionButtonText : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      case QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.buttonLabelClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  /**
   * Checks the value of all required (allowBlank=false) fields on the message tab
   * and if all the required fields have a value, then the Send Test Message button
   * is enabled.
   */
  _maybeEnableSendTestMessageButton : function ()
  {
    var me = this;

    var messageTab = me.down( '#messageTab' );
    var disableButton = false;

    Ext.each( Ext.ComponentQuery.query( 'field', messageTab ), function ( field )
    {
      if ( !field.allowBlank && Ext.isEmpty( field.getValue() ) )
      {
        disableButton = true;
      }
    }, me );

    messageTab.down( '#sendTestMessage' ).setDisabled( disableButton );
  },


  /**
   * Sends a test message.
   */
  _sendTestMessage : function ()
  {
    var me = this;

    var messageChannelProfileCombo = me.down( '#messageChannelProfileCombo' );
    var channelProfileId = messageChannelProfileCombo.getValue();
    var channel = messageChannelProfileCombo.getMessageChannel();

    var recipients = me.down( '#messageToAddr' ).getValue();
    if ( !me._messageChannelSupportsRecipients( channel ) )
      recipients = null;

    var subjectTemplate = me.down( '#messageSubjectTemplate' ).getValue();
    if ( !me._messageChannelSupportsSubject( channel ) )
      subjectTemplate = null;

    var bodyTemplate = me.down( '#messageBodyTemplate' ).getValue();

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.SendTestMessage', {
      channelProfileId : channelProfileId,
      channel : channel,
      recipients : recipients,
      subjectTemplate : subjectTemplate,
      bodyTemplate : bodyTemplate
    } ).execute();
  },


  _messageChannelSupportsRecipients : function ( messageChannel )
  {
    return messageChannel == QDesk.enums.message.MessageChannel.EMAIL
        || messageChannel == QDesk.enums.message.MessageChannel.IM_AIM
        || messageChannel == QDesk.enums.message.MessageChannel.IM_GTALK
        || messageChannel == QDesk.enums.message.MessageChannel.IM_ICQ
        || messageChannel == QDesk.enums.message.MessageChannel.IM_SKYPE
        || messageChannel == QDesk.enums.message.MessageChannel.IM_XMPP
        || messageChannel == QDesk.enums.message.MessageChannel.IM_YAHOO;
  },


  _messageChannelSupportsSubject : function ( messageChannel )
  {
    return messageChannel == QDesk.enums.message.MessageChannel.EMAIL
        || messageChannel == QDesk.enums.message.MessageChannel.SNMPV1_TRAP
        || messageChannel == QDesk.enums.message.MessageChannel.SNMPV2C_TRAP
        || messageChannel == QDesk.enums.message.MessageChannel.SNMPV3_TRAP
        || messageChannel == QDesk.enums.message.MessageChannel.WEB_SERVICE;
  }
} );
