/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.SelectJobGrid', {
  extend : 'Ext.grid.Panel',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.store.scheduler.quartz.QuartzJobDetailStore',
    'QDesk.store.scheduler.quartz.QuartzJobGroupStore',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzSelectJobGrid',

  cls : 'quartzSelectJobGrid',

  viewConfig : {
    stripeRows : true
  },

  scrollable : true,

  //border : false,

  qdesk : {
    /*
     * Scheduler connection ID.
     */
    connectionId : undefined
  },


  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
        '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
        '<tpl if="description">{[Ext.htmlEncode(values.description)]}</tpl>' +
        '<tpl if="!description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
        '</p>'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var jobDetailStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobDetailStore' );

    jobDetailStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    Ext.apply( config, {
      store : jobDetailStore,

      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'xcombo',
          itemId : 'filterByJobGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.fieldLabelFilterByJobGroupCombo' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.emptyTextFilterByJobGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true
        }
      ],

      columns : [
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.textNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'templatecolumn',
          itemId : 'groupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.textGroupNameColumn' ),
          dataIndex : 'group.name',
          tpl : '{group.name}',
          sortable : true,
          groupable : true,
          flex : 0.5
        },
        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.textJobClassColumn' ),
          dataIndex : 'jobClass',
          sortable : true,
          groupable : true,
          flex : 1
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobGroupStore' );

    jobGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByJobGroupCombo' ).store = jobGroupStore;
  },


  /**
   * Changes the scheduler connection of the grid's store and reloads it.
   * @param connectionId a scheduler connection ID.
   */
  changeConnection : function ( connectionId )
  {
    var me = this;
    var store = me.getStore();

    //
    // We cannot change store parameters while the store is loading.
    // If the store is loading, we reinvoke this method later.
    //
    if ( store.isLoading() )
    {
      Ext.defer( me.changeConnection, 200, me );  // try again in 200ms
    }
    else
    {
      store.removeAll();

      if ( connectionId == null )
      {
        me.setDisabled( true );
      }
      else
      {
        me.setDisabled( false );

        store.getProxy().extraParams = {
          connectionId : connectionId
        };

        var filterByJobGroupComboStore = me.down( '#filterByJobGroupCombo' ).getStore();
        filterByJobGroupComboStore.getProxy().extraParams = {
          connectionId : connectionId
        };

        me.refresh();
      }
    }
  },


  /**
   * Refreshes this grid by first reloading the job group combo and then reloading this grid.
   */
  refresh : function ()
  {
    var me = this;

    var filterByJobGroupCombo = me.down( '#filterByJobGroupCombo' );

    // reloads and resets the job group filter combo and then reloads the jobs grid
    filterByJobGroupCombo.getStore().load( {
      scope : me,
      callback : function ( records, operation, success )
      {
        if ( success )
        {
          // reload the jobs grid
          if ( filterByJobGroupCombo.getValue() == null )
          {
            // no need to reset the combo, just reload the grid
            me.getStore().loadPage( 1 );
          }
          else
          {
            // triggers the change event which reloads the jobs grid
            filterByJobGroupCombo.reset( null );
          }
        }
      }
    } );
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    // refresh button
    me.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me.refresh();
        } );

    var store = me.getStore();

    var filterByJobGroupCombo = me.down( '#filterByJobGroupCombo' );
    filterByJobGroupCombo.on( 'change', function ( combo, newValue )
    {
      store.clearFilter( true );

      if ( newValue != null )
      {
        store.addFilter( [
          Ext.create( 'Ext.util.Filter', {
            property : 'group.name',
            value : newValue
          } )
        ], false );
      }

      // apply, or clear the filter and load the first page
      store.loadPage( 1 );
    } );

    store.on( 'beforeload',
        function ( store, operation )
        {
          me.down( '#refreshButton' ).disable();
          filterByJobGroupCombo.disable();

          /*
           * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
           * Therefore we need to force-fire this event by clearing the selection.
           */
          me.getSelectionModel().deselectAll();
        } );

    store.on( 'load',
        function ( store, records, successful, operation )
        {
          me.down( '#refreshButton' ).enable();
          if ( successful )
          {
            filterByJobGroupCombo.enable();
          }
          else
          {
            store.removeAll();

            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.SelectJob.errorLoadingJobsMsg' ) );
          }
        } );
  }
} );
