/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.ExpressionLanguage',
    'QDesk.enums.scheduler.SchedulerType',
    'QDesk.enums.scheduler.JobChainConditionType',
    'QDesk.enums.Weekday',

    'QDesk.store.enums.scheduler.JobChainConditionTypeStore',
    'QDesk.store.enums.ExecStatusStore',
    'QDesk.store.enums.ExpressionLanguageStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel',
    'QDesk.view.main.scheduler.common.JSScriptingObjectsWindow',

    'QDesk.view.main.scheduler.quartz.chain.JobChainTargetsGrid',

    'QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode',
    'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetWindow'
  ],

  alias : 'widget.QuartzEditJobChainForm',

  fieldDefaults : {
    labelWidth : 80
  },

  qdesk : {
    /*
     * Required ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Mode of the form (see EditJobChainFormMode).
     */
    mode : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    var enabledOnWeekdaysPanel =
    {
      xtype : 'panel',
      border : false,
      height : 50,
      layout : {
        type : 'hbox',
        pack : 'start',
        align : 'stretch'
      },
      items : [
        {
          xtype : 'checkboxgroup',
          style : {
            paddingLeft : '10px',
            backgroundColor : '#E6F7E6'
          },
          flex : 2,

          vertical : true,
          columns : 3,
          items : [
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.MONDAY ),
              name : 'enabledOnMonday',
              itemId : 'enabledOnMonday',
              qdesk : {
                value : QDesk.enums.Weekday.MONDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.TUESDAY ),
              name : 'enabledOnTuesday',
              itemId : 'enabledOnTuesday',
              qdesk : {
                value : QDesk.enums.Weekday.TUESDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.WEDNESDAY ),
              name : 'enabledOnWednesday',
              itemId : 'enabledOnWednesday',
              qdesk : {
                value : QDesk.enums.Weekday.WEDNESDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.THURSDAY ),
              name : 'enabledOnThursday',
              itemId : 'enabledOnThursday',
              qdesk : {
                value : QDesk.enums.Weekday.THURSDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.FRIDAY ),
              name : 'enabledOnFriday',
              itemId : 'enabledOnFriday',
              qdesk : {
                value : QDesk.enums.Weekday.FRIDAY
              }
            }
          ]
        },
        {
          xtype : 'checkboxgroup',
          style : {
            paddingLeft : '10px',
            backgroundColor : '#F7E6E6'
          },
          flex : 1,
          vertical : true,
          columns : 1,
          items : [
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.SATURDAY ),
              name : 'enabledOnSaturday',
              itemId : 'enabledOnSaturday',
              qdesk : {
                value : QDesk.enums.Weekday.SATURDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.SUNDAY ),
              name : 'enabledOnSunday',
              itemId : 'enabledOnSunday',
              qdesk : {
                value : QDesk.enums.Weekday.SUNDAY
              }
            }
          ]
        }
      ]
    };


    var enabledFromToPanel =
    {
      xtype : 'panel',
      border : false,
      layout : 'column',
      style : {
        marginBottom : '5px'
      },
      items : [
        {
          xtype : 'timefield',
          itemId : 'enabledFromTime',
          name : 'enabledFromTime',
          labelWidth : 95,
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelEnabledFromTime' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipEnabledFromTime' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtEnabledFromTime' ),
          format : QDesk.util.constant.DateFormat.dayTimeFormat,
          increment : 30
        },
        {
          xtype : 'timefield',
          itemId : 'enabledToTime',
          name : 'enabledToTime',
          labelWidth : 82,
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelEnabledToTime' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipEnabledToTime' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtEnabledToTime' ),
          format : QDesk.util.constant.DateFormat.dayTimeFormat,
          increment : 30,
          margin : '0 0 0 15'
        }
      ]
    };

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          itemId : 'jobChainId',
          name : 'id'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'schedulerObjectName',
          name : 'schedulerObjectName'
        },

        /* Start of Quartz specific fields */
        {
          xtype : 'hiddenfield',
          itemId : 'jobGroupName',
          name : 'jobGroupName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'jobName',
          name : 'jobName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'triggerGroupName',
          name : 'triggerGroupName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'triggerName',
          name : 'triggerName'
        },
        /* End of Quartz specific fields */

        {
          xtype : 'hiddenfield',
          itemId : 'createdAt',
          name : 'createdAt'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'updatedAt',
          name : 'updatedAt'
        },

        {
          xtype : 'textfield',
          itemId : 'name',
          name : 'name',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelName' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipName' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtName' ),
          allowBlank : false
        },
        {
          xtype : 'textareafield',
          itemId : 'description',
          name : 'description',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelDescription' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipDescription' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtDescription' ),
          labelAlign : 'top',
          allowBlank : true
        },

        {
          xtype : 'checkboxfield',
          itemId : 'enabled',
          name : 'enabled',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelEnabled' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipEnabled' ),
          allowBlank : true
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldsetLabelEnabledOnWeekdays' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldsetTooltipEnabledOnWeekdays' ),
          itemId : 'enabledOnWeekdaysFieldSet',
          defaults : {
            labelWidth : 100,
            anchor : '100%'
          },
          padding : '0 10px 7px 10px',
          items : [
            enabledOnWeekdaysPanel
          ]
        },

        enabledFromToPanel,

        {
          xtype : 'tabpanel',
          items : [
            {
              xtype : 'panel',
              padding : '10 10 5 10',
              border : false,
              iconCls : 'quartzJobChainConditionTabIcon',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.tabLabelCondition' ),
              tooltip: QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.tabTooltipCondition' ),
              layout : 'anchor',
              defaults : {
                labelWidth : 120,
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'combo',
                  itemId : 'conditionType',
                  name : 'conditionType',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelConditionType' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipConditionType' ),
                  allowBlank : false,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.scheduler.JobChainConditionTypeStore',
                  valueField : 'value',
                  displayField : 'valueLabel'
                },

                /* Condition exec status */
                {
                  xtype : 'combo',
                  itemId : 'conditionExecStatus',
                  name : 'conditionExecStatus',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelConditionExecStatus' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipConditionExecStatus' ),
                  allowBlank : true,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.ExecStatusStore',
                  valueField : 'value',
                  displayField : 'valueLabel',
                  qdesk : {
                    required : true
                  }
                },

                /* Condition max duration */
                {
                  xtype : 'numberfield',
                  itemId : 'conditionMaxDuration',
                  name : 'conditionMaxDuration',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelMaxDuration' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipMaxDuration' ),
                  emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtMaxDuration' ),
                  minValue : 1,
                  qdesk : {
                    required : true
                  }
                },

                /* Condition expression panel */
                {
                  xtype : 'iconcombo',
                  itemId : 'conditionExpressionLanguage',
                  name : 'conditionExpressionLanguage',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelConditionExpressionLanguage' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipConditionExpressionLanguage' ),
                  allowBlank : true,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.ExpressionLanguageStore',
                  valueField : 'value',
                  displayField : 'valueLabel',
                  iconClsField : 'value',
                  qdesk : {
                    required : true
                  }
                },

                {
                  xtype : 'panel',
                  itemId : 'conditionExpressionPanel',
                  border : false,
                  layout : 'column',
                  items : [
                    {
                      xtype : 'textareafield',
                      itemId : 'conditionExpression',
                      name : 'conditionExpression',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldLabelConditionExpression' ),
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldTooltipConditionExpression' ),
                      emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtConditionExpression' ),
                      labelAlign : 'top',
                      columnWidth : 1,
                      height : 200,
                      qdesk : {
                        required : true
                      }
                    },
                    {
                      xtype : 'button',
                      itemId : 'conditionExpressionObjects',
                      style : {
                        marginLeft : '3px',
                        marginTop : '22px'
                      },
                      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.buttonTooltipConditionExpressionObjects' ),
                      iconCls : 'expressionObjects'
                    }
                  ]
                }
              ]
            },

            {
              xtype : 'panel',
              itemId : 'targetsTab',
              border : false,
              iconCls : 'quartzJobChainTargetTabIcon',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.tabLabelTargets' ),
              tooltip: QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm.tabTooltipTargets' ),
              layout : 'anchor',
              defaults : {
                labelWidth : 105,
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'QuartzJobChainTargetsGrid',
                  border : false,
                  itemId : 'jobChainTargetsGrid',
                  height : 150,
                  qdesk : {
                    connectionId : config.qdesk.connectionId
                  }
                }
              ]
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    /* Defaults */
    me.down( '#enabled' ).setValue( true );
    me.down( '#enabledOnMonday' ).setValue( true );
    me.down( '#enabledOnTuesday' ).setValue( true );
    me.down( '#enabledOnWednesday' ).setValue( true );
    me.down( '#enabledOnThursday' ).setValue( true );
    me.down( '#enabledOnFriday' ).setValue( true );
    me.down( '#enabledOnSaturday' ).setValue( true );
    me.down( '#enabledOnSunday' ).setValue( true );

    me.down( '#conditionType' ).setValue( QDesk.enums.scheduler.JobChainConditionType.EXEC_STATUS );
    me.down( '#conditionExecStatus' ).setValue( QDesk.enums.ExecStatus.SUCCESS );
    me.down( '#conditionExpressionLanguage' ).setValue( QDesk.enums.ExpressionLanguage.JAVASCRIPT );
  },


  /**
   * Invoked from the load-success handler to set special form field values (e.g. the enabledOnWeekdays
   * checkboxes etc.).
   * @param jsonData JSON data.
   */
  loadSpecialFields : function ( jsonData )
  {
    var me = this;

    // set enabledOnWeekdays checkboxes
    Ext.each( Ext.ComponentQuery.query( '#enabledOnWeekdaysFieldSet checkbox', me ),
        function ( checkbox )
        {
          checkbox.setValue( Ext.Array.contains( jsonData.enabledOnWeekday, checkbox.qdesk.value ) );
        } );

    // set enabledFromTime
    if ( jsonData.enabledFromTime )
    {
      var enabledFromDate = QDesk.util.FormatUtils.parseDayTime( jsonData.enabledFromTime );
      me.down( '#enabledFromTime' ).setValue( enabledFromDate );
    }

    // set enabledToTime fields
    if ( jsonData.enabledToTime )
    {
      var enabledToDate = QDesk.util.FormatUtils.parseDayTime( jsonData.enabledToTime );
      me.down( '#enabledToTime' ).setValue( enabledToDate );
    }

    if ( !Ext.isEmpty( jsonData.target ) )
    {
      var jobChainTargetsStore = me.down( '#jobChainTargetsGrid' ).getStore();
      jobChainTargetsStore.loadData( jsonData.target );
    }
  },


  /**
   * Invoked to obtain the array of QDesk.model.scheduler.quartz.QuartzJobChainTarget instances representing
   * current job chain targets.
   */
  getJobChainTargetsData : function ()
  {
    var me = this;

    var result = [];

    Ext.each( me.down( '#jobChainTargetsGrid' ).getStore().getRange(), function ( jobChainTarget )
    {
      result.push( jobChainTarget.getData() );
    } );

    return result;
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    // job chain condition type change
    me.down( '#conditionType' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#conditionExecStatus' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainConditionType.EXEC_STATUS );
        field.allowBlank = newValue != QDesk.enums.scheduler.JobChainConditionType.EXEC_STATUS;
        field.validate();
      } );

      Ext.each( me.down( '#conditionMaxDuration' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainConditionType.MAX_DURATION );
        field.allowBlank = newValue != QDesk.enums.scheduler.JobChainConditionType.MAX_DURATION;
        field.validate();
      } );

      Ext.each( me.down( '#conditionExpression' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainConditionType.EXPRESSION );
        field.allowBlank = newValue != QDesk.enums.scheduler.JobChainConditionType.EXPRESSION;
        field.validate();
      } );

      Ext.each( me.down( '#conditionExpressionLanguage' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainConditionType.EXPRESSION );
        field.allowBlank = newValue != QDesk.enums.scheduler.JobChainConditionType.EXPRESSION;
        field.validate();
      } );

      Ext.each( me.down( '#conditionExpressionObjects' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainConditionType.EXPRESSION );
      } );
    } );

    // expression scripting objects
    me.down( '#conditionExpressionObjects' ).on( 'click', function ( button )
    {
      // for JavaScript language, open the JSScriptingObjectsWindow window
      var selectedExprLang = me.down( '#conditionExpressionLanguage' ).getValue();

      var window;
      switch ( selectedExprLang )
      {
        case QDesk.enums.ExpressionLanguage.JAVASCRIPT:
          window = Ext.widget( 'JSScriptingObjectsWindow', {
            qdesk : {
              schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
            }
          } );
          break;

        default:
          Ext.Error.raise( 'Expression properties window not available for language: ' + selectedExprLang );
      }

      window.down( '#treePanel' ).on( 'itemdblclick', function ( treePanel, record, item, index, event )
      {
        var propertyName = record.get( 'propertyName' );
        var propertyPath = record.getPath( 'propertyName', '.' ).substring( 2 );
        window.close();

        var conditionExpression = me.down( '#conditionExpression' );
        QDesk.util.FormUtils.insertTextAtCursor( conditionExpression, propertyPath );
        event.stopPropagation();
      }, me );

      window.show();
    }, me );
  },


  _getActionButtonText : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.main.scheduler.common.chain.EditJobChainFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.main.scheduler.common.chain.EditJobChainFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      case QDesk.view.main.scheduler.common.chain.EditJobChainFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.buttonLabelClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  }
} );
