/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ExpressionLanguage',
    'QDesk.enums.scheduler.SchedulerType',
    'QDesk.enums.scheduler.JobChainTargetExecType',
    'QDesk.enums.scheduler.JobChainTargetType',

    'QDesk.store.enums.scheduler.JobChainTargetExecTypeStore',

    'QDesk.view.base.BaseFormPanel',
    'QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode',

    'QDesk.view.main.scheduler.quartz.SelectJobGrid',

    'QDesk.view.widget.ConnectionCombo',

    'QDesk.util.FormUtils'
  ],

  alias : 'widget.QuartzEditJobChainTargetForm',

  fieldDefaults : {
    labelWidth : 80
  },

  qdesk : {
    /*
     * Required ID of the current (local) scheduler connection.
     */
    connectionId : undefined,

    /*
     * Mode of the form (see EditJobChainTargetFormMode).
     */
    mode : undefined,

    /*
     * Job chain target instance if EditJobChainTargetFormMode = EDIT
     */
    target : undefined,

    /**
     * Internally used variable with the currently selected connection ID (local or remote)
     */
    currentConnectionId : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( config.qdesk.mode == QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode.EDIT &&
        !config.qdesk.target )
    {
      Ext.Error.raise( 'Required config parameter missing: qdesk.target' );
    }

    /*
     * Use remote connection ID if target uses it.
     */
    var targetConnectionId = config.qdesk.connectionId;
    if ( config.qdesk.target
        && config.qdesk.target.type == QDesk.enums.scheduler.JobChainTargetType.REMOTE
        && config.qdesk.target.remoteConnectionId )
    {
      targetConnectionId = config.qdesk.target.remoteConnectionId;
    }

    Ext.apply( config, {
          items : [
            {
              xtype : 'hiddenfield',
              itemId : 'jobChainTargetId',
              name : 'id'
            },

            {
              xtype : 'fieldset',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldsetLabelConnection' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                /*
                 * Local Connection
                 */
                {
                  xtype : 'radiofield',
                  itemId : 'localConnectionRadio',
                  name : 'type',
                  inputValue : QDesk.enums.scheduler.JobChainTargetType.LOCAL,
                  boxLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.boxLabelLocalConnection' ),
                  boxTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.boxTooltipLocalConnection' )
                },

                /*
                 * Remote Connection
                 */
                {
                  xtype : 'radiofield',
                  itemId : 'remoteConnectionRadio',
                  name : 'type',
                  inputValue : QDesk.enums.scheduler.JobChainTargetType.REMOTE,
                  boxLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.boxLabelRemoteConnection' ),
                  boxTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.boxTooltipRemoteConnection' )
                },
                {
                  xtype : 'fieldcontainer',
                  itemId : 'remoteConnectionFields',
                  padding : '0 0 0 10',
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%',
                    labelWidth : 90
                  },
                  items : [
                    {
                      xtype : 'ConnectionCombo',
                      itemId : 'remoteConnectionCombo',
                      name : 'remoteConnectionId',
                      allowBlank : false,
                      columnWidth : 1
                    }
                  ]
                }
              ]
            },

            {
              xtype : 'checkboxfield',
              itemId : 'enabled',
              name : 'enabled',
              value : true,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.boxLabelEnabled' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.boxTooltipEnabled' ),
              allowBlank : true
            },

            {
              xtype : 'QuartzSelectJobGrid',
              itemId : 'selectJobGrid',
              height : 200,
              qdesk : {
                connectionId : targetConnectionId
              }
            },

            {
              xtype : 'fieldset',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldsetLabelExecType' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%',
                labelWidth : 120
              },
              items : [
                {
                  xtype : 'combo',
                  itemId : 'execType',
                  name : 'execType',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldLabelExecType' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldTooltipExecType' ),
                  allowBlank : false,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.scheduler.JobChainTargetExecTypeStore',
                  valueField : 'value',
                  displayField : 'valueLabel'
                },


                /* Delay */
                {
                  xtype : 'numberfield',
                  itemId : 'execTypeDelay',
                  name : 'execTypeDelay',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldLabelExecTypeDelay' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldTooltipExecTypeDelay' ),
                  emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldEmptyTxtExecTypeDelay' ),
                  minValue : 1,
                  qdesk : {
                    required : true
                  }
                },

                /* Date expression */
                {
                  xtype : 'iconcombo',
                  itemId : 'execTypeDateExpressionLanguage',
                  name : 'execTypeDateExpressionLanguage',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldLabelExecTypeDateExpressionLanguage' ),
                  fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldTooltipExecTypeDateExpressionLanguage' ),
                  allowBlank : true,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.ExpressionLanguageStore',
                  valueField : 'value',
                  displayField : 'valueLabel',
                  iconClsField : 'value',
                  qdesk : {
                    required : true
                  }
                },

                {
                  xtype : 'panel',
                  itemId : 'execTypeDateExpressionPanel',
                  border : false,
                  margin : '0 0 5 0',
                  layout : 'column',
                  items : [
                    {
                      xtype : 'textareafield',
                      itemId : 'execTypeDateExpression',
                      name : 'execTypeDateExpression',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldLabelExecTypeDateExpression' ),
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldTooltipExecTypeDateExpression' ),
                      emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.fieldEmptyTxtExecTypeDateExpression' ),
                      labelAlign : 'top',
                      columnWidth : 1,
                      height : 200,
                      qdesk : {
                        required : true
                      }
                    },
                    {
                      xtype : 'button',
                      itemId : 'execTypeDateExpressionObjects',
                      style : {
                        marginLeft : '3px',
                        marginTop : '22px'
                      },
                      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.buttonTooltipExecTypeDateExpressionObjects' ),
                      iconCls : 'expressionObjects'
                    }
                  ]
                }
              ]
            }
          ],

          buttons : [
            {
              itemId : 'cancelButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
            },
            {
              itemId : 'actionButton',
              text : me._getActionButtonText( config.qdesk.mode ),
              disabled : true
              //formBind : true  // cannot use as we also need to consider if there is a job selected in the job grid...
            }
          ]
        }
    );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    var execTypeDateExprLangCombo = me.down( '#execTypeDateExpressionLanguage' );

    if ( me.qdesk.target )
    {
      // populate the form with data (edit mode)
      me.loadRecord( me.qdesk.target );

      // defaulta for the edit mode
      if ( !execTypeDateExprLangCombo.getValue() )
      {
        execTypeDateExprLangCombo.setValue( QDesk.enums.ExpressionLanguage.JAVASCRIPT );
      }
    }
    else
    {
      // defaults for the add mode
      me.down( '#localConnectionRadio' ).setValue( true );
      me.down( '#remoteConnectionRadio' ).setValue( false );
      me.down( '#execType' ).setValue( QDesk.enums.scheduler.JobChainTargetExecType.IMMEDIATE );
      execTypeDateExprLangCombo.setValue( QDesk.enums.ExpressionLanguage.JAVASCRIPT );
    }
  },


  isValid : function ()
  {
    var me = this;
    var valid = me.callParent();

    // check a job is selected in the grid
    var selectJobGrid = me.down( '#selectJobGrid' );
    var jobIsSelected = !Ext.isEmpty( selectJobGrid.getSelection() );

    return valid && jobIsSelected;
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    /*
     * Upon form validation change event, we enable/disable the action button. Apart
     * from the validity of the form, we also check if a job is selected in the grid.
     */
    me.on( 'validitychange', function ( valid )
    {
      me._maybeDisableActionButton();
    } );


    var selectJobGrid = me.down( '#selectJobGrid' );

    // local / remote connection change
    me.down( '#localConnectionRadio' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#remoteConnectionCombo' ), function ( field )
      {
        field.setDisabled( newValue );
        field.allowBlank = newValue;
        field.validate();
      } );

      if ( newValue )
      {
        me.qdesk.currentConnectionId = me.qdesk.connectionId;

        // reload the selectJobGrid using the local connection
        selectJobGrid.changeConnection( me.qdesk.currentConnectionId );
      }
    } );

    me.down( '#remoteConnectionRadio' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#remoteConnectionCombo' ), function ( field )
      {
        field.setDisabled( !newValue );
        field.allowBlank = !newValue;
        field.validate();
      } );

      if ( newValue )
      {
        // reload the selectJobGrid using the currently selected remote connection
        var remoteConnectionCombo = me.down( '#remoteConnectionCombo' );
        me.qdesk.currentConnectionId = remoteConnectionCombo.getValue();

        selectJobGrid.changeConnection( me.qdesk.currentConnectionId );
      }
    } );


    me.down( '#remoteConnectionCombo' ).on( 'change', function ( combo, newValue, oldValue )
    {
      me.qdesk.currentConnectionId = newValue;
      selectJobGrid.changeConnection( me.qdesk.currentConnectionId );
    } );


    // target execution type change
    me.down( '#execType' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#execTypeDelay' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainTargetExecType.DELAYED );
        field.allowBlank = newValue != QDesk.enums.scheduler.JobChainTargetExecType.DELAYED;
        field.validate();
      } );

      Ext.each( me.down( '#execTypeDateExpression' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainTargetExecType.DATE_EXPRESSION );
        field.allowBlank = newValue != QDesk.enums.scheduler.JobChainTargetExecType.DATE_EXPRESSION;
        field.validate();
      } );

      Ext.each( me.down( '#execTypeDateExpressionLanguage' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainTargetExecType.DATE_EXPRESSION );
        field.allowBlank = newValue != QDesk.enums.scheduler.JobChainTargetExecType.DATE_EXPRESSION;
        field.validate();
      } );

      Ext.each( me.down( '#execTypeDateExpressionObjects' ), function ( field )
      {
        field.setVisible( newValue == QDesk.enums.scheduler.JobChainTargetExecType.DATE_EXPRESSION );
      } );
    } );

    selectJobGrid.on( 'selectionchange', function ( grid, selected )
    {
      me._maybeDisableActionButton();
    } );

    /*
     * Select the target job (if any specified) in the jobs grid upon it load.
     */
    var firstLoad = true;

    var selectJobGridStore = selectJobGrid.getStore();
    selectJobGridStore.on( 'load',
        function ( store, records, successful )
        {
          if ( successful )
          {
            if ( me.qdesk.target )
            {
              var jobName = me.qdesk.target.get( 'jobName' );
              var jobGroupName = me.qdesk.target.get( 'jobGroupName' );

              var recordFound = false;
              Ext.each( records, function ( record )
              {
                if ( record.get( 'name' ) == jobName && record.get( 'group' ).name == jobGroupName )
                {
                  recordFound = true;
                  selectJobGrid.setSelection( record );
                  selectJobGrid.getView().focusRow( record );  // scroll to the record
                  return false;
                }
              } );

              /*
               * Show missing job warning upon the first load in the edit mode if the current connection
               * is the same as the connection specified in the editted target.
               */
              if ( me.qdesk.target && me.qdesk.target.get( 'remoteConnectionId' ) == me.qdesk.currentConnectionId )
              {
                if ( firstLoad && !recordFound )
                {
                  QDesk.util.UxUtils.showWarningDialog(
                      QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm.targetJobNotFound',
                          jobGroupName, jobName ) );
                }
              }
            }

            // we want to be notified only once upon the first store load
            firstLoad = false;
          }
        } );


    // exec type date expression scripting objects
    me.down( '#execTypeDateExpressionObjects' ).on( 'click',
        function ( button )
        {
          // for JavaScript language, open the JSScriptingObjectsWindow window
          var selectedExprLang = me.down( '#execTypeDateExpressionLanguage' ).getValue();

          var window;
          switch ( selectedExprLang )
          {
            case QDesk.enums.ExpressionLanguage.JAVASCRIPT:
              window = Ext.widget( 'JSScriptingObjectsWindow', {
                qdesk : {
                  schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
                }
              } );
              break;

            default:
              Ext.Error.raise( 'Expression properties window not available for language: ' + selectedExprLang );
          }

          window.down( '#treePanel' ).on( 'itemdblclick', function ( treePanel, record, item, index, event )
          {
            var propertyName = record.get( 'propertyName' );
            var propertyPath = record.getPath( 'propertyName', '.' ).substring( 2 );
            window.close();

            var execTypeDateExpression = me.down( '#execTypeDateExpression' );
            QDesk.util.FormUtils.insertTextAtCursor( execTypeDateExpression, propertyPath );
            event.stopPropagation();
          }, me );

          window.show();
        }, me );


    me.down( '#actionButton' ).on( 'click', function ()
    {
      if ( me.isValid() )
      {
        var fieldValues = me.getForm().getFieldValues();

        // set the jobName and jobGroupName (from selected JobDetail model instance)
        var selectedJob = selectJobGrid.getSelection();
        Ext.apply( fieldValues, {
          jobGroupName : selectedJob[ 0 ].get( 'group' ).name,
          jobName : selectedJob[ 0 ].get( 'name' )
        } );

        // set the remote connection name
        if ( fieldValues.type == QDesk.enums.scheduler.JobChainTargetType.REMOTE )
        {
          var remoteConnectionCombo = me.down( '#remoteConnectionCombo' );
          Ext.apply( fieldValues, {
            remoteConnectionName : remoteConnectionCombo.getSelectedRecord().get( 'name' )
          } );
        }

        var jobChainTarget = QDesk.util.quartz.QuartzJobChainUtils.createJobChainTarget( fieldValues );

        if ( me.qdesk.mode == QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode.ADD )
        {
          me.fireEvent( 'jobChainTargetAdded', jobChainTarget );
        }
        else if ( me.qdesk.mode == QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode.EDIT )
        {
          me.fireEvent( 'jobChainTargetUpdated', jobChainTarget, me.qdesk.target );
        }
      }
    } );
  },


  _getActionButtonText : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  _maybeDisableActionButton : function ()
  {
    var me = this;

    // disable action button if the form has validation errors or no job is selected
    me.down( '#actionButton' ).setDisabled( !me.isValid() );
  }
} );
