/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetForm',
    'QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode',
    'QDesk.Msg'
  ],

  alias : 'widget.EditJobChainTargetWindow',

  cls : 'editJobChainTargetWindow',
  iconCls : 'editJobChainTargetWindowIcon',

  modal : true,
  resizable : false,

  minWidth : 600,
  width : 600,

  qdesk : {
    /*
     * Required ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Mode of the form (see EditJobChainTargetFormMode).
     */
    mode : undefined,

    /*
     * Job chain target instance if EditJobChainTargetFormMode = EDIT
     */
    target : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( config.qdesk.mode == QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode.EDIT &&
        !config.qdesk.target )
    {
      Ext.Error.raise( 'Required config parameter missing: qdesk.target' );
    }

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.mode ),
      items : [
        {
          xtype : 'QuartzEditJobChainTargetForm',
          itemId : 'form',

          qdesk : {
            connectionId : config.qdesk.connectionId,
            mode : config.qdesk.mode,
            target : config.qdesk.target
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } )
  },


  /**
   * Returns the title of this window.
   * @param mode a QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode value.
   * @return {string} the title.
   * @private
   */
  _getWindowTitle : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetWindow.windowTitleAdd' );

      case QDesk.view.main.scheduler.common.chain.EditJobChainTargetFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainTargetWindow.windowTitleEdit' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  }
} );
