/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.view.main.scheduler.quartz.chain.EditJobChainForm',
    'QDesk.view.main.scheduler.common.chain.EditJobChainFormMode',

    'QDesk.Msg'
  ],

  alias : 'widget.QuartzEditJobChainWindow',

  cls : 'editJobChainWindow',
  iconCls : 'editJobChainWindowIcon',

  modal : true,
  resizable : false,

  minWidth : 700,
  width : 700,

  qdesk : {
    /*
     * Required ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Mode of the form (see EditJobChainFormMode).
     */
    mode : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.mode ),
      items : [
        {
          xtype : 'QuartzEditJobChainForm',
          itemId : 'form',

          qdesk : {
            connectionId : config.qdesk.connectionId,
            mode : config.qdesk.mode
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } )
  },


  /**
   * Returns the title of this window.
   * @param mode a QDesk.view.main.scheduler.common.chain.EditJobChainFormMode value.
   * @return {string} the title.
   * @private
   */
  _getWindowTitle : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.main.scheduler.common.chain.EditJobChainFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainWindow.windowTitleAdd' );

      case QDesk.view.main.scheduler.common.chain.EditJobChainFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainWindow.windowTitleEdit' );

      case QDesk.view.main.scheduler.common.chain.EditJobChainFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.EditJobChainWindow.windowTitleClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  }
} );
