/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.store.scheduler.quartz.QuartzJobChainStore',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.view.widget.ReadOnlyCheckColumn',

    'QDesk.util.quartz.QuartzJobChainUtils',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzJobChainsGrid',

  scrollable : true,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
        '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
        '<tpl if="description">{[Ext.htmlEncode(values.description)]}</tpl>' +
        '<tpl if="!description">{[QDesk.Msg.get( \"QDesk.notAvailable\" )]}</tpl>' +
        '</p>' +
        '{[QDesk.util.quartz.QuartzJobChainUtils.createJobChainPropertiesTable( values )]}'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    var jobChainStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobChainStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'addJobChainButton',
          scale : 'small',
          iconCls : 'addJobChain',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.tooltipAddJobChain' ),
          disabled : true,
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'addJobChain' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'deleteJobChainButton',
          scale : 'small',
          iconCls : 'deleteJobChain',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.tooltipDeleteJobChain' ),
          disabled : true,
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'deleteJobChain' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'editJobChainButton',
          scale : 'small',
          iconCls : 'editJobChain',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.tooltipEditJobChain' ),
          disabled : true,
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'editJobChain' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'cloneJobChainButton',
          scale : 'small',
          iconCls : 'cloneJobChain',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.tooltipCloneJobChain' ),
          disabled : true,
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'cloneJobChain' );
              }
            }
          }
        },
        '-',
        {
          xtype : 'searchfield',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.emptyTextFieldFilterQueryText' ),
          store : jobChainStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ],


      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.3
        },

        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'enabledColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textEnabledColumn' ),
          dataIndex : 'enabled',
          sortable : false,
          groupable : false,
          width : 100
        },

        {
          itemId : 'enabledOnWeekdaysColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textEnabledOnWeekdaysColumn' ),
          dataIndex : 'enabledOnWeekday',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enums value */
          renderer : function ( value, metadata, record )
          {
            var result = '';
            Ext.each( value, function ( weekday, index, weekdays )
            {
              result += QDesk.util.EnumUtils.getWeekdayShortName( weekday );
              if ( index < weekdays.length - 1 )
                result += ', ';
            } );

            return result;
          },
          flex : 1
        },


        {
          itemId : 'enabledFromTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textEnabledFromTimeColumn' ),
          dataIndex : 'enabledFromTime',
          sortable : false,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'enabledToTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textEnabledToTimeColumn' ),
          dataIndex : 'enabledToTime',
          sortable : false,
          groupable : false,
          flex : 0.5
        },

        {
          itemId : 'conditionTypeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textConditionTypeColumn' ),
          dataIndex : 'conditionType',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enum value */
          renderer : function ( value, metadata, record )
          {
            switch ( value )
            {
              case QDesk.enums.scheduler.JobChainConditionType.EXEC_STATUS:
                var conditionExecStatus = record.get( 'conditionExecStatus' );
                return QDesk.util.EnumUtils.getJobChainConditionTypeName( value ) + ' [' +
                    QDesk.util.EnumUtils.getExecStatusName( conditionExecStatus ) + ']';
                break;

              case QDesk.enums.scheduler.JobChainConditionType.MAX_DURATION:
                var conditionMaxDuration = record.get( 'conditionMaxDuration' );
                return QDesk.util.EnumUtils.getJobChainConditionTypeName( value ) + ' [' +
                    conditionMaxDuration + 'ms]';
                break;

              case QDesk.enums.scheduler.JobChainConditionType.EXPRESSION:
                var conditionExpressionLanguage = record.get( 'conditionExpressionLanguage' );
                return QDesk.util.EnumUtils.getJobChainConditionTypeName( value ) + ' [' +
                    QDesk.util.EnumUtils.getExpressionLanguageName( conditionExpressionLanguage ) + ']';
                break;

              default:
                return QDesk.util.EnumUtils.getJobChainConditionTypeName( value );
            }
          },
          flex : 0.7
        },

        /* Scheduler related columns */
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : false,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'datecolumn',
          itemId : 'createdAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textCreatedAtColumn' ),
          dataIndex : 'createdAt',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'updatedAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.textUpdatedAtColumn' ),
          dataIndex : 'updatedAt',
          sortable : true,
          groupable : false,
          flex : 1
        }
      ],

      store : jobChainStore,

      qdesk : {
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [ config ] );
  },


  /**
   * Invoked to create a context menu for the specified job chain.
   * @param jobChain a job chain.
   * @private
   */
  _createRowCtxMenu : function ( jobChain )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // enable job chain
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.ruleCtxMenuEnable' ),
          iconCls : 'enableJobChain',
          disabled : jobChain.get( 'enabled' ),
          handler : function ()
          {
            me.fireEvent( 'enableJobChain', jobChain );
          }
        },
        // disable job chain
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.ruleCtxMenuDisable' ),
          iconCls : 'disableJobChain',
          disabled : !jobChain.get( 'enabled' ),
          handler : function ()
          {
            me.fireEvent( 'disableJobChain', jobChain );
          }
        },
        '-',
        // delete job chain
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.ruleCtxMenuDelete' ),
          iconCls : 'deleteJobChain',
          handler : function ()
          {
            me.fireEvent( 'deleteJobChain', jobChain );
          }
        },
        // edit job chain
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.ruleCtxMenuEdit' ),
          iconCls : 'editJobChain',
          handler : function ()
          {
            me.fireEvent( 'editJobChain', jobChain );
          }
        },
        // clone rule
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.chain.JobChainsGrid.ruleCtxMenuClone' ),
          iconCls : 'cloneJobChain',
          handler : function ()
          {
            me.fireEvent( 'cloneJobChain', jobChain );
          }
        }
      ]
    } );
  }
} );
