/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.store.scheduler.quartz.QuartzExecHistoryStore',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.view.widget.MaxTodayDateField',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.SearchField'
  ],

  alias : 'widget.QuartzExecHistoryGrid',

  cls : 'execHistoryGrid',

  scrollable : true,

  //border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    if ( !config.qdesk.maxFetchSize )
      Ext.Error.raise( 'Required config parameter missing: qdesk.maxFetchSize' );

    var execHistoryStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzExecHistoryStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'MaxTodayDateField',
          itemId : 'maxFetchDateSelect',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.fieldLabelMaxFetchDateSelect' ),
          editable : false,
          format : QDesk.util.constant.DateFormat.dateFormat,
          value : new Date(),
          maxValue : new Date(),
          labelAlign : 'right',
          labelWidth : 55,
          width : 155,
          disabled : true,
          listeners : {
            change : {
              fn : function ( picker, newValue )
              {
                this.fireEvent( 'changeMaxFetchDate', newValue );
              }
            }
          }
        },
        {
          xtype : 'combo',
          itemId : 'maxFetchSizeSelect',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.fieldLabelMaxFetchSizeSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [10 , '10'],
              [20 , '20'],
              [50 , '50'],
              [100 , '100'],
              [500 , '500'],
              [1000 , '1000']
              // [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 72,
          width : 172,
          editable : false,
          disabled : true,
          listeners : {
            change : {
              fn : function ( combo, newValue )
              {
                this.fireEvent( 'changeMaxFetchSize', newValue );
              }
            }
          }
        },
        {
          xtype : 'searchfield',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.emptyTextFieldFilterQueryText' ),
          store : execHistoryStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ],


      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.6
        },
        {
          xtype : 'datecolumn',
          itemId : 'startedAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textStartedAtColumn' ),
          dataIndex : 'startedAt',
          sortable : true,
          groupable : false,
          flex : 1.7
        },
        {
          xtype : 'datecolumn',
          itemId : 'finishedAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textFinishedAtColumn' ),
          dataIndex : 'finishedAt',
          sortable : true,
          groupable : false,
          flex : 1.7
        },
        {
          //xtype : 'numbercolumn', - if used, then renderer is not called
          itemId : 'durationColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textDurationColumn' ),
          dataIndex : 'duration',
          renderer : function ( value )
          {
            return QDesk.util.FormatUtils.formatDuration( value );
          },
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'execStatusColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textExecStatusColumn' ),
          dataIndex : 'execStatus',
          sortable : true,
          groupable : false,
          flex : 0.6,
          /* Need custom renderer to localize exec status values */
          renderer : function ( value, metadata, record )
          {
            return QDesk.util.EnumUtils.getExecStatusName( value );
          }
        },

        // error
        {
          xtype : 'actioncolumn',
          itemId : 'errorColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textErrorColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textErrorColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          items : [
            {
              icon : 'images/icon/error-16x16.png',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.tooltipErrorColumn' ),
              getClass : function ( value, metadata, record )
              {
                // hide the icon in case the error is not available
                var error = record.get( 'error' );
                return error ? 'clickable' : 'x-hidden';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var error = record.get( 'error' );
                this.fireEvent( 'requestViewException', error );
              }
            }
          ]
        },

        /* Scheduler related columns */
        {
          itemId : 'schedulerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textSchedulerNameColumn' ),
          dataIndex : 'schedulerName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerInstanceIdColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textSchedulerInstanceIdColumn' ),
          dataIndex : 'schedulerInstanceId',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textSchedulerVersionColumn' ),
          dataIndex : 'schedulerVersion',
          renderer : function ( value )
          {
            return value.major + '.' + value.minor + '.' + value.maintenance;
          },
          sortable : true,
          groupable : false,
          flex : 1.1
        },
        /* End of scheduler related columns */

        /* Job related columns */
        {
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJobGroupNameColumn' ),
          dataIndex : 'jobGroupName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJobNameColumn' ),
          dataIndex : 'jobName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJobClassNameColumn' ),
          dataIndex : 'jobClassName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'jobClassLocationColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJobClassLocationColumn' ),
          dataIndex : 'jobClassLocation',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.tooltipJobDataMapColumn' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var jobDataMap = record.get( 'jobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', jobDataMap );
              }
            }
          ]
        },
        /* End of job related columns */

        /* Trigger related columns */
        {
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textTriggerGroupNameColumn' ),
          dataIndex : 'triggerGroupName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textTriggerNameColumn' ),
          dataIndex : 'triggerName',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          itemId : 'calendarNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textCalendarNameColumn' ),
          dataIndex : 'calendarName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        /* End of trigger related columns */

        // result
        {
          xtype : 'actioncolumn',
          itemId : 'resultColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textResultColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textResultColumn' ),
          sortable : false,
          groupable : false,
          width : 80,
          items : [
            {
              icon : 'images/icon/exec-history-result-16x16.png',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.tooltipResultColumn' ),
              getClass : function ( value, metadata, record )
              {
                // hide the icon in case the result is not available
                var resultNull = record.get( 'resultNull' );
                return resultNull ? 'x-hidden' : 'clickable';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var execHistoryId = record.get( 'id' );
                this.fireEvent( 'requestViewResult', execHistoryId );
              }
            }
          ]
        },

        // userData
        {
          xtype : 'actioncolumn',
          itemId : 'userDataColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textUserDataColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textUserDataColumn' ),
          sortable : false,
          groupable : false,
          width : 90,
          items : [
            {
              icon : 'images/icon/exec-history-user-data-16x16.png',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.tooltipUserDataColumn' ),
              getClass : function ( value, metadata, record )
              {
                // hide the icon in case the user data is not available
                var userDataNull = record.get( 'userDataNull' );
                return userDataNull ? 'x-hidden' : 'clickable';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var execHistoryId = record.get( 'id' );
                this.fireEvent( 'requestViewUserData', execHistoryId );
              }
            }
          ]
        },

        // log
        {
          xtype : 'actioncolumn',
          itemId : 'logColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textLogColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textLogColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          items : [
            {
              icon : 'images/icon/exec-history-log-16x16.png',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.tooltipLogColumn' ),
              getClass : function ( value, metadata, record )
              {
                // hide the icon in case the log data is not available
                var logNull = record.get( 'logNull' );
                return logNull ? 'x-hidden' : 'clickable';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var execHistoryId = record.get( 'id' );
                this.fireEvent( 'requestViewLog', execHistoryId );
              }
            }
          ]
        },

        /* Thread related data */
        {
          itemId : 'threadGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textThreadGroupNameColumn' ),
          dataIndex : 'threadGroupName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'threadNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textThreadNameColumn' ),
          dataIndex : 'threadName',
          sortable : true,
          groupable : false,
          flex : 2
        },
        {
          xtype : 'numbercolumn',
          itemId : 'threadPriorityColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textThreadPriorityColumn' ),
          dataIndex : 'threadPriority',
          sortable : true,
          groupable : false,
          flex : 1
        },

        /* Host related data */
        {
          itemId : 'hostNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textHostNameColumn' ),
          dataIndex : 'hostName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },

        /* JVM related data */
        {
          xtype : 'numbercolumn',
          itemId : 'jvmPidColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJvmPidColumn' ),
          dataIndex : 'jvmPid',
          sortable : true,
          groupable : false,
          flex : 0.6
        },
        {
          itemId : 'jvmNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJvmNameColumn' ),
          dataIndex : 'jvmName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmVendorColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJvmVendorColumn' ),
          dataIndex : 'jvmVendor',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJvmVersionColumn' ),
          dataIndex : 'jvmVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmRuntimeVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textJvmRuntimeVersionColumn' ),
          dataIndex : 'jvmRuntimeVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },

        /* OS related data */
        {
          itemId : 'osNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textOsNameColumn' ),
          dataIndex : 'osName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'osVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textOsVersionColumn' ),
          dataIndex : 'osVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'osArchColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textOsArchColumn' ),
          dataIndex : 'osArch',
          sortable : true,
          groupable : false,
          flex : 0.5
        }
      ],

      store : execHistoryStore,

      qdesk : {
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [config] );

    var maxFetchSize = QDesk.ApplicationConfig.get( me.qdesk.gridId + '.maxFetchSize' );
    me.down( '#maxFetchSizeSelect' ).setValue( maxFetchSize );

    // rows have different CSS style that reflect the status
    me.getView().getRowClass = function ( record )
    {
      var execStatus = record.get( 'execStatus' );
      if ( execStatus == QDesk.enums.ExecStatus.SUCCESS )
        return 'statusSuccess';
      else if ( execStatus == QDesk.enums.ExecStatus.VETO )
        return 'statusVeto';
      else if ( execStatus == QDesk.enums.ExecStatus.ERROR )
        return 'statusError';
      else return '';
    };
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var maxFetchSize = me.down( '#maxFetchSizeSelect' ).getValue();

    var extra = {};
    extra[me.qdesk.gridId + '.maxFetchSize'] = maxFetchSize;

    return extra;
  }
} );
