/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.info.InfoForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.view.base.BaseFormPanel',
    'QDesk.view.main.scheduler.common.AgentConfigurationGrid',

    'QDesk.util.license.LicensedProductFeatures'
  ],

  alias : 'widget.QuartzInfoForm',

  cls : 'quartzInfoForm',
  bodyCls : 'quartzInfoFormBody',

  frame : false,

  fieldDefaults : {
    labelWidth : 150
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldsetLabelSchedulerInfo' ),
          collapsible : true,
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          items : [
            {
              xtype : 'displayfield',
              itemId : 'schedulerHost',
              name : 'schedulerHost',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerHost' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerPort',
              name : 'schedulerPort',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerPort' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerJmxServiceUrl',
              name : 'schedulerJmxServiceUrl',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerJmxServiceUrl' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerStatus',
              name : 'schedulerDisplayStatus',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerStatus' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerName',
              name : 'schedulerName',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerName' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerInstanceId',
              name : 'schedulerInstanceId',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerInstanceId' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerObjectName',
              name : 'schedulerObjectName',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerObjectName' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerVersion',
              name : 'schedulerDisplayVersion',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerVersion' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerThreadPoolClassName',
              name : 'schedulerThreadPoolClassName',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerThreadPoolClassName' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerThreadPoolSize',
              name : 'schedulerThreadPoolSize',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerThreadPoolSize' )
            },
            {
              xtype : 'displayfield',
              itemId : 'schedulerJobStoreClassName',
              name : 'schedulerJobStoreClassName',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerJobStoreClassName' )
            },

            {
              xtype : 'displayfield',
              itemId : 'schedulerMonitoringUrl',
              name : 'schedulerMonitoringUrl',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelSchedulerMonitoringUrl' ),
              renderer : function ( value, field )
              {
                if ( Ext.isEmpty( value ) )
                  return value;

                var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

                var tooltip;
                if ( licensedFeatures.PollMonitoring )
                {
                  tooltip =
                      Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.tooltipSchedulerMonitoringUrl' ) );
                  return '<a href="' + value + '" class="newWindow" target="_blank" data-qtip="' +
                      tooltip + '">' + value + '</a>';
                }
                else
                {
                  tooltip =
                      Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.tooltipSchedulerMonitoringUrlNotLicensed' ) );
                  return '<a href="' + value +
                      '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="'
                      + tooltip + '">' + value + '</a>';
                }
              }
            }
          ]
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldsetLabelJvmAgentInfo' ),
          collapsible : true,
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          items : [
            {
              xtype : 'displayfield',
              itemId : 'jvmAgentNotAvailableMsg',
              hidden : true,
              value : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.jvmAgentNotAvailableMsg' )
            },
            {
              xtype : 'displayfield',
              itemId : 'jvmAgentNameAndVersion',
              name : 'jvmAgentNameAndVersion',
              hidden : true,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelJvmAgentNameAndVersion' )
            },
            {
              xtype : 'displayfield',
              itemId : 'jvmAgentVersion',
              name : 'jvmAgentDisplayVersion',
              hidden : true,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelJvmAgentVersion' )
            },
            {
              xtype : 'displayfield',
              itemId : 'jvmAgentEdition',
              name : 'jvmAgentEdition',
              hidden : true,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelJvmAgentEdition' )
            },
            {
              xtype : 'displayfield',
              itemId : 'jvmAgentLocale',
              name : 'jvmAgentLocale',
              hidden : true,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelJvmAgentLocale' ),
              renderer : function ( value )
              {
                if ( Ext.isEmpty( value ) )
                  return value;

                var result = value.id + ' (' + value.displayCountry;

                if ( value.language )
                  result += '/' + value.displayLanguage;

                if ( result.variant )
                  result += '/' + value.displayVariant;

                result += ')';

                return result;
              }
            },
            {
              xtype : 'displayfield',
              itemId : 'jvmAgentTimeZone',
              name : 'jvmAgentTimeZone',
              hidden : true,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldLabelJvmAgentTimeZone' ),
              renderer : function ( value )
              {
                return Ext.isEmpty( value ) ? value : value.displayName;
              }
            },

            {
              xtype : 'fieldset',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldsetLabelJvmAgentClassLoaderInfo' ),
              itemId : 'jvmAgentClassLoaderInfo',
              hidden : true,
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'textareafield',
                  readOnly : true,
                  name : 'jvmAgentClassLoaderInfo'
                }
              ]
            },

            {
              xtype : 'fieldset',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldsetLabelJvmAgentCurrentThreadContextClassLoaderInfo' ),
              itemId : 'jvmAgentCurrentThreadContextClassLoaderInfo',
              hidden : true,
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'textareafield',
                  readOnly : true,
                  name : 'jvmAgentCurrentThreadContextClassLoaderInfo'
                }
              ]
            },

            {
              xtype : 'fieldset',
              title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.info.InfoForm.fieldsetLabelJvmAgentConfiguration' ),
              itemId : 'jvmAgentConfiguration',
              hidden : true,
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'AgentConfigurationGrid',
                  itemId : 'jvmAgentConfigurationGrid',
                  style : {
                    marginBottom : '5px'
                  }
                }
              ]
            }
          ]
        }
      ]
    } );

    me.callParent( [config] );
  }
} );
