/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.JobDetailWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.main.scheduler.quartz.jobs.JobDetailForm',
    'QDesk.view.main.scheduler.quartz.jobs.JobDetailFormMode',
    'QDesk.view.main.scheduler.quartz.JobDataMapGrid',
    'QDesk.util.constant.Const'
  ],

  alias : 'widget.QuartzJobDetailWindow',

  cls : 'quartzJobDetailWindow',
  iconCls : 'quartzJobDetailWindowIcon',

  scrollable : true,

  modal : true,
  resizable : false,

  width : 500,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.jobDetailFormMode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jobDetailFormMode' );

    if ( !config.qdesk.jobDetail )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jobDetail' );

    /*
     * Job Details Form
     */
    var jobDetailForm = {
      xtype : 'QuartzJobDetailForm',
      itemId : 'jobDetailForm',
      qdesk : config.qdesk
    };

    Ext.apply( config, {
      items : [
        jobDetailForm
      ]
    } );

    me.callParent( [config] );

    me._populateJobDetailForm( config.qdesk.jobDetailFormMode, config.qdesk.jobDetail );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } );

    me.down( '#actionButton' ).on( 'click', me._onActionButtonClick, me );
  },


  /**
   * Invoked to handle the actionButton click event. This method creates
   * and populates a new JobDetail instance and passes it to the configured
   * callback method.
   */
  _onActionButtonClick : function ()
  {
    var me = this;

    if ( Ext.isFunction( me.qdesk.callback ) )
    {
      var fieldValues = me.down( '#jobDetailForm' ).getForm().getFieldValues();

      var newJobDetail = Ext.create( 'QDesk.model.scheduler.quartz.QuartzJobDetail',
          {
            name : fieldValues.jobName,
            group : {
              name : fieldValues.jobGroupName
            },
            jobClass : fieldValues.jobClass,
            durability : fieldValues.durability,
            shouldRecover : fieldValues.shouldRecover,
            description : fieldValues.description,
            jobDataMap : []
          } );

      // populate the jobDataMap
      Ext.Array.each( me.down( '#jobDataMapGrid' ).getStore().data.getRange(), function ( entry )
      {
        var entryName = Ext.String.trim( entry.get( 'name' ) );
        var entryValue = Ext.String.trim( entry.get( 'value' ) );

        // copy only entries with non-blank name
        if ( !Ext.isEmpty( entryName ) )
        {
          newJobDetail.get( 'jobDataMap' ).push( {
            name : entryName,
            value : entryValue
          } );
        }
      } );

      var scope = me.qdesk.scope || me;
      me.qdesk.callback.apply( scope, [newJobDetail] );
    }
  },


  /**
   * Populates the Job Detail form with the specified job detail data.
   * @param jobDetail job detail record.
   */
  _populateJobDetailForm : function ( mode, jobDetail )
  {
    var me = this;

    var jobDetailForm = me.down( '#jobDetailForm' );
    // assert jobDetailForm != null

    jobDetailForm.on( {
      beforerender : function ( jobDetailForm )
      {
        // job name
        var jobNameField = jobDetailForm.down( '#jobName' );
        jobNameField.setValue( jobDetail.get( 'name' ) );

        // job group name must be extracted from the JobGroup
        var groupNameField = jobDetailForm.down( '#jobGroupName' );
        groupNameField.setValue( jobDetail.get( 'group' ) ? jobDetail.get( 'group' ).name : null );
      }} );

    // description field contains 'n/a' in case no description is set
    if ( mode == QDesk.view.main.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER )
    {
      jobDetailForm.on( {
        beforerender : function ( jobDetailForm )
        {
          // if job description is empty, then show N/A
          var descriptionField = jobDetailForm.down( '#description' );
          if ( Ext.isEmpty( descriptionField.getValue() ) )
          {
            descriptionField.setValue( QDesk.Msg.get( 'QDesk.notAvailable' ) );
          }
        }
      } );
    }
    else if ( mode == QDesk.view.main.scheduler.quartz.jobs.JobDetailFormMode.ADD )
    {
      jobDetailForm.on( {
        afterrender : function ( jobDetailForm )
        {
          var nameField = jobDetailForm.down( '#jobName' );
          nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
        }
      } );
    }
    else if ( mode == QDesk.view.main.scheduler.quartz.jobs.JobDetailFormMode.CLONE )
    {
      jobDetailForm.on( {
        beforerender : function ( jobDetailForm )
        {
          var nameField = jobDetailForm.down( '#jobName' );
          nameField.setValue( null );
        },

        afterrender : function ( jobDetailForm )
        {
          var nameField = jobDetailForm.down( '#jobName' );
          nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
        }
      } );
    }

    jobDetailForm.loadRecord( jobDetail );
  }
} );