/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',
    'QDesk.store.scheduler.quartz.QuartzJobDetailStore',
    'QDesk.store.scheduler.quartz.QuartzJobGroupStore',

    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.widget.ExecHealthIndicator',
    'QDesk.view.widget.ReadOnlyCheckColumn',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo',
    'Ext.ux.form.SearchField'
  ],

  alias : 'widget.QuartzJobsGrid',

  cls : 'quartzJobsGrid',

  viewConfig : {
    stripeRows : true
  },

  scrollable : true,

  //border : false,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },


  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
        '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
        '<tpl if="description">{[Ext.htmlEncode(values.description)]}</tpl>' +
        '<tpl if="!description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
        '</p>' +
        '<p style="white-space: nowrap">' +
        '<b>{[QDesk.Msg.get( \"QDesk.monitoringUrl\" )]}:</b> ' +
        '{[this.getMonitoringLink( values.monitoringUrl )]}' +
        '</p>',
        {
          getMonitoringLink : function ( monitoringUrl )
          {
            var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
            var tooltip;
            if ( licensedFeatures.PollMonitoring )
            {
              tooltip =
                  Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipMonitoringUrl' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
            }
            else
            {
              tooltip =
                  Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipMonitoringUrlNotLicensed' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                  tooltip + '">' + monitoringUrl + '</a>';
            }
          }
        }
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var jobDetailStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobDetailStore' );

    jobDetailStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    var schedulerVersion = config.qdesk.schedulerInfo.version;
    var jvmAgentAvailableAndCompatible = config.qdesk.jvmAgentInfo.compatible;

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'triggerJobButton',
          scale : 'small',
          iconCls : 'triggerJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipTriggerJobButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'addJobButton',
          scale : 'small',
          iconCls : 'addJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipAddJobButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'deleteJobButton',
          scale : 'small',
          iconCls : 'deleteJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipDeleteJobButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'editJobButton',
          scale : 'small',
          iconCls : 'editJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipEditJobButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'cloneJobButton',
          scale : 'small',
          iconCls : 'cloneJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipCloneJobButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'xcombo',
          itemId : 'filterByJobGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.fieldLabelFilterByJobGroupCombo' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.emptyTextFilterByJobGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'resumeJobGroupButton',
          scale : 'small',
          iconCls : 'resumeJobGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipResumeJobGroupButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'pauseJobGroupButton',
          scale : 'small',
          iconCls : 'pauseJobGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipPauseJobGroupButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'searchfield',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.emptyTextFieldFilterQueryText' ),
          store : jobDetailStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ]
    } );

    var columns = [];

    columns.push(
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'templatecolumn',
          itemId : 'groupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textGroupNameColumn' ),
          dataIndex : 'group.name',
          tpl : '{group.name}',
          sortable : true,
          groupable : true,
          flex : 0.5
        },
        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textJobClassColumn' ),
          dataIndex : 'jobClass',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipJobDataMapColumn' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var jobDetailRecord = grid.getStore().getAt( rowIndex );

                var jobDataMap = jobDetailRecord.get( 'jobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', jobDataMap );
              }
            }
          ]
        } );


    if ( jvmAgentAvailableAndCompatible )
    {
      columns.push(
          {
            xtype : 'widgetcolumn',
            itemId : 'execHealthIndicatorColumn',
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textExecHealthIndicatorColumn' ),
            dataIndex : 'execHealthIndicatorValues',
            sortable : false,
            groupable : false,
            width : 115,
            widget : {
              xtype : 'ExecHealthIndicator'
            }
          }
      );
    }
    else
    {
      /*
       If the agent is not available, we simply hide the column.

       columns.push(
       {
       itemId : 'execHealthIndicatorColumn',
       text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textExecHealthIndicatorColumn' ),
       sortable : false,
       groupable : false,
       width : 115,
       renderer : function ()
       {
       return '<img src="images/exec-health-indicator-na.png"/>';
       }
       }
       );
       */
    }


    columns.push(
        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'durabilityColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textDurabilityColumn' ),
          align : 'center',
          dataIndex : 'durability',
          sortable : false,
          groupable : false,
          width : 90
        },
        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'shouldRecoverColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textShouldRecoverColumn' ),
          align : 'center',
          dataIndex : 'shouldRecover',
          sortable : false,
          groupable : false,
          width : 90
        },

        // include only for Quartz < 2.x
        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'volatilityColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textVolatilityColumn' ),
          align : 'center',
          dataIndex : 'volatility',
          sortable : false,
          groupable : false,
          width : 90
        }
    );

    if ( jvmAgentAvailableAndCompatible )
    {
      columns.push(
          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'ReadOnlyCheckColumn',
            itemId : 'concurrentColumn',
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textConcurrentColumn' ),
            align : 'center',
            dataIndex : 'concurrentExecutionDisallowed',
            sortable : false,
            groupable : false,
            width : 90,
            renderer : function ( value, metaData )
            {
              // we need to invert the concurrentExecutionDisallowed value
              return this.defaultRenderer( !value, metaData );
            }
          },

          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'ReadOnlyCheckColumn',
            itemId : 'persistJobDataAfterExecutionColumn',
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textPersistsJobDataColumn' ),
            align : 'center',
            dataIndex : 'persistJobDataAfterExecution',
            sortable : false,
            groupable : false,
            width : 120
          }
      );
    }

    Ext.apply( config, {
      columns : columns,
      store : jobDetailStore,
      qdesk : {
        gridId : 'quartz.jobsGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobGroupStore' );

    jobGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByJobGroupCombo' ).store = jobGroupStore;
  },


  /**
   * Invoked to create a context menu for the specified job in the JobsGrid row.
   * @param jobDetail a job.
   * @private
   */
  _createRowCtxMenu : function ( jobDetail )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // trigger job
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.jobCtxMenuTrigger' ),
          iconCls : 'triggerJob',
          handler : function ()
          {
            me.fireEvent( 'triggerJob', jobDetail );
          }
        },
        '-',
        // delete job
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.jobCtxMenuDelete' ),
          iconCls : 'deleteJob',
          handler : function ()
          {
            me.fireEvent( 'deleteJob', jobDetail );
          }
        },
        // edit job
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.jobCtxMenuEdit' ),
          iconCls : 'editJob',
          handler : function ()
          {
            me.fireEvent( 'editJob', jobDetail );
          }
        },
        // clone job
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.jobCtxMenuClone' ),
          iconCls : 'cloneJob',
          handler : function ()
          {
            me.fireEvent( 'cloneJob', jobDetail );
          }
        }
      ]
    } );
  }
} );
