/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.JobsTab', {
  extend : 'QDesk.view.base.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Msg',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.main.scheduler.common.WarningMessagePanel',
    'QDesk.view.main.scheduler.quartz.jobs.JobsGrid',
    'QDesk.view.main.scheduler.quartz.jobs.JobsTabViews'
  ],

  alias : 'widget.QuartzJobsTab',

  iconCls : 'quartzJobsSubTabIcon',

  layout : {
    type : 'border'
  },

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var northItems = [];
    if ( licensedFeatures.MaxJobsPerScheduler != QDesk.util.license.LicensedProductFeatures.UNLIMITED_INT )
    {
      northItems.push( {
        xtype : 'WarningMessagePanel',
        style : {
          margin : '5px'
        },
        qdesk : {
          msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.maxJobsPerScheduler',
              QDesk.util.license.LicensedProduct.productName, licensedFeatures.MaxJobsPerScheduler )
        }
      } );
    }

    northItems.push( {
      xtype : 'QuartzJobsGrid',
      itemId : 'jobsGrid',
      flex : 1,
      qdesk : config.qdesk
    } );


    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsTab.title' ),
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsTab.tooltip' ),
      items : [
        {
          xtype : 'panel',
          flex : 1,
          border : false,
          layout : {
            type : 'vbox',
            align : 'stretch'
          },
          split : true,
          region : 'north',
          items : northItems
        },
        {
          xtype : 'QuartzJobsTabViews',
          itemId : 'views',
          flex : 2,
          region : 'center',
          qdesk : config.qdesk
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );