/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.history.JobExecHistoryView', {
  extend : 'QDesk.view.base.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Msg',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.main.scheduler.common.WarningMessagePanel',
    'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid'
  ],

  alias : 'widget.QuartzJobExecHistoryView',

  iconCls : 'quartzJobExecHistoryTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionHistory )
    {
      if ( me.isJvmAgentAvailableAndCompatible( config.qdesk.jvmAgentInfo ) )
      {
        /*
         * Intentionally over-nested a bit so that we use identical layout to other tabs that
         * have a license restrictions related message panel above the grid.
         */
        Ext.apply( config, {
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : 'fit',
              flex : 1,
              items : [
                {
                  xtype : 'panel',
                  border : false,
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  items : [
                    {
                      xtype : 'QuartzExecHistoryGrid',
                      itemId : 'execHistoryGrid',
                      flex : 1,
                      qdesk : {
                        gridId : 'quartz.jobExecHistoryGrid',
                        maxFetchSize : QDesk.ApplicationConfig.get( 'quartz.jobExecHistoryGrid.maxFetchSize' )
                      }
                    }
                  ]
                }
              ]
            }
          ]
        } );
      }
      else
      {
        Ext.apply( config, {
          items : [
            {
              xtype : 'WarningMessagePanel',
              style : {
                margin : '5px'
              },
              qdesk : {
                msg : me.getJvmAgentWarningMessage( config.qdesk.jvmAgentInfo )
              }
            }
          ]
        } );
      }
    }
    else
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'WarningMessagePanel',
            style : {
              margin : '5px'
            },
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.history.JobExecHistoryView.tooltip' )
    } );

    me.callParent( [ config ] );
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.history.JobExecHistoryView.title' );
    if ( !licensedFeatures.ExecutionHistory )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );