/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.triggers.JobTriggersView', {
  extend : 'QDesk.view.base.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Msg',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.main.scheduler.common.WarningMessagePanel',
    'QDesk.view.main.scheduler.quartz.jobs.triggers.JobTriggersGrid'
  ],

  alias : 'widget.QuartzJobTriggersView',

  iconCls : 'quartzJobTriggersTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var items = [];
    if ( licensedFeatures.MaxTriggersPerJob != QDesk.util.license.LicensedProductFeatures.UNLIMITED_INT )
    {
      items.push( {
        xtype : 'WarningMessagePanel',
        style : {
          margin : '5px'
        },
        qdesk : {
          msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.maxTriggersPerJob',
              QDesk.util.license.LicensedProduct.productName, licensedFeatures.MaxTriggersPerJob )
        }
      } );
    }

    items.push( {
      xtype : 'QuartzJobTriggersGrid',
      itemId : 'jobTriggersGrid',
      qdesk : config.qdesk,
      flex : 1
    } );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.triggers.JobTriggersView.title' ),
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.triggers.JobTriggersView.tooltip' ),
      items : [
        {
          xtype : 'panel',
          border : false,
          layout : 'fit',
          flex : 1,
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : {
                type : 'vbox',
                align : 'stretch'
              },
              items : items
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );