/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.message.MessageChannel',
    'QDesk.enums.NotificationSeverity',
    'QDesk.store.scheduler.quartz.QuartzExecNotificationRuleStore',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.view.widget.ReadOnlyCheckColumn',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzExecNotificationRulesGrid',

  cls : 'execNotificationRulesGrid',

  scrollable : true,

  //border : false,


  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
            '<p>' +
            '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
            '<tpl if="description">{[Ext.htmlEncode(values.description)]}</tpl>' +
            '<tpl if="!description">{[QDesk.Msg.get( \"QDesk.notAvailable\" )]}</tpl>' +
            '</p>'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    var notificationRuleStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzExecNotificationRuleStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'addNotificationRuleButton',
          scale : 'small',
          iconCls : 'addNotificationRule',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipAddExecNotificationRule' ),
          disabled : true,
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'addNotificationRule' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'deleteNotificationRuleButton',
          scale : 'small',
          iconCls : 'deleteNotificationRule',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipDeleteExecNotificationRule' ),
          disabled : true,
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'deleteNotificationRule' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'editNotificationRuleButton',
          scale : 'small',
          iconCls : 'editNotificationRule',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEditExecNotificationRule' ),
          disabled : true,
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'editNotificationRule' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'cloneNotificationRuleButton',
          scale : 'small',
          iconCls : 'cloneNotificationRule',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipCloneExecNotificationRule' ),
          disabled : true,
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'cloneNotificationRule' );
              }
            }
          }
        },
        '-',
        {
          xtype : 'searchfield',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextFieldFilterQueryText' ),
          store : notificationRuleStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ],


      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.3
        },

        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'actioncolumn',
          itemId : 'severityColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSeverityColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSeverityColumn' ),
          sortable : false,
          groupable : false,
          width : 80,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metadata, record )
          {
            var severity = record.get( 'severity' );
            metadata.tdAttr = 'data-qtip="' + QDesk.util.EnumUtils.getNotificationSeverityName( severity ) + '"';
            return value;
          },
          getClass : function ( value, metadata, record )
          {
            var severity = record.get( 'severity' );
            if ( severity == QDesk.enums.NotificationSeverity.INFO )
              return 'notificationSeverityInfo';
            else if ( severity == QDesk.enums.NotificationSeverity.WARNING )
              return 'notificationSeverityWarning';
            else if ( severity == QDesk.enums.NotificationSeverity.ERROR )
              return 'notificationSeverityError';
            else if ( severity == QDesk.enums.NotificationSeverity.FATAL )
              return 'notificationSeverityFatal';
          }
        },

        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'enabledColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledColumn' ),
          dataIndex : 'enabled',
          sortable : false,
          groupable : false,
          width : 100
        },

        {
          itemId : 'enabledOnWeekdaysColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledOnWeekdaysColumn' ),
          dataIndex : 'enabledOnWeekday',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enums value */
          renderer : function ( value, metadata, record )
          {
            var result = '';
            Ext.each( value, function ( weekday, index, weekdays )
            {
              result += QDesk.util.EnumUtils.getWeekdayShortName( weekday );
              if ( index < weekdays.length - 1 )
                result += ', ';
            } );

            return result;
          },
          flex : 1
        },


        {
          itemId : 'enabledFromTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledFromTimeColumn' ),
          dataIndex : 'enabledFromTime',
          sortable : false,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'enabledToTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledToTimeColumn' ),
          dataIndex : 'enabledToTime',
          sortable : false,
          groupable : false,
          flex : 0.5
        },

        {
          itemId : 'conditionTypeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textConditionTypeColumn' ),
          dataIndex : 'conditionType',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enum value */
          renderer : function ( value, metadata, record )
          {
            switch ( value )
            {
              case QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS:
                var conditionExecStatus = record.get( 'conditionExecStatus' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    QDesk.util.EnumUtils.getExecStatusName( conditionExecStatus ) + ']';
                break;

              case QDesk.enums.scheduler.ExecNotificationRuleConditionType.MAX_DURATION:
                var conditionMaxDuration = record.get( 'conditionMaxDuration' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    conditionMaxDuration + 'ms]';
                break;

              case QDesk.enums.scheduler.ExecNotificationRuleConditionType.EXPRESSION:
                var conditionExpressionLanguage = record.get( 'conditionExpressionLanguage' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    QDesk.util.EnumUtils.getExpressionLanguageName( conditionExpressionLanguage ) + ']';
                break;

              default:
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value );
            }
          },
          flex : 0.7
        },

        /* Scheduler related columns */
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : false,
          groupable : false,
          flex : 1
        },

        {
          itemId : 'messageChannelProfileColumn',
          dataIndex : 'messageChannelProfileName',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessageChannelProfileColumn' ),
          sortable : false,
          groupable : false,
          flex : 1,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metadata, record )
          {
            var iconClass, tooltipText;

            if ( value )
            {
              var messageChannel = record.get( 'messageChannelProfileChannel' );
              tooltipText = QDesk.util.EnumUtils.getMessageChannelName( messageChannel );

              if ( messageChannel == QDesk.enums.message.MessageChannel.EMAIL )
                iconClass = 'messageChannelEmail';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_AIM )
                iconClass = 'messageChannelImAim';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_GTALK )
                iconClass = 'messageChannelImGtalk';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_ICQ )
                iconClass = 'messageChannelImIcq';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_SKYPE )
                iconClass = 'messageChannelImSkype';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_XMPP )
                iconClass = 'messageChannelImXmpp';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.IM_YAHOO )
                iconClass = 'messageChannelImYahoo';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.SNMPV1_TRAP )
                iconClass = 'messageChannelSnmpV1Trap';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.SNMPV2C_TRAP )
                iconClass = 'messageChannelSnmpV2cTrap';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.SNMPV3_TRAP )
                iconClass = 'messageChannelSnmpV3Trap';
              else if ( messageChannel == QDesk.enums.message.MessageChannel.WEB_SERVICE )
                iconClass = 'messageChannelWebService';
            }
            else
            {
              value = QDesk.Msg.get( 'QDesk.notAvailable' );
              iconClass = 'messageChannelUnknown';
              var messageChannelProfileId = record.get( 'messageChannelProfileId' );
              tooltipText =
                  QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessageChannelProfileNotFound',
                      messageChannelProfileId );
            }

            metadata.tdAttr = 'data-qtip="' + tooltipText + '"';

            return '<div class="x-action-col-icon ' + iconClass +
                '" style="padding-top: 1px; background-position: 0 1px">' +
                '<span style="padding-left: 20px;">' + value + '</span></div>';
          }
        },

        {
          xtype : 'datecolumn',
          itemId : 'createdAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textCreatedAtColumn' ),
          dataIndex : 'createdAt',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'updatedAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textUpdatedAtColumn' ),
          dataIndex : 'updatedAt',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'actioncolumn',
          itemId : 'messagesColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessagesColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessagesColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/message-queue-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessagesColumn' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var notifRuleId = record.get( 'id' );
                this.fireEvent( 'requestViewMessages', notifRuleId );
              }
            }
          ]
        }
      ],

      store : notificationRuleStore,

      qdesk : {
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [config] );
  },


  /**
   * Invoked to create a context menu for the specified execution notification rule.
   * @param execNotifRule an execution notification rule.
   * @private
   */
  _createRowCtxMenu : function ( execNotifRule )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // enable rule
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuEnable' ),
          iconCls : 'enableExecNotificationRule',
          disabled : execNotifRule.get( 'enabled' ),
          handler : function ()
          {
            me.fireEvent( 'enableRule', execNotifRule );
          }
        },
        // disable rule
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuDisable' ),
          iconCls : 'disableExecNotificationRule',
          disabled : !execNotifRule.get( 'enabled' ),
          handler : function ()
          {
            me.fireEvent( 'disableRule', execNotifRule );
          }
        },
        '-',
        // delete rule
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuDelete' ),
          iconCls : 'deleteExecNotificationRule',
          handler : function ()
          {
            me.fireEvent( 'deleteRule', execNotifRule );
          }
        },
        // edit rule
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuEdit' ),
          iconCls : 'editExecNotificationRule',
          handler : function ()
          {
            me.fireEvent( 'editRule', execNotifRule );
          }
        },
        // clone rule
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuClone' ),
          iconCls : 'cloneExecNotificationRule',
          handler : function ()
          {
            me.fireEvent( 'cloneRule', execNotifRule );
          }
        }
      ]
    } );
  }
} );
