/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.stats.ExecCountChart', {
  extend : 'Ext.chart.CartesianChart',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.scheduler.quartz.SchedulerStatisticsType',
    'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.view.main.scheduler.quartz.stats.theme.ExecCountChartTheme'
  ],

  style : 'background:#fff',

  animate : true,

  border : false,
  bodyBorder : false,

  shadow : true,

  theme : 'QuartzExecCountChartTheme',

  legend : {
    boxStroke : '#444444',
    labelColor : '#444444',
    docked : 'right'
  },

  qdesk : {
    /**
     * Required config parameter with the statistics period length for this chart.
     */
    periodLength : undefined,

    /**
     * Required config parameter with the statistics period start date for this chart.
     */
    periodStart : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.periodLength )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodLength' );

    if ( !config.qdesk.periodStart )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodStart' );

    Ext.apply( config, {
      store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore' ),

      axes : [
        {
          type : 'numeric',
          minimum : 0,
          position : 'left',
          fields : ['countSuccess', 'countVeto', 'countError'],
          title : {
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.stats.ExecCountChart.labelValueAxis' ),
            fontSize : 16
          },
          minorTickSteps : 1,
          grid : {
            odd : {
              globalAlpha : 1,
              fillStyle : '#ddd',
              strokeStyle : '#bbb',
              lineWidth : 0.5
            }
          }
        },

        {
          type : 'category',
          position : 'bottom',
          fields : ['periodChartDisplayName'],
          title : {
            text : QDesk.Msg.get( 'QDesk.charts.labelPeriodAxis.' + config.qdesk.periodLength ),
            fontSize : 16
          }
        }
      ],

      series : {
        type : 'bar',
        axis : 'bottom',
        stacked : true,
        xField : 'periodChartDisplayName',
        yField : ['countSuccess', 'countVeto', 'countError'],
        title : [
          QDesk.util.EnumUtils.getExecStatusName( QDesk.enums.ExecStatus.SUCCESS ),
          QDesk.util.EnumUtils.getExecStatusName( QDesk.enums.ExecStatus.VETO ),
          QDesk.util.EnumUtils.getExecStatusName( QDesk.enums.ExecStatus.ERROR )
        ],
        tooltip : {
          dismissDelay : 0,
          trackMouse : true,
          renderer : function ( tooltip, storeItem, ctx )
          {
            var html = QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.stats.ExecCountChart.tooltipBar',
                QDesk.util.FormatUtils.parseAndFormatTimestamp( storeItem.get( 'period' ).from ),
                QDesk.util.FormatUtils.parseAndFormatTimestamp( storeItem.get( 'period' ).to ),
                Ext.util.Format.number( storeItem.get( 'countSuccess' ), '0,000' ),
                Ext.util.Format.number( storeItem.get( 'timeSuccess' ), '0,000' ),
                Ext.util.Format.number( storeItem.get( 'avgTimeSuccess' ), '0,000.00' ),
                Ext.util.Format.number( storeItem.get( 'countError' ), '0,000' ),
                Ext.util.Format.number( storeItem.get( 'timeError' ), '0,000' ),
                Ext.util.Format.number( storeItem.get( 'avgTimeError' ), '0,000.00' ),
                Ext.util.Format.number( storeItem.get( 'countVeto' ), '0,000' ),
                Ext.util.Format.number( storeItem.get( 'countSuccessError' ), '0,000' ),
                Ext.util.Format.number( storeItem.get( 'avgTimeSuccessError' ), '0,000.00' ) );

            tooltip.setHtml( html );
          }
        }
      }
    } );

    me.callParent( [config] );
  }
} );