/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.stats.ExecTimeChart', {
  extend : 'Ext.chart.CartesianChart',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.scheduler.quartz.SchedulerStatisticsType',
    'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore',
    'QDesk.util.EnumUtils',

    'QDesk.view.main.scheduler.quartz.stats.theme.ExecTimeChartTheme'
  ],

  style : 'background:#fff',

  animate : true,

  border : false,
  bodyBorder : false,

  shadow : true,

  theme : 'QuartzExecTimeChartTheme',

  legend : {
    boxStroke : '#444444',
    labelColor : '#444444',
    docked : 'right'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore' ),

      axes : [
        {
          type : 'numeric',
          minimum : 0,
          position : 'left',
          fields : ['avgTimeSuccess', 'avgTimeError'],
          title : {
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.stats.ExecTimeChart.labelValueAxis' ),
            fontSize : 16
          },
          minorTickSteps : 1,
          grid : {
            odd : {
              globalAlpha : 1,
              fillStyle : '#ddd',
              strokeStyle : '#bbb',
              lineWidth : 0.5
            }
          }
        },

        {
          type : 'category',
          position : 'bottom',
          fields : ['periodChartDisplayName'],
          title : {
            text : QDesk.Msg.get( 'QDesk.charts.labelPeriodAxis.' + config.qdesk.periodLength ),
            fontSize : 16
          }
        }
      ],

      series : [
        {
          type : 'bar',
          title : [
            QDesk.util.EnumUtils.getExecStatusName( QDesk.enums.ExecStatus.SUCCESS ),
            QDesk.util.EnumUtils.getExecStatusName( QDesk.enums.ExecStatus.ERROR )
          ],
          axis : 'left',
          smooth : true,
          xField : 'periodChartDisplayName',
          yField : ['avgTimeSuccess', 'avgTimeError'],

          tooltip : {
            dismissDelay : 0,
            trackMouse : true,
            renderer : function ( tooltip, storeItem, ctx )
            {
              if ( ctx.field == 'avgTimeSuccess' )
              {
                var html = QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.stats.ExecTimeChart.tooltipSuccessBar',
                    QDesk.util.FormatUtils.parseAndFormatTimestamp( storeItem.get( 'period' ).from ),
                    QDesk.util.FormatUtils.parseAndFormatTimestamp( storeItem.get( 'period' ).to ),
                    Ext.util.Format.number( storeItem.get( 'countSuccess' ), '0,000' ),
                    Ext.util.Format.number( storeItem.get( 'timeSuccess' ), '0,000' ),
                    Ext.util.Format.number( storeItem.get( 'avgTimeSuccess' ), '0,000.00' ),
                    Ext.util.Format.number( storeItem.get( 'countSuccessError' ), '0,000' ),
                    Ext.util.Format.number( storeItem.get( 'avgTimeSuccessError' ), '0,000.00' ) );

                tooltip.setHtml( html );
              }
              else if ( ctx.field == 'avgTimeError' )
              {
                var html = QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.stats.ExecTimeChart.tooltipErrorBar',
                    QDesk.util.FormatUtils.parseAndFormatTimestamp( storeItem.get( 'period' ).from ),
                    QDesk.util.FormatUtils.parseAndFormatTimestamp( storeItem.get( 'period' ).to ),
                    Ext.util.Format.number( storeItem.get( 'countError' ), '0,000' ),
                    Ext.util.Format.number( storeItem.get( 'timeError' ), '0,000' ),
                    Ext.util.Format.number( storeItem.get( 'avgTimeError' ), '0,000.00' ),
                    Ext.util.Format.number( storeItem.get( 'countSuccessError' ), '0,000' ),
                    Ext.util.Format.number( storeItem.get( 'avgTimeSuccessError' ), '0,000.00' ) );

                tooltip.setHtml( html );
              }
            }
          }
        }
      ]
    } );

    me.callParent( [config] );
  }
} );



