/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.stats.MostFrequentlyExecutedJobsChart', {
  extend : 'Ext.chart.PolarChart',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.scheduler.quartz.SchedulerStatisticsType',
    'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore',
    'QDesk.util.EnumUtils',

    'QDesk.view.main.scheduler.quartz.stats.theme.MostFrequentlyExecutedJobsChartTheme'
  ],

  style : 'background:#fff',

  animate : true,

  border : false,
  bodyBorder : false,

  innerPadding : 10,
  insetPadding : 90,

  shadow : true,

  theme : 'QuartzMostFrequentlyExecutedJobsChartTheme',

  legend : {
    boxStroke : '#444444',
    labelColor : '#444444',
    docked : 'right'
  },

  qdesk : {
    /**
     * Required config parameter with the statistics period length for this chart.
     */
    periodLength : undefined,

    /**
     * Required config parameter with the statistics period start date for this chart.
     */
    periodStart : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.periodLength )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodLength' );

    if ( !config.qdesk.periodStart )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodStart' );

    Ext.apply( config, {
      store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore' ),

      series : [
        {
          type : 'pie',
          angleField : 'countSuccessError',
          donut : 20,
          tooltip : {
            dismissDelay : 0,
            trackMouse : true,
            renderer : function ( tooltip, storeItem, ctx )
            {
              var html = QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.stats.MostFrequentlyExecutedJobsChart.tooltipPieSection',
                  storeItem.get( 'jobGroupName' ),
                  storeItem.get( 'jobName' ),
                  Ext.util.Format.number( storeItem.get( 'countSuccess' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'timeSuccess' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeSuccess' ), '0,000.00' ),
                  Ext.util.Format.number( storeItem.get( 'countError' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'timeError' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeError' ), '0,000.00' ),
                  Ext.util.Format.number( storeItem.get( 'countVeto' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'countSuccessError' ), '0,000' ) );

              tooltip.setHtml( html );
            }
          },
          highlight : true,
          label : {
            field : 'jobDisplayName',
            calloutLine : {
              length : 80,
              width : 3
            },
            display : 'outside',
            contrast : true
            //font : '15px Verdana'
          }
        }
      ]
    } );

    me.callParent( [config] );
  }
} );