/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm', {
  extend : 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm',

  requires : [
    'QDesk.Msg',

    'QDesk.enums.scheduler.quartz.RepeatIntervalUnit',

    'QDesk.store.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstructionStore',
    'QDesk.store.enums.scheduler.quartz.RepeatIntervalUnitStore',

    'QDesk.util.CommonUtils',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel'
  ],

  alias : 'widget.QuartzEditCalendarIntervalTriggerForm',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    me.callParent( [config] );
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    var repeatIntervalUnitStore = Ext.create( 'QDesk.store.enums.scheduler.quartz.RepeatIntervalUnitStore', {
      qdesk : {
        schedulerVersion : qdesk.schedulerInfo.version
      }
    } );

    var items = [
      {
        xtype : 'combo',
        itemId : 'repeatIntervalUnit',
        name : 'repeatIntervalUnit',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldLabelRepeatIntervalUnit' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldTooltipRepeatIntervalUnit' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldEmptyTxtRepeatIntervalUnit' ),
        allowBlank : false,
        editable : false,
        forceSelection : true,
        queryMode : 'local',
        store : repeatIntervalUnitStore,
        valueField : 'value',
        displayField : 'valueLabel'
      },
      {
        xtype : 'numberfield',
        itemId : 'repeatInterval',
        name : 'repeatInterval',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldLabelRepeatInterval' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldTooltipRepeatInterval' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldEmptyTxtRepeatInterval' ),
        minValue : 0,
        allowBlank : false
      }
    ];

    // version >= 2.1.3
    if ( QDesk.util.CommonUtils.versionCompare( qdesk.schedulerInfo.version,
        { major : 2, minor : 1, maintenance : 3} ) >= 0 )
    {
      var timeZoneStore = Ext.create( 'Ext.data.ArrayStore', {
        model : 'QDesk.model.TimeZone'
      } );

      timeZoneStore.loadData( qdesk.jvmAgentInfo.availableTimeZones );

      items.push(
          {
            xtype : 'combo',
            itemId : 'timeZone',
            name : 'timeZone',
            fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldLabelTimeZone' ),
            fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldTooltipTimeZone' ),
            emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.fieldEmptyTxtTimeZone' ),
            allowBlank : false,
            editable : false,
            forceSelection : true,
            queryMode : 'local',
            store : timeZoneStore,
            valueField : 'id',
            displayField : 'displayName',
            value : qdesk.jvmAgentInfo.timeZone.id
          } );

      items.push(
          {
            xtype : 'checkboxfield',
            itemId : 'preserveHourOfDayAcrossDaylightSavings',
            name : 'preserveHourOfDayAcrossDaylightSavings',
            boxLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.boxLabelPreserveHourOfDayAcrossDaylightSavings' ),
            boxTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.boxTooltipPreserveHourOfDayAcrossDaylightSavings' ),
            labelAlign : 'right',
            allowBlank : true
          }
      );

      items.push(
          {
            xtype : 'checkboxfield',
            itemId : 'skipDayIfHourDoesNotExist',
            name : 'skipDayIfHourDoesNotExist',
            boxLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.boxLabelSkipDayIfHourDoesNotExist' ),
            boxTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm.boxTooltipSkipDayIfHourDoesNotExist' ),
            labelAlign : 'right',
            allowBlank : true
          }
      );
    }

    return items;
  },


  /**
   * @private
   * @override
   */
  loadRecord : function ( trigger )
  {
    var me = this;
    me.callParent( arguments );

    if ( trigger.get( 'timeZone' ) != null )
    {
      me.getForm().setValues( {
        timeZone : trigger.get( 'timeZone' ).id
      } );
    }
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    return Ext.create( 'QDesk.store.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstructionStore', {
      qdesk : {
        schedulerVersion : schedulerVersion
      }
    } );
  },


  /**
   * @private
   * @override
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
    var me = this;
    me.callParent( arguments );

    /* QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction.SMART_POLICY */
    me.down( '#misfireInstruction' ).setValue( 0 );

    me.down( '#repeatIntervalUnit' ).setValue( QDesk.enums.scheduler.quartz.RepeatIntervalUnit.DAY );
    me.down( '#repeatInterval' ).setValue( 1 );

    var preserveHourOfDayAcrossDaylightSavings = me.down( '#preserveHourOfDayAcrossDaylightSavings' );
    if ( preserveHourOfDayAcrossDaylightSavings )
      preserveHourOfDayAcrossDaylightSavings.setValue( false );

    var skipDayIfHourDoesNotExist = me.down( '#skipDayIfHourDoesNotExist' );
    if ( skipDayIfHourDoesNotExist )
      skipDayIfHourDoesNotExist.setValue( false );
  }
} );
