/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.EditCronTriggerForm', {
  extend : 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm',

  requires : [
    'QDesk.Msg',

    'QDesk.store.enums.scheduler.quartz.CronTriggerMisfireInstructionStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel'
  ],

  alias : 'widget.QuartzEditCronTriggerForm',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    me.callParent( [config] );
  },


  /**
   * @private
   * @override
   */
  loadRecord : function ( trigger )
  {
    var me = this;
    me.callParent( arguments );

    if ( trigger.get( 'timeZone' ) != null )
    {
      me.getForm().setValues( {
        timeZone : trigger.get( 'timeZone' ).id
      } );
    }
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    var timeZoneStore = Ext.create( 'Ext.data.ArrayStore', {
      model : 'QDesk.model.TimeZone'
    } );

    timeZoneStore.loadData( qdesk.jvmAgentInfo.availableTimeZones );

    return [
      {
        xtype : 'textfield',
        itemId : 'cronExpression',
        name : 'cronExpression',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCronTriggerForm.fieldLabelCronExpression' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCronTriggerForm.fieldTooltipCronExpression' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCronTriggerForm.fieldEmptyTxtCronExpression' ),
        allowBlank : false
      },

      {
        xtype : 'combo',
        itemId : 'timeZone',
        name : 'timeZone',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCronTriggerForm.fieldLabelTimeZone' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCronTriggerForm.fieldTooltipTimeZone' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditCronTriggerForm.fieldEmptyTxtTimeZone' ),
        allowBlank : false,
        editable : false,
        forceSelection : true,
        queryMode : 'local',
        store : timeZoneStore,
        valueField : 'id',
        displayField : 'displayName',
        value : qdesk.jvmAgentInfo.timeZone.id
      }
    ];
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    return Ext.create( 'QDesk.store.enums.scheduler.quartz.CronTriggerMisfireInstructionStore', {
      qdesk : {
        schedulerVersion : schedulerVersion
      }
    } );
  },


  /**
   * @private
   * @override
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
    var me = this;
    me.callParent( arguments );

    /* QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction.SMART_POLICY */
    me.down( '#misfireInstruction' ).setValue( 0 );
  }
} );

