/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm', {
  extend : 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm',

  requires : [
    'QDesk.Msg',

    'QDesk.enums.scheduler.quartz.RepeatIntervalUnit',

    'QDesk.store.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstructionStore',
    'QDesk.store.enums.scheduler.quartz.RepeatIntervalUnitStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel'
  ],

  alias : 'widget.QuartzEditDailyTimeIntervalTriggerForm',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    me.callParent( [ config ] );
  },


  /**
   * @private
   * @override
   */
  loadRecord : function ( trigger )
  {
    var me = this;
    me.callParent( arguments );

    var repeatCount = trigger.get( 'repeatCount' );
    me.down( '#repeatCountIndefinitely' ).setValue( repeatCount == -1 );
    me.down( '#repeatCount' ).setValue( repeatCount == -1 ? null : repeatCount );

    var daysOfWeek = trigger.get( 'daysOfWeek' );
    Ext.each( Ext.ComponentQuery.query( '#daysOfWeek checkbox', me ), function ( checkbox )
    {
      checkbox.setValue( Ext.Array.contains( daysOfWeek, checkbox.qdesk.value ) );
    } );
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    var repeatIntervalUnitStore = Ext.create( 'QDesk.store.enums.scheduler.quartz.RepeatIntervalUnitStore', {
      qdesk : {
        schedulerVersion : qdesk.schedulerInfo.version
      }
    } );

    var startAndEndTimeOfDayPanel =
    {
      xtype : 'panel',
      border : false,
      layout : 'column',
      style : {
        marginBottom : '5px'
      },
      items : [
        {
          xtype : 'timefield',
          itemId : 'startTimeOfDay',
          name : 'startTimeOfDay',
          labelWidth : 135,
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldLabelStartTimeOfDay' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldTooltipStartTimeOfDay' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldEmptyTxtStartTimeOfDay' ),
          allowBlank : false,
          format : QDesk.util.constant.DateFormat.dayTimeFormat,
          margin : '0 0 5 0'
        },
        {
          xtype : 'timefield',
          itemId : 'endTimeOfDay',
          name : 'endTimeOfDay',
          labelWidth : 135,
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldLabelEndTimeOfDay' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldTooltipEndTimeOfDay' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldEmptyTxtEndTimeOfDay' ),
          allowBlank : false,
          format : QDesk.util.constant.DateFormat.dayTimeFormat
        }
      ]
    };

    var daysOfWeekPanel =
    {
      xtype : 'panel',
      border : false,
      height : 50,
      layout : {
        type : 'hbox',
        pack : 'start',
        align : 'stretch'
      },
      items : [
        {
          xtype : 'checkboxgroup',
          style : {
            paddingLeft : '10px',
            backgroundColor : '#E6F7E6'
          },
          flex : 2,

          vertical : true,
          columns : 3,
          items : [
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.MONDAY ),
              name : 'dayOfWeekMonday',
              itemId : 'dayOfWeekMonday',
              qdesk : {
                value : QDesk.enums.Weekday.MONDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.TUESDAY ),
              name : 'dayOfWeekTuesday',
              itemId : 'dayOfWeekTuesday',
              qdesk : {
                value : QDesk.enums.Weekday.TUESDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.WEDNESDAY ),
              name : 'dayOfWeekWednesday',
              itemId : 'dayOfWeekWednesday',
              qdesk : {
                value : QDesk.enums.Weekday.WEDNESDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.THURSDAY ),
              name : 'dayOfWeekThursday',
              itemId : 'dayOfWeekThursday',
              qdesk : {
                value : QDesk.enums.Weekday.THURSDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.FRIDAY ),
              name : 'dayOfWeekFriday',
              itemId : 'dayOfWeekFriday',
              qdesk : {
                value : QDesk.enums.Weekday.FRIDAY
              }
            }
          ]
        },
        {
          xtype : 'checkboxgroup',
          style : {
            paddingLeft : '10px',
            backgroundColor : '#F7E6E6'
          },
          flex : 1,
          vertical : true,
          columns : 1,
          items : [
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.SATURDAY ),
              name : 'dayOfWeekSaturday',
              itemId : 'dayOfWeekSaturday',
              qdesk : {
                value : QDesk.enums.Weekday.SATURDAY
              }
            },
            {
              boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.enums.Weekday.SUNDAY ),
              name : 'dayOfWeekSunday',
              itemId : 'dayOfWeekSunday',
              qdesk : {
                value : QDesk.enums.Weekday.SUNDAY
              }
            }
          ]
        }
      ]
    };

    return [
      {
        xtype : 'combo',
        itemId : 'repeatIntervalUnit',
        name : 'repeatIntervalUnit',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldLabelRepeatIntervalUnit' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldTooltipRepeatIntervalUnit' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldEmptyTxtRepeatIntervalUnit' ),
        allowBlank : false,
        editable : false,
        forceSelection : true,
        queryMode : 'local',
        store : repeatIntervalUnitStore,
        valueField : 'value',
        displayField : 'valueLabel'
      },
      {
        xtype : 'numberfield',
        itemId : 'repeatInterval',
        name : 'repeatInterval',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldLabelRepeatInterval' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldTooltipRepeatInterval' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldEmptyTxtRepeatInterval' ),
        minValue : 0,
        allowBlank : false
      },
      {
        xtype : 'checkboxfield',
        itemId : 'repeatCountIndefinitely',
        name : 'repeatCountIndefinitely',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldLabelRepeatCountIndefinitely' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldTooltipRepeatCountIndefinitely' ),
        allowBlank : true
      },
      {
        xtype : 'numberfield',
        itemId : 'repeatCount',
        name : 'repeatCount',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldLabelRepeatCount' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldEmptyTxtRepeatCount' ),
        minValue : 0,
        allowBlank : false
      },

      startAndEndTimeOfDayPanel,

      {
        xtype : 'fieldset',
        title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldsetLabelDaysOfWeek' ),
        tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm.fieldsetTooltipDaysOfWeek' ),
        itemId : 'daysOfWeek',
        defaults : {
          labelWidth : 100,
          anchor : '100%'
        },
        padding : '0 10px 7px 10px',
        items : [
          daysOfWeekPanel
        ]
      }
    ];
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    return Ext.create( 'QDesk.store.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstructionStore', {
      qdesk : {
        schedulerVersion : schedulerVersion
      }
    } );
  },


  /**
   * @private
   * @override
   */
  _bindTriggerSpecificEventHandlers : function ()
  {
    var me = this;
    me.callParent( arguments );

    me.down( '#repeatCountIndefinitely' ).on( 'change', function ( combo, newValue )
    {
      // if the repeatCountIndefinitely checkbox is checked, then disable the repeatCountField field
      var repeatCountField = me.down( '#repeatCount' );
      repeatCountField.setValue( null );

      repeatCountField.setDisabled( newValue );
      repeatCountField.allowBlank = newValue;
      repeatCountField.clearInvalid();
      repeatCountField.validate();
    } );
  },


  /**
   * @private
   * @override
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
    var me = this;
    me.callParent( arguments );

    /* QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction.SMART_POLICY */
    me.down( '#misfireInstruction' ).setValue( 0 );

    me.down( '#repeatIntervalUnit' ).setValue( QDesk.enums.scheduler.quartz.RepeatIntervalUnit.MINUTE );
    me.down( '#repeatInterval' ).setValue( 1 );

    me.down( '#repeatCountIndefinitely' ).setValue( true );

    Ext.each( Ext.ComponentQuery.query( '#daysOfWeek checkbox', me ), function ( checkbox )
    {
      checkbox.setValue( true );
    } );

    me.down( '#startTimeOfDay' ).setValue( '00:00:00' );
    me.down( '#endTimeOfDay' ).setValue( '23:59:59' );
  }
} );
