/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.EditDateIntervalTriggerForm', {
  extend : 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm',

  requires : [
    'QDesk.Msg',

    'QDesk.enums.scheduler.quartz.RepeatIntervalUnit',

    'QDesk.store.enums.scheduler.quartz.DateIntervalTriggerMisfireInstructionStore',
    'QDesk.store.enums.scheduler.quartz.RepeatIntervalUnitStore',

    'QDesk.util.CommonUtils',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel'
  ],

  alias : 'widget.QuartzEditDateIntervalTriggerForm',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    me.callParent( [config] );
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    var repeatIntervalUnitStore = Ext.create( 'QDesk.store.enums.scheduler.quartz.RepeatIntervalUnitStore', {
      qdesk : {
        schedulerVersion : qdesk.schedulerInfo.version
      }
    } );

    return [
      {
        xtype : 'combo',
        itemId : 'repeatIntervalUnit',
        name : 'repeatIntervalUnit',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldLabelRepeatIntervalUnit' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldTooltipRepeatIntervalUnit' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldEmptyTxtRepeatIntervalUnit' ),
        allowBlank : false,
        editable : false,
        forceSelection : true,
        queryMode : 'local',
        store : repeatIntervalUnitStore,
        valueField : 'value',
        displayField : 'valueLabel'
      },
      {
        xtype : 'numberfield',
        itemId : 'repeatInterval',
        name : 'repeatInterval',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldLabelRepeatInterval' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldTooltipRepeatInterval' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldEmptyTxtRepeatInterval' ),
        minValue : 0,
        allowBlank : false
      }
    ];
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    return Ext.create( 'QDesk.store.enums.scheduler.quartz.DateIntervalTriggerMisfireInstructionStore', {
      qdesk : {
        schedulerVersion : schedulerVersion
      }
    } );
  },


  /**
   * @private
   * @override
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
    var me = this;
    me.callParent( arguments );

    /* QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction.SMART_POLICY */
    me.down( '#misfireInstruction' ).setValue( 0 );

    me.down( '#repeatIntervalUnit' ).setValue( QDesk.enums.scheduler.quartz.RepeatIntervalUnit.DAY );
    me.down( '#repeatInterval' ).setValue( 1 );
  }
} );
