/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm', {
  extend : 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm',

  requires : [
    'QDesk.Msg',

    'QDesk.store.enums.scheduler.quartz.SimpleTriggerMisfireInstructionStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel',

    'QDesk.view.main.scheduler.quartz.JobDataMapGrid',

    'Ext.ux.form.field.DateTime'
  ],

  alias : 'widget.QuartzEditSimpleTriggerForm',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    me.callParent( [config] );
  },


  /**
   * @private
   * @override
   */
  loadRecord : function ( trigger )
  {
    var me = this;
    me.callParent( arguments );

    var repeatCount = trigger.get( 'repeatCount' );
    me.down( '#repeatCountIndefinitely' ).setValue( repeatCount == -1 );
    me.down( '#repeatCount' ).setValue( repeatCount == -1 ? null : repeatCount );
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    return [
      {
        xtype : 'numberfield',
        itemId : 'repeatInterval',
        name : 'repeatInterval',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm.fieldLabelRepeatInterval' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm.fieldTooltipRepeatInterval' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm.fieldEmptyTxtRepeatInterval' ),
        minValue : 0,
        allowBlank : false
      },

      {
        xtype : 'checkboxfield',
        itemId : 'repeatCountIndefinitely',
        name : 'repeatCountIndefinitely',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm.fieldLabelRepeatCountIndefinitely' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm.fieldTooltipRepeatCountIndefinitely' ),
        allowBlank : true
      },
      {
        xtype : 'numberfield',
        itemId : 'repeatCount',
        name : 'repeatCount',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm.fieldLabelRepeatCount' ),
        fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm.fieldTooltipRepeatCount' ),
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm.fieldEmptyTxtRepeatCount' ),
        minValue : 0,
        allowBlank : false
      }
    ];
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    return Ext.create( 'QDesk.store.enums.scheduler.quartz.SimpleTriggerMisfireInstructionStore', {
      qdesk : {
        schedulerVersion : schedulerVersion
      }
    } );
  },


  /**
   * @private
   * @override
   */
  _bindTriggerSpecificEventHandlers : function ()
  {
    var me = this;
    me.callParent( arguments );

    me.down( '#repeatCountIndefinitely' ).on( 'change', function ( combo, newValue )
    {
      // if the repeatCountIndefinitely checkbox is checked, then disable the repeatCountField field
      var repeatCountField = me.down( '#repeatCount' );
      repeatCountField.setValue( null );

      repeatCountField.setDisabled( newValue );
      repeatCountField.allowBlank = newValue;
      repeatCountField.clearInvalid();
      repeatCountField.validate();
    } );
  },


  /**
   * @private
   * @override
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
    var me = this;
    me.callParent( arguments );

    /* QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.SMART_POLICY */
    me.down( '#misfireInstruction' ).setValue( 0 );

    me.down( '#repeatCountIndefinitely' ).setValue( true );
  }
} );
