/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',

    'QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode',

    'QDesk.view.main.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm',
    'QDesk.view.main.scheduler.quartz.triggers.EditCronTriggerForm',
    'QDesk.view.main.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm',
    'QDesk.view.main.scheduler.quartz.triggers.EditDateIntervalTriggerForm',
    'QDesk.view.main.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm',
    'QDesk.view.main.scheduler.quartz.triggers.EditSimpleTriggerForm'
  ],

  alias : 'widget.QuartzEditTriggerWindow',

  cls : 'quartzEditTriggerWindow',
  iconCls : 'quartzEditTriggerWindowIcon',

  modal : true,
  resizable : false,

//  minWidth : 500,
  width : 600,

  qdesk : {
    /*
     * Scheduler connection ID.
     */
    connectionId : undefined,

    /*
     * Mode of the form (see EditTriggerFormMode).
     */
    mode : undefined,

    /*
     * Trigger type (see QDesk.enums.scheduler.quartz.TriggerType).
     */
    triggerType : undefined,

    /*
     * Trigger job group name.
     */
    triggerJobGroupName : undefined,

    /*
     * Trigger job name.
     */
    triggerJobName : undefined,

    /*
     * QuartzDesk JVM Agent info object (see QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo).
     */
    jvmAgentInfo : undefined,

    /*
     * Scheduler info object (see QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo).
     */
    schedulerInfo : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.mode ),
      items : [
        {
          // form xtype depends on the requested trigger type
          xtype : me._getFormXType( config.qdesk.triggerType ),
          itemId : 'form',

          qdesk : {
            connectionId : config.qdesk.connectionId,
            mode : config.qdesk.mode,
            triggerType : config.qdesk.triggerType,
            triggerJobGroupName : config.qdesk.triggerJobGroupName,
            triggerJobName : config.qdesk.triggerJobName,
            jvmAgentInfo : config.qdesk.jvmAgentInfo,
            schedulerInfo : config.qdesk.schedulerInfo
          }
        }
      ]
    } );

    me.callParent( [config] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } )
  },


  /**
   * Returns the title of this window.
   * @param mode a QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode value.
   * @return {string} the title.
   * @private
   */
  _getWindowTitle : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerWindow.windowTitleAdd' );

      case QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerWindow.windowTitleEdit' );

      case QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerWindow.windowTitleClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  },


  /**
   * Returns the XType of the edit form for the specified trigger type.
   * @param triggerType a trigger type.
   * @returns {string} the XType.
   * @private
   */
  _getFormXType : function ( triggerType )
  {
    switch ( triggerType )
    {
      case QDesk.enums.scheduler.quartz.TriggerType.SIMPLE:
        return 'QuartzEditSimpleTriggerForm';

      case QDesk.enums.scheduler.quartz.TriggerType.CRON:
        return 'QuartzEditCronTriggerForm';

      case QDesk.enums.scheduler.quartz.TriggerType.CALENDAR_INTERVAL:
        return 'QuartzEditCalendarIntervalTriggerForm';

      case QDesk.enums.scheduler.quartz.TriggerType.DAILY_TIME_INTERVAL:
        return 'QuartzEditDailyTimeIntervalTriggerForm';

      case QDesk.enums.scheduler.quartz.TriggerType.DATE_INTERVAL:
        return 'QuartzEditDateIntervalTriggerForm';

      case QDesk.enums.scheduler.quartz.TriggerType.NTH_INCLUDED_DAY:
        return 'QuartzEditNthIncludedDayTriggerForm';

      default:
        Ext.Error.raise( 'Unrecognized trigger type: ' + triggerType );
        return null;
    }
  }
} );
