/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.enums.scheduler.quartz.TriggerType',

    'QDesk.util.FormatUtils',

    'QDesk.store.scheduler.quartz.QuartzTriggerGroupStore',
    'QDesk.store.scheduler.quartz.QuartzTriggerWithJobDetailStore',

    'QDesk.view.widget.ExecHealthIndicator',
    'QDesk.view.widget.ReadOnlyCheckColumn',

    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.quartz.QuartzEnumUtils',
    'QDesk.util.quartz.QuartzTriggerUtils',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo',
    'Ext.ux.form.SearchField'
  ],

  alias : 'widget.QuartzTriggersGrid',

  cls : 'quartzTriggersGrid',

  scrollable : true,

  //border : false,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var triggersStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzTriggerWithJobDetailStore' );

    triggersStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    var schedulerVersion = config.qdesk.schedulerInfo.version;
    var jvmAgentAvailableAndCompatible = config.qdesk.jvmAgentInfo.compatible;

    Ext.apply( config, {
      listeners : {
        itemcontextmenu : function ( grid, record, item, index, event )
        {
          var me = this;
          event.stopEvent();
          var rowCtxMenu = me._createRowCtxMenu( record, jvmAgentAvailableAndCompatible );
          rowCtxMenu.showAt( event.getXY() );
        }
      }
    } );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'resumeTriggerButton',
          scale : 'small',
          iconCls : 'resumeTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipResumeTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerButton',
          scale : 'small',
          iconCls : 'pauseTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipPauseTriggerButton' ),
          disabled : true
        },
        '-',

        {
          xtype : 'button',
          itemId : 'addTriggerButton',
          scale : 'small',
          iconCls : 'addTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipAddTriggerButton' ),
          menu : {
            items : QDesk.util.quartz.QuartzTriggerUtils.createAddTriggerButtonMenuItems( schedulerVersion )
          },
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'deleteTriggerButton',
          scale : 'small',
          iconCls : 'deleteTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipDeleteTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'editTriggerButton',
          scale : 'small',
          iconCls : 'editTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipEditTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'cloneTriggerButton',
          scale : 'small',
          iconCls : 'cloneTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipCloneTriggerButton' ),
          disabled : true
        },
        '-',

        {
          xtype : 'xcombo',
          itemId : 'filterByTriggerGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.filterByJobGroupComboLabel' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.emptyTextFilterByTriggerGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'resumeTriggerGroupButton',
          scale : 'small',
          iconCls : 'resumeTriggerGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipResumeTriggerGroupButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerGroupButton',
          scale : 'small',
          iconCls : 'pauseTriggerGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipPauseTriggerGroupButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'searchfield',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.emptyTextFieldFilterQueryText' ),
          store : triggersStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ]
    } );

    var columns = [];

    columns.push(
        {
          xtype : 'templatecolumn',
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerNameColumn' ),
          dataIndex : 'trigger.name',
          tpl : '{trigger.name}',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerGroupNameColumn' ),
          dataIndex : 'trigger.group.name',
          tpl : '{trigger.group.name}',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerTypeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerTypeColumn' ),
          dataIndex : 'trigger.type',
          tpl : '{[Ext.isEmpty( values.trigger.type ) ? QDesk.Msg.get("QDesk.unknown") : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( values.trigger.type )]}',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerStateColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerStateColumn' ),
          dataIndex : 'trigger.state',
          tpl : '{trigger.state}',
          sortable : true,
          groupable : true,
          flex : 0.5
        }
    );

    if ( jvmAgentAvailableAndCompatible )
    {
      columns.push(
          {
            xtype : 'widgetcolumn',
            itemId : 'triggerExecHealthIndicatorColumn',
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerExecHealthIndicatorColumn' ),
            dataIndex : 'trigger.execHealthIndicatorValues',
            sortable : false,
            groupable : false,
            width : 115,
            widget : {
              xtype : 'ExecHealthIndicator'
            },
            onWidgetAttach : function ( column, widget, record )
            {
              var trigger = record.get( 'trigger' );
              if ( trigger.execHealthIndicatorValues )
              {
                widget.setValues( trigger.execHealthIndicatorValues );
              }
            }
          }
      );
    }
    else
    {
      /*
       If the agent is not available, we simply hide the column.

       columns.push(
       {
       itemId : 'triggerExecHealthIndicatorColumn',
       text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerExecHealthIndicatorColumn' ),
       dataIndex : 'trigger.execHealthIndicatorValues',
       sortable : false,
       groupable : false,
       width : 115,
       renderer : function ()
       {
       return '<img src="images/exec-health-indicator-na.png"/>';
       }
       }
       );
       */
    }

    columns.push(
        {
          xtype : 'actioncolumn',
          itemId : 'triggerJobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerJobDataMapColumn' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var triggerWithJobDetailRecord = grid.getStore().getAt( rowIndex );
                var trigger = triggerWithJobDetailRecord.get( 'trigger' );

                var jobDataMap = trigger.jobDataMap;
                var effectiveJobDataMap = trigger.effectiveJobDataMap;
                this.fireEvent( 'requestViewJobDataMap', jobDataMap, effectiveJobDataMap );
              }
            }
          ]
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerStartTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerStartTimeColumn' ),
          dataIndex : 'trigger.startTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.startTime ) ]}',
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerEndTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerEndTimeColumn' ),
          dataIndex : 'trigger.endTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.endTime ) ]}',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerPreviousFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerPreviousFireTimeColumn' ),
          dataIndex : 'trigger.previousFireTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.previousFireTime ) ]}',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerNextFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerNextFireTimeColumn' ),
          dataIndex : 'trigger.nextFireTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.nextFireTime ) ]}',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerFinalFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerFinalFireTimeColumn' ),
          dataIndex : 'trigger.finalFireTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.finalFireTime ) ]}',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerCalendarNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerCalendarNameColumn' ),
          dataIndex : 'trigger.calendar',
          tpl : '{trigger.calendar.displayName}',
          sortable : false,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerPriorityColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerPriorityColumn' ),
          dataIndex : 'trigger.priority',
          tpl : '{trigger.priority}',
          sortable : false,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'triggerMisfireInstructionColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerMisfireInstructionColumn' ),
          dataIndex : 'trigger.misfireInstruction',
          sortable : false,
          groupable : false,
          /* Need custom renderer to display misfire instruction name */
          renderer : function ( value, metadata, record )
          {
            var trigger = record.get( 'trigger' );

            if ( trigger.type )
            {
              var misfireInstruction =
                  QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionFromNumber( trigger.type,
                      trigger.misfireInstruction );

              return QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionName( trigger.type,
                  misfireInstruction );
            }
            else
            {
              // if we do not know the trigger type, then we are not able to determine the misfire instruction name;
              // so we display the numeric constant instead
              return QDesk.Msg.get( 'QDesk.unknown' ) + ' (' + trigger.misfireInstruction.toString() + ')';
            }
          },
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobNameColumn' ),
          dataIndex : 'jobDetail.name',
          tpl : '{jobDetail.name}',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobGroupNameColumn' ),
          dataIndex : 'jobDetail.group.name',
          tpl : '{jobDetail.group.name}',
          sortable : true,
          groupable : true,
          flex : 1
        }
    );


    if ( jvmAgentAvailableAndCompatible )
    {
      columns.push(
          {
            xtype : 'widgetcolumn',
            itemId : 'jobExecHealthIndicatorColumn',
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobExecHealthIndicatorColumn' ),
            dataIndex : 'jobDetail.execHealthIndicatorValues',
            sortable : false,
            groupable : false,
            width : 115,
            widget : {
              xtype : 'ExecHealthIndicator'
            }
          }
      );
    }
    else
    {
      /*
       If the agent is not available, we simply hide the column.

       columns.push(
       {
       itemId : 'jobExecHealthIndicatorColumn',
       text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobExecHealthIndicatorColumn' ),
       dataIndex : 'jobDetail.execHealthIndicatorValues',
       sortable : false,
       groupable : false,
       width : 115,
       renderer : function ()
       {
       return '<img src="images/exec-health-indicator-na.png"/>';
       }
       }
       );
       */
    }


    columns.push(
        {
          xtype : 'templatecolumn',
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobClassColumn' ),
          dataIndex : 'jobDetail.jobClass',
          tpl : '{jobDetail.jobClass}',
          sortable : true,
          groupable : true,
          flex : 1
        },

        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'jobDurabilityColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobDurabilityColumn' ),
          dataIndex : 'jobDetail.durability',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.durability, metaData );  // does not work for nested properties - we use renderer
          }
        },
        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'jobShouldRecoverColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobShouldRecoverColumn' ),
          dataIndex : 'jobDetail.shouldRecover',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.shouldRecover, metaData );  // does not work for nested properties - we use renderer
          }
        },
        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'jobVolatilityColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobVolatilityColumn' ),
          dataIndex : 'jobDetail.volatility',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.volatility, metaData );  // does not work for nested properties - we use renderer
          }
        }
    );

    if ( jvmAgentAvailableAndCompatible )
    {
      columns.push(
          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'ReadOnlyCheckColumn',
            itemId : 'jobConcurrentColumn',
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobConcurrentColumn' ),
            align : 'center',
            dataIndex : 'jobDetail.concurrentExecutionDisallowed',  // does not work for nested properties - we use renderer
            sortable : false,
            groupable : false,
            width : 110,
            renderer : function ( value, metaData )
            {
              var jobDetail = metaData.record.get( 'jobDetail' );
              // we need to invert the concurrentExecutionDisallowed value
              return this.defaultRenderer( !jobDetail.concurrentExecutionDisallowed, metaData );
            }
          },

          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'ReadOnlyCheckColumn',
            itemId : 'jobPersistJobDataAfterExecutionColumn',
            text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobPersistsJobDataColumn' ),
            align : 'center',
            dataIndex : 'jobDetail.persistJobDataAfterExecution',  // does not work for nested properties - we use renderer
            sortable : false,
            groupable : false,
            width : 140,
            renderer : function ( value, metaData )
            {
              var jobDetail = metaData.record.get( 'jobDetail' );
              return this.defaultRenderer( jobDetail.persistJobDataAfterExecution, metaData );
            }
          }
      );
    }


    Ext.apply( config, {
      plugins : [
        {
          ptype : 'rowexpander',
          rowBodyTpl : [
            '<p>' +
            '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
            '<tpl if="trigger.description">{[Ext.htmlEncode(values.trigger.description)]}</tpl>' +
            '<tpl if="!trigger.description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
            '</p>' +

            '{[QDesk.util.quartz.QuartzTriggerUtils.createTriggerPropertiesTable( values.trigger )]}' +

            '<p style="white-space: nowrap">' +
            '<b>{[QDesk.Msg.get( \"QDesk.monitoringUrl\" )]}:</b> ' +
            '{[this.getMonitoringLink( values.trigger.monitoringUrl )]}' +
            '</p>',
            {
              getMonitoringLink : function ( monitoringUrl )
              {
                var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
                var tooltip;
                if ( licensedFeatures.PollMonitoring )
                {
                  tooltip =
                      Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipMonitoringUrl' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
                }
                else
                {
                  tooltip =
                      Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipMonitoringUrlNotLicensed' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                      tooltip + '">' + monitoringUrl + '</a>';
                }
              }
            }
          ]
        }
      ]
    } );

    Ext.apply( config, {
      columns : columns,
      store : triggersStore,

      qdesk : {
        gridId : 'quartz.triggersGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var triggerGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzTriggerGroupStore' );

    triggerGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByTriggerGroupCombo' ).store = triggerGroupStore;
  },


  /**
   * Invoked to create a context menu for the specified trigger in the TriggersGrid row.
   * @param triggerWithJobDetail a trigger with a job detail.
   * @param jvmAgentAvailableAndCompatible a flag indicating if the JVM agent is available and compatible.
   * @private
   */
  _createRowCtxMenu : function ( triggerWithJobDetail, jvmAgentAvailableAndCompatible )
  {
    var me = this;

    var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // resume trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuResume' ),
          iconCls : 'resumeTrigger',
          disabled : trigger.get( 'state' ) != QDesk.enums.scheduler.quartz.TriggerState.PAUSED,
          handler : function ()
          {
            me.fireEvent( 'resumeTrigger', trigger );
          }
        },
        // pause trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuPause' ),
          iconCls : 'pauseTrigger',
          disabled : trigger.get( 'state' ) != QDesk.enums.scheduler.quartz.TriggerState.NORMAL,
          handler : function ()
          {
            me.fireEvent( 'pauseTrigger', trigger );
          }
        },
        '-',
        // delete trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuDelete' ),
          iconCls : 'deleteTrigger',
          handler : function ()
          {
            me.fireEvent( 'deleteTrigger', trigger );
          }
        },
        // edit trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuEdit' ),
          iconCls : 'editTrigger',
          disabled : !jvmAgentAvailableAndCompatible,
          handler : function ()
          {
            me.fireEvent( 'editTrigger', trigger );
          }
        },
        // clone trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuClone' ),
          iconCls : 'cloneTrigger',
          disabled : !jvmAgentAvailableAndCompatible,
          handler : function ()
          {
            me.fireEvent( 'cloneTrigger', trigger );
          }
        }
      ]
    } );
  }
} );
