/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.enums.scheduler.quartz.TriggerStatisticsType',
    'QDesk.model.FixedTimePeriod',
    'QDesk.store.scheduler.quartz.stats.QuartzStatisticsTimePeriodLengthStore',
    'QDesk.store.scheduler.quartz.stats.QuartzTriggerStatisticsTimePeriodStore',
    'QDesk.store.scheduler.quartz.stats.QuartzTriggerStatisticsTypeStore',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.main.scheduler.common.WarningMessagePanel'
  ],

  alias : 'widget.QuartzTriggerStatisticsPanel',

  //border : false,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  qdesk : {
    /**
     * Required configuration parameter containing the scheduler connection ID.
     */
    connectionId : undefined
  },

  items : [
    {
      xtype : 'WarningMessagePanel',
      itemId : 'warningMessagePanel',
      hidden : true,
      style : {
        margin : '5px'
      }
    },
    {
      xtype : 'panel',
      itemId : 'chartDescription',
      layout : 'fit',
      border : 0,
      cls : 'chartDescription',
      hidden : true
    },
    {
      xtype : 'panel',
      itemId : 'chartContainer',
      layout : 'fit',
      border : 0,
      flex : 1
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'previewChartButton',
          scale : 'small',
          iconCls : 'previewChart',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.tooltipPreviewChartButton' ),
          disabled : true
        },
        //{
        //  xtype : 'button',
        //  itemId : 'saveChartButton',
        //  scale : 'small',
        //  iconCls : 'save',
        //  tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.tooltipSaveChartButton' ),
        //  menu : {
        //    items : [
        //      {
        //        itemId : 'saveChartAsJpeg',
        //        iconCls : 'saveChartAsJpeg',
        //        text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.saveChartButtonMenuAsJpeg' )
        //      },
        //      {
        //        itemId : 'saveChartAsPng',
        //        iconCls : 'saveChartAsPng',
        //        text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.saveChartButtonMenuAsPng' )
        //      },
        //      {
        //        itemId : 'saveChartAsSvg',
        //        iconCls : 'saveChartAsSvg',
        //        text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.saveChartButtonMenuAsSvg' )
        //      }
        //    ]
        //  },
        //  disabled : true
        //},
        {
          xtype : 'combo',
          itemId : 'statisticsTypeSelect',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.fieldLabelStatisticsTypeSelect' ),
          store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzTriggerStatisticsTypeStore' ),
          value : QDesk.enums.scheduler.quartz.TriggerStatisticsType.TRIGGER_EXEC_TIME,
          displayField : 'valueLabel',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 50,
          width : 280,
          editable : false
          //disabled : true
        },
        {
          xtype : 'combo',
          itemId : 'periodLengthSelect',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.fieldLabelPeriodLengthSelect' ),
          store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzStatisticsTimePeriodLengthStore' ),
          displayField : 'valueLabel',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 40,
          width : 120,
          editable : false
          //disabled : true
        },
        {
          xtype : 'panel',
          border : false,
          html : '&ndash;',
          bodyStyle : {
            backgroundColor : 'transparent'
          }
        },

        // DAY
        {
          xtype : 'datefield',
          itemId : 'dayPeriodSelect',
          editable : false,
          format : QDesk.util.constant.DateFormat.dateFormat,
          width : 100,
          hidden : true
        },

        // WEEK
        {
          xtype : 'combo',
          itemId : 'weekPeriodSelect',
          store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzTriggerStatisticsTimePeriodStore' ),
          displayField : 'displayName',
          valueField : 'from',
          labelAlign : 'right',
          width : 290,
          editable : false,
          forceSelection : true,
          hidden : true
        },

        // MONTH
        {
          xtype : 'combo',
          itemId : 'monthPeriodSelect',
          store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzTriggerStatisticsTimePeriodStore' ),
          displayField : 'displayName',
          valueField : 'from',
          labelAlign : 'right',
          width : 120,
          editable : false,
          forceSelection : true,
          hidden : true
        },

        // QUARTER
        {
          xtype : 'combo',
          itemId : 'quarterPeriodSelect',
          store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzTriggerStatisticsTimePeriodStore' ),
          displayField : 'displayName',
          valueField : 'from',
          labelAlign : 'right',
          width : 80,
          editable : false,
          forceSelection : true,
          hidden : true
        },

        // YEAR
        {
          xtype : 'combo',
          itemId : 'yearPeriodSelect',
          store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzTriggerStatisticsTimePeriodStore' ),
          displayField : 'displayName',
          valueField : 'from',
          labelAlign : 'right',
          width : 60,
          editable : false,
          forceSelection : true,
          hidden : true
        }
      ]
    } );

    me.callParent( [config] );
  },


  initComponent : function ()
  {
    var me = this;
    me.callParent();

    /*
     * Bind the event handlers lazily so that the period change event
     * that triggers is not fired before the panel is rendered.
     */
    me.on( 'beforerender', function ()
    {
      me._bindEventHandlers();
    } );
  },


  /**
   * Returns the currently selected statistics type.
   * @return the currently selected statistics type.
   */
  getStatisticsType : function ()
  {
    var me = this;
    return me.down( '#statisticsTypeSelect' ).getValue();
  },


  getPeriodLength : function ()
  {
    var me = this;
    return me.down( '#periodLengthSelect' ).getValue();
  },


  getPeriodStart : function ()
  {
    var me = this;

    var dayPeriodSelect = me.down( '#dayPeriodSelect' );
    var weekPeriodSelect = me.down( '#weekPeriodSelect' );
    var monthPeriodSelect = me.down( '#monthPeriodSelect' );
    var quarterPeriodSelect = me.down( '#quarterPeriodSelect' );
    var yearPeriodSelect = me.down( '#yearPeriodSelect' );

    var periodLength = me.getPeriodLength();

    if ( periodLength == null )
      return null;

    switch ( periodLength )
    {
      case QDesk.enums.TimePeriodLength.DAY:
        return dayPeriodSelect.getValue();
        break;

      case QDesk.enums.TimePeriodLength.WEEK:
        return weekPeriodSelect.getValue();
        break;

      case QDesk.enums.TimePeriodLength.MONTH:
        return monthPeriodSelect.getValue();
        break;

      case QDesk.enums.TimePeriodLength.QUARTER:
        return quarterPeriodSelect.getValue();
        break;

      case QDesk.enums.TimePeriodLength.YEAR:
        return yearPeriodSelect.getValue();
        break;

      default:
        Ext.Error.raise( 'Unexpected period length: ' + periodLength );
    }

    return null;  // to suppress warning
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    me.down( '#statisticsTypeSelect' ).on( 'change', function ( combo, newValue )
    {
      me._onStatisticsTypeChange( newValue );
    }, me );

    me.down( '#periodLengthSelect' ).on( 'change', function ( combo, newValue )
    {
      me._onPeriodLengthChange( newValue );
    }, me );


    // periods
    me.down( '#dayPeriodSelect' ).on( 'change', me._onPeriodChange, me );

    me.down( '#weekPeriodSelect' ).on( 'change', me._onPeriodChange, me );

    me.down( '#monthPeriodSelect' ).on( 'change', me._onPeriodChange, me );

    me.down( '#quarterPeriodSelect' ).on( 'change', me._onPeriodChange, me );

    me.down( '#yearPeriodSelect' ).on( 'change', me._onPeriodChange, me );

    /*
     * Default period length to be initially selected. This triggers a 'change' event
     * whose handler loads the available periods.
     */
    me.down( '#periodLengthSelect' ).setValue( QDesk.enums.TimePeriodLength.DAY );
  },


  /**
   * Invoked when the statistics type changes.
   * @param statisticsType the new statistics type length.
   * @private
   */
  _onStatisticsTypeChange : function ( statisticsType )
  {
    var me = this;
    //alert( 'Statistics type changed: ' + statisticsType );
    me._reloadAvailablePeriods();
  },


  /**
   * Invoked when the period length changes.
   * @param periodLength the new period length.
   * @private
   */
  _onPeriodLengthChange : function ( periodLength )
  {
    var me = this;

    //alert( 'Period length changed: ' + periodLength );

    var dayPeriodSelect = me.down( '#dayPeriodSelect' );
    var weekPeriodSelect = me.down( '#weekPeriodSelect' );
    var monthPeriodSelect = me.down( '#monthPeriodSelect' );
    var quarterPeriodSelect = me.down( '#quarterPeriodSelect' );
    var yearPeriodSelect = me.down( '#yearPeriodSelect' );

    switch ( periodLength )
    {
      case QDesk.enums.TimePeriodLength.DAY:
        weekPeriodSelect.setVisible( false );
        monthPeriodSelect.setVisible( false );
        quarterPeriodSelect.setVisible( false );
        yearPeriodSelect.setVisible( false );
        dayPeriodSelect.setVisible( true );
        break;

      case QDesk.enums.TimePeriodLength.WEEK:
        dayPeriodSelect.setVisible( false );
        monthPeriodSelect.setVisible( false );
        quarterPeriodSelect.setVisible( false );
        yearPeriodSelect.setVisible( false );
        weekPeriodSelect.setVisible( true );
        break;

      case QDesk.enums.TimePeriodLength.MONTH:
        dayPeriodSelect.setVisible( false );
        weekPeriodSelect.setVisible( false );
        quarterPeriodSelect.setVisible( false );
        yearPeriodSelect.setVisible( false );
        monthPeriodSelect.setVisible( true );
        break;

      case QDesk.enums.TimePeriodLength.QUARTER:
        dayPeriodSelect.setVisible( false );
        weekPeriodSelect.setVisible( false );
        monthPeriodSelect.setVisible( false );
        yearPeriodSelect.setVisible( false );
        quarterPeriodSelect.setVisible( true );
        break;

      case QDesk.enums.TimePeriodLength.YEAR:
        dayPeriodSelect.setVisible( false );
        weekPeriodSelect.setVisible( false );
        monthPeriodSelect.setVisible( false );
        quarterPeriodSelect.setVisible( false );
        yearPeriodSelect.setVisible( true );
        break;

      default:
        Ext.Error.raise( 'Unexpected period length: ' + periodLength );
    }

    me._reloadAvailablePeriods();
  },


  /**
   * Invoked when the period changes.
   * @param field the field whose change triggered the change event upon
   * which this method was invoked.
   * @param period the new period (the period start time). If null, then there is no
   * available time period.
   * @private
   */
  _onPeriodChange : function ( field, period )
  {
    var me = this;

    //alert( 'Period changed: ' + period );

    var statisticsType = me.down( '#statisticsTypeSelect' ).getValue();
    var periodLength = me.down( '#periodLengthSelect' ).getValue();

    me.fireEvent( 'statisticsCriteriaChange', statisticsType, periodLength, period );
  },


  /**
   * Reloads the contents of the list of available time periods for the currently selected
   * statistics type and period length.
   * @private
   */
  _reloadAvailablePeriods : function ()
  {
    var me = this;

    var dayPeriodSelect = me.down( '#dayPeriodSelect' );
    var weekPeriodSelect = me.down( '#weekPeriodSelect' );
    var monthPeriodSelect = me.down( '#monthPeriodSelect' );
    var quarterPeriodSelect = me.down( '#quarterPeriodSelect' );
    var yearPeriodSelect = me.down( '#yearPeriodSelect' );

    var periodCombo;
    var periodComboStore;

    var periodLength = me.down( '#periodLengthSelect' ).getValue();
    switch ( periodLength )
    {
      case QDesk.enums.TimePeriodLength.DAY:
        break;

      case QDesk.enums.TimePeriodLength.WEEK:
        periodCombo = weekPeriodSelect;
        periodComboStore = weekPeriodSelect.getStore();
        break;

      case QDesk.enums.TimePeriodLength.MONTH:
        periodCombo = monthPeriodSelect;
        periodComboStore = monthPeriodSelect.getStore();
        break;

      case QDesk.enums.TimePeriodLength.QUARTER:
        periodCombo = quarterPeriodSelect;
        periodComboStore = quarterPeriodSelect.getStore();
        break;

      case QDesk.enums.TimePeriodLength.YEAR:
        periodCombo = yearPeriodSelect;
        periodComboStore = yearPeriodSelect.getStore();
        break;

      default:
        Ext.Error.raise( 'Unexpected period length: ' + periodLength );
    }

    var statisticsType = me.down( '#statisticsTypeSelect' ).getValue();

    // reload period select
    if ( periodComboStore )
    {
      // empty the store
      periodComboStore.loadRecords( [] );

      me.setLoading( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.maskLoadPeriodsMsg' ) );

      periodComboStore.load( {
        params : {
          connectionId : me.qdesk.connectionId,
          statisticsType : statisticsType,
          periodLength : periodLength
        },
        callback : function ( records, operation, success )
        {
          me.setLoading( false );

          var warningMsgPanel = me.down( '#warningMessagePanel' );
          warningMsgPanel.setVisible( false );
          warningMsgPanel.setMessage( '' );

          if ( success )
          {
            if ( Ext.isEmpty( records ) )
            {
              warningMsgPanel.setMessage( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.warnNoPeriodsMsg' ) );
              warningMsgPanel.setVisible( true );
            }
            else
            {
              periodCombo.setValue( records[0] )
            }
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.errorLoadPeriodMsg' ) );
          }
        }
      } );
    }
    else
    {
      // DAY period - set the min and max selectable dates
      me.setLoading( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.maskLoadPeriodsMsg' ) );
      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.quartz.triggerStatisticsActionListPeriods,
        params : {
          connectionId : me.qdesk.connectionId,
          statisticsType : statisticsType,
          periodLength : periodLength
        },
        callback : function ( opts, success, resp )
        {
          me.setLoading( false );

          var warningMsgPanel = me.down( '#warningMessagePanel' );
          warningMsgPanel.setVisible( false );
          warningMsgPanel.setMessage( '' );

          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              var data = json.data;
              if ( Ext.isEmpty( data ) )
              {
                // disable all dates
                dayPeriodSelect.setValue( null );
                dayPeriodSelect.setDisabledDays( [0, 1, 2, 3, 4, 5, 6] );

                warningMsgPanel.setMessage( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.warnNoPeriodsMsg' ) );
                warningMsgPanel.setVisible( true );
              }
              else
              {
                var maxPeriod = Ext.create( 'QDesk.model.FixedTimePeriod', data[0] );
                var maxDate = maxPeriod.get( 'from' );

                var minPeriod = Ext.create( 'QDesk.model.FixedTimePeriod', data[data.length - 1] );
                var minDate = minPeriod.get( 'from' );

                dayPeriodSelect.setDisabledDays( [] );

                dayPeriodSelect.setMinValue( minDate );
                dayPeriodSelect.setMaxValue( maxDate );

                // temporarily suppress the change event
                dayPeriodSelect.un( 'change', me._onPeriodChange, me );
                dayPeriodSelect.setValue( null );
                dayPeriodSelect.on( 'change', me._onPeriodChange, me );

                dayPeriodSelect.setValue( maxDate );
              }
            }
            else
            {
              QDesk.util.UxUtils.showJsonResponseErrors( json );
            }
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.stats.TriggerStatisticsPanel.errorLoadPeriodMsg' ) );
          }
        }
      } );
    }
  }
} );
