/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.navigation.NavigationPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.customization.gui.navigation.Node',
    'QDesk.model.customization.gui.navigation.Tab',

    'QDesk.view.navigation.help.HelpTreePanel',
    'QDesk.view.navigation.connections.ConnectionsTreePanel',
    'QDesk.view.navigation.settings.SettingsTreePanel'
  ],

  alias : 'widget.NavigationPanel',

  cls : 'navigationPanel',

  layout : {
    type : 'accordion'
  },


  /*
   * Commented out, because with the introduction of custom tabs, the list of tabs is loaded
   * dynamically.
   */
  //items : [
  //  {
  //    xtype : 'ConnectionsTreePanel',
  //    itemId : 'connectionsTreePanel',
  //    //title : QDesk.Msg.get( 'QDesk.view.navigation.NavigationPanel.titleConnectionsPanel' ),
  //    collapsed : false
  //  },
  //  {
  //    xtype : 'SettingsTreePanel',
  //    itemId : 'settingsTreePanel',
  //    //title : QDesk.Msg.get( 'QDesk.view.navigation.NavigationPanel.titleSettingsPanel' ),
  //    collapsed : true
  //  } ,
  //  {
  //    xtype : 'HelpTreePanel',
  //    itemId : 'helpTreePanel',
  //    //title : QDesk.Msg.get( 'QDesk.view.navigation.NavigationPanel.titleHelpPanel' ),
  //    collapsed : true
  //  }
  //],


  initComponent : function ()
  {
    var me = this;
    me.callParent();

    var tabStore = Ext.StoreManager.lookup( 'customization.gui.navigation.TabStore' );
    tabStore.load( {
      scope : this,
      callback : function ( records, operation, success )
      {
        Ext.each( records, function ( record )
        {
          var widget;
          var type = record.get( 'type' );
          switch ( type )
          {
            case 'XTypeTab':
              widget = me._createXTypeTab( record );
              break;

            case 'CustomTab':
              widget = me._createCustomTab( record );
              break;

            default:
              Ext.Error.raise( 'Unsupported tab type: ' + type );
          }

          me.add( widget );
        } );
      }
    } );
  },


  /**
   * Creates a tab from the specified XTYPE tab definition record (QDesk.model.customization.gui.navigation.Tab
   * instance).
   * @param tabDefRecord a QDesk.model.customization.gui.navigation.Tab instance.
   * @returns the created tab.
   * @private
   */
  _createXTypeTab : function ( tabDefRecord )
  {
    var me = this;

    return Ext.widget( tabDefRecord.get( 'xtype' ), {
      itemId : tabDefRecord.get( 'itemId' ),
      title : me._getTabTitle( tabDefRecord ),
      collapsed : tabDefRecord.get( 'collapsed' )
    } );
  },


  /**
   * Creates a tab from the specified CUSTOM tab definition record (QDesk.model.customization.gui.navigation.Tab
   * instance).
   * @param tabDefRecord a QDesk.model.customization.gui.navigation.Tab instance.
   * @returns the created tab.
   * @private
   */
  _createCustomTab : function ( tabDefRecord )
  {
    var me = this;

    var store = Ext.create( 'Ext.data.TreeStore', {
      model : 'QDesk.model.customization.gui.navigation.Node',
      root : {
        expanded : true,
        children : tabDefRecord.get( 'children' )
      }
    } );

    return Ext.widget( 'treepanel', {
      itemId : tabDefRecord.get( 'itemId' ),
      title : me._getTabTitle( tabDefRecord ),
      collapsed : tabDefRecord.get( 'collapsed' ),

      displayField : 'title',

      useArrows : true,
      rootVisible : false,
      store : store
    } );
  },


  /**
   * Returns the title for the specified tab.
   * @param tabDefRecord a QDesk.model.customization.gui.navigation.Tab instance.
   * @returns the tab title.
   * @private
   */
  _getTabTitle : function ( tabDefRecord )
  {
    var title = tabDefRecord.get( 'title' );

    if ( title && title.indexOf( 'i18n:' ) == 0 )
    {
      var titleKey = title.substr( 'i18n:'.length );
      title = QDesk.Msg.get( titleKey );
    }

    return title;
  }
} );
