/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.navigation.connections.ConnectionsTreePanel', {
  extend : 'Ext.tree.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.enums.scheduler.SchedulerType',

    'QDesk.store.ConnectionsTreeStore',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const',

    'QDesk.controller.command.connection.AddConnectionFolder',
    'QDesk.controller.command.connection.AddSchedulerConnection',

    'QDesk.controller.command.connection.EditConnectionFolder',
    'QDesk.controller.command.connection.EditSchedulerConnection',
    'QDesk.controller.command.connection.DeleteConnectionFolder',
    'QDesk.controller.command.connection.DeleteSchedulerConnection',

    'QDesk.controller.command.connection.MoveConnectionFolder',
    'QDesk.controller.command.connection.MoveSchedulerConnection',

    'QDesk.controller.command.connection.RefreshConnectionFolder',

    'QDesk.controller.command.scheduler.common.StartScheduler',
    'QDesk.controller.command.scheduler.common.PauseScheduler',
    'QDesk.controller.command.scheduler.common.StopScheduler',

    'QDesk.controller.command.scheduler.quartz.OpenSchedulerView'
  ],

  alias : 'widget.ConnectionsTreePanel',

  cls : 'connectionsTreePanel',

  //scrollable : true,

  displayField : 'name',

  rootVisible : true,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      root : {
        id : QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID,
        folderId : QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID,
        name : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.rootNodeName' ),
        expanded : true,

        nodeType : 'folder'
      },

      viewConfig : {
        copy : false,

        loadingText : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.maskLoadMsg' ),

        plugins : {
          ptype : 'treeviewdragdrop',
          appendOnly : true,
          dragText : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.dragText' )
        }
      },

      dockedItems : [
        {
          xtype : 'toolbar',
          itemId : 'toolBar',
          dock : 'top',
          items : [
            {
              xtype : 'button',
              itemId : 'refreshButton',
              scale : 'small',
              iconCls : 'refresh',
              tooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.tooltipRefreshButton' ),
              disabled : true
            },
            {
              xtype : 'button',
              itemId : 'collapseButton',
              scale : 'small',
              iconCls : 'collapse',
              tooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.tooltipCollapseButton' ),
              disabled : true
            },
            '-',
            {
              xtype : 'button',
              itemId : 'addFolderOrConnectionButton',
              scale : 'small',
              iconCls : 'addFolderOrConnection',
              tooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.tooltipAddFolderOrConnectionButton' ),
              disabled : true,
              menu : [
                {
                  itemId : 'addFolderButton',
                  text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.addFolderButton' ),
                  iconCls : 'addFolder'
                },
                {
                  itemId : 'addConnectionButton',
                  text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.addConnectionButton' ),
                  iconCls : 'addConnection'
                }
              ]
            },
            {
              xtype : 'button',
              itemId : 'deleteFolderOrConnectionButton',
              scale : 'small',
              iconCls : 'deleteFolderOrConnection',
              // no tooltip, it is set dynamically based on the selection (folder / connection)
              disabled : true
            },
            {
              xtype : 'button',
              itemId : 'editFolderOrConnectionButton',
              scale : 'small',
              iconCls : 'editFolderOrConnection',
              // no tooltip, it is set dynamically based on the selection (folder / connection)
              disabled : true
            }
          ]
        }
      ]
    } );

    me.callParent( [config] );
  },


  initComponent : function ()
  {
    var me = this;

    // important to set the store before callParent as it expects the store
    me.store = Ext.create( 'QDesk.store.ConnectionsTreeStore');

    //
    // Initially we use an empty store URL and later we replace it with the real URL to work around
    // a bug in Chrome/IE where the initial tree panel's loading message does not display.
    //
    me.store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;

    me.callParent();

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    me._bindToolbarEventHandlers();
    me._bindPanelEventHandlers();
  },


  _bindToolbarEventHandlers : function ()
  {
    var me = this;

    // tree reload events
    var treeStore = me.getStore();

    treeStore.on( 'beforeload',
        function ( store, operation )
        {
          var toolBar = me.down( '#toolBar' );
          Ext.each( Ext.ComponentQuery.query( 'button', toolBar ), function ( button )
          {
            button.disable();
          } );
        } );

    treeStore.on( 'load',
        function ( store, node, records, success )
        {
          var toolBar = me.down( '#toolBar' );
          toolBar.down( '#refreshButton' ).enable();
          if ( success )
          {
            toolBar.down( '#collapseButton' ).enable();
          }
        } );

  },


  _bindPanelEventHandlers : function ()
  {
    var me = this;
    var treeView = me.getView();

    treeView.on( 'beforedrop', function ( node, data, overModel, dropPosition, dropFunction, eOpts )
    {
      var draggedRecord = data.records[0];
      var targetRecord = overModel;

      var draggedRecordParent = draggedRecord.parentNode;

      // if node is dragged to its existing parent, then we cancel the operation
      if ( draggedRecordParent.get( 'id' ) == targetRecord.get( 'id' ) )
        return false;

      if ( !me._checkNodeMove( draggedRecord, targetRecord ) )
      {
        return false;
      }

      me._onNodeDrag( draggedRecord, targetRecord );

      return 1;
    } );


    treeView.on( 'itemcontextmenu', function ( view, record, item, index, event )
    {
      var nodeId = record.get( 'id' );
      event.stopEvent();

      var nodeType = record.get( 'nodeType' );
      if ( nodeId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID || nodeType == 'folder' )
      {
        var folderCtxMenu = me._createFolderCtxMenu( record );
        folderCtxMenu.showAt( event.getXY() );
      }
      else if ( nodeType == 'connection' )
      {
        var connectionCtxMenu = me._createConnectionCtxMenu( record );
        connectionCtxMenu.showAt( event.getXY() );
      }
    } );


    treeView.on( 'itemdblclick', function ( view, record )
    {
      var nodeType = record.get( 'nodeType' );
      if ( nodeType == 'folder' )
      {
        me.application.fireEvent( 'connectionsTreeFolderDblClicked', record );
      }
      else if ( nodeType == 'connection' )
      {
        me._openSchedulerView( record );
      }
    } );

    var treeStore = me.getStore();

    //treeStore.on( 'beforeload', function ( store, operation )
    //{
    //  QDesk.util.UxUtils.maskEl( me.getEl(),
    //      QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.maskLoadMsg' ) );
    //} );

    treeStore.on( 'load', function ( store, records, successful, operation, node )
    {
      //QDesk.util.UxUtils.unmaskEl( me.getEl() );

      if ( !successful )
      {
        // node (failed folder) collapsing must be deferred a bit, otherwise the collapse does not work
        Ext.defer( function ()
        {
          node.collapse();
        }, 200 );

        QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.errorLoadMsg' ) );
      }
    } );

    /*
     * Swap the initial empty store URL with the real URL and fire an event to
     * request schedulers tree reloading. The delayed task is used to workaround
     * a problem in IE and Chrome where the tree panel's loading message does not
     * pop up during the initial tree store load.
     */
    treeStore.getProxy().url = QDesk.util.constant.ActionUrl.connectionActionTree;

    // load the connections tree
    var reloadTask = new Ext.util.DelayedTask( function ()
    {
      Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {} ).execute();
    } );
    reloadTask.delay( 200 );
  },


  /**
   * Checks if the specified record can be dragged to the specified target record.
   * @param draggedRecord the dragged folder/connection record.
   * @param targetRecord the target record (target folder).
   * @return {boolean} true if the dragged folder/connection can be moved, false otherwise.
   * @private
   */
  _checkNodeMove : function ( draggedRecord, targetRecord )
  {
    var canBeMoved = true;

    var targetFolderId = targetRecord.get( 'folderId' );
    var targetFolderName = targetRecord.get( 'name' );

    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      var folderId = draggedRecord.get( 'folderId' );
      var folderName = draggedRecord.get( 'name' );

      Ext.each( targetRecord.childNodes, function ( node )
      {
        if ( node.get( 'nodeType' ) == 'folder' && node.get( 'name' ) == folderName )
        {
          QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.folderCannotBeMovedMsg',
              folderName, targetFolderName ) );

          canBeMoved = false;
          return false;
        }
      } );
    }
    else if ( nodeType == 'connection' )
    {
      var connectionId = draggedRecord.get( 'connectionId' );
      var connectionName = draggedRecord.get( 'name' );

      Ext.each( targetRecord.childNodes, function ( node )
      {
        if ( node.get( 'nodeType' ) == 'connection' && node.get( 'name' ) == connectionName )
        {
          QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.connectionCannotBeMovedMsg',
              connectionName, targetFolderName ) );

          canBeMoved = false;
          return false;
        }
      } );
    }

    return canBeMoved;
  },


  /**
   * Invoked when a folder, or connection in the connections tree panel has been dragged.
   * @param draggedRecord the dragged folder/connection record.
   * @param targetRecord the target record (target folder).
   * @private
   */
  _onNodeDrag : function ( draggedRecord, targetRecord )
  {
    var targetFolderId = targetRecord.get( 'folderId' );
    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      var folderId = draggedRecord.get( 'folderId' );
      Ext.create( 'QDesk.controller.command.connection.MoveConnectionFolder', {
        folderId : folderId,
        targetFolderId : targetFolderId
      } ).execute();
    }
    else if ( nodeType == 'connection' )
    {
      var connectionId = draggedRecord.get( 'connectionId' );
      Ext.create( 'QDesk.controller.command.connection.MoveSchedulerConnection', {
        connectionId : connectionId,
        targetFolderId : targetFolderId
      } ).execute();
    }
  },


  /**
   * Returns the ID of the parent folder of the specified connection.
   * @param connectionId a connection ID.
   * @private the parent folder ID, or null if the connection was not found.
   */
  _getParentFolderForConnection : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var connectionNode = store.getNodeById( connectionId );
    var parentFolderNode = connectionNode ? connectionNode.parentNode : null;
    return parentFolderNode ? parentFolderNode.data.id : null;
  },


  /**
   * Creates and returns the context menu for a folder node in the connections tree.
   * @param folderRecord a folder record (see QDesk.model.ConnectionsTreeNode}.
   * @return the created context menu.
   */
  _createFolderCtxMenu : function ( folderRecord )
  {
    var me = this;

    var folderId = folderRecord.get( 'folderId' );
    var isRootFolder = folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.folderCtxMenuAddFolderOrConnection' ),
          menu : {
            items : [
              {
                text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.folderCtxMenuAddFolder' ),
                iconCls : 'addFolder',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.connection.AddConnectionFolder', {
                    parentFolderId : folderId
                  } ).execute();
                }
              },
              {
                text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.folderCtxMenuAddConnection' ),
                iconCls : 'addConnection',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.connection.AddSchedulerConnection', {
                    parentFolderId : folderId,
                    schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
                  } ).execute();
                }
              }
            ]
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.folderCtxMenuRename' ),
          iconCls : 'renameFolder',
          disabled : isRootFolder,
          handler : function ()
          {
            if ( folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID )
            {
              // ignore, root node cannot be renamed
            }
            else
            {
              Ext.create( 'QDesk.controller.command.connection.EditConnectionFolder', {
                folderId : folderId,
                folderName : folderRecord.get( 'name' )
              } ).execute();
            }
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.folderCtxMenuDelete' ),
          iconCls : 'deleteFolder',
          disabled : isRootFolder,
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.DeleteConnectionFolder', {
              folderId : folderId
            } ).execute();
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.folderCtxMenuRefresh' ),
          iconCls : 'refreshFolder',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
              folderId : folderId
            } ).execute();
          }
        }
      ]
    } );
  },


  /**
   * Creates and returns the context menu for a connection node in the connections tree.
   * @param connectionRecord a folder record (see QDesk.model.ConnectionsTreeNode}.
   * @return the created context menu.
   */
  _createConnectionCtxMenu : function ( connectionRecord )
  {
    var me = this;

    var connectionId = connectionRecord.get( 'connectionId' );
    var schedulerStatus = connectionRecord.get( 'schedulerStatus' );
    var schedulerType = connectionRecord.get( 'schedulerType' );

    var menuItemIconOpen = null;
    switch ( schedulerType )
    {
      case  QDesk.enums.scheduler.SchedulerType.QUARTZ :
        menuItemIconOpen = 'openConnectionQuartz';
        break;

      default:
        Ext.Error.raise( 'Unexpected scheduler type: ' + schedulerType );
    }

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // open connection
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.connectionCtxMenuOpen' ),
          iconCls : menuItemIconOpen,
          disabled : schedulerStatus == QDesk.enums.scheduler.SchedulerStatus.UNREACHABLE,
          handler : function ()
          {
            me._openSchedulerView( connectionRecord )
          }
        },
        // edit connection
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.connectionCtxMenuEdit' ),
          iconCls : 'editConnection',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.EditSchedulerConnection', {
              connectionId : connectionId,
              schedulerType : schedulerType
            } ).execute();
          }
        },
        // delete connection
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.connectionCtxMenuDelete' ),
          iconCls : 'deleteConnection',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.DeleteSchedulerConnection', {
              connectionId : connectionId
            } ).execute();
          }
        },
        '-',
        // start scheduler
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.connectionCtxMenuStart' ),
          iconCls : 'startScheduler',
          disabled : schedulerStatus != QDesk.enums.scheduler.SchedulerStatus.PAUSED,
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.StartScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        },
        // pause scheduler
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.connectionCtxMenuPause' ),
          iconCls : 'pauseScheduler',
          disabled : schedulerStatus != QDesk.enums.scheduler.SchedulerStatus.STARTED,
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.PauseScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        },
        // stop scheduler
        {
          text : QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.connectionCtxMenuStop' ),
          iconCls : 'stopScheduler',
          disabled : (schedulerStatus != QDesk.enums.scheduler.SchedulerStatus.STARTED &&
          schedulerStatus != QDesk.enums.scheduler.SchedulerStatus.PAUSED),
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.StopScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        }
      ]
    } );
  },


  _openSchedulerView : function ( record )
  {
    var me = this;

    var schedulerStatus = record.get( 'schedulerStatus' );

    if ( (schedulerStatus == QDesk.enums.scheduler.SchedulerStatus.UNREACHABLE ) )
    {
      // connection unreachable
      QDesk.util.UxUtils.showInfoDialog( QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.schedulerUnreachableMsg' ) );
    }
    else
    {
      var connectionId = record.get( 'connectionId' );
      var schedulerType = record.get( 'schedulerType' );
      var schedulerVersion = record.get( 'schedulerVersion' );

      if ( schedulerType == QDesk.enums.scheduler.SchedulerType.QUARTZ )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView', {
          connectionId : connectionId,
          schedulerInfo : {
            type : schedulerType,
            version : schedulerVersion
          }
        } ).execute();
      }
      else
      {
        Ext.Error.raise( 'Unsupported scheduler type: ' + schedulerType );
      }
    }
  }
} );
