/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.view.navigation.connections.SchedulerConnectionDataForm',
    'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm',
    'QDesk.view.FormNavigationMenuTreePanel',
    'QDesk.util.constant.Const',
    'QDesk.Msg'
  ],

  statics : {
    MODE_ADD : 'MODE_ADD',
    MODE_EDIT : 'MODE_EDIT',

    NAVIG_NODE_ID_ROOT : 'root',
    NAVIG_NODE_ID_CONNECTION_DATA : 'connection',
    NAVIG_NODE_ID_QUARTZ_MAINTENANCE_TASKS : 'quartzMaintenanceTasks'
  },

  alias : 'widget.EditSchedulerConnectionWindow',

  cls : 'editSchedulerConnectionWindow',

  modal : true,
  resizable : false,

  minWidth : 650,
  width : 650,

  height : 500,
  minHeight : 500,

  layout : {
    type : 'border'
  },

  qdesk : {
    /*
     * Mode of the form (see SchedulerConnectionDataForm modes).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.enums.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    // choose window icon based on the scheduler type
    var iconCls = null;
    switch ( config.qdesk.schedulerType )
    {
      case  QDesk.enums.scheduler.SchedulerType.QUARTZ :
        iconCls = 'editSchedulerConnectionQuartzWindowIcon';
        break;

      default:
        Ext.Error.raise( 'Unexpected scheduler type: ' + config.qdesk.schedulerType );
    }

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.mode ),
      iconCls : iconCls,
      items : [
        {
          xtype : 'FormNavigationMenuTreePanel',
          itemId : 'navigationTreeMenuPanel',
          region : 'west',
          split : true,
          header : false,
          collapsible : true,
          collapseMode : 'mini',
          width : 140,
          root : {
            id : QDesk.view.navigation.connections.EditSchedulerConnectionWindow.NAVIG_NODE_ID_ROOT,  // ROOT folder ID
            name : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow.navigationRoot' ),
            expanded : true,

            children : [
              /* Connection */
              {
                id : QDesk.view.navigation.connections.EditSchedulerConnectionWindow.NAVIG_NODE_ID_CONNECTION_DATA,
                name : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow.navigationConnectionData' ),
                leaf : true
              },

              /* Quartz specific Maintenance Tasks */
              {
                id : QDesk.view.navigation.connections.EditSchedulerConnectionWindow.NAVIG_NODE_ID_QUARTZ_MAINTENANCE_TASKS,
                name : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow.navigationMaintenanceTasks' ),
                leaf : true
              }
            ]
          }
        },
        {
          xtype : 'panel',
          itemId : 'cardPanel',
          region : 'center',
          layout : 'card',
          items : [
            /*
             * Generic connection data form usable for all scheduler connections.
             */
            {
              xtype : 'SchedulerConnectionDataForm',
              itemId : 'connectionDataForm',

              qdesk : {
                nodeId : QDesk.view.navigation.connections.EditSchedulerConnectionWindow.NAVIG_NODE_ID_CONNECTION_DATA,
                mode : config.qdesk.mode,
                schedulerType : config.qdesk.schedulerType
              }
            },

            /*
             * Quartz specific scheduler connection maintenance tasks.
             */
            {
              xtype : 'QuartzSchedulerConnectionMaintenanceTasksForm',
              itemId : 'quartzMaintenanceTasksForm',
              border : 0,

              qdesk : {
                nodeId : QDesk.view.navigation.connections.EditSchedulerConnectionWindow.NAVIG_NODE_ID_QUARTZ_MAINTENANCE_TASKS,
                mode : config.qdesk.mode,
                schedulerType : config.qdesk.schedulerType
              }
            }
          ]
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'cancelButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
            },
            {
              xtype : 'button',
              itemId : 'actionButton',
              text : me._getActionButtonText( config.qdesk.mode ),
              formBind : true
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    me.on( 'beforerender', function ()
    {
      /* Keep track of changes to the settings */
      me._bindChangeDetectors();

      /* Sets the initial licensedFormCount and invalidFormCount flags in the navigation tree nodes. */
      me._initializeForms();
    } );

    // select the first node in the navigation panel (Connection)
    me.on( 'afterrender', function ()
    {
      var navigationTreeMenuPanel = me.down( '#navigationTreeMenuPanel' );
      var navigationRootNode = navigationTreeMenuPanel.getStore().getRootNode();

      var selectionModel = navigationTreeMenuPanel.getSelectionModel();
      selectionModel.select( navigationRootNode.firstChild );

      // set focus on the connection name in the connection panel
      var connectionDataFormPanel = me.down( '#connectionDataForm' );
      var nameField = connectionDataFormPanel.down( '#name' );
      nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    // cancel button closes the window
    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } );

    var navigationTreeMenuPanel = me.down( '#navigationTreeMenuPanel' );

    var cardPanel = me.down( '#cardPanel' );

    // selection changes
    navigationTreeMenuPanel.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length > 0 )  // selection change event is also fired after deselection
          {
            var selectedRecord = selectedRecords[ 0 ];
            var nodeId = selectedRecord.get( 'id' );

            var formPanel = me._getFormPanelByNodeId( nodeId );

            if ( formPanel )
              cardPanel.getLayout().setActiveItem( formPanel );
          }
        }, me );
  },


  /**
   * Returns the form panel for the specified node ID.
   * @param nodeId a node ID.
   * @return {Ext.form.Panel} the form panel, or null.
   * @private
   */
  _getFormPanelByNodeId : function ( nodeId )
  {
    var me = this;

    var result = null;
    Ext.each( Ext.ComponentQuery.query( '#cardPanel form', me ), function ( formPanel )
    {
      if ( formPanel.qdesk.nodeId == nodeId )
      {
        result = formPanel;
        return false;
      }
    } );

    return result;
  },


  /**
   * Returns the title of this window.
   * @param mode a QDesk.view.navigation.connections.SchedulerConnectionDataForm mode value.
   */
  _getWindowTitle : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.navigation.connections.EditSchedulerConnectionWindow.MODE_ADD:
        return QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow.windowTitleAdd' );

      case QDesk.view.navigation.connections.EditSchedulerConnectionWindow.MODE_EDIT:
        return QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow.windowTitleEdit' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  _getActionButtonText : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.navigation.connections.EditSchedulerConnectionWindow.MODE_ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.navigation.connections.EditSchedulerConnectionWindow.MODE_EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  /**
   * Bind change detectors to all form fields.
   * @private
   */
  _bindChangeDetectors : function ()
  {
    var me = this;

    var fields = Ext.ComponentQuery.query( '#cardPanel form field', me );
    Ext.each( fields, function ( field )
    {
      if ( field.qdesk && field.qdesk.ignore )
      {
        // skip form fields marked with qdesk.ignore flag (various helper fields)
      }
      else
      {
        field.on( 'change', function ( field, newValue, oldValue )
        {
          var formPanel = field.up( 'form' );
          me._updateInvalidFormCounts( formPanel );
        }, me );
      }
    } );
  },


  /**
   * Iterates through and validates all forms and sets the initial licensedFormCount and
   * invalidFormCount flag values.
   * @private
   */
  _initializeForms : function ()
  {
    var me = this;

    var formPanels = Ext.ComponentQuery.query( '#cardPanel form', me );
    Ext.each( formPanels, function ( formPanel )
    {
      me._updateLicensedFormCounts( formPanel );
      me._updateInvalidFormCounts( formPanel );
    }, me );
  },


  /**
   * Check to see if the specified form panel is licensed and based on the result
   * it updates values of the licensedFormCount flags in the navigation tree menu.
   * @private
   */
  _updateLicensedFormCounts : function ( formPanel )
  {
    var me = this;

    // find the navigation node for the form panel
    var treePanel = me.down( '#navigationTreeMenuPanel' );
    var navigationNode = treePanel.getStore().getNodeById( formPanel.qdesk.nodeId );

    if ( !navigationNode )
      Ext.Error.raise( 'Cannot find navigation node for form panel with nodeId: ' + formPanel.qdesk.nodeId );

    var licensed = formPanel.qdesk.licensed;

    if ( licensed )
    {
      while ( navigationNode != null )
      {
        navigationNode.set( 'licensedFormCount', navigationNode.get( 'licensedFormCount' ) + 1 );
        navigationNode = navigationNode.parentNode;
      }
    }

    var navigationRootNode = treePanel.getRootNode();
    var rootLicensedFormCount = navigationRootNode.get( 'licensedFormCount' );

    var actionButton = me.down( '#actionButton' );
    actionButton.setDisabled( rootLicensedFormCount > 0 );
  },


  /**
   * Validates the specified form panel and based on the result it updates values of the
   * invalidFormCount flags in the navigation tree menu.
   * @param formPanel a form panel to validate.
   * @private
   */
  _updateInvalidFormCounts : function ( formPanel )
  {
    var me = this;

    // find the navigation node for the form panel
    var treePanel = me.down( '#navigationTreeMenuPanel' );
    var navigationNode = treePanel.getStore().getNodeById( formPanel.qdesk.nodeId );

    if ( !navigationNode )
      Ext.Error.raise( 'Cannot find navigation node for form panel with nodeId: ' + formPanel.qdesk.nodeId );

    var invalidFormCount = navigationNode.get( 'invalidFormCount' );

    var validNow = formPanel.isValid();
    var validBefore = invalidFormCount == 0;

    while ( navigationNode != null )
    {
      if ( validNow )
      {
        if ( validBefore )
        {
          // do nothing, form was valid and still is valid
        }
        else
        {
          navigationNode.set( 'invalidFormCount', navigationNode.get( 'invalidFormCount' ) - 1 );
          treePanel.getView().refreshNode( navigationNode ); // important to refresh the node!!!
        }
      }
      else
      {
        if ( validBefore )
        {
          navigationNode.set( 'invalidFormCount', navigationNode.get( 'invalidFormCount' ) + 1 );
          treePanel.getView().refreshNode( navigationNode ); // important to refresh the node!!!
        }
        else
        {
          // do nothing, form was and still is invalid
        }
      }

      navigationNode = navigationNode.parentNode;
    }

    var navigationRootNode = treePanel.getRootNode();
    var rootInvalidFormCount = navigationRootNode.get( 'invalidFormCount' );

    var actionButton = me.down( '#actionButton' );
    actionButton.setDisabled( rootInvalidFormCount > 0 );
  }
} );
