/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm', {
  extend : 'QDesk.view.base.BaseConfigFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.JmxProtocol',
    'QDesk.model.Scheduler',
    'QDesk.view.base.BaseConfigFormPanel'
  ],

  alias : 'widget.SchedulerConnectionDataForm',

  qdesk : {
    /*
     * Mode of the form (see SchedulerConnectionDataFormMode).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.enums.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    // initially empty store used by the schedulerObjectName combo
    var schedulerObjectNameStore = Ext.create( 'Ext.data.ArrayStore', {
      model : 'QDesk.model.Scheduler'
    } );

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'fieldcontainer',
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldLabelName' ),
              labelWidth : 60,
              allowBlank : false,
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldTooltipName' )
            },
            {
              xtype : 'fieldset',
              title : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldsetLabelRemoteJmxService' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'radiofield',
                  itemId : 'serviceParamsRadio',
                  boxLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.boxLabelServiceParams' )
                },
                {
                  xtype : 'fieldcontainer',
                  itemId : 'serviceParamsFields',
                  padding : '0 0 0 10',
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%',
                    labelWidth : 70
                  },
                  items : [
                    {
                      xtype : 'combo',
                      itemId : 'protocol',
                      name : 'protocol',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldLabelProtocol' ),
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldTooltipProtocol' ),
                      //value : QDesk.enums.JmxProtocol.RMI,
                      allowBlank : false,
                      editable : false,
                      forceSelection : true,
                      queryMode : 'local',
                      store : 'enums.JmxProtocolStore',
                      valueField : 'value',
                      displayField : 'valueLabel'
                    },
                    {
                      xtype : 'textfield',
                      itemId : 'host',
                      name : 'host',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldLabelHost' ),
                      allowBlank : false,
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldTooltipHost' )
                    },
                    {
                      xtype : 'numberfield',
                      itemId : 'port',
                      name : 'port',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldLabelPort' ),
                      allowBlank : false,
                      minValue : 1,
                      maxValue : 65535,
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldTooltipPort' )
                    }
                  ]
                },


                {
                  xtype : 'radiofield',
                  itemId : 'serviceUrlRadio',
                  boxLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.boxLabelServiceUrl' )
                },
                {
                  xtype : 'fieldcontainer',
                  itemId : 'serviceUrlFields',
                  padding : '0 0 0 10',
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%',
                    labelWidth : 70
                  },
                  items : [

                    {
                      xtype : 'textfield',
                      itemId : 'serviceUrl',
                      name : 'serviceUrl',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldLabelServiceUrl' ),
                      fieldTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldTooltipServiceUrl' ),
                      allowBlank : false
                    }
                  ]
                },


                {
                  xtype : 'fieldset',
                  collapsible : true,
                  collapsed : true,
                  title : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldsetLabelAuthentication' ),
                  tooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldsetTooltipAuthentication' ),
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%',
                    labelWidth : 70
                  },
                  items : [
                    {
                      xtype : 'textfield',
                      itemId : 'username',
                      name : 'username',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldLabelUsername' ),
                      allowBlank : true
                    },
                    {
                      xtype : 'textfield',
                      itemId : 'password',
                      name : 'password',
                      fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldLabelPassword' ),
                      inputType : 'password',
                      allowBlank : true
                    }
                  ]
                },

                {
                  xtype : 'checkboxfield',
                  itemId : 'secure',
                  name : 'secure',
                  //boxLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.boxLabelSecure_RMI' ),
                  //boxTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.boxTooltipSecure_RMI' ),
                  labelAlign : 'right',
                  allowBlank : true
                }
              ]
            },
            {
              xtype : 'fieldset',
              title : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldsetTitleSchedulerObjectName' ),
              tooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.fieldsetTooltipSchedulerObjectName' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'panel',
                  border : false,
                  layout : {
                    type : 'hbox'
                  },
                  style : {
                    marginBottom : '5px'
                  },
                  items : [
                    {
                      xtype : 'combo',
                      itemId : 'schedulerObjectName',
                      name : 'schedulerObjectName',
                      allowBlank : false,
                      queryMode : 'local',
                      store : schedulerObjectNameStore,
                      valueField : 'objectName',
                      displayField : 'objectName',
                      flex : 1
                    },
                    {
                      xtype : 'button',
                      itemId : 'discoverButton',
                      margin : '0 0 0 5',
                      text : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.buttonLabelDiscover' )
                    }
                  ]
                }
              ]
            },

            {
              xtype : 'panel',
              border : false,
              layout : {
                type : 'hbox',
                pack : 'end'
              },
              items : [
                {
                  xtype : 'button',
                  itemId : 'testButton',
                  text : QDesk.Msg.get( 'QDesk.view.navigation.connections.SchedulerConnectionDataForm.buttonLabelTest' ),
                  formBind : true
                }
              ]
            }
          ],

          /* All fields in the ConnectionData form are available in all product editions. */
          qdesk : {
            licensedFor : [
              QDesk.enums.ProductEdition.LITE,
              QDesk.enums.ProductEdition.STANDARD,
              QDesk.enums.ProductEdition.ENTERPRISE
            ]
          }
        }
      ]
    } );

    me.callParent( [config] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var protocolCombo = me.down( '#protocol' );

    me.on( 'beforerender', function ()
    {
      var serviceUrl = me.getForm().findField( 'serviceUrl' ).getValue();

      if ( Ext.isEmpty( serviceUrl ) )
      {
        me.down( '#serviceParamsRadio' ).setValue( true );
        me.down( '#serviceParamsFields' ).setDisabled( false );

        me.down( '#serviceUrlRadio' ).setValue( false );
        me.down( '#serviceUrlFields' ).setDisabled( true );
      }
      else
      {
        me.down( '#serviceParamsRadio' ).setValue( false );
        me.down( '#serviceParamsFields' ).setDisabled( true );

        me.down( '#serviceUrlRadio' ).setValue( true );
        me.down( '#serviceUrlFields' ).setDisabled( false );
      }

      // if REMOTING-JMX protocol is selected, check and disable the Secure checkbox
      if ( protocolCombo.getValue() == QDesk.enums.JmxProtocol.REMOTING_JMX )
      {
        var secureCheckbox = me.down( '#secure' );
        secureCheckbox.setValue( true );
        secureCheckbox.setDisabled( true );
      }
    } );

    // Discover button
    me.down( '#discoverButton' ).on( 'click', function ()
    {
      var me = this;
      Ext.create( 'QDesk.controller.command.connection.DiscoverSchedulers', {
        form : me,
        schedulerType : me.qdesk.schedulerType
      } ).execute();
    }, me );

    // Test Connection button
    me.down( '#testButton' ).on( 'click', function ()
    {
      var me = this;
      Ext.create( 'QDesk.controller.command.connection.TestSchedulerConnection', {
        form : me,
        schedulerType : me.qdesk.schedulerType
      } ).execute();
    }, me );

    protocolCombo.on( 'change', me._protocolChanged, me );

    me.down( '#host' ).on( 'change', me._maybeEnableButtons, me );
    me.down( '#port' ).on( 'change', me._maybeEnableButtons, me );
    me.down( '#serviceUrl' ).on( 'change', me._maybeEnableButtons, me );
    me.down( '#schedulerObjectName' ).on( 'change', me._maybeEnableButtons, me );

    me.down( '#serviceParamsRadio' ).on( 'change', me._serviceModeChanged, me );
    me.down( '#serviceUrlRadio' ).on( 'change', me._serviceModeChanged, me );

    // set default protocol value (change of the value invokes the _protocolChanged method)
    protocolCombo.setValue( QDesk.enums.JmxProtocol.RMI );
  },


  /**
   * Change the box label and tool tip of the Secure checkbox.
   * @param protocolCombo the protocol selection select box.
   * @param newValue the newly selected protocol value.
   * @param oldValue the previously selected protocol value.
   * @private
   */
  _protocolChanged : function ( protocolCombo, newValue, oldValue )
  {
    var me = this;
    var secureCheckbox = me.down( '#secure' );

    secureCheckbox.setBoxLabel( QDesk.Msg.get(
        'QDesk.view.navigation.connections.SchedulerConnectionDataForm.boxLabelSecure_' + newValue ) );

    if ( newValue == QDesk.enums.JmxProtocol.REMOTING_JMX )
    {
      secureCheckbox.setValue( true );
      secureCheckbox.setDisabled( true );
    }
    else
    {
      secureCheckbox.setValue( false );
      secureCheckbox.setDisabled( false );
    }

    // TODO: change boxTooltip
  },


  _serviceModeChanged : function ( radio, newValue, oldValue )
  {
    var me = this;
    if ( radio.itemId == 'serviceParamsRadio' && newValue )
    {
      me.down( '#serviceUrlRadio' ).setValue( false );
      me.down( '#serviceUrlFields' ).setDisabled( true );

      me.down( '#serviceParamsFields' ).setDisabled( false );
    }

    if ( radio.itemId == 'serviceUrlRadio' && newValue )
    {
      me.down( '#serviceParamsRadio' ).setValue( false );
      me.down( '#serviceParamsFields' ).setDisabled( true );

      me.down( '#serviceUrlFields' ).setDisabled( false );
    }

    me._maybeEnableButtons();
  },


  _maybeEnableButtons : function ()
  {
    var me = this;

    var discoverButton = me.down( '#discoverButton' );
    var testButton = me.down( '#testButton' );

    var protocolField = me.down( '#host' );
    var hostField = me.down( '#host' );
    var portField = me.down( '#port' );

    var serviceUrlField = me.down( '#serviceUrl' );

    var schedulerObjectNameField = me.down( '#schedulerObjectName' );

    var serviceParamsRadio = me.down( '#serviceParamsRadio' );
    var serviceUrlRadio = me.down( '#serviceUrlRadio' );

    if ( serviceParamsRadio.getValue() )
    {
      discoverButton.setDisabled( !(hostField.isValid() && portField.isValid()) );
      testButton.setDisabled( !(hostField.isValid() && portField.isValid() && schedulerObjectNameField.isValid()) );
    }

    if ( serviceUrlRadio.getValue() )
    {
      discoverButton.setDisabled( !serviceUrlField.isValid() );
      testButton.setDisabled( !(serviceUrlField.isValid() && schedulerObjectNameField.isValid()) );
    }
  }
} );
