/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm', {
  extend : 'QDesk.view.base.BaseConfigFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.JmxProtocol',
    'QDesk.enums.ProductEdition',
    'QDesk.model.Scheduler',
    'QDesk.util.license.LicensedProduct',
    'QDesk.view.base.BaseConfigFormPanel'
  ],

  alias : 'widget.QuartzSchedulerConnectionMaintenanceTasksForm',

  qdesk : {
    /*
     * Mode of the form (see SchedulerConnectionDataFormMode).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.enums.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

//    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;
//
//    /* This form panel is licensed for the Standard, or Enterprise edition only. */
//    Ext.apply( config.qdesk, {
//      licensed : licensedProductEdition == QDesk.enums.ProductEdition.STANDARD ||
//          licensedProductEdition == QDesk.enums.ProductEdition.ENTERPRISE
//    } );

    Ext.apply( config, {
      items : [
        /* Start of execHistoryPurge field set */
        {
          xtype : 'fieldset',
          itemId : 'execHistoryPurge',
          title : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldsetLabelExecHistoryPurge' ),
          tooltip: QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldsetTooltipExecHistoryPurge' ),
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          fieldDefaults : {
            labelWidth : 100
          },
          items : [
            {
              xtype : 'checkboxfield',
              itemId : 'execHistoryPurgeEnabled',
              name : 'quartz.execHistory.purge.enabled',
              boxLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.boxLabelExecHistoryPurgeEnabled' ),
              boxTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.boxTooltipExecHistoryPurgeEnabled' ),
              labelAlign : 'right',
              allowBlank : true
            },
            {
              xtype : 'fieldcontainer',
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              disabled : true,
              items : [
                {
                  xtype : 'numberfield',
                  itemId : 'execHistoryPurgeKeepDays',
                  name : 'quartz.execHistory.purge.keepDays',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldLabelExecHistoryPurgeKeepDays' ),
                  fieldTooltip: QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldTooltipExecHistoryPurgeKeepDays' ),
                  allowBlank : false,
                  minValue : 1,
                  maxValue : 65535
                }
              ]
            }
          ],
          qdesk : {
            licensedFor : [
              QDesk.enums.ProductEdition.STANDARD,
              QDesk.enums.ProductEdition.ENTERPRISE
            ]
          }
        },
        /* End of execHistoryPurge field set */

        /* Start of execHistoryPurgeLog field set */
        {
          xtype : 'fieldset',
          itemId : 'execHistoryLogPurge',
          title : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldsetLabelExecHistoryLogPurge' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldsetTooltipExecHistoryLogPurge' ),
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          fieldDefaults : {
            labelWidth : 100
          },
          items : [
            {
              xtype : 'checkboxfield',
              itemId : 'execHistoryLogPurgeEnabled',
              name : 'quartz.execHistory.log.purge.enabled',
              boxLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.boxLabelExecHistoryLogPurgeEnabled' ),
              boxTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.boxTooltipExecHistoryLogPurgeEnabled' ),
              labelAlign : 'right',
              allowBlank : true
            },
            {
              xtype : 'fieldcontainer',
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              disabled : true,
              items : [
                {
                  xtype : 'numberfield',
                  itemId : 'execHistoryPurgeLogKeepDays',
                  name : 'quartz.execHistory.log.purge.keepDays',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldLabelExecHistoryLogPurgeKeepDays' ),
                  fieldTooltip: QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldTooltipExecHistoryLogPurgeKeepDays' ),
                  allowBlank : false,
                  minValue : 1,
                  maxValue : 65535
                }
              ]
            }
          ],
          qdesk : {
            licensedFor : [
              QDesk.enums.ProductEdition.STANDARD,
              QDesk.enums.ProductEdition.ENTERPRISE
            ]
          }
        },
        /* End of execHistoryPurgeLog field set */

        /* Start of messagesProcessedPurge field set */
        {
          xtype : 'fieldset',
          itemId : 'messagesProcessedPurge',
          title : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldsetLabelMessagesProcessedPurge' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldsetTooltipMessagesProcessedPurge' ),
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          fieldDefaults : {
            labelWidth : 100
          },
          items : [
            {
              xtype : 'checkboxfield',
              itemId : 'messagesProcessedPurgeEnabled',
              name : 'messages.processed.purge.enabled',
              boxLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.boxLabelMessagesProcessedPurgeEnabled' ),
              boxTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.boxTooltipMessagesProcessedPurgeEnabled' ),
              labelAlign : 'right',
              allowBlank : true
            },
            {
              xtype : 'fieldcontainer',
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              disabled : true,
              items : [
                {
                  xtype : 'numberfield',
                  itemId : 'messagesProcessedPurgeKeepDays',
                  name : 'messages.processed.purge.keepDays',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldLabelMessagesProcessedPurgeKeepDays' ),
                  fieldTooltip: QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldTooltipMessagesProcessedPurgeKeepDays' ),
                  allowBlank : false,
                  minValue : 1,
                  maxValue : 65535
                }
              ]
            }
          ],
          qdesk : {
            licensedFor : [
              QDesk.enums.ProductEdition.ENTERPRISE
            ]
          }
        },
        /* End of messagesProcessedPurge field set */

        /* Start of messagesFailedPurge field set */
        {
          xtype : 'fieldset',
          itemId : 'messagesFailedPurge',
          title : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldsetLabelMessagesFailedPurge' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldsetTooltipMessagesFailedPurge' ),
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          fieldDefaults : {
            labelWidth : 100
          },
          items : [
            {
              xtype : 'checkboxfield',
              itemId : 'messagesFailedPurgeEnabled',
              name : 'messages.failed.purge.enabled',
              boxLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.boxLabelMessagesFailedPurgeEnabled' ),
              boxTooltip : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.boxTooltipMessagesFailedPurgeEnabled' ),
              labelAlign : 'right',
              allowBlank : true
            },
            {
              xtype : 'fieldcontainer',
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              disabled : true,
              items : [
                {
                  xtype : 'numberfield',
                  itemId : 'messagesFailedPurgeKeepDays',
                  name : 'messages.failed.purge.keepDays',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldLabelMessagesFailedPurgeKeepDays' ),
                  fieldTooltip: QDesk.Msg.get( 'QDesk.view.navigation.connections.quartz.SchedulerConnectionMaintenanceTasksForm.fieldTooltipMessagesFailedPurgeKeepDays' ),
                  allowBlank : false,
                  minValue : 1,
                  maxValue : 65535
                }
              ]
            }
          ],
          qdesk : {
            licensedFor : [
              QDesk.enums.ProductEdition.ENTERPRISE
            ]
          }
        }
        /* End of messagesProcessedPurge field set */
      ]
    } );

    me.callParent( [config] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var taskStatusCheckboxes = [
      me.down( '#execHistoryPurgeEnabled' ),
      me.down( '#execHistoryLogPurgeEnabled' ),
      me.down( '#messagesProcessedPurgeEnabled' ),
      me.down( '#messagesFailedPurgeEnabled' )
    ];

    Ext.each( taskStatusCheckboxes, function ( checkbox )
    {
      checkbox.on( 'change', me._taskStatusChanged, me );
    } );

    // we need to explicitly change the status of individual task configuration fieldcontainers
    // using the current task status values
    Ext.each( taskStatusCheckboxes, function ( checkbox )
    {
      me._taskStatusChanged( checkbox, checkbox.getValue() );
    } );
  },


  _taskStatusChanged : function ( checkbox, taskEnabled )
  {
    checkbox.nextSibling( 'fieldcontainer' ).setDisabled( !taskEnabled );
  }
} );
