/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.navigation.settings.SettingsTreePanel', {
  extend : 'Ext.tree.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ProductEdition',
    'QDesk.model.SettingsTreeNode',
    'QDesk.util.license.LicensedProduct'
  ],

  alias : 'widget.SettingsTreePanel',

  cls : 'settingsTreePanel',

  //scrollable : true,

  displayField : 'name',

  useArrows : true,

  // the settings tree panel has no root node
  rootVisible : false,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;

    var children = [];

    children.push( {
      id : 'appSettings',
      iconCls : 'appSettings',
      leaf : true,

      /* SettingsTreeNode model attributes */
      name : QDesk.Msg.get( 'QDesk.view.navigation.settings.SettingsTreePanel.nodeAppSettings' ),
      licensed : true
    } );

    if ( licensedProductEdition == QDesk.enums.ProductEdition.LITE )
    {
      children.push( {
        id : 'messageChannelProfiles',
        cls : 'notLicensed',
        iconCls : 'messageChannelProfiles',
        leaf : true,

        /* SettingsTreeNode model attributes */
        name : QDesk.Msg.get( 'QDesk.view.navigation.settings.SettingsTreePanel.nodeMessageChannelProfiles' ),
        licensed : false
      } );
    }
    else if ( licensedProductEdition == QDesk.enums.ProductEdition.STANDARD )
    {
      children.push( {
        id : 'messageChannelProfiles',
        cls : 'notLicensed',
        iconCls : 'messageChannelProfiles',
        leaf : true,

        /* SettingsTreeNode model attributes */
        name : QDesk.Msg.get( 'QDesk.view.navigation.settings.SettingsTreePanel.nodeMessageChannelProfiles' ),
        licensed : false
      } );
    }
    else if ( licensedProductEdition == QDesk.enums.ProductEdition.ENTERPRISE )
    {
      children.push( {
        id : 'messageChannelProfiles',
        cls : 'licensed',
        iconCls : 'messageChannelProfiles',
        leaf : true,

        /* SettingsTreeNode model attributes */
        name : QDesk.Msg.get( 'QDesk.view.navigation.settings.SettingsTreePanel.nodeMessageChannelProfiles' ),
        licensed : true
      } );
    }
    else
    {
      Ext.Error.raise( 'Unrecognized product edition: ' + licensedProductEdition );
    }

    // important to set the store before callParent as it expects the store!!!
    var store = Ext.create( 'Ext.data.TreeStore', {

      model : 'QDesk.model.SettingsTreeNode',

      root : {
        expanded : true,
        children : children
      }
    } );

    Ext.apply( config, {
      store : store
    } );

    me.callParent( [config] );
  }
} );
