/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * Tree selection combo box that is used to select the connection.
 *
 * Base on the code published at http://www.sencha.com/forum/showthread.php?38654-Tree-in-a-Combo./page6
 */
Ext.define( 'QDesk.view.widget.ConnectionCombo', {
  extend : 'Ext.ux.form.TreeIconCombo',

  requires : [
    'QDesk.Msg',
    'QDesk.store.ConnectionsTreeStore',
    'QDesk.util.constant.Const'
  ],

  alias : 'widget.ConnectionCombo',

  idValuePrefix : 'connection-',
  valueField : 'connectionId',
  displayField : 'name',
  iconClsField : 'iconCls',


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      emptyValueText : QDesk.Msg.get( 'QDesk.view.widget.ConnectionCombo.emptyValueText' ),
      valueNotFoundText : QDesk.Msg.get( 'QDesk.view.widget.ConnectionCombo.valueNotFoundText' ),
      loadFailedText : QDesk.Msg.get( 'QDesk.view.widget.ConnectionCombo.loadFailedText' ),
      loadingText : QDesk.Msg.get( 'QDesk.view.widget.ConnectionCombo.loadingText' ),

      root : {
        id : QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID,  // ROOT folder ID
        folderId : QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID,
        name : QDesk.Msg.get( 'QDesk.view.widget.ConnectionCombo.rootNodeName' ),
        expanded : true,
        nodeType : 'folder'
      },

      /**
       * The combo must use a store that loads the data eagerly. If the data is loaded lazily,
       * the setValue method may not be able to set value if the combo item is not in the
       * store (because it has not been loaded yet).
       */
      store : Ext.create( 'QDesk.store.ConnectionsTreeStore', {
        qdesk : {
          eagerLoad : true
        }
      } )
    } );

    Ext.apply( this, config );

    me.callParent( [ config ] );
  },


  /**
   * Returns the ID of the currently selected connection.
   * @return the connection ID.
   */
  getConnectionId : function ()
  {
    var me = this;
    var record = me.getSelectedRecord();
    return record == null ? null : record.get( 'id' );
  }
} );